@extends('appstore::user.layout.base')


@section('headCss')
    <title>App Store | Reset Password</title>
    <link rel="stylesheet" href="{{ asset('compiled/css/app.css') }}" />
    <link rel="stylesheet" href="{{ asset('static/fontawesome/css/fontawesome.min.css') }}">
    <link rel="stylesheet" href="{{ asset('static/fontawesome/css/solid.min.css') }}">
    @livewireStyles
@endsection

@section('headJs')
    <script defer src="https://unpkg.com/alpinejs@3.x.x/dist/cdn.min.js"></script>
@endsection

@section('body')
    <body class="relative m-0 p-0">
        @include('appstore::user.logo-title')

        <div class="flex justify-center mx-auto lg:py-0 mt-8">
            <div class="rounded-3xl bg-white w-full h-auto max-w-lg">
                <div class="pb-8">
                    @if($token != null)
                        <form action="{{ route('password.update') }}" method="POST">
                            @csrf
                            <input type="hidden" name="token" value="{{ $token }}">
                            <input type="hidden" name="email" value="{{ request()->get('email') }}">

                            <div class="flex justify-center flex-col w-full">
                                <div>
                                    <h1 class="rounded-tl-3xl rounded-tr-3xl text-center text-white text-xl w-full p-2 bg-{{ config('appstore.frontend.secondaryColor') }}">Imposta Password</h1>
                                    @error('password')<p class="text-red-700 text-center p-4 bg-red-100">{{ $message }}</p> @enderror

                                    <div class="px-12 mt-8">
                                        <div class="mt-2 relative" x-data="{passwordVisible: false}">
                                            <label for="">Nuova Password:</label>
                                            <input :type="passwordVisible ? 'text' : 'password'"  class="login__input input input--lg border border-gray-300 block w-full"
                                                   placeholder="Nuova Password" name="password" />
                                            <i class="fa-solid fa-eye absolute right-3 top-8" x-show="!passwordVisible" @click="passwordVisible = true"></i>
                                            <i class="fa-solid fa-eye-slash absolute right-3 top-8" x-show="passwordVisible" @click="passwordVisible = false"></i>
                                        </div>
                                        <div class="mt-2 relative" x-data="{passwordVisible: false}">
                                            <label for="">Conferma Password:</label>
                                            <input :type="passwordVisible ? 'text' : 'password'" class="login__input input input--lg border border-gray-300 block w-full"
                                                   placeholder="Conferma Password" name="password_confirmation"/>
                                            <i class="fa-solid fa-eye absolute right-3 top-8" x-show="!passwordVisible" @click="passwordVisible = true"></i>
                                            <i class="fa-solid fa-eye-slash absolute right-3 top-8" x-show="passwordVisible" @click="passwordVisible = false"></i>
                                        </div>
                                        <button class="button button--lg text-white bg-{{ config('appstore.frontend.primaryColor') }} align-top w-full mt-8">SALVA</button>
                                    </div>
                                </div>
                            </div>
                        </form>
                    @else
                        <div class="px-12">
                            <h1 class="text-center text-black text-xl w-full my-8">{{ $errorMsg }}</h1>
                            <a href="{{ route('appstore.login') }}"><button class="button button--lg text-white bg-{{ config('appstore.frontend.primaryColor') }} align-top w-full mt-4">TORNA AL LOGIN</button></a>
                        </div>
                    @endif
                </div>
            </div>
        </div>

        @livewireScripts
    </body>
@endsection