<?php

namespace FourWords\AppManager\Appstore\Controllers;

use FourWords\AppManager\Appstore\Models\AppstoreApp;
use FourWords\AppManager\Appstore\Models\AppstoreVersion;
use FourWords\AppManager\Ioc\Events\BonjourPayload;
use FourWords\AppManager\Ioc\Models\IocUser;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Auth;

class APIAppstoreController extends Controller {
    public function info() {
        if(!request()->has('app')) {
            abort(400);
        }

        $app = AppstoreApp::where('active', true)->find(request('app'));
        if($app == null) {
            abort(404);
        }

        $history = [];
        $versions = $app->appstore_version()->where('show_in_history', '=', true)
            ->orderBy('version', 'desc')->get();
        $versionNotActive = $versions->filter(function($value, $key) {
            return !$value->active;
        });
        foreach($versionNotActive as $v) {
            array_push($history, [
                'news' => $v->changelog,
                'version' => $v->version,
                'releasedate' => (string) strtotime($v->created_at)
            ]);
        }

        $appVersion = $versions->first(function($value, $key) {
            return $value->active;
        });

        if(!$appVersion) {
            abort(503);
        }

        $response = [
            'icon' => asset('storage/logo/'.$app->icon),
            'news' => $appVersion->changelog,
            'url' => route('appstore.show', request('app')),
            'version' => $appVersion->version,
            'releasedate' => (string) strtotime($app->created_at),
            'size' => $appVersion->dimension,
            'description' => $app->description,
            'history' => $history
        ];

        return response()->json($response);
    }
}