<?php

namespace FourWords\AppManager\Appstore\Controllers;

use Detection\MobileDetect;
use FourWords\AppManager\Appstore\Models\AppstoreApp;
use FourWords\AppManager\Ioc\Controllers\IocController;
use Illuminate\Support\Facades\Auth;

class AppstoreController extends IocController {

    public function login() {
        $app = $this->getApp();
        if (count($app) == 0) {
            return view('appstore::user.404-app');
        }

        if (Auth::guard('iocweb')->check()) {
            return redirect(route('appstore.index'));
        }

        return view('appstore::user.login', ['app' => $app]);
    }

    public function signup() {
        $app = $this->getApp();
        if (count($app) == 0) {
            return view('appstore::user.404-app');
        }

        if (Auth::guard('iocweb')->check()) {
            return redirect(route('appstore.index'));
        }

        return view('appstore::user.signup');
    }

    public function index() {
        $app = $this->getApp();
        if (count($app) == 0) {
            return view('appstore::user.404-app');
        }

        if(count($app) == 1) {
            return redirect(route('appstore.show', $app->first()->id . '?token='.request()->get('token')));
        }
        return view('appstore::user.index', ['app' => $app]);
    }

    public function show($appId) {
        if($app = AppstoreApp::getAppById($appId)) {
            return view('appstore::user.download', ['app' => $app, 'active' => true]);
        } else {
            return view('appstore::user.download', ['app' => null, 'active' => false]);
        }
    }

    public function getDeviceOS() {
        $detect = new MobileDetect;
        if($detect->isAndroidOS()) {
            return "Android";
        } else if($detect->isiOS()) {
            return "iOS";
        }
        return null;
    }

    private function getApp() {
        $app = AppstoreApp::getAppByOS(request()->has('app') ? (request()->get('app') == '1' ? 'iOS' : 'Android') : $this->getDeviceOS())->where("active", "=", "1");
        if($app->first()) {
            $appActive = [];
            foreach ($app as $a) {
                if(AppstoreApp::checkVersions($a->id)) {
                    $appActive[] .= $a->id;
                }
            }
            if(count($appActive) > 0) {
                return AppstoreApp::all()->find($appActive);
            }
        }
        return AppstoreApp::orderBy('type', 'asc')->where("active", "=", "1")->get();
    }

    public function sendEmailResetPassword() {

    }

    public function logout() {
        Auth::guard('iocweb')->logout();
        return redirect(route('appstore.login'));
    }
}
