<?php

namespace FourWords\AppManager\Appstore\Controllers;

use FourWords\AppManager\Appstore\Mail\SignUp;
use FourWords\AppManager\Appstore\Models\RegistrationEmail;
use FourWords\AppManager\Ioc\Models\IocUser;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;

class SignupController extends Controller {
    public function webSendSignUpLink($pIva, $customerCode, $email) {
        $statusJSON = 'OK';
        $errorJSON = '';

        $columnVat = 'pIva';
        $user = $this->getIocUser($customerCode, $pIva, $columnVat);

        if(!$user->id) {
            $statusJSON = 'ERROR';
            $errorJSON = 'Nessun account è associato con i seguenti dati inseriti';
            session()->flash('appstore_loginErrorMsg', $errorJSON);
        }

        if($user->email) {
            $statusJSON = 'ERROR';
            $errorJSON = 'Questo account è già registrato';
            session()->flash('appstore_loginErrorMsg', $errorJSON);
        }

        if($statusJSON == 'OK') {
            try {
                if(!$this->sendSignupLinkViaMail($customerCode, $email, $user)) {
                    $statusJSON = 'ERROR';
                    $errorJSON = 'La mail di registrazione è già stata inviata, attendi almeno 15 minuti per inviarla di nuovo';
                    session()->flash('appstore_loginErrorMsg', $errorJSON);
                }
            } catch (\Exception $e) {
                Log::error($e->getMessage());
                telegramError($e->getMessage());
                $statusJSON = 'ERROR';
                $errorJSON = 'Errore nell\'invio della mail';
                session()->flash('appstore_loginErrorMsg', $errorJSON);
            }
        }

        return response()
            ->json(['customerCode' => $customerCode, $columnVat => $pIva, 'email' => $email, 'status' => $statusJSON, 'message' => $errorJSON])
            ->header("Content-Type", 'application/json')
            ->header("Accept", 'application/json');
    }

    public function create($code) {
        if($this->getIocUserBycode($code)->email || !RegistrationEmail::where('email', '=', request()->get('email'))->first()) {
            $code = null;
        }

        return view('appstore::user.signup', ['code' => $code]);
    }

    public function store(Request $request)
    {
        $request->validate([
            'email' => 'required|email',
            'code' => 'required',
            'password' => 'required|min:4|confirmed'
        ]);

        $request->only('email', 'password', 'password_confirmation', 'code');

        try {
            $user = $this->getIocUserBycode($request->get('code'));
            $user->email = $request->get('email');
            $user->forceFill([
                'password' => $request->get('password')
            ]);
            $user->active = true;
            $user->save();

            RegistrationEmail::where('iocuser_id', '=', $user->id)->delete();

        } catch (\Exception $e) {
            return back()->withErrors(['password' => 'Errore del sistema, contattare l\'amministratore']);
        }

        Auth::guard('iocweb')->attempt(['username' => $user->username, 'password' => $request->get('password')]);

        return redirect()->route('appstore.login');
    }

    private function sendSignupLinkViaMail($customerCode, $email, $user) {
        $exist = false;
        if($signup = RegistrationEmail::where('email', '=', $email)->first()) {
            $exist = true;

            $dbtimestamp = strtotime($signup->created_at);
            if (time() - $dbtimestamp < 15 * 60) {
                return false;
            }

            $signup->created_at = date('Y-m-d H-i-s');
            $signup->save();
        }

        $signup = app()->make(SignUp::class, [$customerCode, $email, $user->name]);
        Mail::to($email)->send($signup);

        if(!$exist) {
            $signup = new RegistrationEmail();
            $signup->iocuser_id = $user->id;
            $signup->email = $email;
            $signup->created_at = date('Y-m-d H-i-s');
            $signup->save();
        }

        session()->flash('appstore_loginSuccessMsg', 'E stata inviata via email il link per procedere con la registrazione');
        return true;
    }

    protected function getIocUser($customerCode, $pIva, &$columnVat = 'pIva') {
        $user = IocUser::where('code', '=', $customerCode)->where('vat_code', '=', $pIva)->first();
        if(!$user->id) {
            $user = IocUser::where('code', '=', $customerCode)->where('fiscal_code', '=', $pIva)->first();
            $columnVat = 'fiscalCode';
        }
        return $user;
    }

    protected function getIocUserBycode($customerCode) {
        return IocUser::where('code', '=', $customerCode)->first();
    }

}