<?php

namespace FourWords\AppManager\Appstore\Livewire\Admin;

use FourWords\AppManager\Appstore\Models\AppstoreType;
use FourWords\AppManager\Ioc\Livewire\IocComponent;
use FourWords\AppManager\Appstore\Models\AppstoreApp;
use FourWords\AppManager\Appstore\Models\AppstoreVersion;
use Illuminate\Validation\ValidationException;
use Livewire\WithFileUploads;

class CreateVersion extends IocComponent
{
    use WithFileUploads;

    public $app;
    public $showModal = false;
    public $randId = 0;

    private $appExtension;

    public $versionInfo = [
        'version' => '1.0.0',
        'file' => '',
        'dimension' => '',
        'changelog' => '',
        'showInHistory' => true,
        'active' => false
    ];

    protected function rules() {
        $rules = ['versionInfo.version' => 'nullable|regex:/^[0-9.]+$/'];
        if(!$this->app->abm_active) {
            $rules += ['versionInfo.file' => 'required'];
        }
        return $rules;
    }

    protected $validationAttributes = [
        'versionInfo.file' => 'file'
    ];

    protected $listeners = [
        'onCreateVersion'
    ];

    public function render() {
        return view('appstore::admin.livewire.create-version');
    }

    public function createApp() {
        $this->validate();

        $version = $this->versionDB($this->app);
        if($version != null) {
            $this->successToast('Versione dell\'app creata con successo');
            $this->emit('refreshVersion', $version->id);

            $this->showModal = false;
            $this->resetFields();
        }
    }

    private function versionDB($app) {
        if($this->versionInfo['version'] != null and $app != null) {
            try {
                $version = new AppstoreVersion();
                $version->app = $app->id;

                if(!AppstoreVersion::where("app", "=", $version->app)->where("version", "=", $this->versionInfo['version'])->first()) {
                    $version->version = $this->versionInfo['version'];

                    
                    if(!$this->checkFile() && !$app->abm_active) {
                        return null;
                    }
                    
                    if(!$app->abm_active) {
                        $version->file = $this->uploadFile($app);
                        $version->dimension = $this->fileSizeConvertMB($this->versionInfo['file']->getSize());
                    }

                    $version->changelog = $this->versionInfo['changelog'];
                    $this->disableAllVersions($this->versionInfo['active']);
                    $version->active = $this->versionInfo['active'];
                    $version->show_in_history = $this->versionInfo['showInHistory'];

                    $version->save();
                    return $version;
                } else {
                    //$this->showModal = false;
                    $this->errorToast('Questa versione già esiste');
                    return null;
                }
            } catch(\Exception $e) {
                return null;
            }
        }
        return null;
    }

    public function onCreateVersion() {
        $this->showModal = true;
    }

    private function checkFile() {
        if($this->versionInfo['file'] == null) {
            return false;
        }

        $this->appExtension = AppstoreType::find($this->app->type)->extension;
        $fileNameToGetExtension = explode('.',$this->versionInfo['file']->getClientOriginalName());
        if(!in_array($fileNameToGetExtension[count($fileNameToGetExtension) - 1], [$this->appExtension])) {
            $this->errorToast('Il file dell\'app deve essere di tipo .'.$this->appExtension);
            return false;
        }

        return true;
    }

    private function uploadFile($app) {
        $filename = $app->name.'_'.$this->versionInfo['version'].'.'.$this->appExtension;

        $this->versionInfo['file']->storeAs('file', $filename, 'public');
        return $filename;    
    }

    private function disableAllVersions($versionField) {
        if($versionField) {
            $this->app->appstore_version()->where('active', true)->update(['active' => false]);
        }
    }

    public function resetFields() {
        $this->randId++;
        $this->resetErrorBag();
        $this->versionInfo = [
            'version' => '1.0.0',
            'file' => '',
            'dimension' => '',
            'changelog' => '',
            'showInHistory' => true,
            'active' => false
        ];
    }

    private function fileSizeConvertMB($bytes) {
        return number_format($bytes / 1048576, 2) . ' MB';
    }
}
