<?php

namespace FourWords\AppManager\Appstore\Livewire\Admin;

use FourWords\AppManager\Appstore\Models\AppstoreVersion;
use FourWords\AppManager\Appstore\Models\AppstoreApp;
use FourWords\AppManager\Appstore\Models\AppstoreType;
use FourWords\AppManager\Ioc\Livewire\IocComponent;
use Livewire\WithFileUploads;

class FormVersionEdit extends IocComponent {
    use WithFileUploads;

    public $randId = 1;

    public $appVersion;
    public $abm;
    public $versionInfo = [
        'version' => '',
        'file' => '',
        'showFile' => '',
        'dimension' => '',
        'changelog' => '',
        'active' => '',
        'showInHistory' => ''
    ];

    protected $validationAttributes = [
        'versionInfo.file' => 'file'
    ];

    public function mount() {
        AppstoreApp::find($this->appVersion->app)->first()->name;
        $this->versionInfo['version'] = $this->appVersion->version;
        $this->versionInfo['showFile'] = $this->appVersion->file;
        $this->versionInfo['dimension'] = $this->appVersion->dimension;
        $this->versionInfo['changelog'] = $this->appVersion->changelog;
        $this->versionInfo['active'] = $this->appVersion->active;
        $this->versionInfo['showInHistory'] = $this->appVersion->show_in_history;
    }

    public function render() {
        return view('appstore::admin.livewire.form-version-edit');
    }

    public function update() {
        if(!$this->checkPermission('edit-appstore-version')) {
            return;
        }

        try {
            if($this->appDB()) {
                $this->successToast('La versione dell\'app è stata aggiornata con successo');
                //$this->emit('hideFormVersion');
                $this->emit('refreshVersion');
                $this->refreshUploadFile();
            }
        } catch (\Exception $e) {
            $this->errorToast($e->getMessage());
        }
    }

    public function delete() {
        if(!$this->appVersion->active || !$this->versionInfo['active']) {
            $vId = $this->appVersion->id;
            $this->appVersion->delete();
            $this->emit('closeDropdowns');
            $this->emit('deleteVersion', $vId);
            $this->successToast('La versione dell\'app è stata eliminata con successo');
        } else {
            $this->errorToast('Prima di eliminare questa versione rimuovi la versione corrente o assegnala ad un\'altra versione');
        }
    }

    private function appDB() {
        if(!AppstoreVersion::where("app", "=", $this->appVersion->app)->where("version", "=", $this->versionInfo['version'])->first() || $this->appVersion->version == $this->versionInfo['version']) {
            $this->appVersion->version = $this->versionInfo['version'];

            if(!$this->uploadFile()) {
                return false;
            }

            if($this->versionInfo['file'] != null) {
                $this->appVersion->dimension = $this->fileSizeConvertMB($this->versionInfo['file']->getSize());
            } else {
                $this->appVersion->dimension = $this->versionInfo['dimension'];
            }

            $this->appVersion->changelog = $this->versionInfo['changelog'];
            $this->appVersion->show_in_history = $this->versionInfo['showInHistory'];

            $this->disableAllVersions($this->versionInfo['active']);
            $this->appVersion->active = $this->versionInfo['active'];

            $this->appVersion->save();
            return true;
        } else {
            $this->errorToast('Questa versione già esiste');
            return null;
        }
    }

    private function disableAllVersions($versionField) {
        if($versionField && !$this->appVersion->active) {
            AppstoreApp::find($this->appVersion->app)->appstore_version()->where('active', true)->update(['active' => false]);
        }
    }

    private function uploadFile() {
        if($this->versionInfo['file'] != null) {
            if($this->appVersion->file == null) {
                $this->errorToast('File dell\'app mancante');
                return false;
            }

            $appExtension = AppstoreType::find(AppstoreApp::find($this->appVersion->app)->type)->extension;
            $fileNameToGetExtension = explode('.',$this->versionInfo['file']->getClientOriginalName());
            if(!in_array($fileNameToGetExtension[count($fileNameToGetExtension) - 1], [$appExtension])) {
                $this->errorToast('Il file dell\'app deve essere di tipo .'.$appExtension);
                return false;
            }
            
            $appName = AppstoreApp::find($this->appVersion->app)->name;
            $filename = $appName.'_'.$this->appVersion->version.'.'.$appExtension;

            $this->versionInfo['file']->storeAs('file', $filename, 'public');
            $this->appVersion->file = $filename;

            $this->versionInfo['showFile'] = $filename;
        }
        return true;
    }

    private function fileSizeConvertMB($bytes) {
        return number_format($bytes / 1048576, 2) . ' MB';
    }

    private function refreshUploadFile() {
        $this->versionInfo['file'] = '';
        //Per svuotare l'input file serve modificare il suo ID
        //Assegno $randId e lo incremento ogni volta che devo svuotare l'input
        $this->randId++; 
    }
}
