<?php

namespace FourWords\AppManager\Appstore\Livewire\User;

use Detection\MobileDetect;
use Livewire\Component;
use FourWords\AppManager\Appstore\Models\AbmCode;
use FourWords\AppManager\Appstore\Models\AbmCodeRedeemed;
use \Illuminate\Support\Facades\Auth;

class AppstoreAbmDownload extends Component
{
    public $showModal = false;
    public $abmCodeRequested = false;
    public $tooManyAbmRequest = false;
    
    public $app;
    public $userAbmCode;
    public $n_abm_redeemed;

    protected $listeners = ['openAppstoreScreen'];

    public function openAppstoreScreen() {
        $this->showModal = true;
    }

    private function getUserAbmCode() {
        $userId = Auth::guard('iocweb')->user()->id;

        $this->checkAbmRequests();
        
        $this->userAbmCode = AbmCode::where("user_id", "=", $userId)->where("appstore_app_id", "=", $this->app->id)->orderBy("allocation_date", "desc")->first()->code;
        if($this->userAbmCode == null || $this->n_abm_redeemed == 0) {
            $this->requestNewAbmCode();
        }
    }

    public function requestNewAbmCodeButton() {
        $this->requestNewAbmCode();
        $this->abmCodeRequested = true;
    }

    private function requestNewAbmCode() {
        $this->userAbmCode = AbmCode::where("user_id", "=", null)->where("appstore_app_id", "=", $this->app->id)->first()->code;
        if($this->userAbmCode != null || $this->n_abm_redeemed == 0) {
            $this->claimAbmCode();
        }
    }

    public function mount() {
        $this->getUserAbmCode();
    }

    public function render() {
        return view('appstore::user.livewire.abm-download');
    }

    private function claimAbmCode() {
        $userId = Auth::guard('iocweb')->user()->id;

        $abm = AbmCode::where("user_id", "=", null)->where("appstore_app_id", "=", $this->app->id)->first();
        $abm->user_id = $userId;
        $abm->allocation_date = time();
        $abm->save();

        $abm_redeemed = AbmCodeRedeemed::where("user_id", "=", $userId)->where("appstore_app_id", "=", $this->app->id)->first();
        
        if($abm_redeemed == null) {
            $abm_redeemed = new AbmCodeRedeemed();
            $abm_redeemed->user_id = $userId;
            $abm_redeemed->appstore_app_id = $this->app->id;
        }
        
        $abm_redeemed->codes_redeemed += 1;
        $abm_redeemed->save();
    }

    private function checkAbmRequests() {
        $this->n_abm_redeemed = AbmCodeRedeemed::countCodesRedeemed($this->app->id);
        if($this->n_abm_redeemed >= config('appstore.user.maxAbmCodeRedeemed')) {
            $this->tooManyAbmRequest = true;
        }
    }
}
