<?php

namespace FourWords\AppManager\Appstore\Livewire\User;

use FourWords\AppManager\Appstore\Controllers\SignupController;
use FourWords\AppManager\Ioc\Controllers\PasswordController;
use FourWords\AppManager\Ioc\Models\IocUser;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Password;
use Livewire\Component;

class AppstoreLogin extends Component {
    public $app;
    public $appSelected;


    private $errorMessage;
    public $submitted = false;

    public $credentials = [
        'username' => '',
        'password' => '',
    ];

    public $email;

    public
        $pIva,
        $customerCode,
        $signupEmail;

    public $reset = [
        'token' => '',
        'email' => '',
        'new_password' => '',
        'confirm_password' => ''
    ];

    protected $rules = [
        'credentials.username' => 'required|min:3',
        'credentials.password' => 'required|min:3',

        /*'reset.token' => 'required',
        'reset.email' => 'required|email',
        'reset.new_password' => 'required|min:8|confirmed',
        'reset.confirm_password' => 'required|min:8|confirmed'*/
    ];
    protected $validationAttributes = [
        'credentials.username' => 'username',
        'credentials.password' => 'password',
    ];

    protected $listeners = ['updatePassword'];

    public function loginSubmit() {

        $this->submitted = true;
        $this->errorMessage = '';

        $this->validate();
        if(Auth::guard('iocweb')->attempt($this->credentials)) {
            redirect(route('appstore.show', $this->appSelected));
        } else {
            $this->errorMessage = 'Credenziali non corrette';
            session()->flash('appstore_loginErrorMsg',$this->errorMessage);
        }
    }

    public function downloadPage() {
        return redirect(route('appstore.show', $this->appSelected));
    }

    public function updated($name) {
        if($this->submitted) {
            $this->validateOnly($name);
            $this->errorMessage = '';
        }
    }

    public function mount() {
        $this->selectApp();
    }

    public function render() {
        return view('appstore::user.livewire.login');
    }

    private function selectApp() {
        try {
            $this->appSelected = $this->app->id;
        } catch (\Exception $e) {
            $this->appSelected = $this->app->first()->id;
        }
    }

    public function resetPassword() {
        $validated = $this->validate([
            'email' => 'required|email',
        ]);

        $status = Password::sendResetLink(
            ['email' => $this->email]
        );

        if($status == "passwords.sent") {
            $this->errorMessage = 'E stata inviata via email il link per resettare la sua password';
            session()->flash('appstore_loginSuccessMsg',$this->errorMessage);
        } elseif ($status == "passwords.user") {
            $this->errorMessage = 'Nessun account è associato a questa email';
            session()->flash('appstore_loginErrorMsg',$this->errorMessage);
        } else {
            $this->errorMessage = 'Qualcosa è andato storto, conttattare l\'amministratore';
            session()->flash('appstore_loginErrorMsg',$this->errorMessage);
        }

        return $status === Password::RESET_LINK_SENT
                    ? back()->with(['status' => __($status)])
                    : back()->withErrors(['email' => __($status)]);
    }

    public function sendEmailResetPassword() {
        $validated = $this->validate([
            'email' => 'required|email',
        ]);

        $controller = app()->make(PasswordController::class);
        $controller->webSendResetLink($this->email);
    }

    public function sendEmailSignup() {
        $validated = $this->validate([
            'signupEmail' => 'required|email',
            'pIva' => 'required',
            'customerCode' => 'required'
        ]);

        $controller = app()->make(SignupController::class);
        $controller->webSendSignupLink($this->pIva, $this->customerCode, $this->signupEmail);
    }

    private function checkEmailExists() {
        if(!IocUser::where('email', '=', $this->credentials['email'])->first()->id) {
            return false;
        }

        return true;
    }

    public function resetError() {
        $this->resetErrorBag();
        session()->remove('appstore_loginErrorMsg');
        session()->remove('appstore_loginSuccessMsg');
    }
}
