<?php

namespace FourWords\AppManager\Appstore\Providers;

use FourWords\AppManager\Appstore\Controllers\SignupController;
use FourWords\AppManager\Appstore\Livewire\Admin\AbmLimitUserComponent;
use FourWords\AppManager\Appstore\Livewire\Admin\AppAbmLimitUser;
use FourWords\AppManager\Appstore\Livewire\Admin\AppInfo;
use FourWords\AppManager\Appstore\Livewire\Admin\AppList;
use FourWords\AppManager\Appstore\Livewire\Admin\AppName;
use FourWords\AppManager\Appstore\Livewire\Admin\AppVersion;
use FourWords\AppManager\Appstore\Livewire\Admin\CreateApp;
use FourWords\AppManager\Appstore\Livewire\Admin\CreateVersion;
use FourWords\AppManager\Appstore\Livewire\Admin\FormAppEdit;
use FourWords\AppManager\Appstore\Livewire\Admin\FormVersionEdit;
use FourWords\AppManager\Appstore\Livewire\Admin\VersionComponent;
use FourWords\AppManager\Appstore\Livewire\User\AppstoreAbmDownload;
use FourWords\AppManager\Appstore\Livewire\User\AppstoreBackLogout;
use FourWords\AppManager\Appstore\Livewire\User\AppstoreDownload;
use FourWords\AppManager\Appstore\Livewire\User\AppstoreLogin;
use FourWords\AppManager\Appstore\Mail\SignUp;
use FourWords\AppManager\Appstore\Middleware\AppstoreAuthMiddleware;
use FourWords\AppManager\Ioc\Controllers\PasswordController;
use FourWords\AppManager\Ioc\Events\DidCopyFrontendFiles;
use Illuminate\Support\Facades\Event;
use Illuminate\Support\Facades\File;
use Illuminate\Support\ServiceProvider;
use Livewire\Livewire;

class AppstoreServiceProvider extends ServiceProvider {
    public function register() {
        $this->app->bind(PasswordController::class, function ($app, $params) {
            return new PasswordController();
        });

        $this->app->bind(SignupController::class, function ($app, $params) {
            return new SignupController();
        });

        $this->app->bind(SignUp::class, function ($app, $params) {
            return new SignUp(...$params);
        });
    }

    public function boot() {

        app('router')->aliasMiddleware('appstore-auth', AppstoreAuthMiddleware::class);

        //Migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');
        
        //Routes
        $this->loadRoutesFrom(__DIR__ . '/../../routes/routes.php');

        //Views
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'appstore');

        // Includo le configurazioni
        config(require __DIR__ . '/../../config/appstore.php');

        //Livewire Components
        $this->registerLivewireComponents();

        Event::listen(DidCopyFrontendFiles::class, function (DidCopyFrontendFiles $event) {
            $this->updateFrontend();
        });
    }

    public function updateFrontend()
    {

        /*
        |--------------------------------------------------------------------------
        | Logo app di default
        |--------------------------------------------------------------------------
        */
        $sourceFolder = __DIR__ . '/../../resources/appstore-logo';
        $destinationFolder = storage_path('app/public/logo');

        if (!File::isDirectory($destinationFolder)) {
            File::makeDirectory($destinationFolder, 0770, true, true);
        }

        File::copy($sourceFolder.'/logo-app.png', $destinationFolder.'/logo-app1.png');
        File::copy($sourceFolder.'/logo-app.png', $destinationFolder.'/logo-app2.png');

    }

    protected function registerLivewireComponents() {
        //Admin
        Livewire::component('admin.app-list', AppList::class);
        Livewire::component('admin.create-app', CreateApp::class);
        Livewire::component('admin.app-name', AppName::class);
        Livewire::component('admin.version-component', VersionComponent::class);
        Livewire::component('admin.app-version', AppVersion::class);
        Livewire::component('admin.app-info', AppInfo::class);
        Livewire::component('admin.form-app-edit', FormAppEdit::class);
        Livewire::component('admin.form-version-edit', FormVersionEdit::class);
        Livewire::component('admin.create-version', CreateVersion::class);
        Livewire::component('admin.app-abm-limit-user', AppAbmLimitUser::class);
        Livewire::component('admin.abm-limit-user-component', AbmLimitUserComponent::class);

        //User
        Livewire::component('user.appstore-login', AppstoreLogin::class);
        Livewire::component('user.appstore-download', AppstoreDownload::class);
        Livewire::component('user.appstore-back-logout', AppstoreBackLogout::class);
        Livewire::component('user.abm-download', AppstoreAbmDownload::class);
    }
}