<?php

namespace FourWords\AppManager\Catalog\Images;
use Exception;
use Illuminate\Support\Facades\File;
use Intervention\Image\ImageManagerStatic as ImageResizer;
use SplFileInfo;

class ImageManager
{
    public $imageResizingAttributes;
    private $sourcePath;
    private $destinationPath;
    private $filter;
    private $filterClosure;

    private $dryRun = false;

    public function __construct($sourcePath = null) {
        $this->sourcePath = $sourcePath;
        $this->destinationPath = $sourcePath;

        // https://image.intervention.io/v3/modifying/resizing
        ImageResizer::configure(['driver' => 'imagick']);

        $this->imageResizingAttributes = new ImageResizingAttributes();
        //$this->imageResizingAttributes->setAttributes(800, 800, 'jpeg', 75);
    }

    public function getSourcePath() {return $this->sourcePath;}
    public function getDestinationPath() {return $this->destinationPath;}
    public function info() {return $this->imageResizingAttributes;}

    public function setSourcePath($path) {$this->sourcePath = $path;}
    public function setDestinationPath($path) {$this->destinationPath = $path;}

    public function width($value) {$this->imageResizingAttributes->setWidth($value); return $this;}
    public function heigth($value) {$this->imageResizingAttributes->setHeigth($value); return $this;}
    public function format($value) {$this->imageResizingAttributes->setFormat($value); return $this;}
    public function quality($value) {$this->imageResizingAttributes->setQuality($value); return $this;}
    public function resolution($width, $heigth) {$this->imageResizingAttributes->setWidth($width); $this->imageResizingAttributes->setHeigth($heigth); return $this;}
    public function destinationPath($path) {$this->setDestinationPath($path); return $this;}
    public function filterFiles($pattern) {$this->filter = $pattern; return $this;}
    public function filterClosure($closure) {$this->filterClosure = $closure; return $this;}



    public function getFiles($rootOnly = true) {
        if(!File::isDirectory($this->destinationPath)){
            File::makeDirectory($this->destinationPath, 0755, true, true);
        }

        if($this->filter) {
            $filteredFiles = [];

            $files = File::glob($this->sourcePath.'/*');
            foreach($files as $file) {
                if(preg_match($this->filter, basename($file))) {
                    array_push($filteredFiles, new SplFileInfo($file));
                }
            }

            return $filteredFiles;
        }

        if($rootOnly) {
            return File::files($this->sourcePath);
        }

        return File::allFiles($this->sourcePath);
    }

    public function resizeImages($files) {

        $counter = 1;
        foreach($files as $file) {
            $imageResizingAttributes = $this->imageResizingAttributes;
            if(is_callable($this->filterClosure)) {
                $imageResizingAttributes = ($this->filterClosure)($file, $this->imageResizingAttributes);
            }

            $this->resizeOneImage($file, $imageResizingAttributes);

            if ($this->dryRun && $counter >= 50) break;
            $counter += 1;

        }
    }

    public function resizeOneImage($file, $imageResizingAttributes) {
        $img = ImageResizer::make($file);

        $originalWidth = $img->getWidth();
        $originalHeight = $img->getHeight();

        // Se non ho impostato la qualità, ridimensiono solo se necessario
        if (!$this->imageResizingAttributes->getQuality() && ($this->imageResizingAttributes->getWidth() || $this->imageResizingAttributes->getHeigth())) {
            if ($originalWidth <= $this->imageResizingAttributes->getWidth() && $originalHeight <= $this->imageResizingAttributes->getHeigth()) {
                return;
            }
        }

        // Imposto larghezza e altezza
        $this->setResolution($img);

        $filename = explode('.'.$file->getExtension(), $file->getFilename())[0];
        $img->save($this->dryRun ? $this->destinationPath.'/DRYRUN_'.$filename.'.'.($imageResizingAttributes->getFormat() ?? $file->getExtension()) : $this->destinationPath.'/'.$filename.'.'.($imageResizingAttributes->getFormat() ?? $file->getExtension()), $imageResizingAttributes->getQuality(), $imageResizingAttributes->getFormat());
    }

    public function dryRun()
    {
        $this->dryRun = true;
        return $this->start();
    }

    public function start() {

        if (!$this->getSourcePath()) throw new Exception("Path sorgente non definito!");

        $this->resizeImages($this->getFiles());
        return true;
    }

    protected function setResolution(&$img) {
        $width = $this->imageResizingAttributes->getWidth();
        $height = $this->imageResizingAttributes->getHeigth();

        if ($width || $height) {
            $img->resize($width, $height, function ($constraint) {
                $constraint->aspectRatio(); // Mantengo le proporzioni
                $constraint->upsize(false); // Evito l'upsizing se una foto è più piccola
            });
        }

    }
}