<?php


namespace FourWords\AppManager\Catalog\Importer;

use DateTimeZone;
use Monolog\Formatter\LineFormatter;
use Monolog\Handler\RotatingFileHandler;
use Monolog\Logger;

class CatalogImporter
{

    /** @var AgentImporter Importo i nomi degli agenti */
    protected $agentImporter;

    /** @var AttributesValuesImporter Importo i valori delle classifiche */
    protected $avImporter;

    /** @var ProductImporter Importo i prodotti con tutte le codifiche delle classifiche, più unità di misura, pezzi per confezione, iva e min_qta */
    protected $productImporter;

    /** @var CategoryImporter Creo una o più alberature di categorie dalle classifiche scelte M:M
     *
     * Category (id, code, name, parent_id)
     * CategoryProduct (category_id, product_id)
     *
     * */
    protected $categoryImporter;

    /** @var FeatureImporter Creo delle caratteristiche da collegare ai prodotti M:M
     *
     * Feature (id, code, name)
     * FeatureProduct (feature_id, product_id, value)
     */
    protected $featureImporter;

    /** @var CustomerFeatureImporter Creo delle caratteristiche da collegare ai clienti M:M
     *
     * CustomerFeature (id, code, name)
     * CustomerHasFeature (feature_id, customer_id, value)
     */
    protected $customerFeatureImporter;

    /** @var Logger */
    public $logger;

    public function __construct(AgentImporter $agentImporter, AttributesValuesImporter $avImporter, ProductImporter $productImporter, CategoryImporter $categoryImporter, FeatureImporter $featureImporter, CustomerFeatureImporter $customerFeatureImporter)
    {
        $this->agentImporter = $agentImporter;
        $this->avImporter = $avImporter;
        $this->productImporter = $productImporter;
        $this->categoryImporter = $categoryImporter;
        $this->featureImporter = $featureImporter;
        $this->customerFeatureImporter = $customerFeatureImporter;

        // Log
        $this->logger = new Logger('catalog-importer');
        $this->logger->setTimezone(new DateTimeZone('Europe/Rome'));

        if (!file_exists(config('catalog.log.dir'))) {
            mkdir(config('catalog.log.dir'),config('catalog.log.dirPermissions'), true);
        }

        // debug.log
        $LdebugHandler = new RotatingFileHandler(config('catalog.log.dir').'debug.log',14,Logger::DEBUG, true, config('catalog.log.filePermissions'));
        $debugDateFormat = "Y-m-d H:i:s";
        $debugOutput = "[%datetime%] %channel%.%level_name% : %message% \n";
        $debugFormatter = new LineFormatter($debugOutput, $debugDateFormat,true);
        $LdebugHandler->setFormatter($debugFormatter);
        $this->logger->pushHandler($LdebugHandler);
    }

    public function pushHandler($handler) {
        $this->logger->pushHandler($handler);
    }

    public function import() {
        $this->agentImporter->import();
        $this->avImporter->import();
        $this->productImporter->import();
        $this->categoryImporter->import();
        $this->featureImporter->import();
        $this->customerFeatureImporter->import();
    }

}
