<?php


namespace FourWords\AppManager\Catalog\Importer;

use Exception;
use FourWords\AppManager\Catalog\Model\AttributeValue;
use FourWords\AppManager\Catalog\Model\Category;
use FourWords\AppManager\Catalog\Model\CategoryProduct;
use FourWords\AppManager\Catalog\Model\Product;
use FourWords\AppManager\DataExtractor\DataExtractor;
use Illuminate\Support\Facades\DB;

class CategoryImporter extends SyncableAbstractImporter
{

    protected $productsCategory;

    public function __construct(Category $model, DataExtractor $dataExtractor)
    {
        parent::__construct($model, $dataExtractor);
    }

    function getCategoriesTrees() {
        return [
            [
                "classifica" => ['CLA1MA','CLA2MA'],    // Classifica Galileo
                "name" => 'Famiglie',
            ]
        ];
    }

    protected function createRootCategories()
    {
        $radice = $this->model->sync(['code' => 1, 'name' => 'Radice']);
        $this->productsCategory = $this->model->sync(['code' => 2, 'name' => 'Home', 'parent_id' => $radice->id]);
    }

    public function import()
    {
        try {

            DB::beginTransaction();

            $this->model->setAllIsSynchronized(false);
            $this->truncate(CategoryProduct::class);

            $this->createCategoriesTree();

            $this->model->deleteNotSynchronized();

            DB::commit();

        } catch (Exception $e) {
            DB::rollBack();
            throw $e;
        }

    }

    protected function createCategoriesTree()
    {


        $this->createRootCategories();

        Product::chunk(5000, function ($products) {

            foreach ($products as $product) {

                foreach ($this->getCategoriesTrees() as $categoryTree) {

                    // Creo la root dell'albero di categoria specificato
                    $treeRootCategory = $this->model->sync([
                        'code' => $categoryTree['category_code'] ?? \Str::slug($categoryTree['name'])
                    ], ['code' => $categoryTree['category_code'] ?? \Str::slug($categoryTree['name']), 'name' => $categoryTree['name'], 'parent_id' => $this->productsCategory->id]);

                    $ultimaCategoriaCreata = $treeRootCategory;

                    // Per ogni codice di questo albero di categoria
                    foreach ($categoryTree['classifica'] as $classifica) {

                        // Controllo che sia valorizzata la codifica
                        if (!empty($product->{strtolower($classifica)})) {

                            // Se lo è prendo il valore che corrisponde a questa classifica+codifica
                            $valoreAttributo = AttributeValue::where('classifica', '=', strtolower($classifica))->where('codifica', '=', $product->{strtolower($classifica)})->first();

                            // Se l'ho precedentemente importata
                            if ($valoreAttributo) {
                                // Creo la categoria con quel nome e metto come parent la categoria precedente
                                $ultimaCategoriaCreata = $this->model->sync([
                                    'code' => $valoreAttributo->classifica . $valoreAttributo->codifica
                                ], ['code' => $valoreAttributo->classifica . $valoreAttributo->codifica, 'name' => $valoreAttributo->valore, 'parent_id' => $ultimaCategoriaCreata->id]);
                            }
                        }

                    }

                    // Collego il prodotto all'ultima categoria creata (o $treeRootCategory)
                    $ultimaCategoriaCreata->products()->attach($product->id);
                }

            }

        });

    }


    public function getRecords()
    {
    }
}
