<?php


namespace FourWords\AppManager\Catalog\Importer;

use Exception;
use FourWords\AppManager\Catalog\Model\AttributeValue;
use FourWords\AppManager\Catalog\Model\Feature;
use FourWords\AppManager\Catalog\Model\FeatureProduct;
use FourWords\AppManager\Catalog\Model\Product;
use FourWords\AppManager\DataExtractor\DataExtractor;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\DB;

class FeatureImporter extends SyncableAbstractImporter
{

    protected $throwIfEmptyResult = true;
    protected $featureIdsByCode = [];
    /** @var Collection */
    protected $productIdsByCode = [];

    protected $relationModel;

    public function __construct(Feature $model, FeatureProduct $relationModel, DataExtractor $dataExtractor)
    {
        parent::__construct($model, $dataExtractor);
        $this->relationModel = $relationModel;
    }

    /** Mappatura Nome Feature => Nome alias nella query */
    function getCustomFeature() {
        return [
            'Pezzi per confezione'              => 'items_per_carton',
//            "Peso"		 			            => "peso",
            "IVA"				 	            => "vat_value",
            "Unità di Misura"		            => "um",
            "Unità di Misura alternativa"		=> "other_um",
            "Quantità minima"                   => "min_qta"
        ];
    }

    public function getRecords()
    {
        $lib = config('catalog.as.library');

//        PEUNMA AS peso,

        $query = "SELECT DISTINCT
    CDARMA AS product_code,
    ROUND(CAST(".$lib."MGSPE00F.PZCFMS AS DOUBLE), 0) AS items_per_carton,
    ALIVMA AS vat_value,
    UMBAMA AS um,
    UMALMA AS other_um,
    QTMNMS AS min_qta
     FROM ".$lib."MGART00F
		LEFT JOIN ".$lib."MGSPE00F ON CDDTMA = CDDTMS AND CDARMA = CDARMS AND CDCLMS = ''
        WHERE STATMA = ' ' AND DSARMA <> '' AND LEFT(DSARMA,1) <> '.' AND LEFT(CDARMA,1) <> '.'";
        return $this->dataExtractor->execute($query);
    }

    public function import()
    {

        try {

            DB::beginTransaction();

            $this->model->setAllIsSynchronized(false);
            $this->truncate($this->relationModel);

            // Creo le Feature specificate in getCustomFeature()
            $this->createFeatures();

            $this->connectFeaturesToProducts();

            $this->model->deleteNotSynchronized();

            DB::commit();

        } catch (Exception $e) {
            DB::rollBack();
            throw $e;
        }

    }

    protected function createFeatures() {
        foreach ($this->getCustomFeature() as $featureName => $featureCode ) {
            // Creo la Feature
            $feature = $this->model->sync(['code' => $featureCode, 'name' => $featureName]);

            // Salvo l'id con chiave 'code', che riutilizzerò per collegare i valori della feature al prodotto
            $this->featureIdsByCode[$featureCode] = $feature->id;
        }
    }

    protected function connectFeaturesToProducts() {
        // Prendo codici e id prodotto importati precedentemente
        $this->productIdsByCode = Product::pluck('id','code');

        // Prendo il risultato della query
        $productAndFeaturesValues = $this->getRecords();

        // Aggiungo ai codici prodotto i relativi id, ottenuti importando i prodotti nel DB Laravel
        $productIdsByCode = $this->productIdsByCode;
        $productAndFeaturesValues = array_map(function ($row) use ($productIdsByCode) {
            $row['id'] = $productIdsByCode[$row['product_code']];
            return $row;
        }, $productAndFeaturesValues);

        // Per ogni record con codice prodotto e valori della feature
        foreach ($productAndFeaturesValues as $product) {

            if (!$product['id']) continue;

            // Per ogni valore feature
            foreach ($this->featureIdsByCode as $featureCode => $featureId) {
                // Collego il prodotto alla feature con il relativo valore
                if (!empty($product[$featureCode])) {
                    $this->relationModel->create(['feature_id' => $featureId, 'product_id' => $product['id'], 'value' => $product[$featureCode]]);
                }

            }
        }
    }
}
