<?php


namespace FourWords\AppManager\Catalog\Importer\Pim;

use Exception;
use FourWords\AppManager\Catalog\Importer\SyncableAbstractImporter;
use FourWords\AppManager\Catalog\Model\Category;
use FourWords\AppManager\Catalog\Pim\PimApi;
use FourWords\AppManager\DataExtractor\DataExtractor;
use Illuminate\Support\Facades\DB;

class PimCategoryImporter extends SyncableAbstractImporter
{

    protected $productsCategory;

    public function __construct(Category $model, DataExtractor $dataExtractor)
    {
        parent::__construct($model, $dataExtractor);
    }

    function namedPimCategoriesTrees() {
        return [
            [
                "category_code" => 'prodotti',
                "name" => 'Prodotti',
            ]
        ];
    }

    protected function createRootCategories()
    {
        $radice = $this->model->sync(['code' => 1, 'name' => 'Radice']);
        $this->productsCategory = $this->model->sync(['code' => 2, 'name' => 'Home', 'parent_id' => $radice->id]);
    }

    public function import()
    {
        try {

            DB::beginTransaction();

            $this->model->setAllIsSynchronized(false);
//            $this->truncate(CategoryProduct::class);

            $this->createCategoriesTree();

            $this->model->deleteNotSynchronized();

            DB::commit();

        } catch (Exception $e) {
            DB::rollBack();
            throw $e;
        }

    }

    protected function createCategoriesTree()
    {


        $this->createRootCategories();

            /** @var PimApi $pimApi */
            $pimApi = app()->make(PimApi::class);

            $pimCategories = $pimApi->getCategoriesList();

            if (count($pimCategories) == 0) throw new Exception("Risultato categorie da PIM vuoto");

            foreach ($this->namedPimCategoriesTrees() as $pimCategoryTree) {

                // Creo la root dell'albero di categoria specificato
                $treeRootCategory = $this->model->sync([
                    'code' => $pimCategoryTree['category_code'] ?? \Str::slug($pimCategoryTree['name'])
                ], ['code' => $pimCategoryTree['category_code'] ?? \Str::slug($pimCategoryTree['name']), 'name' => $pimCategoryTree['name'], 'parent_id' => $this->productsCategory->id]);

                foreach ($pimCategories as $categoryTree) {

                    // Riparto dalla categoria root
                    $ultimaCategoriaCreata = $treeRootCategory;

                    foreach ($categoryTree as $category) {

                        // Creo la root dell'albero di categoria specificato
                        $name = $category['name']['it'];
                        $subCategory = $this->model->sync([
                            'code' => $category['code'] ?? \Str::slug($name)
                        ], ['code' => $category['code'] ?? \Str::slug($name), 'name' => $name, 'parent_id' => $ultimaCategoriaCreata->id]);

                        $ultimaCategoriaCreata = $subCategory;

                    }

    //                // Collego il prodotto all'ultima categoria creata (o $treeRootCategory)
    //                $ultimaCategoriaCreata->products()->attach($product->id);
                }
            }


    }


    public function getRecords()
    {
    }
}
