<?php


namespace FourWords\AppManager\Catalog\Importer\Pim;

use Exception;
use FourWords\AppManager\Catalog\Importer\SyncableAbstractImporter;
use FourWords\AppManager\Catalog\Model\Feature;
use FourWords\AppManager\Catalog\Pim\PimApi;
use FourWords\AppManager\DataExtractor\DataExtractor;
use Illuminate\Support\Facades\DB;

class PimFeatureImporter extends SyncableAbstractImporter
{

    protected $throwIfEmptyResult = true;

    protected $pimApi;

    public function __construct(Feature $model, DataExtractor $dataExtractor)
    {
        parent::__construct($model, $dataExtractor);
        /** @var PimApi $pimApi */
        $this->pimApi = app()->make(PimApi::class);
    }

    public function getRecords()
    {
    }

    public function import()
    {

        try {

            DB::beginTransaction();

            $this->model->setAllIsSynchronized(false);

            // Creo le Feature dal PIM
            $this->createFeatures();

            $this->model->deleteNotSynchronized();

            DB::commit();

        } catch (Exception $e) {
            DB::rollBack();
            throw $e;
        }

    }

    protected function createFeatures() {

        $pimFeatures = $this->pimApi->getFeaturesList();

        $this->validateRecords($pimFeatures);

        foreach ($pimFeatures as $pimFeature) {
            // Creo la Feature
            $this->model->sync(['code' => $pimFeature['feature_id']], ['name' => $pimFeature['name']['it']]);
        }
    }
}
