<?php

namespace FourWords\AppManager\Catalog\Model;

use FourWords\AppManager\Catalog\Interfaces\Syncable;
use Illuminate\Database\Eloquent\Model;

class SyncableModel extends Model implements Syncable
{

    public $distinctKey = '';

    protected $isNew = false;
    protected $wasUpdated = false;

    public $isSyncedProperty = 'synchronized';

    public function setIsSynchronized($isSynchronized) {
        $this->setAttribute($this->isSyncedProperty, $isSynchronized);
    }

    public function setAllIsSynchronized($isSynchronized){
        $this->newQuery()->update([$this->isSyncedProperty => $isSynchronized]);
    }

    /**
     * @return int
     */
    public function getSynchronizedCount(){
        return $this->where($this->isSyncedProperty, true)->count();
    }

    public function getNotSynchronizedCount(){
        return $this->where($this->isSyncedProperty, false)->count();
    }

    /**
     * @return float|int
     */
    public function getSynchronizedPercentage() {

        $totalModels = $this->count();
        $synchronizedModels = $this->getSynchronizedCount();

        if ($totalModels > 0) {
            $percentage = ($synchronizedModels / $totalModels) * 100;
        } else {
            $percentage = 0;
        }

        return $percentage;

    }

    /**
     * @return float|int
     */
    public function getNotSynchronizedPercentage() {
        return 100 - $this->getSynchronizedPercentage();
    }

    public function isSynchronized() {
        return $this->{$this->isSyncedProperty};
    }

    public function deleteNotSynchronized() {
        return $this->where($this->isSyncedProperty,'false')->delete();
    }

    public function setSynchronizedAndSave($bool) {
        return $this->fill([$this->isSyncedProperty => $bool])->save();
    }

    public function setSynchronizedWithoutSave($bool) {
        return $this->fill([$this->isSyncedProperty => $bool]);
    }

    public function isNew()
    {
        return $this->isNew;
    }

    public function wasUpdated()
    {
        return $this->wasUpdated;
    }

    /**
     * Aggiorna il model che combacia con $attributes e aggiorna i $values se serve + synchronized = true, o ne crea uno nuovo con entrambi i kv.
     * @param array $attributes Gli attributi kv che il model deve avere per determinare se esiste già, altrimenti sarà creato un model con questi valori più $values
     * @param array $values I valori kv da aggiornare se esiste già il model (e i valori sono diversi), oppure da unire a $attributes nella creazione del model
     * @return mixed
     */
    public function sync($attributes, $values = []) {

        // Verifico se esiste un immagine prodotto con stessa dimensione e nomefile
        $model = $this->where($attributes)->first();

        // Se non esiste la creo
        if (!$model) {

            $model = $this->create(
                array_merge(
                    $attributes,
                    $values,
                    [$this->isSyncedProperty => true]
                )
            );

            $model->isNew = true;

        } else {

            $model->fill($values);
            if (!empty($model->getDirty())) {
                $model->wasUpdated = true;
            }
            $model->fill([$this->isSyncedProperty => true]);
            $model->update($model->getDirty());

        }
        return $model;
    }
}