<?php
/**
 * Copyright (c) 2020/2
 * Created by Federico Papetti
 */

namespace FourWords\AppManager\Catalog\Providers;

use FourWords\AppManager\Director\DatabaseManager;
use Illuminate\Support\Facades\File;
use Illuminate\Support\ServiceProvider;

class CatalogServiceProvider extends ServiceProvider
{

    public function register() {
        $this->bindClasses('Model');
        $this->bindClasses('Importer');
        $this->bindClasses('Importer/Pim');
        $this->bindClasses('Images');
    }

    public function bindClasses($type) {
        $filesDirectory = __DIR__.'/../'.$type;

        $filesToInclude = File::files($filesDirectory);

        foreach ($filesToInclude as $classFile) {
            $className = pathinfo($classFile, PATHINFO_FILENAME);
            $modelClass = "FourWords\\AppManager\\Catalog\\".str_replace("/","\\",$type)."\\{$className}";

            // Binding
            $this->app->bind($modelClass, $modelClass);
        }
    }

    public function boot() {

        // Includo le configurazioni
        config(require __DIR__.'/../../config/catalog.php');

        //Migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        DatabaseManager::addConnection([
            'ip' => config('catalog.as.ip'),
            'user' => config('catalog.as.user'),
            'pwd' => config('catalog.as.password'),
            'lib' => '*LIBL',
            'driver' => 'com.ibm.as400.access.AS400JDBCDriver',
            'connection-warning' => 'false',
            'jdbc' => 'jdbc:as400://'.config('catalog.as.ip').';libraries=*LIBL;naming=system;prompt=false;errors=full'
        ],'catalog');

    }

}
