<?php
namespace FourWords\AppManager\DataExtractor;

use Exception;
use FourWords\AppManager\Director\FWUtils;

class DataExtractor extends Db {

    protected static $type = 'dataextractor';
    
    const HOST = 'host';
    const META = 'meta';

    protected $dataFolder;

    protected $columnsName;
    
    protected $serviceUrl;

    protected $debug = false;

    protected $config;

    protected $meta;

    protected $request;
    
    protected $requestList = [];
    
//    public function __construct($ref) {
//        parent::__construct($ref);
//
//        $this->dataFolder = _CBSYNC_FOLDER_LOG_DATAEXTRACTOR_DATA_;
//    }
    /**
     * DataExtractor constructor.
     * @param string|array $config
     * @param string $meta
     */
    public function __construct($config = null, $meta=1, $serviceUrl = null, $dataFolder = null)
    {
        if ($config == null) {
            $config = config('dataextractor.connectionName');
        }
        $this->cleanTempFolder();

        $this->setConfig($config);
        $this->meta = $meta;
        $this->serviceUrl = $serviceUrl == null ? config('dataextractor.absoluteUrl') : $serviceUrl;
        $this->dataFolder = $dataFolder == null ? config('dataextractor.dataDir') : $dataFolder;
        $this->debug = config('dataextractor.debug');
    }

    /**
     * @return array
     */
    public function getConfig()
    {
        return $this->config;
    }

    /**
     * @param array|string $config
     */
    public function setConfig($config)
    {
        if (!is_array($config)) {
            $config = config('database.connections.'.$config);
        }
        $this->config = $config;
    }



    /**
     * @return mixed
     */
    public function getServiceUrl()
    {
        return $this->serviceUrl;
    }

    /**
     * @param mixed $serviceUrl
     */
    public function setServiceUrl($serviceUrl)
    {
        $this->serviceUrl = $serviceUrl;
    }

    /**
     * @return mixed
     */
    public function getMeta()
    {
        return $this->meta;
    }

    /**
     * @param mixed $meta
     */
    public function setMeta($meta)
    {
        $this->meta = $meta;
    }

    /**
     * @return mixed
     */
    public function getDebug()
    {
        return $this->debug;
    }

    /**
     * @param mixed $debug
     */
    public function setDebug($debug)
    {
        $this->debug = $debug;
    }

    /**
     * @throws Exception
     */
    public function query($sql) {

        $query = $this->getEncodedQuery($sql);

        $this->result = false;
        $this->request = md5($query);

        if(!isset($this->requestList[$this->request])) {

//            $response = $this->sendCurlRequest($query,$this->request);

//            $className = 'dataExtractor'; //TODO:>> Aggiungere dataExtractor/bonjour. Bisogna ricompilare jar e aggiungere il parametro alla chiamata qui sotto

            $request = md5($query);

            if($query == '') {
                return '';
            }

            //rimuovo l'estensione se presente
            $file = pathinfo($request, PATHINFO_FILENAME);
            $outputFile = config('dataextractor.dataDir') . $file;

            if (!file_exists(config('dataextractor.dataDir'))) {
                mkdir(config('dataextractor.dataDir'),0777,true);
            }

            $debug = $this->getDebug() ? '1' : '0';
            $meta = $this->getMeta() ? '1' : '0';

            $redirectOutput = $this->getDebug() ? (' >> ' . config('dataextractor.debugOutputFile') . ' 2>&1') : ' 2>&1';

            //eseguo la query
            $response = shell_exec('cd '.config('dataextractor.dir').'/data-extractor/ && JFILE="$(which java)"; $JFILE -Xshare:off -jar DataExtractor.jar dataExtractor ' . base64_encode(json_encode($this->getConfig())) .' ' . $query . ' ' . $outputFile . ' '. $debug .' ' . $meta . $redirectOutput);

            if ($this->getDebug()) {
                $response = shell_exec('tail -n 1 ' . config('dataextractor.debugOutputFile'));

//                $lines = explode("\n", $output);
//                $startIndex = null;
//
//                foreach ($lines as $index => $line) {
//                    if (strpos($line, "SQL: ") === 0) {
//                        $startIndex = $index;
//                        break;
//                    }
//                }
//
//                if ($startIndex) {
//                    $response = implode("\n", array_slice($lines, $startIndex + 1));
//                }
            }

            //se non ha eseguito correttamente lancio un eccezione
            if ($response != "OK\n") {
                throw new Exception('Errore DE - '.($response ?? 'Responso NULL, attivare debug'));
            }

            //se è stata ritornata una stringa vuota
            if($response === '') {
                return [];
            }
        }

        $this->result = fopen(rtrim($this->dataFolder,DIRECTORY_SEPARATOR).DIRECTORY_SEPARATOR.$this->request.'.txt', 'r');

        $this->requestList[$this->request] = true;

        $this->getMetaValues();

        return $this->result;
    }


    /**
     * @param $user
     * @param $pwd
     * @return bool
     */
    public function bonjour($user, $pwd)
    {
        $res = shell_exec('cd '.config('dataextractor.dir').'/data-extractor/ && JFILE="$(which java)"; $JFILE -Xshare:off -jar DataExtractor.jar bonjour' . base64_encode(json_encode($this->getConfig())) . ' '. $user . ' ' . $pwd . ' 1');
        if (strpos($res, "OK")) return true;
        return false;
    }

    public function getMetaValues(){

        if (!$this->meta)
            return;

        // recupero la prima riga
        $row = fgets($this->result);

        if (!$row)
            return;

        $this->columnsName = array_map("strtolower", json_decode($row, true));

    }

    public function execute($sql)
    {

        if (strlen(trim($sql)) > 5) {
            if (strpos(strtolower(substr(trim($sql), 0, 6)), 'select') !== false || strpos(strtolower(substr(trim($sql), 0, 4)), 'with') !== false) {
                return $this->executeS($sql);
            }
        }

        return $this->executeI($sql);
    }

    public function executeS($sql) {
        
        $this->result = $this->query($sql);
        
        $return = array();
        
        while ($row = fgets($this->result)) {

            $row = $this->formatRow(json_decode($row));
				
			//decodifico la riga nella lista di ritorno
			$return[] = $row;
        }
        
        return $return;
    }

    public function formatRow($row){

        if (!$this->meta)
            return $row;

        $ret = array();
        foreach ($row as $key => $col){
            $ret[$this->columnsName[$key]] = $col;
        }

        return $ret;
    }

    public function getRow($sql) {
        
        $sql = rtrim($sql, " \t\n\r\0\x0B;").' FETCH FIRST 1 ROWS ONLY';
        
        $this->result = $this->query($sql);
        
        $row = fgets($this->result);
        
        $row = $this->formatRow(json_decode($row));

        unset($this->requestList[$this->request]);
        
        return $row;
    }

    public function getValue($sql) {
        
        if (!$result = $this->getRow($sql)) {
            return false;
        }
        
        return array_shift($result);
    }

    /**
     * @param $sql
     * @return int
     */
    public function executeI($sql) {

        //non essendo una select non reperisco i nomi delle colonne
        $meta = $this->meta;
        $this->meta = 0;

        //eseguo la query d'inserimento
        $this->query($sql);

        //ripristino la configurazione originale della proprietà meta
        $this->meta = $meta;

        $return = '';
        while (!feof($this->result)) {
            $return .= fread($this->result, 4096);  // use a buffer of 4KB
        }

        //chiudo il collegamento al file della risposta
        fclose($this->result);

        //rimuovo il file
        unlink(rtrim($this->dataFolder,DIRECTORY_SEPARATOR).DIRECTORY_SEPARATOR.$this->request.'.txt');

        //rimuovo la richiesta (query) dalla lista di quelle già eseguite
        unset($this->requestList[$this->request]);

        if (str_starts_with($return,'OK Rows:')) {
            return (int) explode( ":", $return)[1];
        }

        return $return;
    }
    
    public function nextRow($result = false) {
        
        if (!$result) {
            $result = $this->result;
        }
        
        if (!is_resource($result) || get_resource_type($result) != 'stream') {
            return false;
        }
        
        $row = fgets($result);
        
        return json_decode($row);
    }
    
    // --- metodi non neccessari --- //
    
    public function optimizeTables($prefix) {}
    
    public function beginTransaction() {}
    
    public function commitTransaction() {}
    
    public function rollbackTransaction() {}
    
    // --- --- --- --- --- --- --- --- //

    /**
     * @throws Exception
     */
    public function sendCurlRequest($query, $request = null, $serviceUrl = null) {

        if ($this->config == null) {
            throw new Exception('La configurazione dataextractor non è stata trovata in config(database.connections.\<nome\>)');
        }
         
        //se non è stato specificato un servizio uso quello di classe
        if(is_null($serviceUrl)) {
            $serviceUrl = $this->serviceUrl;
        }
        
        //se non è stata passato un nome specifico della richiesta, utilizzo come nome l'md5 della query
        if(is_null($request)) {
            $request = md5($query);
        }
    
        //se la query è una stringa vuota ritorno un risultato vuoto
        if($query == '') {
            return '';
        }

        $config = base64_encode(json_encode((object) $this->config));

        //parametri
        $postFields = array(
            "project" => $config,
            "sql" => $query,
            "file" => $request,
            "meta" => $this->meta,
        );

        //eseguo la sincronizzazione dei db
        $ch = curl_init();

        //setto le impostazioni del curl
        curl_setopt($ch, CURLOPT_URL, $serviceUrl);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($postFields));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

        //eseguo il curl
        $result = curl_exec($ch);

        if(curl_getinfo($ch, CURLINFO_HTTP_CODE) != 200) {
            throw new Exception("Impossibile eseguire curl: ".$serviceUrl." -- ".$result);      //TODO: recupera il contenuto del file (ora viene usato il binario...)
        }

        curl_close($ch);

        return $result;
    }

    public function cleanTempFolder() {
        FWUtils::recursiveDeleteDir(config('dataextractor.dataDir'));
    }
    
    protected function createRequestFile($result,$request) {
    
        //creo il nome del file dove inserire il contenuto
        $filename = $request.'.zip';
    
        //path completo del file
        $filePath = rtrim($this->dataFolder,DIRECTORY_SEPARATOR).DIRECTORY_SEPARATOR.$filename;

        if(!is_dir($this->dataFolder) && !mkdir($this->dataFolder,0775,true)) {
            throw new Exception('impossibile creare la cartella '.$this->dataFolder);
        }
    
        //controllo se esiste il file e lo salvo
        if(file_exists($filePath)) {
            unlink($filePath);
        }
    
        //creo il file zip
        $file = fopen($filePath, 'a');
        fwrite($file, $result);
        fclose($file);
    
        //scompatto il file zip e lo rimuovo
        return $this->unzipFile($filePath);
    
    }
    
    protected function unzipFile($filePath) {
        
        $folder = dirname($filePath);
        
        if(!is_dir($folder) && !mkdir($folder,0775,true)) {
            throw new Exception('impossibile creare la cartella '.$folder);
        }

        //creo un nuovo oggetto di tipo archivio zip
		$zip = new \ZipArchive();
	
		//creo il file zip con il contenuto ricevuto
		$zip->open($filePath);
	
		//estraggo l'archivio
		if($zip->extractTo($folder)) {
			$zip->close();
			return unlink($filePath);
		}
		else return false;
    }
    
    protected function createEmptyRequestFile($request) {
        return $this->createTxtFile($request);
    }
    
    protected function createTxtFile($request, $content = '') {
        
        $filename = rtrim($this->dataFolder,DIRECTORY_SEPARATOR).DIRECTORY_SEPARATOR.$request.'.txt';
        
        if(!is_dir($this->dataFolder) && !mkdir($this->dataFolder,0775,true)) {
            throw new Exception('impossibile creare la cartella '.$this->dataFolder);
        }
    
        $file = fopen($filename, 'a');
        fwrite($file, $content);
        return fclose($file);
    }
    
    protected function getEncodedQuery($sql) {

        $sql = $this->cleanQuery($sql);

        return base64_encode($sql);
    }
    
    protected function close($request = null) {
        
        if(is_null($request)) {
            $request = $this->request;
        }
        
        @fclose($request);
        
        $this->requestList[$request] = false;
    }
    
    protected function delete($request = null) {

        if(is_null($request)) {
            $request = $this->request;
        }
        
        @fclose($request);
        
        unset($this->requestList[$request]);
    }
    
    protected function setLastQuery($sql) {
        parent::setLastQuery($sql);
        
        $filename = config('dataextractor.queryDir').$this->request.'.txt';
        
        if(!is_dir(config('dataextractor.queryDir')) && !mkdir(config('dataextractor.queryDir'),0775,true)) {
            throw new Exception('impossibile creare la cartella '.config('dataextractor.queryDir'));
        }
        
        //scrivo la query che sta per esser eseguita nel file di log 
        file_put_contents($filename, $sql);
    }
    
    public function getErrorMessage() {
        throw new Exception('metodo getErrorMessage non implementato!');
    }
}