<?php
namespace FourWords\AppManager\DataExtractor;

abstract class Db {

    protected static $type;

    protected static $instance;
    
    protected $result = false;
    
    private $_last_query;

    /**
     * metodo che pulisce la query
     *
     * @param string $query
     * @return string
     */
    public function cleanQuery($query) {
    
        //rimuovo tutti i commenti
        $query = preg_replace('@(--[^\r\n]*)@ms','',$query);
    
        //rimuovo gli a capo
        $query = str_replace("\r\n", ' ', $query);
        //rimuovo i tab
        $query = str_replace("\t", ' ', $query);
        //rimuovo gli spazzi prima di una virgola
        $query = str_replace(' ,', ',', $query);
        //rimuovo gli spazzi in eccesso
        $query = preg_replace('/\s+/', ' ', $query);
        //rimuovo eventuali spazi da inizio e fine stringa
        $query = trim($query);
    
        return $query;
    }
    
    /**
     * metodo statico che costruisce la stringa da utilizzare nella IN di una query data una lista di valori
     *
     * @param string|array $list
     * @return string
     */
    public static function getQueryInFromArray($list) {
    
        //se non è un array lo creo
        if(!is_array($list)) {
            $list = array($list);
        }

        return "'".implode("','", $list)."'";
    }
    
    protected function setLastQuery($sql) {
        $this->_last_query = $sql;
    }

    public static function escape($string) {
        return str_replace(array("'"), array("''"), $string);
    }

    public function prepareSaveQueryTable($table,$row,$searchConditions) {

        if(!$searchConditions) {
            throw new \Exception(sprintf("%s: condizione necessaria per verificare l'esistenza del record in tabella %s",__METHOD__,$table));
        }

        if(!is_array($searchConditions)) $searchConditions = array($searchConditions);

        $andWhere = '';
        foreach ($searchConditions as $column) {
            $andWhere .= sprintf(" AND %s = '%s'",$column,static::escape($row[$column]));
        }

        //controllo se è già presente il record
        $exists = $this->getRow(sprintf("SELECT 1 FROM %s WHERE 1=1 %s",$table,$andWhere));

        //ritorno la query
        return !$exists ? $this->prepareInsertQueryTable($table,$row) : $this->prepareUpdateQueryTable($table,$row,$searchConditions);
    }

    public function prepareInsertQueryTable($table,$row) {

        return sprintf("INSERT INTO %s (%s) VALUES (%s)",$table,
            implode(',',array_map(function($key) {return strtoupper($key);},array_keys($row))),
            implode(',',array_map(function($val) {return "'".static::escape($val)."'";},array_values($row)))
        );
    }

    public function prepareUpdateQueryTable($table,$row,$searchConditions) {

        if(!$searchConditions) {
            throw new \Exception(sprintf("%s: condizione necessaria per verificare l'esistenza del record in tabella %s",__METHOD__,$table));
        }

        $andWhere = '';
        foreach ($searchConditions as $column) {
            $andWhere .= sprintf(" AND %s = '%s'",$column,static::escape($row[$column]));
        }

        $set = '';
        foreach ($row as $col => $val) {
            $set .= sprintf(",%s = '%s'",strtoupper($col),static::escape($val));
        }
        $set = ltrim($set,',');

        return sprintf("UPDATE %s SET %s WHERE 1=1 %s",$table,$set,$andWhere);
    }
    
    abstract public function execute($sql);
    
    abstract public function executeS($sql);
    
    abstract public function getRow($sql);
    
    abstract public function getValue($sql);
    
    abstract public function nextRow($result = false);
    
    abstract public function query($sql);

    public function exec($sql) {throw new \Exception('metodo exec non definito per la connessione corrente');}
    
    abstract public function beginTransaction();
    
    abstract public function commitTransaction();
    
    abstract public function rollbackTransaction();
    
    abstract public function optimizeTables($prefix);
    
    abstract public function getErrorMessage();
}
