<?php
namespace FourWords\AppManager\DataExtractor;

use Exception;

class OldDataExtractor extends Db {

    protected static $type = 'dataextractor';
    
    const HOST = 'host';
    const META = 'meta';
    const PARAMS = 'params';

    protected $dataFolder;

    protected $columnsName;
    
    protected $serviceUrl;

    protected $configName;

    protected $meta;

    protected $defaultParams;

    protected $request;
    
    protected $requestList = [];
    
//    public function __construct($ref) {
//        parent::__construct($ref);
//
//        $this->dataFolder = _CBSYNC_FOLDER_LOG_DATAEXTRACTOR_DATA_;
//    }
    /**
     * DataExtractor constructor.
     * @param string $serviceUrl
     * @param string $meta
     * @param string $configName
     * @param array $defaultParams
     * @param null $dataFolder
     */
    public function __construct($serviceUrl, $meta, $configName, $defaultParams = [], $dataFolder = null)
    {
//        parent::__construct();
        $this->serviceUrl = $serviceUrl;
        $this->meta = $meta;
        $this->configName = $configName;
        $this->defaultParams = $defaultParams;
        $this->dataFolder = config('dataextractor.dataDir');
    }

    /**
     * @return mixed
     */
    public function getServiceUrl()
    {
        return $this->serviceUrl;
    }

    /**
     * @param mixed $serviceUrl
     */
    public function setServiceUrl($serviceUrl)
    {
        $this->serviceUrl = $serviceUrl;
    }

    /**
     * @return mixed
     */
    public function getMeta()
    {
        return $this->meta;
    }

    /**
     * @param mixed $meta
     */
    public function setMeta($meta)
    {
        $this->meta = $meta;
    }

    /**
     * @return mixed
     */
    public function getDefaultParams()
    {
        return $this->defaultParams;
    }

    /**
     * @param mixed $defaultParams
     */
    public function setDefaultParams($defaultParams)
    {
        $this->defaultParams = $defaultParams;
    }

    /**
     * @return string
     */
    public function getConfigName(): string
    {
        return $this->configName;
    }

    /**
     * @param string $configName
     */
    public function setConfigName(string $configName)
    {
        $this->configName = $configName;
    }



    public function query($sql) {
        
        $query = $this->getEncodedQuery($sql);

        $this->result = false;
        $this->request = md5($query);
        
        $this->setLastQuery($sql);
        
        if(!isset($this->requestList[$this->request])) {
        
            $response = $this->sendCurlRequest($query,$this->request);
            
            //se è stata ritornata una stringa vuota creo un file txt vuoto
            if($response == '')
                $this->createEmptyRequestFile($this->request);
            //altrimenti salvo il contenuto del file zip che è stato ritornato
            else $this->createRequestFile($response,$this->request);
        }
        
        $this->result = fopen(rtrim($this->dataFolder,DIRECTORY_SEPARATOR).DIRECTORY_SEPARATOR.$this->request.'.txt', 'r');
        
        $this->requestList[$this->request] = true;
        
        $this->getMetaValues();
        
        return $this->result;
    }

    public function getMetaValues(){

        if (!$this->meta)
            return;

        // recupero la prima riga
        $row = fgets($this->result);

        if (!$row)
            return;

        $this->columnsName = array_map("strtolower", json_decode($row, true));

    }

    public function executeS($sql) {
        
        if(strpos(strtolower($sql), 'insert') !== false || strpos(strtolower($sql), 'update') !== false) {
            throw new Exception('DataExtractor execute: utilizzare il metodo execute per eseguire la query '.$sql);
        }
        
        $this->result = $this->query($sql);
        
        $return = array();
        
        while ($row = fgets($this->result)) {

            $row = $this->formatRow(json_decode($row));
				
			//decodifico la riga nella lista di ritorno
			$return[] = $row;
        }
        
        return $return;
    }

    public function formatRow($row){

        if (!$this->meta)
            return $row;

        $ret = array();
        foreach ($row as $key => $col){
            $ret[$this->columnsName[$key]] = $col;
        }

        return $ret;
    }

    public function getRow($sql) {
        
        $sql = rtrim($sql, " \t\n\r\0\x0B;").' FETCH FIRST 1 ROWS ONLY';
        
        $this->result = $this->query($sql);
        
        $row = fgets($this->result);
        
        $row = $this->formatRow(json_decode($row));

        unset($this->requestList[$this->request]);
        
        return $row;
    }

    public function getValue($sql) {
        
        if (!$result = $this->getRow($sql)) {
            return false;
        }
        
        return array_shift($result);
    }
    
    public function execute($sql) {

        if(strpos(strtolower($sql), 'insert') === false && strpos(strtolower($sql), 'update') === false) {
            throw new Exception('DataExtractor execute: utilizzare il metodo executeS per eseguire la query '.$sql);
        }

        //non essendo una select non reperisco i nomi delle colonne
        $meta = $this->meta;
        $this->meta = 0;

        //eseguo la query d'inserimento
        $this->query($sql);

        //ripristino la configurazione originale della proprietà meta
        $this->meta = $meta;

        $return = '';
        while (!feof($this->result)) {
            $return .= fread($this->result, 4096);  // use a buffer of 4KB
        }

        //chiudo il collegamento al file della risposta
        fclose($this->result);

        //rimuovo il file
        unlink(rtrim($this->dataFolder,DIRECTORY_SEPARATOR).DIRECTORY_SEPARATOR.$this->request.'.txt');

        //rimuovo la richiesta (query) dalla lista di quelle già eseguite
        unset($this->requestList[$this->request]);

        return $return;
    }
    
    public function nextRow($result = false) {
        
        if (!$result) {
            $result = $this->result;
        }
        
        if (!is_resource($result) || get_resource_type($result) != 'stream') {
            return false;
        }
        
        $row = fgets($result);
        
        return json_decode($row);
    }
    
    // --- metodi non neccessari --- //
    
    public function optimizeTables($prefix) {}
    
    public function beginTransaction() {}
    
    public function commitTransaction() {}
    
    public function rollbackTransaction() {}
    
    // --- --- --- --- --- --- --- --- //
    
    public function sendCurlRequest($query,$request = null,$serviceUrl = null) {
         
        //se non è stato specificato un servizio uso quello di classe
        if(is_null($serviceUrl)) {
            $serviceUrl = $this->serviceUrl;
        }
        
        //se non è stata passato un nome specifico della richiesta, utilizzo come nome l'md5 della query
        if(is_null($request)) {
            $request = md5($query);
        }
    
        //se la query è una stringa vuota ritorno un risultato vuoto
        if($query == '') {
            return '';
        }

        //parametri
        $postFields = array(
            "project" => $this->configName,
            "sql" => $query,
            "file" => $request,
            "meta" => $this->meta,
        );

        //recupero eventuali parametri di default da aggiungere alla richiesta
        if($this->defaultParams != null && is_array($this->defaultParams) && !empty($this->defaultParams)) {
            foreach ($this->defaultParams as $param => $value) {
                $postFields[$param] = $value;
            }
        }

        if($this->defaultParams != null && is_array($this->defaultParams) && !empty($this->defaultParams)) {
            foreach ($this->defaultParams as $param => $value) {
                $postFields[$param] = $value;
            }
        }

        //eseguo la sincronizzazione dei db
        $ch = curl_init();

        //setto le impostazioni del curl
        curl_setopt($ch, CURLOPT_URL, $serviceUrl);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($postFields));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

        //eseguo il curl
        $result = curl_exec($ch);

        if(curl_getinfo($ch, CURLINFO_HTTP_CODE) != 200) {
            throw new Exception("Impossibile eseguire curl: ".$serviceUrl." -- ".$result);      //TODO: recupera il contenuto del file (ora viene usato il binario...)
        }

        curl_close($ch);

        return $result;
    }

    public function cleanTempFolder() {
        $ch = curl_init();

        $tempCleanerUrl = str_replace('DataExtractor.php','TempCleaner.php',$this->serviceUrl);

        //setto le impostazioni del curl
        curl_setopt($ch, CURLOPT_URL, $tempCleanerUrl);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

        //eseguo il curl
        $result = curl_exec($ch);
        $result = $result ? strip_tags($result) : '';

        curl_close($ch);

        return $result;
    }
    
    protected function createRequestFile($result,$request) {
    
        //creo il nome del file dove inserire il contenuto
        $filename = $request.'.zip';
    
        //path completo del file
        $filePath = rtrim($this->dataFolder,DIRECTORY_SEPARATOR).DIRECTORY_SEPARATOR.$filename;

        if(!is_dir($this->dataFolder) && !mkdir($this->dataFolder,0775,true)) {
            throw new Exception('impossibile creare la cartella '.$this->dataFolder);
        }
    
        //controllo se esiste il file e lo salvo
        if(file_exists($filePath)) {
            unlink($filePath);
        }
    
        //creo il file zip
        $file = fopen($filePath, 'a');
        fwrite($file, $result);
        fclose($file);
    
        //scompatto il file zip e lo rimuovo
        return $this->unzipFile($filePath);
    
    }
    
    protected function unzipFile($filePath) {
        
        $folder = dirname($filePath);
        
        if(!is_dir($folder) && !mkdir($folder,0775,true)) {
            throw new Exception('impossibile creare la cartella '.$folder);
        }

        //creo un nuovo oggetto di tipo archivio zip
		$zip = new \ZipArchive();
	
		//creo il file zip con il contenuto ricevuto
		$zip->open($filePath);
	
		//estraggo l'archivio
		if($zip->extractTo($folder)) {
			$zip->close();
			return unlink($filePath);
		}
		else return false;
    }
    
    protected function createEmptyRequestFile($request) {
        return $this->createTxtFile($request);
    }
    
    protected function createTxtFile($request, $content = '') {
        
        $filename = rtrim($this->dataFolder,DIRECTORY_SEPARATOR).DIRECTORY_SEPARATOR.$request.'.txt';
        
        if(!is_dir($this->dataFolder) && !mkdir($this->dataFolder,0775,true)) {
            throw new Exception('impossibile creare la cartella '.$this->dataFolder);
        }
    
        $file = fopen($filename, 'a');
        fwrite($file, $content);
        return fclose($file);
    }
    
    protected function getEncodedQuery($sql) {

        $sql = $this->cleanQuery($sql);

        return base64_encode($sql);
    }
    
    protected function close($request = null) {
        
        if(is_null($request)) {
            $request = $this->request;
        }
        
        @fclose($request);
        
        $this->requestList[$request] = false;
    }
    
    protected function delete($request = null) {

        if(is_null($request)) {
            $request = $this->request;
        }
        
        @fclose($request);
        
        unset($this->requestList[$request]);
    }
    
    protected function setLastQuery($sql) {
        parent::setLastQuery($sql);
        
        $filename = config('dataextractor.queryDir').$this->request.'.txt';
        
        if(!is_dir(config('dataextractor.queryDir')) && !mkdir(config('dataextractor.queryDir'),0775,true)) {
            throw new \Exception('impossibile creare la cartella '.config('dataextractor.queryDir'));
        }
        
        //scrivo la query che sta per esser eseguita nel file di log 
        file_put_contents($filename, $sql);
    }
    
    public function getErrorMessage() {
        throw new Exception('metodo getErrorMessage non implementato!');
    }
}