<?php
/**
 * Copyright (c) 2020/3
 * Created by Federico Papetti
 */

namespace FourWords\AppManager\Director;

use Closure;
use Illuminate\Support\Facades\Request;

class Director
{
    protected $prefix = '4words/';
    protected $fwProviders = [];
    protected $fwProvidersByModule = [];
    protected $composerPackages = [];
    protected $composerPackagesByName = [];

    public $clientName = null;
    public $modules = [];
    public $jobs = [];
    protected $modulesJsonPath = '';

    public function __construct()
    {
        $this->readComposer();
    }

    /**
     * Leggo il composer.lock e valorizzo la proprietà $composerPackages
     * Metodo chiamato nel boot del service DirectorServiceProvider
     */
    public function readComposer() {
        $composerLock = base_path('composer.lock');
        if (file_exists($composerLock)) {
            $composerLockObj = json_decode(file_get_contents($composerLock));
            $this->composerPackages = $composerLockObj->packages;

            foreach ($this->composerPackages as $i => $v) {
                $this->composerPackagesByName[$v->name] = $v;
            }

            if (empty($this->composerPackages) && file_exists(app()->getCachedPackagesPath())) {
                $this->composerPackages = include app()->getCachedPackagesPath();
            }

        } else {
            throw new \Exception("File composer.lock non trovato!");
        }
    }

    /**
     * @param string $packageName Il nome del pacchetto 4words o altro ( "api" oppure "dataextractor" oppure "illuminate/support"... )
     * @return bool true se è installato (presente nel composer.lock)
     */
    public function isInstalled($packageName) {

        // Aggiungo il prefisso dei package 4words se non ne è stato specificato uno
        if (strpos($packageName,'/') === false) {
            $packageName = $this->prefix.$packageName;
        }

        return array_search( $packageName, array_column($this->composerPackages,'name')) !== false;
    }

    /**
     * @param string $packageName Il nome del pacchetto 4words o altro ( "api" oppure "dataextractor" oppure "illuminate/support"... )
     * @return array I provider registrati nella proprietà "extra" del composer.json nel modulo,
     */
    public function getProviders($packageName, $depth = 1) {

        // Aggiungo il prefisso dei package 4words se non ne è stato specificato uno
        if (strpos($packageName,'/') === false) {
            $packageName = $this->prefix.$packageName;
        }
        $packageIndex = array_search( $packageName, array_column($this->composerPackages,'name'));

        $providers = [];

        if ($packageIndex === false) {
            return $providers;
        }

        // Aggiungo i providers in "require"
        if ($depth == 1 && property_exists($this->composerPackages[$packageIndex],'require')) {
            $subModules = array_keys((array) $this->composerPackages[$packageIndex]->require);
            $subModules = array_filter($subModules,function ($module) {
                return strpos($module,'/') !== false;
            });

            $subModulesProviders = [];

            foreach ($subModules as $subModule) {
                $depth += 1;
                $subModulesProviders = array_merge($subModulesProviders,$this->getProviders($subModule, $depth));
            }

            $providers = array_merge($providers, $subModulesProviders);

        }

        // Aggiungo i providers del modulo
        if (!property_exists($this->composerPackages[$packageIndex],'extra') ||
            !property_exists($this->composerPackages[$packageIndex]->extra,'laravel') ||
            !property_exists($this->composerPackages[$packageIndex]->extra->laravel,'providers')) {
            return $providers;
        }

        $providers = array_merge($providers,$this->composerPackages[$packageIndex]->extra->laravel->providers);

        return $providers;
    }

    public function map4wPackages() {

        $packagesPath = base_path('vendor/4words');
        $allPackages = scandir($packagesPath);

        $allPackages = array_filter($allPackages,function($dirName) {
           return substr($dirName,0,1) != '.';
        });

        foreach ($allPackages as $packageName) {

                $this->modules[] = $packageName;

            // Determino il cliente e valorizzo l'ordine dei moduli in $modules
//            $modulesJsonPath = $packagesPath.'/'.$packageName.'/modules.json';
//            if (file_exists($modulesJsonPath)) {
//
//                $this->modulesJsonPath = $modulesJsonPath;
//                $this->clientName = $packageName;
//
//                $modules = json_decode(file_get_contents($modulesJsonPath)) ?? [];
//                if (!property_exists($modules,'modules')) { throw new \Exception('Proprietà modules non presente in modules.json'); }
//
//                if (array_search($packageName,$this->modules) === false) {
//                    $modules->modules[] = $packageName;
//                }
//
//                $this->modules = $modules->modules;
//
//            }

            if (!property_exists($this->composerPackagesByName['4words/'.$packageName],'autoload')) { continue; }
            $autoload = $this->composerPackagesByName['4words/'.$packageName]->autoload;
            if (!property_exists($autoload,'psr-4')) { continue; }

            $autoload = (array) $autoload;
            $providerNamespace = array_keys((array) $autoload['psr-4'])[0];


            // Salvo il package
            if ($packageName != 'director') {

                // Prendo tutti i Jobs del package
                $jobsPath = $packagesPath.'/'.$packageName.'/src/Jobs';
                if (file_exists($jobsPath)) {

                    $allJobs = collect(scandir($jobsPath))->filter(function ($file) {
                        return substr($file,0,1) != '.' && !str_ends_with($file, 'IocJob.php');
                    })->map(function ($file) use ($providerNamespace) {
                        return $providerNamespace.'Jobs\\'.explode('.',$file)[0];
                    });

                    $this->jobs = array_merge($this->jobs, $allJobs->toArray());

                }

                // Prendo tutti i provider del package
                $providersPath = $packagesPath.'/'.$packageName.'/src/Providers';
                if (file_exists($providersPath)) {

                    // Escludo i file che iniziano con "."
                    $allProviders = collect(scandir($providersPath))->filter(function ($file) {
                        return substr($file,0,1) != '.';
                    })->map(function ($file) use ($providerNamespace) {
                        return $providerNamespace.'Providers\\'.explode('.',$file)[0];
                    });

                    foreach ($allProviders as $provider) {
                        if (class_exists($provider)) {
                            $this->fwProvidersByModule[$packageName][] = $provider;
                        }
                    }

                }


            }

        }

        // Dopo aver mappato package e providers, ordino i providers in base al modules.json
//        foreach ($this->modules as $module) {
//            foreach ($this->fwProvidersByModule[$module] as $provider) {
//                $this->fwProviders[] = $provider;
//            }
//        }

    }

    public function getClientName() {
        return $this->clientName;
    }

    public function get4wProviders() {
        return $this->fwProviders;
    }

    /**
     * @param string $packageName Il nome del pacchetto 4words o altro ( "api" oppure "dataextractor" oppure "illuminate/support"... )
     * @param Closure $closure
     * @return mixed|null
     */
    public function whenInstalled($packageName, Closure $closure) {
        if ($this->isInstalled($packageName)) {
            return $closure();
        } else {
            return null;
        }
    }

    public function getCommand() {
        $command = Request::server('argv', null);
//        if (is_array($command)) {
//            return $command = implode(' ', $command);
//        }

        return (array) $command;
    }

    /**
     * @param string|array $commandNames
     * @param string|array $exclusions
     * @return bool
     */
    public function commandContains($commandNames, $exclusions = null) {

            $command = $this->getCommand();

            if (!empty(array_intersect($command, $commandNames)) &&
                ($exclusions == null || empty(array_intersect($command, $exclusions)))) {
                return true;
            }

            return false;
    }


}
