<?php

namespace FourWords\AppManager\IocClient\Jobs;

use FourWords\AppManager\DataExtractor\DataExtractor;
use FourWords\AppManager\Ioc\Contracts\UserImporter;
use FourWords\AppManager\Ioc\Jobs\IocJob;
use FourWords\AppManager\Ioc\Models\IocUser;
use Illuminate\Database\Eloquent\Model;

class ImportIocUsers extends IocJob implements UserImporter
{

    protected $newUsers = 0;
    protected $editedUsers = 0;

    public function handleJob()
    {
        parent::handleJob();

        $this->import();

        $this->logger->info('Nuovi utenti: '.$this->newUsers.' | Utenti modificati: '.$this->editedUsers);

    }

    public function parameters() {
        return [
            ['id' => 'codice-cliente', 'name' => 'Filtro su codici cliente',  'description' => '1234 oppure 1234, 5678, 9999']
        ];
    }

    public function getModel()
    {
        return app()->make(IocUser::class);
    }

    public function recordToUserMap()
    {
        return [
            'username' => 'username',
            'password' => 'password',
            'firstname' => 'name',
            'codice_agente' => 'agent',
            'codice_cliente' => 'code',
            'codice_cliente_fatturazione' => 'billing_code',
            'email' => 'email',
            'tipo_cliente' => 'type',
            'vat_code' => 'vat_code',
            'fiscal_code' => 'fiscal_code',
            'address' => 'address',
            'city' => 'city',
            'province' => 'province',
            'cap' => 'cap',
        ];
    }

    public function getUsers()
    {
        $de = new DataExtractor();

        $columns = [
            'username' => 'CONTCA',
            'codice_cliente' => 'CONTCA',
            'codice_cliente_fatturazione' => 'CSPECA',
            'password' => '\'\'',
//            'flag_sospensione' => 'FLSPAW',                   // 'S' se sospeso
            'firstname' => 'SUBSTR(DSCOCP, 1,32)',
            'codice_agente' => 'CDAGCA',
            'tipo_cliente' => 'CLFOCP',                  // 'C' cliente 'D' destinazione
            'email' => 'INELCA',
            'vat_code' => 'PIVACA',
            'fiscal_code' => 'CDFICA',
            'address' => 'INDICA',
            'city' => 'LOCACA',
            'province' => 'PROVCA',
            'cap' => 'CAPOCA',
        ];

        $columnsString = "";
        foreach ($columns as $alias => $col) {
            $columnsString .= $col . ' as ' . $alias . ', ';
        }
        $columnsString = substr($columnsString,0, -2);

        $paramFilter = !empty($this->params['codice-cliente']) ? " AND CONTCA IN ('".implode("','",explode(",",str_replace(' ','',$this->params['codice-cliente'])))."')" : "";

        // Prendo i clienti, senza codici che iniziano con 0299 (template), senza sospesi
        $users = $de->executeS("SELECT ".$columnsString."
    FROM " . config('ioc.as.library') . "CGANA00F
    JOIN " . config('ioc.as.library') . "WWAWB00F ON (CONTCA = CDGEWB)
    JOIN " . config('ioc.as.library') . "CGANA03F ON (CONTCA = CONTAD AND CDDTCA = CDDTAD)
    JOIN " . config('ioc.as.library') . "CGPCO00F ON (CONTCA = CONTCP AND CDDTCP = CDDTCA)
    WHERE CDDTCA = '01' AND TPGEWB = 'CL'
    AND SUBSTR(CONTCA, 1, 4) <> '0299'".$paramFilter);

        return $users;
    }

    public function prepareUserData(&$userData) {
        // Rimuovo spazi in eccesso, caratteri speciali e sostituisco spazi con trattini per creare la mail
        $username = &$userData['username'];
        $username = preg_replace('/\s+/', ' ', $username);
        $username = str_replace(' ', '-', $username);
        $username = preg_replace('/[^A-Za-z0-9\-]/', '', $username);

        // Rimuovo alcuni caratteri speciali dal firstname e sostituisco spazi multipli con uno spazio solo
        $firstname = &$userData['firstname'];
        $firstname = preg_replace('/[^A-Za-z&\'\s.-]/', ' ', $firstname);
        $firstname = preg_replace('/\s+/', ' ', $firstname);

        $userData['username'] = strtolower($userData['username']);

        return $userData;
    }
    public function saveOrUpdateUser(&$userData)
    {

        $userData = $this->prepareUserData($userData);

        /** @var Model $model */
        $model = $this->getModel();
        $user = $model->firstOrNew([
            'username' => $userData['username']
        ]);

        foreach ($this->recordToUserMap() as $recordColumn => $modelProperty) {

            if ($modelProperty == 'password') {

                if (empty($userData[$recordColumn])) continue;

                $userData[$recordColumn] = trim($userData[$recordColumn]);
            }

            $user->{$modelProperty} = $userData[$recordColumn];
        }

        if (!$user->exists) {
            $this->logger->info('Nuovo utente: '.json_encode($user->toArray()));
            $this->newUsers += 1;
        } else {
            if ($user->isDirty()) {
                $this->logger->info('Modificato utente '.$user->username.' con codice '.$user->code.'. Dati modificati: '. implode(',', array_keys($user->getDirty()) ) );
                $this->editedUsers += 1;
            }
        }

        if (empty($user->password)) {
            $user->password = env('APP_KEY');
        }

        $user->save();
    }

    public function import()
    {
        $users = $this->getUsers();

        foreach ($users as $user) {
            $this->saveOrUpdateUser($user);
        }
    }
}
