<?php

return [
    'ioc' => [

        /*
        |--------------------------------------------------------------------------
        | Configurazione dei log
        |--------------------------------------------------------------------------
        |
        | dir => Path dove vengono salvati i file di log
        |-------------------------------------------------------------------------- */
        'log' => [
            'dir' =>  storage_path(env('IOC_LOG_DIR','logs/ioc/')),
            'dirPermissions' =>  env('IOC_LOG_FILE_PERMISSIONS',0770),
            'filePermissions' =>  env('IOC_LOG_FILE_PERMISSIONS',0660)
        ],

        /*
        |--------------------------------------------------------------------------
        | Configurazioni del cliente
        |--------------------------------------------------------------------------
        |
        | maxDevicesForUser => Il numero massimo di dispositivi per utente (default infiniti), superato questo limite il dispositivo viene bloccato e richiede approvazione manuale
        |-------------------------------------------------------------------------- */
        'maxDevicesForUser' => env('IOC_MAX_DEVICES_FOR_USER', 0),

        /*
        |--------------------------------------------------------------------------
        | B2B
        |--------------------------------------------------------------------------
        | url => Indirizzo http/https dell'url completo di /icm/ finale es. http://10.200.100.82:8080/B2BTestWeb/icm/
        | noAuthLaravel => Salta l'autenticazione su laravel e passa solamente per il B2B
        |-------------------------------------------------------------------------- */
        'b2b' => [
            'url' => env('IOC_B2B_URL'),
            'appConfigPath' => env('IOC_B2B_APPCONFIG_PATH'),
            'isGdprEnabled' => env('IOC_B2B_GDPR_ENABLED'),
            'version' => env('IOC_B2B_VERSION'),
            'noAuthLaravel' => env('IOC_B2B_NO_AUTH_LARAVEL', false),
        ],

        /*
        |--------------------------------------------------------------------------
        | JGalileo
        |--------------------------------------------------------------------------
        |
        | library => Libreria AS400 da utilizzare per estrarre dati es. "PRO90DAT." N.B. Tenere il punto dopo la stringa
        |-------------------------------------------------------------------------- */
        'as' => [
            'ip' => env('IOC_AS_IP', ''),
            'library' => env('IOC_AS_LIBRARY', ''),
            'user' => env('IOC_AS_USER', ''),
            'password' => env('IOC_AS_PASSWORD', ''),
            'headTable' => env('IOC_AS_HEAD_TABLE', 'WBOTEP0F'),
            'rowsTable' => env('IOC_AS_ROWS_TABLE', 'WBORIP0F'),
            'commentTable' => env('IOC_AS_COMMENT_TABLE', 'WBCOMP0F'),
        ],

        /*
        |--------------------------------------------------------------------------
        | Pim
        |-------------------------------------------------------------------------- */
        'pim' => [
            'apiUrl' => env('IOC_PIM_API_URL', ''),
            'version' => env('IOC_PIM_VERSION', ''),
            'user' => env('IOC_PIM_USER', ''),
            'password' => env('IOC_PIM_PASSWORD', '')
        ],

         /*
        |--------------------------------------------------------------------------
        | Integrazione con i moduli ICM di Prestashop
        |--------------------------------------------------------------------------
        |
        | dir => Path della root di prestashop, partendo dalla root di laravel
        |
        | Riparto sempre dalla base_path del modulo poiché potrei dover prendere immagini e/o sqlite su altri ps, quindi creo una variabile d'ambiente per ogni cartella
        |-------------------------------------------------------------------------- */
        'ps' => [

            // Directory
            'dir' => base_path(env('PS_DIR', '../')),
            'modulesDir' => base_path(env('PS_MODULES_DIR', '../modules/')),
            'icmSqliteDir' => base_path(env('PS_SQLITE_DIR', '../modules/icm/sqlite/')),
            'downloadDir' => base_path(env('PS_DOWNLOAD_DIR', '../download/')),
            'icmImgDir' => base_path(env('PS_IMG_DIR', '../modules/icm/img/')),
            'icmImgCategoriesDir' => base_path(env('PS_IMG_CATEGORIES_DIR', '../modules/icm/img/c/')),
            'icmImgProductsDir' => base_path(env('PS_IMG_PRODUCTS_DIR', '../modules/icm/img/p/')),
            'icmImgGalleriesDir' => base_path(env('PS_IMG_GALLERIES_DIR','../modules/icm/img/g/')),
            'icmImgCombinationsDir' => base_path(env('PS_IMG_COMBINATIONS_DIR','../modules/icm/img/co/')),
            'icmImgTmpDir' => base_path(env('PS_IMG_TMP_DIR', '../modules/icm/img/tmp/')),

            // Comportamento
            'loginViaB2B' => env('PS_LOGIN_VIA_B2B', false),

            // Cliente
            'client' => [
                'name' => env('PS_CLIENT', env('APP_NAME', 'nomecliente')),
                'mailSuffix' => env('PS_CLIENT_MAIL_SUFFIX', '@nomecliente.com')
            ],
        ],

        /*
       |--------------------------------------------------------------------------
       | Configurazioni invio mail
       |--------------------------------------------------------------------------
       | from => l'indirizzo mail da cui vengono spedite le mail
       |-------------------------------------------------------------------------- */
        'mail' => [
            'from' => env('IOC_MAIL_FROM', 'ioc@domain.com'),
        ],

    ]
];
