@php use Illuminate\Support\Facades\App; @endphp
@extends('ioc::layout.base')

@section('headJs')
    <script defer src="https://unpkg.com/alpinejs@3.x.x/dist/cdn.min.js"></script>
    @livewireScripts
@endsection

@section('headCss')
    <link rel="stylesheet" href="{{ asset('static/fontawesome/css/fontawesome.min.css') }}">
    <link rel="stylesheet" href="{{ asset('static/fontawesome/css/light.min.css') }}">
    <link rel="stylesheet" href="{{ asset('static/fontawesome/css/solid.min.css') }}">
    <link rel="stylesheet" href="{{ asset('static/fontawesome/css/brands.min.css') }}">
@endsection

@section('body')
    <body class="app">
    @include('ioc::layout.components.mobile-menu')
    <div class="flex">
        <!-- BEGIN: Side Menu -->
        <nav class="side-nav">
            <a href="" class="intro-x flex items-center pl-5 pt-4">
                <img alt="iOC Admin Panel" class="w-8" src="{{ asset('static/images/logo.svg') }}">
                <span class="hidden xl:block text-white text-lg ml-3">
                        iOC <span class="font-medium">Admin</span>
                    </span>
            </a>
            @if (App::environment() !== 'production')
                @livewire('refresh-sidebar')
            @endif
            <div class="side-nav__devider my-6"></div>
            <ul>
                @foreach ($side_menu as $menu)

                    @if($menu->level != 0)
                        @continue
                    @endif

                    @php $hasSubMenuOpen = array_search(Route::currentRouteName(), $menu->allSubRoutes()) !== false; @endphp

                    @if ($menu->name == 'devider')
                        <li class="side-nav__devider my-6"></li>
                    @elseif ($menu->permission == null || auth()->user()->can($menu->permission))
                        <li>
                            <a href="{{ Route::has($menu->route) ? route($menu->route) : '#' }}"
                               class="{{ $hasSubMenuOpen ? 'side-menu--open' : '' }} {{ preg_match('/^('. $menu->allRouteBase() . ')/', Route::currentRouteName()) ? 'side-menu side-menu--active' : 'side-menu' }}">
                                <div class="side-menu__icon">
                                    <i class="{{ $menu->icon }}"></i>
                                    {{--                                  <i ... {{ $menu->icon }} ...> --}}
                                </div>
                                <div class="side-menu__title">
                                    {{ $menu->name }}
                                    @if ($menu->children->count() > 0)
                                        <i data-feather="chevron-down"
                                           class="side-menu__sub-icon {{ $hasSubMenuOpen ? 'transform rotate-180' : '' }}"></i>
                                    @endif
                                </div>
                            </a>
                            @if ($menu->children->count() > 0)

                                <ul class="{{ $hasSubMenuOpen ? 'side-menu__sub-open' : '' }}">
                                    @foreach ($menu->children as $subMenu)
                                        @if($subMenu->permission != '')
                                            @cannot($subMenu->permission)
                                                @continue
                                            @endcannot
                                        @endif

                                        <li>
                                            <a href="{{ Route::has($subMenu->route) ? route($subMenu->route) : '#' }}"
                                               class="{{ $subMenu->route == Route::currentRouteName() ? 'side-menu side-menu--active' : 'side-menu' }}">
                                                <div class="side-menu__icon">
                                                    <i class="{{ $subMenu->icon }}"></i>
                                                    {{--                                  <i ... {{ $menu->icon }} ...> --}}
                                                </div>
                                                <div class="side-menu__title">
                                                    {{ $subMenu->name }}
                                                    {{--                                                    @if (isset($subMenu['sub_menu']))--}}
                                                    {{--                                                        <i data-feather="chevron-down" class="side-menu__sub-icon"></i>--}}
                                                    {{--                                                    @endif--}}
                                                </div>
                                            </a>
                                            {{--                                            @if (isset($subMenu['sub_menu']))--}}
                                            {{--                                                <ul class="{{ $second_page_name == $subMenu['page_name'] ? 'side-menu__sub-open' : '' }}">--}}
                                            {{--                                                    @foreach ($subMenu['sub_menu'] as $lastSubMenu)--}}
                                            {{--                                                        <li>--}}
                                            {{--                                                            <a href="{{ isset($lastSubMenu['layout']) ? route('page', ['layout' => $lastSubMenu['layout'], 'theme' => $theme, 'pageName' => $lastSubMenu['page_name']]) : 'javascript:;' }}" class="{{ $third_page_name == $lastSubMenu['page_name'] ? 'side-menu side-menu--active' : 'side-menu' }}">--}}
                                            {{--                                                                <div class="side-menu__icon">--}}
                                            {{--                                                                    <i data-feather="zap"></i>--}}
                                            {{--                                                                </div>--}}
                                            {{--                                                                <div class="side-menu__title">{{ $lastSubMenu['title'] }}</div>--}}
                                            {{--                                                            </a>--}}
                                            {{--                                                        </li>--}}
                                            {{--                                                    @endforeach--}}
                                            {{--                                                </ul>--}}
                                            {{--                                            @endif--}}
                                        </li>
                                    @endforeach
                                </ul>
                            @endif
                        </li>
                    @endif
                @endforeach
            </ul>
        </nav>
        <!-- END: Side Menu -->
        <!-- BEGIN: Content -->
        <div class="content">
            <!-- BEGIN: Top Bar -->
            <div class="top-bar">
                <!-- BEGIN: Breadcrumb -->
                <div class="-intro-x breadcrumb mr-auto hidden sm:flex">
                    <a href="{{ route('home') }}" class="">iOC</a>
                    @yield('breadcrumb')
                </div>
                <!-- END: Breadcrumb -->
                @include('ioc::layout.components.top-bar')
            </div>
            <!-- END: Top Bar -->
            @yield('content')
        </div>
        <!-- END: Content -->
    </div>
    {{--        @include('../layout/components/dark-mode-switcher')--}}

    <!-- BEGIN: JS Assets-->
    {{-- <script src="https://developers.google.com/maps/documentation/javascript/examples/markerclusterer/markerclusterer.js"></script>
    <script src="https://maps.googleapis.com/maps/api/js?key=["your-google-map-api"]&libraries=places"></script> --}}



    <script src="{{ asset('static/fontawesome/js/light.min.js') }}"></script>
    <script src="{{ asset('static/fontawesome/js/fontawesome.min.js') }}"></script>
    <script src="{{ asset('compiled/js/app.js') }}"></script>
    <!-- END: JS Assets-->


    @yield('script')
    </body>
@endsection
