<div>

    @can('create-app-users')
        <div class="mt-2 flex">
            <button wire:click="$emit('onCreateClick')" class="ml-auto button text-white bg-theme-1 shadow-md">Nuovo Utente App</button>
        </div>
    @endcan

    <x-modal :static="true">
        <x-slot name="content">
            <div class="p-6">

                <div class="hidden sm:block absolute top-0 right-0 pt-4 pr-4">
                    <button @click="open = false" type="button"
                            class="text-gray-400 hover:text-gray-500 focus:outline-none focus:text-gray-500 transition ease-in-out duration-150">
                        <svg class="h-6 w-6" stroke="currentColor" fill="none" viewBox="0 0 24 24">
                            <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2"
                                  d="M6 18L18 6M6 6l12 12" />
                        </svg>
                    </button>
                </div>

                <h2 class="text-lg font-medium mr-auto mb-4">{{ $title }}</h2>

                <div class="grid grid-cols-4 gap-4 row-gap-3">
                    <div class="col-span-4 sm:col-span-2 @error('name') has-error @enderror">
                        <label for="name">Nome:</label>
                        <input type="text" class="input w-full border mt-2" id="name" placeholder="Mario Rossi" wire:model.defer="name">
                        @error('name')
                        <div class="text-theme-6 mt-2">{{ $message }}</div>
                        @enderror
                    </div>
                    <div class="col-span-4 sm:col-span-2 @error('username') has-error @enderror">
                        <label for="username">Username:</label>
                        <input type="text" class="input w-full border mt-2" id="username" wire:model.defer="username" placeholder="mariorossi">
                        @error('username')
                        <div class="text-theme-6 mt-2">{{ $message }}</div>
                        @enderror
                    </div>
                    <div class="col-span-4 sm:col-span-2 @error('code') has-error @enderror">
                        <label for="code">ID Gestionale:</label>
                        <input type="text" class="input w-full border mt-2" id="code" wire:model.defer="code" placeholder="012345">
                        @error('code')
                        <div class="text-theme-6 mt-2">{{ $message }}</div>
                        @enderror
                    </div>
                    <div class="col-span-4 sm:col-span-2 @error('agent') has-error @enderror">
                        <label for="code">Agente:</label>
                        <input type="text" class="input w-full border mt-2" id="code" wire:model.defer="agent" placeholder="123">
                        @error('agent')
                        <div class="text-theme-6 mt-2">{{ $message }}</div>
                        @enderror
                    </div>
                    <div class="col-span-4 sm:col-span-2 @error('email') has-error @enderror">
                        <label for="username">Email:</label>
                        <input type="text" class="input w-full border mt-2" id="username" wire:model.defer="email" placeholder="mariorossi">
                        @error('email')
                        <div class="text-theme-6 mt-2">{{ $message }}</div>
                        @enderror
                    </div>
                    <div class="col-span-4 sm:col-span-2 @error('vat_code') has-error @enderror">
                        <label>P.IVA:</label>
                        <input type="text" class="input w-full border mt-2" placeholder="Partita IVA" wire:model="vat_code">
                        @error('vat_code') <span class="mt-2 text-red-500">{{ $message }}</span> @enderror
                    </div>
                    <div class="col-span-4 sm:col-span-2 @error('vat_code') has-error @enderror">
                        <label>Codice Fiscale:</label>
                        <input type="text" class="input w-full border mt-2" placeholder="Codice Fiscale" wire:model="fiscal_code">
                        @error('fiscal_code') <span class="mt-2 text-red-500">{{ $message }}</span> @enderror
                    </div>
                    <div class="col-span-4 sm:col-span-2 @error('type') has-error @enderror">
                        <label for="type">Tipo:</label>
                        {{--                        <input type="text" class="input w-full border mt-2" id="type" wire:model.defer="type" placeholder="daily | everyThirtyMinutes | cron ...">--}}
                        <select class="input w-full border mt-2" name="type" id="type" wire:model="type">
                            <option value="">Seleziona tipo</option>
                            <option value="C">Cliente</option>
                            <option value="D">Destinazione</option>
                        </select>

                        @error('type')
                        <div class="text-theme-6 mt-2">{{ $message }}</div>
                        @enderror
                    </div>
                    <div class="col-span-4 sm:col-span-2 @error('password') has-error @enderror">
                        <label>Password:</label>
                        <input type="password" class="input w-full border mt-2" placeholder="Password" wire:model="password">
                        @error('password') <span class="mt-2 text-red-500">{{ $message }}</span> @enderror
                    </div>
                    <div class="col-span-4 sm:col-span-2 @if(empty($password)) hidden @endif @error('confirmPassword') has-error @enderror">
                        <label>Conferma password</label>
                        <input type="password" class="input w-full border mt-2" placeholder="Password" wire:model.defer="confirmPassword">
                        @error('confirmPassword') <span class="mt-2 text-red-500">{{ $message }}</span> @enderror
                    </div>
                    <div class="col-span-4 sm:col-span-4 @error('active') has-error @enderror">
                        <label for="parameters">Attivo:</label>
                        <input type="checkbox" class="input input--switch border" id="parameters" wire:model.defer="active">
                        @error('active')
                        <div class="text-theme-6 mt-2">{{ $message }}</div>
                        @enderror
                    </div>

                    @if($this->modelId)
                        <!-- BEGIN: Dispositivi -->
                        <div class="col-span-4 sm:col-span-4">
                            @livewire('user-devices', ['user' => $this->modelId], key($this->modelId))
                        </div>
                        <!-- END: Dispositivi -->
                    @endif

                </div>
            </div>
            <div class="px-5 py-3 text-right border-t border-gray-200 dark:border-dark-5">
                <button @click="open = false" class="button w-24 border dark:border-dark-5 text-gray-700 dark:text-gray-300 ml-1">Annulla</button>
                <button wire:click.prevent="$emit('onSave')" class="button w-20 bg-theme-1 text-white ml-auto">Salva</button>
            </div>
        </x-slot>
    </x-modal>


</div>