<?php

use FourWords\AppManager\Ioc\Controllers\AdminController;
use FourWords\AppManager\Ioc\Controllers\AuthController;
use FourWords\AppManager\Ioc\Controllers\DashboardController;
use FourWords\AppManager\Ioc\Controllers\DeviceController;
use FourWords\AppManager\Ioc\Controllers\IocUserController;
use FourWords\AppManager\Ioc\Controllers\PasswordController;
use FourWords\AppManager\Ioc\Controllers\PermissionController;
use FourWords\AppManager\Ioc\Controllers\PermissionGroupController;
use FourWords\AppManager\Ioc\Controllers\RoleController;
use FourWords\AppManager\Ioc\Controllers\TaskController;
use FourWords\AppManager\Ioc\Controllers\TaskLogController;
use FourWords\AppManager\Ioc\Ps\GetAppRelease;
use FourWords\AppManager\Ioc\Ps\GetResource;
use FourWords\AppManager\Ioc\Ps\ImportPsUsers;
use FourWords\AppManager\Ioc\Ps\LoadOrder;
use FourWords\AppManager\Ioc\Ps\PSBonjour;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;

/*
|--------------------------------------------------------------------------
| Rotte WEB
|--------------------------------------------------------------------------
*/

Route::middleware('web')->group(function () {

    Route::middleware('loggedin')->group(function() {
        Route::get('login', [AuthController::class, 'loginView'])->name('login-view');
        Route::post('login', [AuthController::class, 'login'])->name('login');
        //Route::get('register', [AuthController::class, 'registerView'])->name('register-view');
        //Route::post('register', [AuthController::class, 'register'])->name('register');
        Route::get('password-dimenticata', [AuthController::class, 'forgotPasswordShow'])->name('forgotPassword.show');
        Route::post('password-dimenticata', [AuthController::class, 'forgotPasswordSendEmail'])->name('forgotPassword.email');

        Route::get('/reset-password/{token}', [AuthController::class, 'resetPassword'])->name('admin.password.reset');
        Route::post('/reset-password', [AuthController::class, 'updatePassword'])->name('admin.password.update');
    });

    Route::middleware('auth')->group(function() {

        Route::get('/', function () {
            return redirect('/dashboard');
        })->name('home');

        Route::get('/dashboard', [DashboardController::class, 'index'])->name('dashboard');

        Route::get('logout', [AuthController::class, 'logout'])->name('logout');
//    Route::get('page/{layout}/{theme}/{pageName}', [PageController::class, 'loadPage'])->name('page');

        // GEOLOCALIZZAZIONE
        Route::prefix('dashboard')->group(function() {
            Route::get('geo', [DashboardController::class, 'geo'])->name('dashboard.geo');
            Route::get('geo/failed', [DashboardController::class, 'geoFailed'])->name('dashboard.geo-failed');
        });

        // AMMINISTRATORI
        Route::prefix('administrator')->group(function() {

            Route::get('', [AdminController::class, 'index'])->name('admin.index');
            Route::get('{user}/edit', [AdminController::class, 'edit'])->name('admin.edit');

        });

        // UTENTI APP
        Route::prefix('user')->group(function() {

            Route::get('', [IocUserController::class, 'index'])->name('user.index');

        });

        // DISPOSITIVI APP
        Route::prefix('device')->group(function() {

            Route::get('', [DeviceController::class, 'index'])->name('device.index');

        });

        // TASK
        Route::prefix('task')->group(function() {

            Route::get('', [TaskController::class, 'index'])->name('task.index');
            Route::get('log', [TaskLogController::class, 'index'])->name('task.log.index');

        });

        // PERMESSI
        Route::prefix('permissions')->group(function() {

            Route::get('', [PermissionController::class, 'index'])->name('permission.index');

        });

        // GRUPPI PERMESSI
        Route::prefix('permission-group')->group(function() {

            Route::get('', [PermissionGroupController::class, 'index'])->name('permission-group.index');

        });

        // RUOLI
        Route::prefix('roles')->group(function() {

            Route::get('', [RoleController::class, 'index'])->name('role.index');

        });

//    Route::put('user/{user}/update', [UserController::class, 'update'])->name('user-update');
    });

});




/*
|--------------------------------------------------------------------------
| Rotte API
|--------------------------------------------------------------------------
*/

Route::prefix('api/ioc')->middleware('api')->group(function () {


    /*
    |--------------------------------------------------------------------------
    | Rotte libere
    |--------------------------------------------------------------------------
    */
    Route::get('bonjour', 'BonjourController@bonjour');

    Route::post('/forgot-password', [PasswordController::class, 'apiSendResetLink'])->name('password.forgot');

    /*
    |--------------------------------------------------------------------------
    | Rotte protette con HMAC / Bearer Token
    |--------------------------------------------------------------------------
    */
    Route::middleware('hmacorbearer')->group(function () {
        Route::get('getResource', 'ResourceController@get');
    });

    /*
    |--------------------------------------------------------------------------
    | Rotte protette con Bearer Token
    |--------------------------------------------------------------------------
    */
    Route::middleware('iocapi')->group(function () {
        Route::get('bonjourToken', 'BonjourController@bonjour');
    });

});


/*
|--------------------------------------------------------------------------
| Rotte API Prestashop
|--------------------------------------------------------------------------
*/

Route::prefix('api')->middleware('api')->group(function () {

    Route::get('user/import', function () {
        app()->make(ImportPsUsers::class)->handle();
    });

    Route::get('bonjour', function () {

        $requiredKeys = ['oudid', 'system_name', 'system_version', 'app_version', 'model'];
        foreach ($requiredKeys as $requiredKey) {
            if (!request()->has($requiredKey)) abort(400);
        }

        try {

            $response = app()->make(PSBonjour::class)->login();
            return response()->json($response);

        } catch (Exception $e) {
            return response()->json(['status' => 'ERROR', 'message' => 'Impossibile accedere. Si prega di riprovare più tardi']);
        }

    });

    Route::post('loadOrder', function () {
        return app()->make(LoadOrder::class)->send();
    });

    Route::get('getAppRelease', function (Request $request) {

        $token = $request->input('token');
        $app = $request->input('app');

        if(!$token || !$app) {
            abort(400);
        }

        try {
            $response = app()->make(GetAppRelease::class)->get($app, $token);
            return response()->json($response);

        } catch (Exception $e) {

            switch ($e->getMessage()) {
                case 'unavailable':
                    $status = 503;
                    break;
                case 'unauthorized':
                    $status = 401;
                    break;
                default:
                    $status = 404;
                    break;
            }
            return response()->make('',$status);
        }

    });

    Route::get('getResource', function (Request $request) {

        $token = $request->input('token');
        $type = $request->input('type');
        $file_name = $request->input('file');
        $userCode = $request->input('user');

        if(!$token || !$type || !$file_name) {
            abort(400);
        }

        try {
            $file = app()->make(GetResource::class)->get($file_name, $type, $token, $userCode);

            $size = intval(sprintf("%u", filesize($file)));
            if ($size + 1 >  134217728 && intval($size * 1.5) <= 1073741824) { //Not higher than 1GB
                ini_set('memory_limit', intval($size * 1.5));
            }
            @ini_set('zlib.output_compression', 0);


            return response()->download($file,$file_name); // ["Content-Type" => "application/octet-stream"] ?

        } catch (Exception $e) {

            $status = 404;
            if ($e->getMessage() == 'unauthorized') {
                $status = 401;
            }

            return response()->make('',$status);
        }

    });
});