<?php

namespace FourWords\AppManager\Ioc\Bonjour;
use FourWords\AppManager\Ioc\Models\IocUser;
use FourWords\AppManager\Orders\B2B;
use Illuminate\Support\Facades\Auth;

class B2BBonjour extends IocBonjour
{

    /** @var B2B */
    protected $b2b;

    public function __construct()
    {
        parent::__construct();
        $this->b2b = app()->make(B2B::class);
    }

    public function shouldAuthenticateRemotely()
    {
        return true;
    }

    public function authenticate()
    {
        // Mi autentico anche su Laravel a meno che la config noAuthLaravel sia true
        return config('ioc.b2b.noAuthLaravel') ? IocUser::where('username',request('user'))->first() : parent::authenticate();
    }

    public function authenticateRemotely() {

        try {

            $params = (request('user') && request('pwd')) ? request()->all() : ['user' => Auth::user()->username, 'pwd' => Auth::user()->password];

            // Autentico nel B2B
            $response = $this->b2b->authenticate($params);

            if ($response['status'] != 'OK') {
                $this->logger->error('Autenticazione fallita con b2b: '.$response['data']['error']);
            }
//                $token = $response['token'];
//                $subUser = $response['data']['subUser'] ?? "";
//                $userCode = $response['data']['userCode'];
//                $userRole = $response['data']['userRole'];
//                $subUsersList = $response['data']['subUsersList'] ?? [];
//                $actions = $response['data']['actions'];

            return $response;

        } catch (\Exception $e) {
            $this->logger->error($e->getMessage());
            telegramError('Errore di autenticazione nel b2b: '.$e->getMessage()."\n\n".json_encode(request()->all(), JSON_PRETTY_PRINT));
            $response = ['status' => 'ERROR', 'message' => 'Errore di autenticazione. Si prega di riprovare più tardi.'];
        }

        return $response;

    }

}
