<?php

namespace FourWords\AppManager\Ioc\Bonjour;

use Carbon\Carbon;
use FourWords\AppManager\Ioc\Events\BonjourFailedEvent;
use FourWords\AppManager\Ioc\Events\BonjourPayload;
use FourWords\AppManager\Ioc\Models\Device;
use FourWords\AppManager\Ioc\Models\IocUser;
use Illuminate\Support\Facades\Auth;

class IocBonjour extends AbstractBonjour
{

    public function hasLicenses() {
        return true;
    }

    public function getDevice() {
        return Device::where('identifier', request('oudid'))->first();
    }

    public function createDevice() {
        $this->device = Device::create([
            'identifier' => request('oudid'),
            'name' => request('system_name'),
            'version' => request('system_version'),
            'model' => request('model'),
            'app_id' => request('app'),
            'app_version' => request('app_version'),
            'push_device_token' => request('push_device_token'),
            'push_status' => request('push_status')
        ]);
        //TODO: Campi Notifiche Push?
    }

    public function setUserDeviceStatus($status) {
        $this->userDevicePivot->status = $status;
        $this->userDevicePivot->save();
    }

    public function updateUserDeviceLastAccess() {
        $this->userDevicePivot->last_access = Carbon::now();
        $this->userDevicePivot->save();
    }

    public function updateDevice() {
        $this->device->update([
            'app_version' => request('app_version'),
            'identifier' => request('oudid'),
            'name' => request('system_name'),
            'version' => request('system_version'),
            'model' => request('model'),
            'app_id' => request('app'),
            'push_device_token' => request('push_device_token'),
            'push_status' => request('push_status')
        ]);
    }

    public function getCurrentUserDevice() {
        return $this->user->devices()->withPivot('status','last_access')->where('user_id',$this->user->id)->where('ioc_devices.id',$this->device->id)->first()->pivot;
    }

    public function getOtherActiveUserDevicesCount() {
        return $this->user->devices()->withPivot('status','last_access')->where('user_id',$this->user->id)->where('status','OK')->count();
    }

    public function isDeviceEnabled() {
        return $this->device->active;
    }

    public function isUserEnabled()
    {
        return $this->user->active;
    }

    public function shouldWipe() {
        return $this->device->wipe;
    }

    public function createToken() {
        $this->user->tokens()->where('name', $this->device->identifier)->delete();
        return $this->user->createToken($this->device->identifier)->plainTextToken;
    }

    public function shouldWaitApproval() {
        if (!$this->isDemo() && intval(config('ioc.maxDevicesForUser')) > 0) {

            $otherActiveUserDevicesCount = $this->getOtherActiveUserDevicesCount();

            if ($this->userDevicePivot->status == "WAIT" && $otherActiveUserDevicesCount >= intval(config('ioc.maxDevicesForUser'))) {
                return true;
            }
        }

        return false;
    }

    public function isDemo() {
        return strtolower(request('user')) == "clappconnect" || strtolower(request('user')) == "clitest";
    }

    public function isUserDeviceEnabled() {
        return $this->userDevicePivot->status !== 'DISABLED';
    }

    public function authenticate() {
        if (Auth::user()) {
            $this->authenticatingWithToken = true;
            return Auth::user();
        } else {
            return IocUser::where('username',request('user'))->where('password', request('pwd'))->first();
        }
    }

    public function attachUserToDevice() {
        $this->device->users()->attach($this->user->id, ['status' => 'OK']);
    }

    public function getUserData() {
        $loggedUser = [];
        $loggedUser['id'] = $this->user->id;
        $loggedUser['clientCode'] = $this->user->code;
        $loggedUser['clientType'] = $this->user->type;
        $loggedUser['userCode'] = $this->user->code;
        $loggedUser['firstname'] = $this->user->name;
        $loggedUser['lastname'] = "";
        $loggedUser['username'] = $this->user->username;
        $loggedUser['email'] = $this->user->email ?? '';
        return $loggedUser;
    }

    public function sendResponse($status = 'OK', $message = '', $response = []) {

        if (!empty($response)) {
            $this->response = $response;
        }

        $this->response['status'] = $status;

        if (!empty($message)) {
            $this->response['message'] = $message;
        }

        return $status === 'OK' ? pushEvent(new BonjourPayload($this->response)) : pushEvent(new BonjourFailedEvent($this->response));
    }

}
