<?php

namespace FourWords\AppManager\Ioc\Commands;

use FourWords\AppManager\Ioc\Events\DidCopyFrontendFiles;
use FourWords\AppManager\Ioc\Helpers\Client;
use FourWords\AppManager\Ioc\Helpers\IocPackageManifest;
use FourWords\AppManager\IocClient\Providers\IocClientServiceProvider;
use Illuminate\Console\Command;
use Illuminate\Contracts\Filesystem\FileNotFoundException;
use Illuminate\Filesystem\Filesystem;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Artisan;
use Illuminate\Support\Facades\Event;
use Illuminate\Support\Facades\File;

class UpdateCommand extends Command
{

    const PACKAGE_DEPENDENCIES = [
        "@popperjs/core" => "^2.4.4",
        "cash-dom" => "^8.0.0",
        "chart.js" => "^2.9.3",
        "cross-env" => "^7.0.0",
        "dayjs" => "^1.8.33",
        "dropzone" => "^5.7.0",
        "feather-icons" => "^4.26.0",
        "laravel-mix" => "^6.0.6",
        "litepicker" => "^1.5.7",
        "postcss-loader" => "3",
        "pristinejs" => "^0.1.9",
        "raw-loader" => "3",
        "style-loader" => "1",
        "svg-loaders" => "^0.2.0",
        "tailwindcss" => "2.2.19",
        "tiny-slider" => "^2.9.3",
        "tippy.js" => "^6.2.6",
        "toastify-js" => "^1.12.0",
        "velocity-animate" => "^1.5.2",
        "vue-template-compiler" => "^2.6.11",
        "xlsx" => "^0.16.6",
        "zoom-vanilla.js" => "^2.0.6",
    ];

    const PACKAGE_DEV_DEPENDENCIES = [
        "axios" => "^0.21",
        "lodash" => "^4.17.19",
        "postcss" => "^8.1.14",
        "cross-env" =>  "^7.0.0", //TODO: Rimuovere quando funzionerà yarn run prod su vecchi server
        "resolve-url-loader" => "^3.1.0",
        "sass-loader" => "^12.1.0",
        "sass" => "^1.15.2",
    ];

    /**
     * The console command name.
     *
     * @var string
     */
    protected $name = 'ioc:update';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Aggiorna il Modulo iOC';

    /**
     * Execute the console command.
     *
     * @param Filesystem $filesystem
     *
     * @return void
     * @throws FileNotFoundException
     */
    public function handle(Filesystem $filesystem)
    {

        $this->call('optimize:clear');
        $this->call(ReorderPackages::class);

        // Pubblico gli assets del Modulo iOC
        $this->info('Pubblico gli assets del modulo iOC');
        $this->updateFrontend();
        $this->info('Pubblico gli assets del modulo CLIENTE iOC');
        $this->updateClientFrontend();
        Event::dispatch(new DidCopyFrontendFiles());

        // Modifico il file package.json per includere i pacchetti necessari
        $this->info('Includo dipendenze nel package.json');
        if (!$this->handlePackageJson($filesystem)) {
            $this->error("Errore sostituzione dipendenze package.json");
            return;
        }

        // Sostituisco il Webpack per combinare e compilare risorse js/css
        $this->info('Sostituisco il file webpack.mix.js');
        if (!$this->replaceWebPack($filesystem)) {
            $this->error('Impossibile sostituire webpack.mix.js');
            return;
        }

        // Sostituisco il file di configurazione Tailwind
        $this->info('Sostituisco il file tailwind.config.js');
        if (!$this->replaceTailwind($filesystem)) {
            $this->error('Impossibile sostituire tailwind.config.js');
            return;
        }

        $this->call(DumpAutoload::class);

        $this->call(ReorderPackages::class);
        $this->call('migrate');

        $this->info('Eseguo yarn install...');
        $res = shell_exec('cd '.base_path().' && yarn install');
        $this->info($res);

        $this->info('Eseguo yarn run...');
        $res = shell_exec('yarn run '.(App::environment() !== 'production' ? 'dev' : 'prod'));
        $this->info($res);

        $this->call('cache:clear');

        // fine installazione con successo!
        $this->info('Modulo iOC aggiornato con successo!');



    }

    public function updateFrontend()
    {

        /*
        |--------------------------------------------------------------------------
        | File da compilare
        |--------------------------------------------------------------------------
        | Il contenuto della cartella 'ioc/resources/to-compile' verrà copiato in resources/to-compile
        */
        File::copyDirectory(__DIR__ . '/../../resources/to-compile', resource_path('to-compile'));

        /*
        |--------------------------------------------------------------------------
        | File interni da copiare solamente in resources
        |--------------------------------------------------------------------------
        | Il contenuto della cartella 'ioc/resources/internal' verrà copiato in /resources
        */
        File::copyDirectory(__DIR__ . '/../../resources/internal', resource_path());

        /*
        |--------------------------------------------------------------------------
        | File statici
        |--------------------------------------------------------------------------
        | Il contenuto della cartella 'ioc/resources/static' verrà copiato in public/static
        */
        if (File::exists(public_path('static'))) {
            File::deleteDirectories(public_path('static'));
        }

        File::copyDirectory(__DIR__ . '/../../resources/static', public_path('static'));
        $this->extractFontAwesome();

        /*
        |--------------------------------------------------------------------------
        | File pubblici in storage
        |--------------------------------------------------------------------------
        | Il contenuto della cartella 'ioc/resources/storage/public' verrà copiato in storage/app/public
        */
        File::copyDirectory(__DIR__ . '/../../resources/storage/public', storage_path('app/public'));

    }

    public function updateClientFrontend()
    {
        /*
        |--------------------------------------------------------------------------
        | File da compilare
        |--------------------------------------------------------------------------
        | Il contenuto della cartella 'ioc/resources/to-compile' verrà copiato in resources/to-compile
        */
        if (File::exists(Client::getToCompileDir())) {
            File::copyDirectory(Client::getToCompileDir(), resource_path('to-compile'));
        }

        /*
        |--------------------------------------------------------------------------
        | File interni da copiare solamente in resources
        |--------------------------------------------------------------------------
        | Il contenuto della cartella 'ioc/resources/internal' verrà copiato in /resources
        */
        if (File::exists(Client::getInternalResourcesDir())) {
            File::copyDirectory(Client::getInternalResourcesDir(), resource_path());
        }

        /*
        |--------------------------------------------------------------------------
        | File statici
        |--------------------------------------------------------------------------
        | Il contenuto della cartella 'ioc-<nomecliente>/resources/static' verrà copiato in public/static
        */
        if (File::exists(Client::getStaticResourcesDir())) {
            File::copyDirectory(Client::getStaticResourcesDir(), public_path('static'));
        }

        /*
        |--------------------------------------------------------------------------
        | File pubblici in storage
        |--------------------------------------------------------------------------
        | Il contenuto della cartella 'ioc-<nomecliente>/resources/storage/public' verrà copiato in storage/app/public
        */
        if (File::exists(Client::getStoragePublicDir())) {
            File::copyDirectory(Client::getStoragePublicDir(), storage_path('app/public'));
        }

    }

    public function extractFontAwesome() {

        $path = __DIR__ . '/../../resources/fontawesome.zip';

        $zip = new \ZipArchive();

        if ($zip->open($path) === TRUE) {
            $zip->extractTo(public_path('static'));
            $zip->close();
        } else {
            $this->error('Impossibile estrarre Fontawesome');
            $this->error($zip->getStatusString());
            return;
        }
    }

    /**
     * @param Filesystem $filesystem
     *
     * @return bool
     */
    protected function handlePackageJson($filesystem)
    {
        $path = base_path('package.json');
        if (!$filesystem->exists($path)) {
            return false;
        } else {
            $json = json_decode(file_get_contents(base_path('package.json')));
            $json->dependencies = self::PACKAGE_DEPENDENCIES;
            $json->devDependencies = self::PACKAGE_DEV_DEPENDENCIES; //array_merge((array) $json->devDependencies, self::PACKAGE_DEV_DEPENDENCIES);
            $filesystem->put($path, json_encode($json));

            if (defined(IocClientServiceProvider::class.'::PACKAGE_DEPENDENCIES') && !empty(IocClientServiceProvider::PACKAGE_DEPENDENCIES)) {
                $json->dependencies = IocClientServiceProvider::PACKAGE_DEPENDENCIES;
                $filesystem->put($path, json_encode($json));
            }

            if (defined(IocClientServiceProvider::class.'::PACKAGE_DEV_DEPENDENCIES') && !empty(IocClientServiceProvider::PACKAGE_DEV_DEPENDENCIES)) {
                $json->devDependencies = IocClientServiceProvider::PACKAGE_DEV_DEPENDENCIES;
                $filesystem->put($path, json_encode($json));
            }

            $this->info('package.json aggiornato correttamente!');
            return true;
        }
    }

    /**
     * @param Filesystem $filesystem
     *
     * @return bool
     */
    protected function replaceWebPack($filesystem)
    {
        $mainWebPack = base_path() . '/webpack.mix.js';
        $iocWebPack = Client::hasWebpack() ? Client::webPack() : __DIR__ . '/../../webpack.mix.js';

        return $filesystem->copy($iocWebPack, $mainWebPack);
    }

    /**
     * @param Filesystem $filesystem
     *
     * @return bool
     */
    protected function replaceTailwind($filesystem)
    {
        $mainTailwind = base_path() . '/tailwind.config.js';
        $iocTailwind = Client::hasTailwindConfig() ? Client::tailwindConfig() : __DIR__ . '/../../tailwind.config.js';

        return $filesystem->copy($iocTailwind, $mainTailwind);
    }

}
