<?php

namespace FourWords\AppManager\Ioc\Controllers;

use App\Http\Controllers\Controller;
use FourWords\AppManager\Ioc\Models\User;
use FourWords\AppManager\Ioc\Request\LoginRequest;
use Illuminate\Auth\Events\PasswordReset;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Password;
use Illuminate\Http\Request;

class AuthController extends Controller
{
    public function loginView()
    {
        return view('ioc::login.main', [
            'theme' => 'light',
            'page_name' => 'auth-login',
            'layout' => 'login'
        ]);
    }

    /**
     * Authenticate login user.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function login(LoginRequest $request)
    {
        if (!Auth::attempt([
            'email' => $request->email,
            'password' => $request->password
        ], $request->get('remember_me') ?? false)) {
            throw new \Exception('E-mail o password errate.');
        }
    }

    /**
     * Logout user.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Http\RedirectResponse|\Illuminate\Http\Response|\Illuminate\Routing\Redirector
     */
    public function logout()
    {
        Auth::logout();
        return redirect('login');
    }

    public function forgotPasswordShow() {
        return view('ioc::login.forgot-password');
    }

    public function forgotPasswordSendEmail($email) {
        $passwordBroker = app('auth.password')->broker('users');

        try {
            $status = $passwordBroker->sendResetLink(
                ['email' => $email]
            );
        } catch (\Exception $e) {
            $error = 'Impossibile inviare la mail, contattare l\'amministratore';
            session()->flash('appstore_loginErrorMsg', $error);
            return response()->json(['status' => 'ERROR', 'message' => $error]);
        }

        $errorJSON = '';
        if($status == "passwords.sent") {
            $statusJSON = 'OK';
            session()->flash('appstore_loginSuccessMsg', 'E stata inviata via email il link per resettare la sua password');
        } elseif ($status == "passwords.user") {
            $statusJSON = 'ERROR';
            $errorJSON = 'Nessun account è associato a questa email';
            session()->flash('appstore_loginErrorMsg', $errorJSON);
        } elseif ($status == "passwords.throttled") {
            $statusJSON = 'ERROR';
            $errorJSON = 'Troppi tentativi di invio, si prega di riprovare più tardi.';
            session()->flash('appstore_loginErrorMsg', $errorJSON);
        } else {
            $statusJSON = 'ERROR';
            $errorJSON = 'Qualcosa è andato storto, conttattare l\'amministratore';
            session()->flash('appstore_loginErrorMsg', $errorJSON);
        }

        /*return $status === Password::RESET_LINK_SENT
            ? back()->with(['status' => __($status)])
            : back()->withErrors(['email' => __($status)]);*/

        return response()
            ->json(['status' => $statusJSON, 'message' => $errorJSON]);
    }

    public function resetPassword($token) {
        $passwordResets = DB::table('password_resets')->where('token', md5($token))->first();
        if(!$passwordResets){
            return view('ioc::login.reset-password', ['token' => null, 'errorMsg' => 'Link scaduto.']);
        }

        //dd(round((time() - strtotime($timeDB)) / 60,2));
        if (round((time() - strtotime($passwordResets->created_at)) / 60,2) >= 15){
            return view('ioc::login.reset-password', ['token' => null, 'errorMsg' => 'Link scaduto.']);
        }

        if(!User::where('email', '=', request()->get('email'))->first()) {
            return view('ioc::login.reset-password', ['token' => null, 'errorMsg' => 'Utente non trovato.']);
        }

        return view('ioc::login.reset-password', ['token' => $token]);
    }

    public function updatePassword(Request $request) {
        $request->validate([
            'token' => 'required',
            'email' => 'required|email',
            'password' => 'required|min:4|confirmed'
        ]);

        $status = Password::reset(
            $request->only('email', 'password', 'password_confirmation', 'token'),
            function ($user, $password) {
                $user->forceFill([
                    'password' => $password
                ]);
                $user->save();

                Auth::attempt(['email' => $user->email, 'password' => $password]);

                event(new PasswordReset($user));
            }
        );

        return $status === Password::PASSWORD_RESET
            ? redirect()->route('login-view')->with('status', __($status))
            : back()->withErrors(['email' => [__($status)]]);
    }
}
