<?php

namespace FourWords\AppManager\Ioc\Controllers;

use FourWords\AppManager\Ioc\Models\IocUser;
use Illuminate\Auth\Events\PasswordReset;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Password;

class PasswordController extends Controller {
    public function webSendResetLink($email) {
        $passwordBroker = app('auth.password')->broker('ioc_users');

        try {
            $status = $passwordBroker->sendResetLink(
                ['email' => $email]
            );
        } catch (\Exception $e) {
            $error = 'Impossibile inviare la mail, contattare l\'amministratore';
            session()->flash('appstore_loginErrorMsg', $error);
            return response()->json(['status' => 'ERROR', 'message' => $error]);
        }

        $statusJSON = '';
        $errorJSON = '';
        if($status == "passwords.sent") {
            $statusJSON = 'OK';
            session()->flash('appstore_loginSuccessMsg', 'E stata inviata via email il link per resettare la sua password');
        } elseif ($status == "passwords.user") {
            $statusJSON = 'ERROR';
            $errorJSON = 'Nessun account è associato a questa email';
            session()->flash('appstore_loginErrorMsg', $errorJSON);
        } elseif ($status == "passwords.throttled") {
            $statusJSON = 'ERROR';
            $errorJSON = 'Troppi tentativi di invio, si prega di riprovare più tardi.';
            session()->flash('appstore_loginErrorMsg', $errorJSON);
        } else {
            $statusJSON = 'ERROR';
            $errorJSON = 'Qualcosa è andato storto, conttattare l\'amministratore';
            session()->flash('appstore_loginErrorMsg', $errorJSON);
        }

        /*return $status === Password::RESET_LINK_SENT
            ? back()->with(['status' => __($status)])
            : back()->withErrors(['email' => __($status)]);*/

        return response()
            ->json(['status' => $statusJSON, 'message' => $errorJSON]);
    }

    public function apiSendResetLink(Request $request) {
        $validated = $request->validate([
            'email' => 'required|email',
        ]);

        return $this->webSendResetLink($request->get('email'));
    }

    public function resetPassword($token) {
        $passwordResets = DB::table('password_resets')->where('token', md5($token))->first();
        if(!$passwordResets){
            return view('appstore::user.reset-password', ['token' => null, 'errorMsg' => 'Link scaduto.']);
        }

        //dd(round((time() - strtotime($timeDB)) / 60,2));
        if (round((time() - strtotime($passwordResets->created_at)) / 60,2) >= 15){
            return view('appstore::user.reset-password', ['token' => null, 'errorMsg' => 'Link scaduto.']);
        }

        if(!IocUser::where('email', '=', request()->get('email'))->first()) {
            return view('appstore::user.reset-password', ['token' => null, 'errorMsg' => 'Utente non trovato.']);
        }

        return view('appstore::user.reset-password', ['token' => $token]);
    }

    public function updatePassword(Request $request) {
        $request->validate([
            'token' => 'required',
            'email' => 'required|email',
            'password' => 'required|min:4|confirmed'
        ]);

        $passwordBroker = app('auth.password')->broker('ioc_users');

        $status = $passwordBroker->reset(
            $request->only('email', 'password', 'password_confirmation', 'token'),
            function ($user, $password) {
                $user->forceFill([
                    'password' => $password
                ]);
                $user->save();

                Auth::guard('iocweb')->attempt(['username' => $user->username, 'password' => $password]);

                event(new PasswordReset($user));
            }
        );

        return $status === Password::PASSWORD_RESET
                ? redirect()->route('appstore.login')->with('status', __($status))
                : back()->withErrors(['email' => [__($status)]]);
    }
}