<?php

namespace FourWords\AppManager\Ioc\Database\Seeders;

use Faker\Factory;
use FourWords\AppManager\Ioc\Models\BonjourHistory;
use FourWords\AppManager\Ioc\Models\Device;
use FourWords\AppManager\Ioc\Geo\GeoTown;
use FourWords\AppManager\Ioc\Models\IocUser;
use FourWords\AppManager\Ioc\Models\Role;
use FourWords\AppManager\Ioc\Models\User;
use FourWords\AppManager\Orders\Model\Order;
use FourWords\AppManager\Orders\Model\OrderRow;
use FourWords\AppManager\PushNotifications\Model\PushNotification;
use FourWords\AppManager\PushNotifications\Model\PushNotificationGroup;
use Illuminate\Database\Seeder;
use Illuminate\Support\Str;

class DemoDataSeeder extends Seeder
{
    /**
     * Run the database seeds.
     *
     * @return void
     */
    public function run()
    {
        $faker = Factory::create('it_IT');

        User::create([
            'name' => 'Manager',
            'email' => 'manager@test.com',
            'email_verified_at' => now(),
            'role_id' => Role::where('name', '=', 'Manager')->first()->id,
            'password' => 'manager',
            'remember_token' => Str::random(10)
        ]);

//        User::factory()->times(15)->create();

        $iocUser = IocUser::create(
            [
                'name' => 'API Tester',
                'username' => 'apitester',
                'code' => str_pad('999',6,'0',STR_PAD_LEFT),
                'password' => 'password',
                'email' => $faker->unique()->safeEmail(),
                'active' => true
            ]
        );

        $name = $faker->randomElement(['iPhone ','Samsung Galaxy S']) . $faker->numberBetween(7,14);

        $iocUser->devices()->attach(Device::create([
            'identifier' => $faker->unique()->uuid,
            'name' => $name,
            'version' => rand(12,20),
            'app_id' => str_starts_with($name,'iPhone')  ? 1 : 2,
            'app_version' => '1.'.rand(0,8).'.0',
            'model' => strtolower($name).'-'.$faker->words(2, true),
        ]), ['last_access' => now()->subHours(rand(1,48)),
            'status' => 'OK',]);

        // ***
        //TODO: Rimuovere
        $iocUser = IocUser::create(
            [
                'name' => 'BITIFER SNC DI BRUSINELLI & TEST',
                'username' => 'cli7',
                'code' => '0201012861',
                'password' => 'Provacli7!',
                'vat_code' => '08563820151',
                'active' => true
            ]
        );
        $iocUser->devices()->attach(Device::create([
            'identifier' => '46AB7999-6F4D-428A-8CE1-7F614A5463B7',
            'name' => 'iPad 9 Dev',
            'version' => '15.0',
            'app_id' => 1,
            'app_version' => '1.0.4',
            'model' => 'iPad 6',
            'push_device_token' => '01d8bd3c7b727e1077317032e4cda0812f99fa8b8a54200a22c9994170e6215c'
        ]), ['last_access' => now()->subHours(rand(1,48)),
            'status' => 'OK',]);
        // ***

        User::create([
            'name' => $faker->name(),
            'email' => $faker->unique()->safeEmail(),
            'email_verified_at' => now(),
            'role_id' => Role::where('name', '=', 'Manager')->first()->id,
            'password' => 'password',
            'remember_token' => Str::random(10)
        ]);


        for ($i = 0; $i < 25; $i++) {

            $geo = GeoTown::inRandomOrder()->first();

            $iocUser = IocUser::create(
                [
                    'name' => substr($faker->company,0,24),
                    'code' => str_pad($faker->unique()->numberBetween(100,800),6,'0',STR_PAD_LEFT),
                    'username' => str_replace(' ','',strtolower($faker->name)),
                    'email' => $faker->unique()->safeEmail(),
                    'agent' => $faker->randomElement(['001','AGE','ELE']),
                    'city' => $geo->name,
                    'vat_code' => $faker->vat(),
                    'address' => $faker->streetAddress,
                    'cap' => $geo->cap,
                    'province' => $geo->prov,
                    'active' => $faker->boolean(70),
                    'password' => '5f4dcc3b5aa765d61d8327deb882cf99', // password
                ]
            );

            $name = $faker->randomElement(['iPhone ','Samsung Galaxy S']) . $faker->numberBetween(7,14);

            $iocUser->devices()->attach($device = Device::create([
                'identifier' => $faker->unique()->uuid,
                'name' => $name,
                'version' => rand(12,20),
                'app_id' => str_starts_with($name,'iPhone') ? 1 : 2,
                'app_version' => '1.'.rand(0,8).'.0',
                'model' => strtolower($name).'-'.$faker->words(2, true),
            ]), ['last_access' => now()->subHours(rand(1,48)),
                'status' => $faker->randomElement(['DISABLED','OK']),]);

            for ($j = 0; $j < rand(50, 70); $j++) {
                BonjourHistory::create([
                    'user' => $iocUser->id,
                    'device' => $device->id,
                    'created_at' => $faker->dateTimeBetween('-2 months')
                ]);
            }
        }

        for ($i = 0; $i < 200; $i++) {

            $status = $faker->boolean(85) ? 'OK' : $faker->randomElement(['WAIT', 'ERROR']);

            $userRandom = IocUser::inRandomOrder()->first();

            $date = $faker->dateTimeBetween('-2 months', 'now');
            $thirtyDaysAgo = (new \DateTime())->sub(new \DateInterval('P30D'));
            $order = Order::create([
                'creation_date' => $date,
                'username' => $userRandom->username,
                'code' => $userRandom->code,
                'reference' => $status == 'OK' ? $faker->numberBetween(111111, 999999) : '',
                'attempts' => $status == 'ERROR' ? $faker->numberBetween(1,5) : 1,
                'response_error' => $status == 'ERROR' ? $faker->sentence : '',
                'total' => $date <= $thirtyDaysAgo ? $faker->randomFloat(2, 30, 60) : $faker->randomFloat(2, 45, 80),
                'quantity_total' => $date <= $thirtyDaysAgo ?  $faker->numberBetween(1, 5) : $faker->numberBetween(3, 8),
                'status' => $status,
                'created_at' => $date
            ]);

            $qtTotal = 0;
            $total = 0;
            $packages = 0;

            for ($j = 0; $j < rand(1,6); $j++) {

                $q = $faker->numberBetween(1,9);
                $qtTotal += $q;

                $t = $faker->randomFloat(2,50,100);
                $total += $t;

                $p = $faker->numberBetween(1,5);
                $packages += $p;

                $order->rows()->create([
                    'product' => $faker->ean13(),
                    'row_type' => 'VE',
                    'row' => $j+1,
                    'product_name' => $faker->words(3, true),
                    'um' => 'PZ',
                    'gross_price' => $faker->randomFloat(2,100,150),
                    'reduced_price' => $t,
                    'quantity' => $q,
                    'packages' => $p,
                    'price_list' => $faker->numberBetween(10,99)
                ]);
            }

            $order->quantity_total = $qtTotal;
            $order->total = $total;
            $order->colli = $packages;
            $order->save();

        }

        for($i = 0; $i < 18; $i++) {

            $date = $faker->dateTimeBetween('-1 week', 'now');
            $userRandom = IocUser::inRandomOrder()->first();

            $order = Order::create([
                'creation_date' => $date,
                'username' => $userRandom->username,
                'code' => $userRandom->code,
                'reference' => $faker->numberBetween(111111, 999999),
                'attempts' => 1,
                'response_error' => '',
                'total' => $faker->randomFloat(2, 45, 100),
                'quantity_total' => $faker->numberBetween(3, 10),
                'status' => 'OK',
                'created_at' => $date
            ]);

            $qtTotal = 0;
            $total = 0;
            $packages = 0;

            for ($j = 0; $j < rand(1,6); $j++) {

                $q = $faker->numberBetween(1,9);
                $qtTotal += $q;

                $t = $faker->randomFloat(2,50,100);
                $total += $t;

                $p = $faker->numberBetween(1,5);
                $packages += $p;

                $order->rows()->create([
                    'product' => $faker->ean13(),
                    'row_type' => 'VE',
                    'row' => $j+1,
                    'product_name' => $faker->words(3, true),
                    'um' => 'PZ',
                    'gross_price' => $faker->randomFloat(2,100,150),
                    'reduced_price' => $t,
                    'quantity' => $q,
                    'packages' => $p,
                    'price_list' => $faker->numberBetween(10,99)
                ]);
            }

            $order->quantity_total = $qtTotal;
            $order->total = $total;
            $order->colli = $packages;
            $order->save();


        }

        for ($i = 0; $i < 130; $i++) {
            BonjourHistory::create([
                'user' => IocUser::inRandomOrder()->first()->id,
                'device' => Device::inRandomOrder()->first()->id,
                'created_at' => $faker->dateTimeBetween('-1 week')
            ]);
        }

        $pushNotification = PushNotification::create([
            'name' => 'Notifica di prova',
            'title' => 'Notifica di prova',
            'subtitle' => 'App iOC',
            'message' => 'Questa è una notifica di prova, ignorala pure.',
            'ios_send' => true,
            'android_send' => true,
            'draft' => true,
            'send_at' => now()->addMonth()
        ]);

        $group = PushNotificationGroup::create([
           'name' => 'Gruppo di prova'
        ]);
        $group->users()->attach(\App\Models\User::inRandomOrder()->limit(3)->get('id')->pluck('id'));
        $pushNotification->groups()->attach($group->id);

//        IocUser::factory()->times(15)
//        ->hasAttached(
//            Device::factory()->count(rand(1,3)),
//            ['last_access' => now()->subHours(rand(1,48)), 'status' => 'OK' ]
//        )->create();
//
//        /** @var IocUser $iocUser */
//        $iocUser = IocUser::create(
//            [
//                'name' => 'API Tester',
//                'username' => 'apitester',
//                'code' => str_pad('999',6,'0',STR_PAD_LEFT),
//                'password' => '5f4dcc3b5aa765d61d8327deb882cf99', // Password: 'password'
//            ]
//        );
//
//        $iocUser->devices()->attach(Device::factory()->times(1)->create(),
//            ['last_access' => now()->subHours(rand(1,48)), 'status' => 'OK' ]);

    }
}
