<?php

namespace FourWords\AppManager\Ioc\Exceptions;

use FourWords\AppManager\Ioc\Models\Task;
use FourWords\AppManager\Ioc\Models\TaskLog;
use FourWords\AppManager\Ioc\Telegram\iOCTelegramBot;
use Illuminate\Foundation\Exceptions\Handler;
use Illuminate\Queue\MaxAttemptsExceededException;
use Symfony\Component\HttpFoundation\Exception\BadRequestException;
use Throwable;

class IocExceptionHandler extends Handler
{

    protected $dontReport = [
        BadRequestException::class
    ];

    public function shouldReport(Throwable $e)
    {
        // Evito spam di messaggi della coda se il server ha problemi/rallentamenti
        if (str_starts_with($e->getMessage(),"SQLSTATE[HY000]: General error: 1205 Lock wait timeout exceeded")) {
            return false;
        }

        return parent::shouldReport($e);
    }

    protected function onMaxAttemptsExceeded($exception) {
        $task = app()->make(Task::class)->where('target_class', substr($exception->getMessage(),0,strpos($exception->getMessage(),' has been attempted')))->first();

        if ($task) {
            $task->running = false;
            $task->save();

            $joblog = app()->make(TaskLog::class);
            $joblog->task_id = $task->id;
            $joblog->start = time();
            $joblog->end = time();
            $joblog->isOK = false;
            $joblog->error = "Possibile esecuzione PHP in timeout, controllare log PHP";
            $joblog->save();
        }
    }

    public function report(Throwable $exception)
    {
        parent::report($exception);

        $errorMessage = $exception->getMessage();

        // Aggiungo tempi di esecuzione / timeout
        $currentExecutionTime = microtime(true) - $_SERVER['REQUEST_TIME_FLOAT'];
        $maxExecutionTime = ini_get('max_execution_time');
        $errorMessage = $errorMessage."\n\nExecution Time: ".round($currentExecutionTime,2)." / {$maxExecutionTime} secondi";

        // Salvo un log del task in caso di (presunta) esecuzione PHP in timeout
        if ($exception instanceof MaxAttemptsExceededException) {
            $this->onMaxAttemptsExceeded($exception);
        }

        if ($this->shouldReport($exception)) {

            if (app()->runningInConsole()) {
                echo $errorMessage;
            }

            /** @var iOCTelegramBot $tgBot */
            if (!env('IOC_TELEGRAM_SEND_MESSAGES')) return;

            $tgBot = app()->make(iOCTelegramBot::class);
            $tgBot->sendError($errorMessage);

        }
    }

}