<?php

namespace FourWords\AppManager\Ioc\Geo;

use Illuminate\Support\Facades\Http;

class HereGeocoder extends AbstractGeocoderService
{

    protected $version = "v1";
    protected $apiUrlBase = "https://geocode.search.hereapi.com/";

    protected $apiUrl = '';

    protected $geocodeEndpoint = 'geocode';

    public function __construct($cap = null, $province = null, $city = null, $address = null)
    {
        parent::__construct($cap, $province, $city, $address);
        $this->apiUrl = $this->apiUrlBase.$this->version.'/'.$this->geocodeEndpoint;
    }


    public function geolocalize(): ?GeocodedData
   {
       $apiKey = env('GEO_HERE_API_KEY');

       if (!$apiKey) {
           throw new \Exception('API Key per here.com non definita nell\'env');
       }


       $completeAddressString = ($this->address ? $this->address.',' : '') . $this->cap.' '.$this->city.(($this->city || $this->cap) ? ',' : '').' Italy';
       $errorMessage = 'Impossibile eseguire geocode su: '.$completeAddressString;

       try {
               $response = Http::get($this->apiUrl, [
                   'q' => $completeAddressString,
                   'apiKey' => $apiKey,
               ])->throw();
       } catch (\Exception $e) {
           $statusCode = $e->getCode();
           if ($statusCode == 429) {
               throw new ThrottleException($e->getMessage(), $e->getCode());
           } else if ($statusCode == 403) {
               throw new ExceededLimitsException($e->getMessage(), $e->getCode());
           }
       }
       $data = $response->json();

       if (empty($data['items'])) {
           throw new GeocodingFailedException($errorMessage."\nResponso: ".$response->body());
       }

       $firstResult = $data['items'][0];

       if (!isset($firstResult['position']['lat']) || !isset($firstResult['position']['lng'])) {
           throw new GeocodingFailedException($errorMessage."\nResponso: ".$response->body());
       }

       $latitude = $firstResult['position']['lat'];
       $longitude = $firstResult['position']['lng'];
       $cap = '';
       $city = '';
       $prov = '';
       $address = null;
       $countryCode = null;

       if (isset($firstResult['address']['postalCode'])) {

           $addressResponse = $firstResult['address'];

           $cap = $addressResponse['postalCode'];
           $city = $addressResponse['city'];
           $prov = $addressResponse['countyCode'];
           $address = $addressResponse['street'].(isset($addressResponse['houseNumber']) ? ' '.$addressResponse['houseNumber'] : '');
           $countryCode = $addressResponse['countryCode'];
       }

       return new GeocodedData($city, $cap, $prov, $latitude, $longitude, $countryCode, $address);
   }
}