<?php

namespace FourWords\AppManager\Ioc\Helpers;

use Exception;
use FourWords\AppManager\Ioc\Jobs\IocJob;
use FourWords\AppManager\Ioc\Models\Task;
use Illuminate\Contracts\Container\BindingResolutionException;
use Illuminate\Support\Facades\Gate;

class TaskManager
{

    /**
     * @param Task $task
     * @return void
     * @throws Exception
     */
    public static function runTask($task, $params = []) {

        if(Gate::denies('run-tasks')) {
            throw new Exception('Non hai il permesso per questa azione');
        }

        if(!$task) {
            throw new Exception('Task non trovato');
        }

        try {

            $iocJob = self::getJobFromClass($task->target_class, $params);
            self::dispatch($iocJob);

        } catch (Exception $ex) {
            throw new Exception('Errore esecuzione Task');
        }
    }

    /**
     * @param $taskId
     * @return void
     * @throws Exception
     */
    public static function runTaskId($taskId, $params = []) {
        $task = Task::findOrFail($taskId);
        self::runTask($task, $params);
    }

    /**
     * @param string $jobNamespace Namespace del job da lanciare
     * @return void
     */
    public static function runJob(string $jobNamespace, $params = []) {

        if(Gate::denies('run-tasks')) {
            throw new Exception('Non hai il permesso per questa azione');
        }

        try {

            $iocJob = self::getJobFromClass($jobNamespace, $params);
            self::dispatch($iocJob);

        } catch (Exception $ex) {
            throw new Exception('Errore esecuzione Task');
        }
    }

    /**
     * @param $namespaceClass
     * @return IocJob
     * @throws BindingResolutionException
     */
    protected static function getJobFromClass($namespaceClass, $params): IocJob
    {
        if (class_exists(trim($namespaceClass))) {
            /** @var IocJob $iocJob */
            $iocJob = app()->make($namespaceClass);
            $iocJob->setParameters($params);

        } else {
            $iocJob = app()->make(IocJob::class);
            $iocJob->setCommand($namespaceClass);
        }

        return $iocJob;
    }

    /**
     * @param IocJob $job
     * @return void
     */
    protected static function dispatch($job) {

        if ($job->task) {
            $job->task->running = true;
            $job->task->save();
        }

        if (strtolower(config('queue.default')) == 'sync') {
            dispatch_now($job);
        } else {
            dispatch($job);
        }
    }
}