<?php

namespace FourWords\AppManager\Ioc\Jobs;

use FourWords\AppManager\Ioc\Models\BonjourHistory;
use FourWords\AppManager\Ioc\Models\Device;
use FourWords\AppManager\Ioc\Models\IocUser;
use FourWords\AppManager\Ioc\Ps\Traits\PSConnection;
use Illuminate\Database\Connection;

class ImportDevices extends ImportJob
{
    use PSConnection;
    protected Connection $psDB;

    public function getEntityName(): string {
        return 'Dispositivo';
    }

    public function getPrimaryKeys($entity): array {
        return [
            'identifier' => $entity->code,
        ];
    }

    public function entityMapFields(): array {
        return [
            'code' => 'identifier',
            'system_name' => 'name',
            'system_version' => 'version',
            'model' => 'model',
            'id_app_info' => 'app_id',
            'app_version' => 'app_version',
            'status' => 'active',
            'clean_data' => 'wipe',
            'push_device_token' => 'push_device_token',
            'push_status' => 'push_status',
        ];
    }

    public function getEntities(): array {
        $this->psDB = $this->loadPSConnection();
        return $this->psDB->select('SELECT * FROM icm_devices');
    }

    public function getModel() : string {
        return Device::class;
    }

    protected function changeModelData(&$model, $entity) {
        foreach ($this->entityMapFields() as $recordColumn => $modelProperty) {
            if($recordColumn == 'status') {
                if($entity->{$recordColumn} == 'ENABLED') {
                    $model->active = 1;
                } else {
                    $model->active = 0;
                }
                continue;
            }

            $model->{$modelProperty} = $entity->{$recordColumn};
        }
    }

    public function handleJob() {

        $entities = $this->getEntities();

        foreach ($entities as $entity) {
            $modelType = app()->make($this->getModel());
            $model = $modelType->firstOrNew($this->getPrimaryKeys($entity));
            // Aggiungo solamente device nuovi
            if (!$model->exists) {
                $this->changeModelData($model, $entity);
                $this->logNewModel($model);
                $this->newEntities += 1;
                $model->save();
            }

        }
        $this->logTotals();

        // Collegamento Utente/Dispositivo
        $entities = $this->psDB->select('SELECT pe.id_employee, ide.id_device, email, id.code, ide.status, ide.last_access FROM icm_devices_employees AS ide
                                            JOIN ps_employee AS pe ON pe.id_employee = ide.id_employee
                                            JOIN icm_devices AS id ON id.id_device = ide.id_device');

        $devicesMap = Device::select('id', 'identifier')->whereIn('identifier', array_unique(array_column($entities, 'code')))->get()->pluck('id', 'identifier')->toArray();
        $users = IocUser::select('id', 'username')->whereIn('username', array_map(function($value) { return explode('@', $value)[0]; }, array_unique(array_column($entities, 'email'))))->get();
        $usersMap = $users->pluck('id', 'username')->toArray();

        $devicesIdsMap = [];
        $usersIdsMap = [];

        foreach ($entities as &$entity) {
            $deviceId = $devicesMap[$entity->code];
            $userId = $usersMap[explode('@',$entity->email)[0]];

            $usersIdsMap[$entity->id_employee] = $userId;
            $devicesIdsMap[$entity->id_device] = $deviceId;

            $user = $users->find($userId);
            if($user) {
                if($user->devices->contains($deviceId)) {
                    $user->devices()->updateExistingPivot($deviceId, ['status' => $entity->status, 'last_access' => $entity->last_access]);
                } else {
                    $user->devices()->attach($deviceId, ['status' => $entity->status, 'last_access' => $entity->last_access]);
                }
            }
        }

        // Statistiche bonjour
        $psBonjours = $this->psDB->select('SELECT * from icm_stats_bonjour');

        foreach ($psBonjours as $bonjour) {
            $user = IocUser::find($usersIdsMap[$bonjour->id_employee]);
            if ($user) {
                $deviceId = $devicesIdsMap[$bonjour->id_device];
                $bonjourHistory = BonjourHistory::firstOrNew(['user' => $user->id, 'device' => $deviceId, 'created_at' => $bonjour->date_access]);
                $bonjourHistory->created_at = $bonjour->date_access;
                $bonjourHistory->save();
            }
        }

    }
}
