<?php

namespace FourWords\AppManager\Ioc\Jobs;


use FourWords\AppManager\Ioc\Interfaces\Importable;

abstract class ImportJob extends IocJob implements Importable {
    protected int
        $newEntities = 0,
        $editedEntities = 0;

    public function handleJob() {
        parent::handleJob();

        $entities = $this->getEntities();

        foreach ($entities as $entity) {
            $modelType = app()->make($this->getModel());
            $model = $modelType->firstOrNew($this->getPrimaryKeys($entity));

            $this->changeModelData($model, $entity);

            if (!$model->exists) {
                $this->logNewModel($model);
                $this->newEntities += 1;
            } else {
                if ($model->isDirty()) {
                    $this->logEditedModel($model);
                    $this->editedEntities += 1;
                }
            }

            $model->save();
        }
        $this->logTotals();
    }

    protected function changeModelData(&$model, $entity) {
        foreach ($this->entityMapFields() as $recordColumn => $modelProperty) {
            $model->{$modelProperty} = $entity->{$recordColumn};
        }
    }

    protected function logNewModel($model) {
        $this->logger->info('Nuovo '.$this->getEntityName().': '.json_encode($model->toArray()));
    }

    protected function logEditedModel($model) {
        $this->logger->info('Modificato '.$this->getEntityName().' Dati modificati: '. implode(',', array_keys($model->getDirty()) ) );
    }

    protected function logTotals() {
        $this->logger->info("Nuove entità ".$this->getEntityName().": ".$this->newEntities);
        $this->logger->info("Entità ".$this->getEntityName()." modificate: ".$this->editedEntities);
    }
}