<?php

namespace FourWords\AppManager\Ioc\Jobs;

use Carbon\Carbon;
use FourWords\AppManager\Ioc\Ps\Traits\PSConnection;
use FourWords\AppManager\Orders\Model\Order;
use FourWords\AppManager\Orders\Model\OrderRow;
use Illuminate\Database\Connection;
use Illuminate\Support\Facades\File;

class ImportOrders extends ImportJob
{
    use PSConnection;
    protected Connection $psDB;

    public function getEntityName(): string {
        return 'Ordine';
    }

    public function getPrimaryKeys($entity): array {
        return ['creation_date' => $entity->creation_date, 'username' => $entity->username];
    }

    public function entityMapFields(): array {
        return [
            'date_upd' => 'updated_at',
            'date_add' => 'created_at',
            'creation_date' => 'creation_date',
            'username' => 'username',
            'customer_code' => 'code',
            'status' => 'status',
            'reference' => 'reference',
            'db_creation_date' => 'sqlite_db_date',
            'date_responce' => 'response_date',
            'attempts' => 'attempts'
        ];
    }

    public function getEntities(): array {
        $this->psDB = $this->loadPSConnection();
        return $this->psDB->select('SELECT * FROM iofa_sent_orders');
    }

    public function getModel() : string {
        return Order::class;
    }

    public function handleJob() {
        parent::handleJob();

        $this->getTotalsFromJson();
    }

    private function getTotalsFromJson() {
        $logsDir = config('ioc.ps.modulesDir').'icmordersfromas/logs/';

        $months = [];
        $fromMonth = Carbon::now()->subMonths(2);
        while (true) {
            $months[] = $fromMonth->format('Y-m');
            $fromMonth->addMonth();
            if($fromMonth > Carbon::now()) {
                break;
            }
        }

        foreach ($months as $month) {
            $files = File::glob($logsDir . $month . '_*_orders.txt');

            foreach ($files as $file) {
                if($file == $logsDir . $month . '_all_sent_orders.txt') {
                    continue;
                }

                $lines = File::lines($file);

                foreach ($lines as $line) {
                    if (empty($line)) {
                        continue;
                    }

                    $logData = json_decode(preg_replace('/^\[\d{2}-[A-Za-z]{3}-\d{4} \d{2}:\d{2}:\d{2}\] /', '', $line));
                    if ($order = app()->make($this->getModel())->where('creation_date', '=', $logData->orderInfo->order->dataCreazioneOrdine)->first()) {
                        $total = 0.0;
                        $quantity = 0;
                        $colli = 0;

                        foreach ($logData->orderInfo->order->rows as $orderRow) {
                            $quantity += $orderRow->qta;
                            $colli += $orderRow->colli;
                            $total += $orderRow->prezzoNetto * $orderRow->qta;

                            $orderRowModel = OrderRow::fromApp($orderRow, $order->id, $order->created_at);
                            $orderRowModel->save();

                        }

                        $order->total = round($total, 2);
                        $order->quantity_total = $quantity;
                        $order->colli = round($colli, 2);
                        $order->save();
                    }
                }

            }
        }
    }
}
