<?php

namespace FourWords\AppManager\Ioc\Livewire\Admin;

use FourWords\AppManager\Ioc\Livewire\IocComponent;
use FourWords\AppManager\Ioc\Models\Role;
use FourWords\AppManager\Ioc\Models\User;

class AdminCreateEditComponent extends IocComponent
{
    protected $modelName = "amministratore";
    protected $model = User::class;

    public
        $name,
        $email,
        $roles,
        $roleId,
        $password,
        $confirmPassword,
        $active;

    public $modelId;

    public $showModal = false;

    public $title = "Crea nuovo";

    protected $listeners = [
        'onCreateClick',
        'onEditClick',
        'onSave',
        'onCancel'
    ];

    public $fields = [
        'id',
        'name',
        'email',
        'role_id',
        'password',
        'confirmPassword',
        'active'
    ];

    protected $rules = [
        'name' => 'required',
        'email' => 'required|email'
    ];

    public function mount() {
        $this->title = "Crea nuovo ".$this->modelName;
        
        $this->roles = Role::all();
        $this->roleId = 1;
    }

    /**
     * Reseting all inputted fields
     * @return void
     */
    public function resetFields(){

        $this->resetErrorBag();

        foreach ($this->fields as $field) {

            if ($field == 'id') {
                $this->modelId = null;
            } else if ($field == 'active') {
                $this->{$field} = false;
            } else {
                $this->{$field} = '';
            }
        }

//        $this->emit('refreshUserDevices', $this->modelId);
    }

    public function render()
    {
        return view('ioc::livewire.admin.create-edit');
    }

    public function onCreateClick() {
        if(!$this->checkPermission('create-admin')) {
            return;
        }

        $this->resetFields();
        $this->title = "Crea ".$this->modelName;
        $this->showModal = true;
    }

    public function onEditClick($modelId) {
        if(!$this->checkPermission('edit-other-admin')) {
            return;
        }

        $this->resetFields();
        $this->title = "Modifica ".$this->modelName;

        try {
            $model = $this->model::findOrFail($modelId);
            if(!$model) {
                $this->errorToast($this->modelName.' non trovato');
                return;
            }

            foreach ($this->fields as $field) {

                if ($field == 'password' || $field == 'confirmPassword') {
                    continue;
                }

                if ($field == 'id') {
                    $this->modelId = $model->id;
                } else {
                    $this->{$field} = $model->{$field};
                }
            }

            $this->showModal = true;

        } catch (\Exception $ex) {
            $this->errorToast('Errore modifica '.$this->modelName);
        }


    }

    public function onSave() {
        if ($this->modelId) {
            if($this->checkPermission('edit-other-admin')) {
                $this->update();
            }
        } else {
            if($this->checkPermission('create-admin')) {
                $this->store();
            }
        }
    }

    public function store()
    {
        $this->beforeValidate();
        $this->validate();
        
        try {
            $admin = new $this->model();

            $admin->name = $this->name;
            $admin->email = $this->email;
            $admin->role()->associate(Role::where('id','=',$this->roleId)->first());
            $admin->password = $this->password;
            $admin->save();

            $this->emit('refreshLivewireDatatable');
            $this->showModal = false;
            $this->successToast($this->modelName.' creato con successo');
            $this->resetFields();
        } catch (\Exception $ex) {
            $this->errorToast($ex->getMessage());
            $this->errorToast('Errore nella creazione '.$this->modelName);
        }
    }

    public function beforeValidate() {

        if (!$this->modelId) {
            $this->rules['password'] = 'required';
        }

        if (!$this->modelId || $this->password != null) {
            $this->rules['confirmPassword'] = 'required|same:password';
        }
    }

    public function update()
    {
        $this->beforeValidate();
        $this->validate();
        try {

            $model = $this->model::findOrFail($this->modelId);

            foreach ($this->fields as $field) {

                if ($field == 'confirmPassword') {
                    continue;
                }

                if ($field == 'id') {
                    $model->id = $this->modelId;
                    continue;
                }

                if ($field == 'password' && empty($this->password)) {
                    continue;
                }

                $model->{$field} = $this->{$field};
            }

            $model->save();
            $this->emit('refreshLivewireDatatable');
            $this->showModal = false;
            $this->successToast($this->modelName.' aggiornato con successo');
            $this->resetFields();
        } catch (\Exception $ex) {
            $this->errorToast('Errore nella modifica '.$this->modelName);
        }
    }

}
