<?php

namespace FourWords\AppManager\Ioc\Livewire\Admin;

use FourWords\AppManager\Ioc\Livewire\IocComponent;
use FourWords\AppManager\Ioc\Models\Permission;
use FourWords\AppManager\Ioc\Models\RolePermission;
use FourWords\AppManager\Ioc\Models\User;
use FourWords\AppManager\Ioc\Models\UserPermission;

class PermissionSetting extends IocComponent
{
    public
        $user,
        $permissions = [],
        $userPermissions = [],
        $rolePermissions = [];

    public function mount() {
        $this->getPermissions();
        $this->getUserPermissions();
    }

    public function render() {
        return view("ioc::livewire.admin.permission-settings");
    }

    public function getPermissions() {
        $this->permissions = Permission::orderBy('group_id', 'asc')->get();
    }

    public function getUserPermissions() {
        $this->userPermissions = User::find($this->user->id)->getUserPermissions();
        $this->rolePermissions = RolePermission::where("role_id", "=", $this->user->role_id)->where("active", "=", true)->get()->pluck('permission_id')->toArray();

    }

    public function save() {
        $addPermissions = array_diff($this->userPermissions, $this->rolePermissions);
        $removePermissions = array_diff($this->rolePermissions, $this->userPermissions);

        $model = UserPermission::where('user_id', '=', $this->user->id);
        $model->delete();

        foreach ($addPermissions as $addPermission) {
            $model = new UserPermission();
            $model->user_id = $this->user->id;
            $model->permission_id = $addPermission;
            $model->action = true;
            $model->save();
        }

        foreach ($removePermissions as $removePermission) {
            $model = new UserPermission();
            $model->user_id = $this->user->id;
            $model->permission_id = $removePermission;
            $model->action = false;
            $model->save();
        }
        $this->successToast('I permessi dell\'utente sono stati aggiornati con successo');
    }

    public function reset() {
        $model = UserPermission::where('user_id', '=', $this->user->id);
        $model->delete();
        $this->userPermissions = RolePermission::where("role_id", "=", $this->user->role_id)->where("active", "=", true)->get()->pluck('permission_id')->toArray();
        $this->successToast('I permessi dell\'utente sono stati resettati con successo');
    }

    public function selectAllPermissions($groupId) {
        $permissionNotSelected = array_diff(Permission::where("group_id", "=", $groupId)->where("active", "=", true)->get()->pluck('id')->toArray(), $this->userPermissions);

        foreach ($permissionNotSelected as $p) {
            $this->userPermissions[] = $p;
        }
    }

    public function deselectAllPermissions($groupId) {
        $this->userPermissions = array_values(array_diff($this->userPermissions, Permission::where("group_id", "=", $groupId)->where("active", "=", true)->get()->pluck('id')->toArray()));
    }

    public function countPermissions($groupId)
    {
        $permissions = Permission::where("group_id", "=", $groupId)->where("active", "=", true)->get()->pluck('id')->toArray();
        $permissionNotSelected = array_diff($permissions, $this->userPermissions);
        return '[ ' . (count($permissions) - count($permissionNotSelected)) . ' / ' . count($permissions) . ' ]';
    }

    public function getPermissionsSelected($groupId) {
        $permissions = Permission::where("group_id", "=", $groupId)->where("active", "=", true)->get()->pluck('id')->toArray();
        $permissionNotSelected = array_diff($permissions, $this->userPermissions);
        return (count($permissions) - count($permissionNotSelected));
    }

    public function getMaxPermissions($groupId) {
        return count(Permission::where("group_id", "=", $groupId)->where("active", "=", true)->get()->pluck('id')->toArray());
    }
}

