<?php

namespace FourWords\AppManager\Ioc\Livewire\Dashboard\Charts;

use Carbon\Carbon;
use FourWords\AppManager\Ioc\Livewire\Dashboard\GeneralStats;
use FourWords\AppManager\Ioc\Livewire\IocComponent;

class AccessiGiornalieriChart extends IocComponent {
    public string
        $title,
        $column,
        $currentPeriodName ='aaa',
        $lastPeriodName = 'aaaa';

    public int
        $currentPeriodValue,
        $lastPeriodValue;

    public string
        $jsonUniqueAccessData,
        $jsonAllAccessData;

    private int $dayStats = 30;

    protected $listeners = ['periodChanged', 'refreshStats'];

    public function mount() {
        $this->title = 'Ordini Ricevuti';
        $this->column = 'quantity_total';

        $this->fillData();
    }

    public function render() {
        return view('ioc::dashboard.livewire.daily-access-chart');
    }

    public function fillData() {
        $users = GeneralStats::getBonjour($this->dayStats);
        $usersUnique = GeneralStats::getUniqueBonjour()
            ->where('login_date', '>=', Carbon::today()->subDays($this->dayStats));

        $this->weeklyUserAccess($usersUnique);
        $this->uniqueUserAccess($usersUnique);
        $this->allUserAccess($users);
    }

    private function weeklyUserAccess($users) {
        $this->setPeriodName();
        $this->currentPeriodValue = 0;
        $this->lastPeriodValue = 0;

        $date = Carbon::today()->subDays(($this->dayStats/2)-1);
        foreach($users as $user) {
            if($user->login_date >= $date->format('Y-m-d')) {
                $this->currentPeriodValue += $user->logins;
            } else {
                $this->lastPeriodValue += $user->logins;
            }
        }
    }

    private function uniqueUserAccess($usersUnique) {
        $data = new LineChartData();
        $data->label = 'Unici';
        $data->color = '#1C3FAA';
        $data->bgColor = '#1C3FAA33';

        $allUsersAccessData = $this->fillAllData($usersUnique, Carbon::today()->subDays($this->dayStats-1), Carbon::today());
        ksort($allUsersAccessData);

        $days = [];
        $logins = [];
        setlocale(LC_TIME, 'it_IT.UTF-8');
        foreach($allUsersAccessData as $date => $user) {
            $ts = strtotime($date);
            $isToday = date('Y-m-d',time()) == date('Y-m-d',$ts);
            $days[] = $isToday ? 'Oggi' : strftime("%a %d %b.", $ts);
            $logins[] = $user;
        }

        $data->x = $days;
        $data->y = $logins;
        $this->jsonUniqueAccessData = json_encode($data);
    }

    private function allUserAccess($users) {
        $data = new LineChartData();
        $data->label = 'Totali';
        $data->color = '#8DA9BE';
        $data->bgColor = '#8DA9BE33';

        $allUsersAccessData = $this->fillAllData($users, Carbon::today()->subDays($this->dayStats-1), Carbon::today());
        ksort($allUsersAccessData);

        $days = [];
        $logins = [];
        foreach($allUsersAccessData as $date => $user) {
            $days[] = date("d M.", strtotime($date));
            $logins[] = $user;
        }

        $data->x = $days;
        $data->y = $logins;
        $this->jsonAllAccessData = json_encode($data);
    }

    private function fillAllData($usersData, $startDate, $endDate) {
        $allData = [];
        foreach($usersData as $user) {
            $allData[$user->login_date] = $user->logins;
        }

        foreach ($startDate->daysUntil($endDate) as $date) {
            if(!isset($allData[$date->format('Y-m-d')])) {
                $allData[$date->format('Y-m-d')] = 0;
            }
        }
        return $allData;
    }

    private function setPeriodName() {
        switch ($this->dayStats) {
            case 14:
                $this->currentPeriodName = 'Ultima Settimana';
                $this->lastPeriodName = 'Scorsa Settimana';
                break;

            case 30:
                $this->currentPeriodName = 'Ultimi 15 Giorni';
                $this->lastPeriodName = 'Precedenti 15 Giorni';
                break;

            default:
                $this->currentPeriodName = 'Ultimi '.$this->dayStats.' Giorni';
                $this->lastPeriodName = 'Precedenti '.$this->dayStats.' Giorni';
        }
    }

    public function periodChanged($value) {
        if($value != -1) {
            $this->dayStats = $value;
        }

        $this->fillData();
        $this->createAccessLineChart();
    }

    public function refreshStats($period) {
        $this->periodChanged($period);
    }

    public function createAccessLineChart() {
        $this->dispatchBrowserEvent('dailyAccessLineChartChanged', ['uniqueAccessData' => json_decode($this->jsonUniqueAccessData), 'allAccessData' => json_decode($this->jsonAllAccessData)]);
    }
}

