<?php

namespace FourWords\AppManager\Ioc\Livewire\Dashboard\Charts;

use FourWords\AppManager\Ioc\Livewire\Dashboard\GeneralStats;
use FourWords\AppManager\Ioc\Livewire\IocComponent;

class OrdiniRicevutiChart extends IocComponent {
    public string
        $title,
        $currentPeriodName,
        $lastPeriodName;

    public int
        $currentPeriodValue,
        $lastPeriodValue;

    public string $jsonData;

    private int $dayStats = 30;

    protected $listeners = ['periodChanged', 'refreshStats'];

    public function mount() {
        $this->title = 'Ordini Ricevuti';
        $this->fillData();
    }

    public function render() {
        return view('ioc::dashboard.livewire.orders-chart');
    }

    public function fillData() {
        $date = \Carbon\Carbon::today()->subDays($this->dayStats - 1)->startOfDay();
        $orders = GeneralStats::getOrders()->where('created_at','>=',$date)->sortByDesc('creation_date');

        $this->weeklyOrders($orders);
        $this->chartOrders($orders);
    }

    private function weeklyOrders($orders) {
        $this->setPeriodName();
        $this->currentPeriodValue = 0;
        $this->lastPeriodValue = 0;

        $date = \Carbon\Carbon::today()->subDays(($this->dayStats/2)-1);
        foreach($orders as $order) {
            if($order->created_at >= $date) {
                $this->currentPeriodValue++;
            } else {
                $this->lastPeriodValue++;
            }
        }
    }

    private function chartOrders($orders) {
        $data = new LineChartData();
        $data->label = 'Inviati';
        $data->color = '#1C3FAA';
        $data->bgColor = '#1C3FAA33';

        $endDate = \Carbon\Carbon::today();
        $startDate = \Carbon\Carbon::today()->subDays($this->dayStats-1);

        $orderData = [];
        foreach($orders as $order) {
            $date = explode(' ', $order->created_at)[0];

            if($date < $startDate->format('Y-m-d')) {
                break;
            }

            if(!isset($orderData[$date])) {
                $orderData[$date] = 0;
            }

            $orderData[$date]++;
        }

        foreach ($startDate->daysUntil($endDate) as $date) {
            if(!isset($orderData[$date->format('Y-m-d')])) {
                $orderData[$date->format('Y-m-d')] = 0;
            }
        }

        $days = [];
        $orders = [];

        ksort($orderData);
        setlocale(LC_TIME, 'it_IT.UTF-8');
        foreach($orderData as $date => $order) {
            $ts = strtotime($date);
            $isToday = date('Y-m-d',time()) == date('Y-m-d',$ts);
            $days[] = $isToday ? 'Oggi' : strftime("%a %d %b.", $ts);
            $orders[] = $order;
        }

        $data->x = $days;
        $data->y = $orders;
        $this->jsonData = json_encode($data);
    }

    private function setPeriodName() {
        switch ($this->dayStats) {
            case 14:
                $this->currentPeriodName = 'Ultima Settimana';
                $this->lastPeriodName = 'Scorsa Settimana';
                break;

            case 30:
                $this->currentPeriodName = 'Ultimi 15 Giorni';
                $this->lastPeriodName = 'Precedenti 15 Giorni';
                break;

            default:
                $this->currentPeriodName = 'Ultimi '.$this->dayStats.' Giorni';
                $this->lastPeriodName = 'Prece
                denti '.$this->dayStats.' Giorni';
        }
    }

    public function periodChanged($value) {
        if($value != -1) {
            $this->dayStats = $value;
        }

        $this->fillData();
        $this->createLineChart();
    }

    public function refreshStats($period) {
        $this->periodChanged($period);
    }

    public function createLineChart() {
        $this->dispatchBrowserEvent('ordersLineChartChanged', ['data' => json_decode($this->jsonData)]);
    }
}
