<?php

namespace FourWords\AppManager\Ioc\Livewire\Dashboard;

use Carbon\Carbon;
use FourWords\AppManager\Ioc\Livewire\IocComponent;
use FourWords\AppManager\Ioc\Models\BonjourHistory;
use FourWords\AppManager\Ioc\Models\IocUser;
use FourWords\AppManager\Orders\Model\Order;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\DB;

class GeneralStats extends IocComponent {
    public string
        $title = "Statistiche",
        $version = '';
    public $statsPeriod = 30;

    public function mount() {

    }

    public function render() {
        return view('ioc::dashboard.livewire.general-stats');
    }

    public static function getOrders($days = 60): Collection
    {
        return Cache::remember('stats_orders_'.$days, 60*6, function() use($days) {

            $q = app()->make(Order::class)->where('status', 'OK')->with('rows');
            if ($days > 0) {
                $q = $q->where('created_at', '>=', Carbon::today()->subDays($days-1));
            }
            return $q->orderBy('created_at')->get();

        });
    }

    public function updating($name, $value) {
        if($name == 'statsPeriod') {
            if ($this->statsPeriod != $value) {
                $this->statsPeriod = $value;
                $this->emit('periodChanged', $value);
            }
        }
    }

    public static function getUsers(): Collection
    {
        return Cache::remember('stats_users', 60*2, function () {
            return IocUser::where('active', true)->get();
        });
    }

    /*
    SELECT DATE(created_at) AS login_date, COUNT(DISTINCT user) AS unique_logins
FROM ioc_bonjours
GROUP BY login_date
ORDER BY login_date;
    */

    public static function getBonjour($days = 14): Collection
    {
        return Cache::remember('stats_bonjour_'.$days, 60*6, function() use($days) {

            $q = BonjourHistory::selectRaw('DATE(created_at) AS login_date')
                ->selectRaw('COUNT(*) AS logins');
            if ($days > 0) {
                $q = $q->where('created_at', '>=', Carbon::today()->subDays($days-1));
            }
            return $q->groupBy('login_date')
                ->orderBy('login_date')
                ->get();
        });
    }

    public static function getUniqueBonjour(): Collection
    {
        return Cache::remember('stats_unique_bonjour', 60*6, function() {
            return BonjourHistory::selectRaw('DATE(created_at) AS login_date')
                ->selectRaw('COUNT(DISTINCT user) AS logins')
                ->groupBy('login_date')
                ->orderBy('login_date')
                ->get();
        });
    }

//    public static function getActiveClientsBonjour($days = 60)
//    {
//        return Cache::remember('stats_active_clients_bonjour_'.$days, 60*6, function() use ($days) {
//
//            // Periodo più lontano
//            $q = BonjourHistory::selectRaw('COUNT(*) / COUNT(DISTINCT DATE(created_at)) AS media_bonjour_giornalieri');
//
//            if ($days > 0) {
//                $result[] = (int) $q
//                    ->where('created_at', '>=', Carbon::today()->subDays($days-1))
//                    ->where('created_at', '<=', Carbon::today()->subDays(($days/2)-1))
//                    ->first()->media_bonjour_giornalieri ?? 0.0;
//            } else {
//                $result[] = 0;
//            }
//
//            // Periodo più vicino
//            $q = BonjourHistory::selectRaw('COUNT(*) / COUNT(DISTINCT DATE(created_at)) AS media_bonjour_giornalieri');
//
//            if ($days > 0) {
//                $result[] = (int) $q
//                    ->where('created_at', '>=', Carbon::today()->subDays(($days/2)-1))
//                    ->first()->media_bonjour_giornalieri ?? 0.0;
//            } else {
//                $result[] = (int) $q->first()->media_bonjour_giornalieri ?? 0.0;
//            }
//
//
//            return $result;
//        });
//    }

    public static function getBestClients(): Collection
    {
        return Cache::remember('stats_best_clients', 60*6, function () {

            return app()->make(Order::class)->select('username', DB::raw('SUM(total) as total_spent'))
                ->where('status', 'OK')
//                ->where('created_at', '>=', Carbon::today()->subDays($days-1))
                ->groupBy('username')
                ->orderByDesc('total_spent')
                ->limit(5)
                ->get();
        });
    }

    public function refreshStats() {

        foreach(['60','28','-1'] as $d) {
            Cache::forget('stats_orders_'.$d);
            Cache::forget('stats_users');
            Cache::forget('stats_bonjour_'.$d);
            Cache::forget('stats_unique_bonjour');
            Cache::forget('stats_active_clients_bonjour_'.$d);
            Cache::forget('stats_best_clients');
        }

        $this->emit('refreshStats', intval($this->statsPeriod));
    }

}
