<?php

namespace FourWords\AppManager\Ioc\Livewire\Dashboard\Stats;

use Carbon\Carbon;
use FourWords\AppManager\Ioc\Livewire\IocComponent;
use FourWords\AppManager\Ioc\Models\BonjourHistory;
use FourWords\AppManager\Ioc\Models\IocUser;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\DB;

class AttiviOraStats extends IocComponent {

    public $activeNow = [];
    public $minutesAgo = 60;


    public function mount() {
        $this->getActiveNow();
    }

    public function getActiveNow()
    {
        $dataToRetrieve = 'user, device, ioc_users.name as name, ioc_users.username as username, ioc_users.agent as agent, ioc_devices.app_id as app_id, ioc_devices.name as device_name';
        $groupBy = 'user, device, ioc_users.name, ioc_users.username, ioc_users.agent, ioc_devices.app_id, ioc_devices.name';

        if (env('APP_DEBUG')) {
            $this->activeNow = DB::select('SELECT '.$dataToRetrieve.', ioc_bonjours.created_at
FROM ioc_bonjours
JOIN ioc_users on user = ioc_users.id
JOIN ioc_devices on device = ioc_devices.id
GROUP BY '.$groupBy.', ioc_bonjours.created_at
ORDER BY ioc_bonjours.created_at DESC
LIMIT ?
', [rand(3,7)]);
            return;
        }

        $this->activeNow = Cache::remember('stats_active_now', 60, function() use ($dataToRetrieve, $groupBy) {
            return DB::select('SELECT '.$dataToRetrieve.'
FROM ioc_bonjours
JOIN ioc_users on user = ioc_users.id
JOIN ioc_devices on device = ioc_devices.id
WHERE ioc_bonjours.created_at >= ?
GROUP BY '.$groupBy.'
', [Carbon::now()->subMinutes($this->minutesAgo)]);
        });
    }

    public function render() {
        return view('ioc::dashboard.livewire.active-now-stats');
    }
}
