<?php

namespace FourWords\AppManager\Ioc\Livewire\Dashboard\Stats;

use Carbon\Carbon;
use FourWords\AppManager\Ioc\Livewire\Dashboard\GeneralStats;

class MediaOrdiniStats extends SingleValueBoxStats {

    public function mount() {
        $this->fromDay = 30;
        $this->title = 'Media ordini giornalieri';
        $this->column = 'logins';
        $this->icon = '<svg class="svg-inline--fa fa-chart-waterfall fa-w-16 text-theme-10 report-box__icon" aria-hidden="true" focusable="false" data-prefix="fa-light" data-icon="chart-waterfall" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512" data-fa-i2svg=""><path fill="currentColor" d="M256 256C264.828 256 272 248.828 272 240V144C272 135.172 264.828 128 256 128S240 135.172 240 144V240C240 248.828 247.172 256 256 256ZM352 192C360.828 192 368 184.828 368 176V80C368 71.172 360.828 64 352 64S336 71.172 336 80V176C336 184.828 343.172 192 352 192ZM160 352C168.828 352 176 344.828 176 336V240C176 231.172 168.828 224 160 224S144 231.172 144 240V336C144 344.828 151.172 352 160 352ZM496 448H80C53.531 448 32 426.469 32 400V48C32 39.172 24.828 32 16 32S0 39.172 0 48V400C0 444.125 35.875 480 80 480H496C504.828 480 512 472.828 512 464S504.828 448 496 448ZM448 352C456.828 352 464 344.828 464 336V80C464 71.172 456.828 64 448 64S432 71.172 432 80V336C432 344.828 439.172 352 448 352Z"></path></svg>';
        $this->link = '#ordini-ricevuti';
        if (!$this->lazy) $this->fillData();
    }

    protected function calculateAverage($orders, $start, $end) {
        $countsByDay = [];

        // Aggiungo il count dei valori ad un array che ha come chiave il giorno
        foreach ($orders->groupBy(function ($order) {
            return Carbon::parse($order->created_at)->toDateString();
        }) as $date => $groupedOrders) {
            $countsByDay[$date] = $groupedOrders->count();
        }

        // Aggiungo i giorni senza valori, con valore 0 per contarli nella media
        $missingDays = Carbon::create($start);
        while ($missingDays->lte($end)) {
            $dateString = $missingDays->toDateString();
            if (!isset($countsByDay[$dateString])) {
                $countsByDay[$dateString] = 0;
            }
            $missingDays->addDay();
        }

        // Ritorno la media
        return collect($countsByDay)->avg();
    }

    protected function fillData() {

        $calculatePercent = true;

        if ($this->fromDay <= 0) {
            $this->fromDay = 30;
            $calculatePercent = false;
        }

        $orders = GeneralStats::getOrders($this->fromDay > 0 ? $this->fromDay*2 : -1);

        $endDate = Carbon::now()->startOfDay();
        $startDate = Carbon::now()->subDays($this->fromDay - 1)->startOfDay();
        $latestOrders = $orders->whereBetween('created_at', [$startDate->toDateString(), Carbon::create($endDate)->addDay()->toDateString()]);

        $this->result = (int) round($this->calculateAverage($latestOrders, $startDate, $endDate));


        $endDatePreceding = Carbon::parse(Carbon::create($startDate)->subDay())->startOfDay();
        $startDatePreceding = Carbon::now()->subDays($this->fromDay*2 -1)->startOfDay();
        $precedingOrders = $orders->whereBetween('created_at', [$startDatePreceding->toDateString(), Carbon::create($endDatePreceding)->addDay()->toDateString()]);

        $lastPeriodData = round($this->calculateAverage($precedingOrders, $startDatePreceding, $endDatePreceding));


        // Evito divisione per 0
        if ($lastPeriodData == 0 || !$calculatePercent) {
            $this->percent = 0;
            return;
        }

        $this->percent = round((($this->result - $lastPeriodData) / $lastPeriodData) * 100);

    }

}
