<?php

namespace FourWords\AppManager\Ioc\Livewire\Dashboard\Stats;

use Carbon\Carbon;
use FourWords\AppManager\Ioc\Livewire\Dashboard\GeneralStats;

class RigheMedieOrdiniStats extends SingleValueBoxStats {

    public function mount() {
        $this->fromDay = 30;
        $this->title = 'Righe medie ordini';
        $this->column = 'logins';
        $this->icon = '<svg class="svg-inline--fa fa-chart-bar fa-w-16 text-theme-10 report-box__icon" aria-hidden="true" focusable="false" data-prefix="fa-light" data-icon="chart-bar" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512" data-fa-i2svg=""><path fill="currentColor" d="M144 240H304C312.828 240 320 232.828 320 224S312.828 208 304 208H144C135.172 208 128 215.172 128 224S135.172 240 144 240ZM144 144H368C376.828 144 384 136.828 384 128S376.828 112 368 112H144C135.172 112 128 119.172 128 128S135.172 144 144 144ZM496 448H80C53.531 448 32 426.469 32 400V48C32 39.172 24.828 32 16 32S0 39.172 0 48V400C0 444.125 35.875 480 80 480H496C504.828 480 512 472.828 512 464S504.828 448 496 448ZM144 336H432C440.828 336 448 328.828 448 320S440.828 304 432 304H144C135.172 304 128 311.172 128 320S135.172 336 144 336Z"></path></svg>';
        $this->link = route('orders.index');
        if (!$this->lazy) $this->fillData();
    }

    protected function fillData() {

        $calculatePercent = true;

        if ($this->fromDay <= 0) {
            $this->fromDay = 30;
            $calculatePercent = false;
        }

        $orders = GeneralStats::getOrders($this->fromDay > 0 ? $this->fromDay*2 : -1);

        $endDate = Carbon::now()->startOfDay();
        $startDate = Carbon::now()->subDays($this->fromDay - 1)->startOfDay();
        $latestOrders = $orders->whereBetween('created_at', [$startDate->toDateString(),  Carbon::create($endDate)->addDay()->toDateString()]);

        $this->result = (int) $latestOrders->map(function ($order) {
            return count($order->rows);
        })->avg();

        $endDatePreceding = Carbon::parse(Carbon::create($startDate)->subDay())->startOfDay();
        $startDatePreceding = Carbon::now()->subDays($this->fromDay*2 - 1)->startOfDay();
        $precedingOrders = $orders->whereBetween('created_at', [$startDatePreceding->toDateString(), Carbon::create($endDatePreceding)->addDay()->toDateString()]);

        $lastPeriodData = (int) $precedingOrders->map(function ($order) {
            return count($order->rows);
        })->avg();

        // Evito divisione per 0
        if ($lastPeriodData == 0 || !$calculatePercent) {
            $this->percent = 0;
            return;
        }

        $this->percent = round((($this->result - $lastPeriodData) / $lastPeriodData) * 100);

    }

}
