<?php

namespace FourWords\AppManager\Ioc\Livewire\Dashboard\Stats;

use Carbon\Carbon;
use FourWords\AppManager\Ioc\Livewire\Dashboard\GeneralStats;
use FourWords\AppManager\Ioc\Livewire\IocComponent;

class SingleValueBoxStats extends IocComponent {
    public
        $lazy = false,
        $title,
        $column,
        $icon,
        $link,
        $fromDay = 30,
        $toDay,

        $result = null,
        $percent = 0;

    public string $version, $classes = "";

    protected $listeners = ['periodChanged', 'refreshStats'];

    public function periodChanged($value) {
        $this->fromDay = $value;
        $this->fillData();
    }

    public function refreshStats($period) {
        $this->periodChanged($period);
    }

    public function render() {
        if($this->version == 'horizontal') {
            return view('ioc::dashboard.livewire.horizontal-single-value-box-stats');
        }

        return view('ioc::dashboard.livewire.single-value-box-stats');
    }

    protected function getData() {
        return GeneralStats::getOrders($this->fromDay > 0 ? $this->fromDay*2 : -1);
    }

    protected function fillData() {

        $orders = $this->getData();

        $date = Carbon::today()->subDays(($this->fromDay * 2)-1);

        if ($this->fromDay > 0) {
            $orders = $orders->where('created_at', '>=', $date);
        }

        $lastPeriodData = $this->sumValues($orders);

        if ($this->fromDay <= 0) {
            $lastPeriodData = 0;
        }

        if ($lastPeriodData == 0) {
            $this->percent = 0;
            return;
        }

        $this->percent = round((($this->result - $lastPeriodData) / $lastPeriodData) * 100);
    }

    protected function sumValues($orders) {
        $lastPeriodData = 0;
        $periodData = 0;
        $date = \Carbon\Carbon::today()->subDays($this->fromDay-1);
        foreach($orders as $order) {
            if($this->fromDay <= 0 || $order->created_at >= $date) {
                $periodData += $order->{$this->column};
            } else {
                $lastPeriodData += $order->{$this->column};
            }
        }
        $this->result = round($periodData);
        return $lastPeriodData;
    }

    public function fieldEdited($rowId, $column) {}
}
