<?php

namespace FourWords\AppManager\Ioc\Livewire\Dashboard\Stats;

use Carbon\Carbon;
use FourWords\AppManager\Ioc\Livewire\Dashboard\GeneralStats;

class ValoreMedioOrdiniStats extends SingleValueBoxStats {

    public function mount() {
        $this->fromDay = 30;
        $this->title = 'Valore medio ordini';
        $this->column = 'logins';
        $this->icon = '<svg class="svg-inline--fa fa-chart-line fa-w-16 text-theme-11 report-box__icon" aria-hidden="true" focusable="false" data-prefix="fa-light" data-icon="chart-line" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512" data-fa-i2svg=""><path fill="currentColor" d="M123.328 315.328L224 214.625L308.672 299.297C314.92 305.545 325.047 305.545 331.295 299.299L475.312 155.311C481.562 149.062 481.562 138.934 475.314 132.686S458.937 126.437 452.691 132.686L320 265.375L235.33 180.674C229.107 174.449 218.926 174.449 212.703 180.672L100.703 292.672C94.469 298.906 94.469 309.062 100.703 315.297C106.938 321.562 117.062 321.562 123.328 315.328ZM496 448H80C53.531 448 32 426.469 32 400V48C32 39.172 24.828 32 16 32S0 39.172 0 48V400C0 444.125 35.875 480 80 480H496C504.828 480 512 472.828 512 464S504.828 448 496 448Z"></path></svg>';
        $this->link = route('orders.index');
        if (!$this->lazy) $this->fillData();
    }

    protected function fillData() {

        $calculatePercent = true;

        if ($this->fromDay <= 0) {
            $this->fromDay = 30;
            $calculatePercent = false;
        }

        $orders = GeneralStats::getOrders($this->fromDay > 0 ? $this->fromDay*2 : -1);

        $endDate = Carbon::now()->startOfDay();
        $startDate = Carbon::now()->subDays($this->fromDay - 1)->startOfDay();
        $latestOrders = $orders->whereBetween('created_at', [$startDate->toDateString(),  Carbon::create($endDate)->addDay()->toDateString()]);

        $this->result = $latestOrders->avg('total');

        $endDatePreceding = Carbon::parse(Carbon::create($startDate)->subDay())->startOfDay();
        $startDatePreceding = Carbon::now()->subDays($this->fromDay*2 - 1)->startOfDay();
        $precedingOrders = $orders->whereBetween('created_at', [$startDatePreceding->toDateString(), Carbon::create($endDatePreceding)->addDay()->toDateString()]);

        $lastPeriodData = $precedingOrders->avg('total');


        // Evito divisione per 0
        if ($lastPeriodData == 0 || !$calculatePercent) {
            $this->percent = 0;
            $this->result = currency($this->result).'€';
            return;
        }

        $this->percent = round((($this->result - $lastPeriodData) / $lastPeriodData) * 100);

        $this->result = currency($this->result).'€';

    }

}
