<?php

namespace FourWords\AppManager\Ioc\Livewire\Permission;

use FourWords\AppManager\Ioc\Livewire\IocComponent;
use FourWords\AppManager\Ioc\Models\Permission;
use FourWords\AppManager\Ioc\Models\PermissionGroup;
use Illuminate\Support\Str;

class PermissionCreateComponent extends IocComponent
{
    protected $modelName = "permesso";
    protected $model = Permission::class;

    public

        $name,
        $slug,
        $description,
        $permissionGroups,
        $permissionGroupId = '1',

        $showAddPermissionGroup = false,
        $slugChaged = false;

    public $modelId;

    public $showModal = false;

    public $title = "Crea nuovo";

    protected $listeners = [
        'onCreateClick',
        'onEditClick',
        'onSave',
        'onCancel',
        'togglePermissionGroupAddComponent',
        'createPermissionOnNewGroup'
    ];

    public $fields = [
        'id',
        'name',
        'slug',
        'description',
        'group_id',
        'active'
    ];

    protected $rules = [
        'name' => 'required',
        'description' => 'required'
    ];

    public function mount() {
        $this->title = "Crea un nuovo ".$this->modelName;
        $this->permissionGroups = PermissionGroup::all();


    }

    public function updated() {
        if($this->permissionGroupId == 'new') {
            $this->showAddPermissionGroup = true;
        } else {
            $this->showAddPermissionGroup = false;
        }

        if(!$this->slugChaged) {
            $this->slug = Str::slug($this->name);
        }
    }

    /**
     * Reseting all inputted fields
     * @return void
     */
    public function resetFields(){

        $this->resetErrorBag();

        foreach ($this->fields as $field) {

            if ($field == 'id') {
                $this->modelId = null;
            } else {
                $this->{$field} = '';
            }
        }
        $this->showAddPermissionGroup = false;
    }

    public function render() {
        return view('ioc::livewire.permission.create');
    }

    public function onCreateClick() {
        if(!$this->checkPermission('create-permissions')) {
            return null;
        }

        $this->resetFields();
        $this->title = "Crea ".$this->modelName;
        $this->showModal = true;
    }

    public function onSave() {
        if($this->permissionGroupId == 'new') {
            $this->emit('addPermissionGroup');
            return;
        }

        if ($this->modelId) {
            $this->update();
        } else {
            $this->store();
        }
    }

    public function store()
    {
        $this->validate();
        
        try {
            $this->createPermissionDB();

            $this->emit('refreshLivewireDatatable');
            $this->showModal = false;
            $this->successToast($this->modelName.' creato con successo');
            $this->resetFields();
        } catch (\Exception $ex) {
            //$this->errorToast($ex->getMessage());
            $this->errorToast('Errore nella creazione '.$this->modelName);
        }
    }

    private function createPermissionDB() {
        $permission = new $this->model();
        $permission->name = $this->name;
        $permission->slug = $this->slug ?: Str::slug($this->name);
        $permission->group_id = $this->permissionGroupId ?: 1;
        $permission->description = $this->description;
        $permission->save();
    }


    public function onEditClick($modelId) {
        if(!$this->checkPermission('edit-permissions')) {
            return null;
        }

        $this->resetFields();
        $this->title = "Modifica ".$this->modelName;

        try {
            $model = $this->model::findOrFail($modelId);
            if(!$model) {
                $this->errorToast($this->modelName.' non trovato');
                return;
            }

            foreach ($this->fields as $field) {
                if ($field == 'id') {
                    $this->modelId = $model->id;
                } else {
                    $this->{$field} = $model->{$field};
                }

                if($field == 'group_id') {
                    $this->permissionGroupId = $model->group_id;
                }
            }
            $this->showModal = true;
        } catch (\Exception $ex) {
            $this->errorToast('Errore modifica '.$this->modelName);
        }
    }

    public function update()
    {
        $this->validate();
        try {
            $model = $this->model::findOrFail($this->modelId);

            foreach ($this->fields as $field) {
                if ($field == 'id') {
                    $model->id = $this->modelId;
                    continue;
                }

                if($field == 'group_id') {
                    $model->{$field} = $this->permissionGroupId;
                    continue;
                }

                $model->{$field} = $this->{$field};
            }

            $model->save();
            $this->emit('refreshLivewireDatatable');
            $this->showModal = false;
            $this->successToast($this->modelName.' aggiornato con successo');
            $this->resetFields();
        } catch (\Exception $ex) {
            $this->errorToast('Errore nella modifica '.$this->modelName);
        }
    }

    public function togglePermissionGroupAddComponent($flag) {
        $this->showAddPermissionGroup = $flag;
    }

    public function slugChanged() {
        $this->slugChaged = true;
    }

    public function createPermissionOnNewGroup($permissionGroup) {
        $this->permissionGroups = PermissionGroup::all();
        $this->permissionGroupId = $permissionGroup['id'];

        try {
            if ($this->modelId) {
                $this->update();
            } else {
                $this->store();
            }
        } catch(\Exception $ex) {
            $this->showAddPermissionGroup = false;
        }
    }
}
