<?php

namespace FourWords\AppManager\Ioc\Livewire\Permission;

use FourWords\AppManager\Ioc\Livewire\IocComponent;
use FourWords\AppManager\Ioc\Models\Permission;
use FourWords\AppManager\Ioc\Models\RolePermission;

class PermissionListComponent extends IocComponent
{
    public
        $roleId,
        $permissionGroupsUsed,

        $permissions = [],
        $rolePermissions = [];

    protected $listeners = ['setRoleId'];

    public function mount($rolePermissions) {
        $this->rolePermissions = $rolePermissions;
        $this->getPermissionGroups();

        $this->getPermissions();
    }

    public function render() {
        return view("ioc::livewire.permission.list");
    }

    public function setRoleId($id){
        $this->roleId = $id;
        $this->getPermissionGroups();
    }

    public function getPermissionGroups() {
        $this->permissionGroupsUsed = Permission::select("group_id")->distinct()->get();

        $this->getRolePermissions();
    }


    public function getPermissions() {
        $this->permissions = Permission::orderBy('group_id', 'asc')->get();
    }

    public function getRolePermissions() {
        $this->rolePermissions = array_diff($this->rolePermissions, $this->rolePermissions);
        if ($this->roleId) {
            $this->rolePermissions = RolePermission::where("role_id", "=", $this->roleId)->where("active", "=", true)->get()->pluck('permission_id')->toArray();
        }
    }

    public function updatedRolePermissions() {
        $this->emitUp('setRolePermissions', $this->rolePermissions);
    }

    public function countPermissions($groupId)
    {
        $permissions = Permission::where("group_id", "=", $groupId)->where("active", "=", true)->get()->pluck('id')->toArray();
        $permissionNotSelected = array_diff($permissions, $this->rolePermissions);
        return '[ ' . (count($permissions) - count($permissionNotSelected)) . ' / ' . count($permissions) . ' ]';
    }

    public function getPermissionsSelected($groupId) {
        $permissions = Permission::where("group_id", "=", $groupId)->where("active", "=", true)->get()->pluck('id')->toArray();
        $permissionNotSelected = array_diff($permissions, $this->rolePermissions);
        return (count($permissions) - count($permissionNotSelected));
    }

    public function getMaxPermissions($groupId) {
        return count(Permission::where("group_id", "=", $groupId)->where("active", "=", true)->get()->pluck('id')->toArray());
    }

    public function selectAllPermissions($groupId) {
        $permissionNotSelected = array_diff(Permission::where("group_id", "=", $groupId)->where("active", "=", true)->get()->pluck('id')->toArray(), $this->rolePermissions);

        foreach ($permissionNotSelected as $p) {
            $this->rolePermissions[] = $p;
        }
        $this->emitUp('setRolePermissions', $this->rolePermissions);
    }

    public function deselectAllPermissions($groupId) {
        $this->rolePermissions = array_values(array_diff($this->rolePermissions, Permission::where("group_id", "=", $groupId)->where("active", "=", true)->get()->pluck('id')->toArray()));
        $this->emitUp('setRolePermissions', $this->rolePermissions);
    }
}

