<?php

namespace FourWords\AppManager\Ioc\Livewire\PermissionGroup;

use FourWords\AppManager\Ioc\Livewire\IocComponent;
use FourWords\AppManager\Ioc\Models\PermissionGroup;

class PermissionGroupAddComponent extends IocComponent
{
    protected $modelName = "gruppo di permessi";
    protected $model = PermissionGroup::class;

    public
        $name,
        $description;

    protected $listeners = [
        'addPermissionGroup'
    ];

    public $fields = [
        'id',
        'name',
        'description'
    ];

    protected $rules = [
        'name' => 'required',
        'description' => 'required'
    ];

    public function mount() {

    }

    /**
     * Reseting all inputted fields
     * @return void
     */
    public function resetFields(){

        $this->resetErrorBag();

        foreach ($this->fields as $field) {

            if ($field == 'id') {
                $this->modelId = null;
            } else {
                $this->{$field} = '';
            }
        }
    }

    public function render() {
        return view('ioc::livewire.permissionGroup.add');
    }

    public function addPermissionGroup()
    {
        $this->validate();

        try {
            $permission = new $this->model();

            $permission->name = $this->name;
            $permission->description = $this->description;
            $permission->save();

            $this->emitUp('createPermissionOnNewGroup', $permission);
            $this->emitUp('togglePermissionGroupAddComponent', false);

            $this->successToast($this->modelName.' creato con successo');
            $this->resetFields();
        } catch (\Exception $ex) {
            $this->errorToast($ex->getMessage());
            $this->errorToast('Errore nella creazione '.$this->modelName);
        }
    }
}
