<?php

namespace FourWords\AppManager\Ioc\Livewire\PermissionGroup;

use FourWords\AppManager\Ioc\Livewire\IocComponent;
use FourWords\AppManager\Ioc\Models\PermissionGroup;

class PermissionGroupCreateComponent extends IocComponent
{
    protected $modelName = "gruppo di permessi";
    protected $model = PermissionGroup::class;

    public
        $name,
        $description;

    public $modelId;

    public $showModal = false;

    public $title = "Crea nuovo";

    protected $listeners = [
        'onCreateClick',
        'onEditClick',
        'onSave',
        'onCancel'
    ];

    public $fields = [
        'id',
        'name',
        'description'
    ];

    protected $rules = [
        'name' => 'required',
        'description' => 'required'
    ];

    public function mount() {
        $this->title = "Crea un nuovo ".$this->modelName;
    }

    /**
     * Reseting all inputted fields
     * @return void
     */
    public function resetFields(){

        $this->resetErrorBag();

        foreach ($this->fields as $field) {

            if ($field == 'id') {
                $this->modelId = null;
            } else {
                $this->{$field} = '';
            }
        }
    }

    public function render() {
        return view('ioc::livewire.permissionGroup.create');
    }

    public function onCreateClick() {
        if(!$this->checkPermission('create-permission-groups')) {
            return null;
        }

        $this->resetFields();
        $this->title = "Crea ".$this->modelName;
        $this->showModal = true;
    }

    public function onSave() {
        if ($this->modelId) {
            $this->update();
        } else {
            $this->store();
        }
    }

    public function store()
    {
        $this->validate();
        
        try {
            $permission = new $this->model();

            $permission->name = $this->name;
            $permission->description = $this->description;
            $permission->save();

            $this->emit('refreshLivewireDatatable');
            $this->showModal = false;
            $this->successToast($this->modelName.' creato con successo');
            $this->resetFields();
        } catch (\Exception $ex) {
            $this->errorToast($ex->getMessage());
            $this->errorToast('Errore nella creazione '.$this->modelName);
        }
    }


    public function onEditClick($modelId) {
        if(!$this->checkPermission('edit-permission-groups')) {
            return null;
        }

        $this->resetFields();
        $this->title = "Modifica ".$this->modelName;

        try {
            $model = $this->model::findOrFail($modelId);
            if(!$model) {
                $this->errorToast($this->modelName.' non trovato');
                return;
            }

            foreach ($this->fields as $field) {
                if ($field == 'id') {
                    $this->modelId = $model->id;
                } else {
                    $this->{$field} = $model->{$field};
                }
            }
            $this->showModal = true;
        } catch (\Exception $ex) {
            $this->errorToast('Errore modifica '.$this->modelName);
        }
    }

    public function update()
    {
        $this->validate();
        try {
            $model = $this->model::findOrFail($this->modelId);

            foreach ($this->fields as $field) {
                if ($field == 'id') {
                    $model->id = $this->modelId;
                    continue;
                }

                $model->{$field} = $this->{$field};
            }

            $model->save();
            $this->emit('refreshLivewireDatatable');
            $this->showModal = false;
            $this->successToast($this->modelName.' aggiornato con successo');
            $this->resetFields();
        } catch (\Exception $ex) {
            $this->errorToast('Errore nella modifica '.$this->modelName);
        }
    }
}
