<?php

namespace FourWords\AppManager\Ioc\Livewire\Role;

use FourWords\AppManager\Ioc\Livewire\IocComponent;
use FourWords\AppManager\Ioc\Models\Permission;
use FourWords\AppManager\Ioc\Models\PermissionGroup;
use FourWords\AppManager\Ioc\Models\Role;
use FourWords\AppManager\Ioc\Models\RolePermission;
use Illuminate\Support\Str;

class RoleCreateComponent extends IocComponent
{
    protected $modelName = "ruolo";
    protected $model = Role::class;

    public
        $name,
        $description,
        $rolePermissions = [];

    public $modelId;

    public $showModal = false;

    public $title = "Crea nuovo";

    protected $listeners = [
        'onCreateClick',
        'onEditClick',
        'onSave',
        'onCancel',
        'setRolePermissions'
    ];

    public $fields = [
        'id',
        'name',
        'description'
    ];

    protected $rules = [
        'name' => 'required',
        'description' => 'required'
    ];

    public function mount() {
        $this->title = "Crea un nuovo ".$this->modelName;
    }

    /**
     * Reseting all inputted fields
     * @return void
     */
    public function resetFields(){

        $this->resetErrorBag();

        foreach ($this->fields as $field) {

            if ($field == 'id') {
                $this->modelId = null;
            } else if ($field == 'active') {
                $this->{$field} = false;
            } else {
                $this->{$field} = '';
            }
        }
    }

    public function render() {
        return view('ioc::livewire.role.create');
    }

    public function onCreateClick() {
        if(!$this->checkPermission('create-roles')) {
            return null;
        }

        $this->resetFields();
        $this->emit('setRoleId', null);

        $this->title = "Crea ".$this->modelName;
        $this->showModal = true;
    }

    public function onSave() {
        if ($this->modelId) {
            $this->update();
        } else {
            $this->store();
        }
    }

    public function store()
    {
        $this->validate();
        
        try {
            $role = new $this->model();

            $role->name = $this->name;
            $role->description = $this->description;

            $role->save();

            $model = $this->model::findOrFail($role->id);
            $model->permissions()->sync($this->rolePermissions);

            $model->save();

            //$rolePermissions = new RolePermission();


            $this->emit('refreshLivewireDatatable');
            $this->showModal = false;
            $this->successToast($this->modelName.' creato con successo');
            $this->resetFields();
        } catch (\Exception $ex) {
            $this->errorToast($ex->getMessage());
            $this->errorToast('Errore nella creazione '.$this->modelName);
        }
    }

    public function onEditClick($modelId) {
        if(!$this->checkPermission('edit-roles')) {
            return null;
        }

        $this->resetFields();
        $this->title = "Modifica ".$this->modelName;

        try {
            $model = $this->model::findOrFail($modelId);
            if(!$model) {
                $this->errorToast($this->modelName.' non trovato');
                return;
            }

            foreach ($this->fields as $field) {
                if ($field == 'id') {
                    $this->modelId = $model->id;
                } else {
                    $this->{$field} = $model->{$field};
                }
            }
            $this->showModal = true;

            $role = Role::where('name', "=", $this->name)->first()->id;
            $this->emit('setRoleId', $role);
            $this->rolePermissions = $model->permissions()->where('role_id', '=', $role)->pluck('permission_id')->toArray();
        } catch (\Exception $ex) {
            $this->errorToast('Errore modifica '.$this->modelName);
        }
    }

    public function update()
    {
        $this->validate();
        try {
            $model = $this->model::findOrFail($this->modelId);

            foreach ($this->fields as $field) {
                if ($field == 'id') {
                    $model->id = $this->modelId;
                    continue;
                }

                $model->{$field} = $this->{$field};
            }

            $model->permissions()->sync($this->rolePermissions);

            $model->save();

            $this->emit('refreshLivewireDatatable');
            $this->showModal = false;
            $this->successToast($this->modelName.' aggiornato con successo');
            $this->resetFields();
        } catch (\Exception $ex) {
            $this->errorToast('Errore nella modifica '.$this->modelName);
        }
    }

    public function setRolePermissions($rolePermissions) {
        $this->rolePermissions = $rolePermissions;
    }
}
