<?php

namespace FourWords\AppManager\Ioc\Livewire\Table;

use FourWords\AppManager\Ioc\Models\User;
use FourWords\AppManager\Ioc\Models\Role;
use Illuminate\Support\Facades\Auth;
use Mediconesystems\LivewireDatatables\Column;

class AdminTable extends IocTable
{

    public $hideable = "select";

    public function builder()
    {
        return User::join('ioc_roles', 'ioc_roles.id', 'users.role_id');
    }

    public function edited($value, $key, $column, $rowId)
    {
        parent::edited($value, $key, $column, $rowId);
        if ($column == 'name' && $rowId == Auth::id()) {
            $this->emit('refreshUser');
        }
    }

    public function columns()
    {
        return [

            Column::checkbox(),

            Column::name('id')
                ->label('ID')
                ->filterable()
                ->defaultSort('asc'),

            Column::name('ioc_roles.name')
                ->label('Ruolo')
                ->filterable($this->roles)->alignRight(),

//            Column::name('devices.name')
//                ->label('Device')
//                ->filterable($this->devices)->alignRight(),


//            Column::name('role.region.name')
//                ->label('Region')
//                ->filterable($this->regions)
//                ->truncate(8)
//                ->searchable(),

            Column::name('name')
                ->label('Nome')
                ->filterable()
                ->editable()
                ->searchable(),

//            Column::name('username')
//                ->searchable()
//                ->editable()
//                ->filterable(),

            Column::name('email')
                ->searchable()
                ->editable()
                ->filterable(),

            Column::callback('id', function ($id) {
                return view('ioc::livewire.icon-edit', ['id' => $id]);
            })->label('Azioni')
                ->excludeFromExport()
        ];

    }

    public function edit($id) {
        if(!$this->checkPermission('edit-other-admin')) {
            return null;
        }

        return redirect(route('admin.edit', ['user' => $id]));
    }

    public function getRolesProperty()
    {
        return Role::pluck('name');
    }

}
