<?php

namespace FourWords\AppManager\Ioc\Livewire\Table;

use FourWords\AppManager\Ioc\Models\Device;
use FourWords\AppManager\Ioc\Models\IocUser;
use Illuminate\Support\Facades\DB;
use Mediconesystems\LivewireDatatables\BooleanColumn;
use Mediconesystems\LivewireDatatables\Column;
use Mediconesystems\LivewireDatatables\DatetimeColumn;

class DeviceTable extends IocTable
{

    public function builder()
    {
        return Device::query()
            ->leftJoin('ioc_bonjours', function ($leftJoin) {
                $leftJoin
                    ->on('ioc_devices.id', '=', 'ioc_bonjours.device')
                    ->where('ioc_bonjours.created_at','=', DB::raw("(SELECT max(created_at) FROM ioc_bonjours WHERE device = ioc_devices.id)"));
            });
    }

    public function columns()
    {
        return [

            Column::checkbox(),

            Column::name('id')
                ->label('ID')
//                ->filterable()
                ->defaultSort('desc')
                ->hide(),

            BooleanColumn::name('active')
                ->label('Attivo')
                ->togglable(),

//            Column::callback(['ioc_users_devices.'], function ($os) {
//                return AppstoreType::where('os')
//            })->label('OS'),

            Column::name('identifier')
                ->label('Codice univoco')
                ->hide()
                ->filterable(),
//                ->searchable(),

            Column::name('users.name')
                ->label('Utenti collegati')
                ->filterable(),

            Column::name('name')
                ->label('Nome')
                ->filterable()
                ->searchable(),

            Column::name('model')
                ->label('Modello')
                ->filterable()
                ->searchable(),

            Column::name('app_version')
                ->label('Versione App')
                ->filterable(),
//                ->editable()
//                ->searchable(),

//        Column::name('appstore_types.name')
//            ->label('OS')
//            ->filterable(AppstoreType::all()->pluck('name')->toArray()),

            Column::callback(['app_id'], function ($appId) {
                return $appId == 1 ? 'iOS' : 'Android';
            })->label('OS'),

            Column::name('version')
                ->label('Versione OS')
                ->filterable(),

            BooleanColumn::name('wipe')
                ->label('Cancella dati')
                ->togglable()
                ->excludeFromExport()
                ->hide(),

            Column::name('push_device_token')
                ->label('Push Token')
                ->filterable()
                ->hide(),

            Column::callback(['push_status'], function ($pushStatus) {
                    return $pushStatus >= 2 ? '<svg class="h-5 w-5 stroke-current text-green-600 mx-auto" fill="none" viewBox="0 0 24 24" stroke="currentColor">
    <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M9 12l2 2 4-4m6 2a9 9 0 11-18 0 9 9 0 0118 0z"></path>
</svg>' : '<svg class="h-5 w-5 stroke-current text-red-300 mx-auto" fill="none" viewBox="0 0 24 24" stroke="currentColor">
    <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M10 14l2-2m0 0l2-2m-2 2l-2-2m2 2l2 2m7-2a9 9 0 11-18 0 9 9 0 0118 0z"></path>
</svg>';
            })->label('Notifiche push')
                ->exportCallback(function ($value) {
                    return $value;
                })
            ->filterable(['0','1','2']),

            DatetimeColumn::name('ioc_bonjours.created_at')
                ->label('Ultimo accesso')
                ->format(null, true)
                ->sortable(),

            Column::callback(['id', 'name'], function ($id, $name) {
                return view('datatables::delete', [
                    'value' => $id,
                    'name' => $name,
                    'permission' => 'delete-app-devices'
                ]);
            })->label('Azioni')
                ->unsortable()
                ->excludeFromExport(),

        ];

    }

    public function users() {
        return $this->belongsToMany(IocUser::class, 'ioc_users_devices', 'device_id','user_id');
    }

}
