<?php

namespace FourWords\AppManager\Ioc\Livewire\Table;

use FourWords\AppManager\Ioc\Models\IocUser;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Storage;
use Mediconesystems\LivewireDatatables\BooleanColumn;
use Mediconesystems\LivewireDatatables\Column;
use Mediconesystems\LivewireDatatables\NumberColumn;

class GeoTable extends IocTable
{
    protected $fetchAllData = true; // Recupero tutti i dati, non solo quelli che mostro con il paginatore. Max 1000

//    public $defaultFilters = [
//        'active' => '1',
//    ];

    protected function onAllDataFetched()
    {
        $jsonEncodedData = json_encode($this->prepareMarkers($this->allData));
        $this->emit('mapDataChanged', $jsonEncodedData);
        Cache::forever('mapData', $jsonEncodedData);
        $this->dispatchBrowserEvent('mapDataUpdated', $jsonEncodedData);
    }

    public function prepareMarkers($rows) {

        $markers = [];

        foreach ($rows as $client) {
            $markers[] = [
                'iconUrl' => $client->active ? env('APP_URL').Storage::url('geo/marker-green.png') : env('APP_URL').Storage::url('geo/marker-red.png'),
                'latitude' => $client->{'geo_users.latitude'},
                'longitude' => $client->{'geo_users.longitude'},
                'iconSize' => [50, 50],
                'iconAnchor' => [25, 50],
                'popupAnchor' => [0, -50],
                'popup' => '<b>'.$client->name.'</b>'.
                    '<br>'.$client->code
                    .'<br>Agente: '.$client->agent
                    .'<br>'.$client->{'geo_users.city'}.($client->{'geo_users.prov'} ? ' ('.$client->{'geo_users.prov'}.')' : '')
                    .($client->{'geo_users.address'} ? '<br>'.$client->{'geo_users.address'} : ''),
                'data' => $client
            ];
        }

        return $markers;
    }

    public function builder()
    {
        return IocUser::query()
            ->join('geo_users', 'ioc_users.code', '=', 'geo_users.code')
            ->leftJoin('catalog_agents', 'catalog_agents.code', '=', 'ioc_users.agent')
            ->whereNotNull('agent')
            ->where('agent','!=','');
    }

//    public function getResultsProperty()
//    {
//        $results = parent::getResultsProperty();
//        $this->dispatchBrowserEvent('updateMapData', json_encode($this->allData));
////        $this->emit('updateMapData', json_encode($this->allData));
//        return $results;
//    }
//
//    public function render()
//    {
//        $view = parent::render();
//        $this->dispatchBrowserEvent('updateMapData', json_encode($this->allData));
//        return $view;
//    }

    public function columns()
    {
        return [

            Column::checkbox(),

            BooleanColumn::name('active')
                ->label('Attivo')
                ->booleanFilterable(),

//            Column::name('id')
//                ->label('ID')
//                ->hide(),
//                ->filterable()
//                ->defaultSort('asc'),

            Column::name('code')
                ->label('ID Gestionale')
                ->filterable()
                ->searchable(),

            Column::name('agent')
                ->label('Codice Agente')
                ->filterable()
                ->hide(),

            Column::name('catalog_agents.name')
                ->label('Agente')
                ->filterable(),

            Column::callback(['type'], function ($type) {
                return $type == 'D' ? 'Destinazione' : 'Cliente';
            })
                ->label('Tipo')
                ->filterable(['C' => 'Cliente', 'D' => 'Destinazione'])
                ->filterName('type')
                ->hide(),


            Column::name('name')
                ->label('Nome')
                ->filterable()
                ->searchable(),

            Column::name('ioc_users.username')
                ->label('Username')
                ->filterName('username')
                ->filterable()
                ->searchable()
                ->hide(),

            NumberColumn::name('orders.username:count')
                ->label('Ordini inviati')
                ->filterable()
                ->filterName('orders')
                ->defaultSort('desc'),

            NumberColumn::name('orders.total:sum')
                ->label('Totale Ordinato')
                ->filterable()
                ->sortable()
                ->filterName('orders-total')
                ->hide(),

//            NumberColumn::callback(['orders.total:sum'], function ($total) {
//                return currency($total)."€";
//            })
//            ->label('Totale Ordinato')
//            ->filterable()
//            ->sortable()
////            ->filterName('orders-total')
//            ->defaultSort('desc'),

            Column::name('vat_code')
                ->label('Partita IVA')
                ->filterable()
                ->searchable()
                ->hide(),

            Column::name('fiscal_code')
                ->label('Codice Fiscale')
                ->filterable()
                ->searchable()
                ->hide(),

            Column::name('geo_users.prov')
                ->label('Provincia')
                ->filterable(),

            Column::name('geo_users.city')
                ->label('Città')
                ->filterable(),

            Column::name('geo_users.cap')
                ->label('Cap')
                ->filterable(),

            Column::name('geo_users.address')
                ->label('Indirizzo')
                ->filterable(),

            Column::name('geo_users.latitude')
                ->label('Latitudine')
                ->hide(),

            Column::name('geo_users.longitude')
                ->label('Longitudine')
                ->hide(),

            Column::name('email')
                ->label('Email')
                ->filterable()
                ->searchable()
                ->hide(),

            Column::name('created_at')
                ->label('Creato il')
                ->hide(),
//                ->filterable()
//                ->searchable(),

//            Column::callback(['id', 'name'], function ($id, $name) {
//                return view('datatables::table-actions', [
//                    'id' => $id,
//                    'name' => $name,
//                    'permission' => 'delete-app-users'
//                ]);
//            })
//                ->label('Azioni')
//                ->unsortable()
//                ->excludeFromExport(),
        ];

    }

}
