<?php

namespace FourWords\AppManager\Ioc\Livewire\Table;

use FourWords\AppManager\Ioc\Models\Device;
use FourWords\AppManager\Ioc\Models\IocUser;
use Illuminate\Support\Facades\DB;
use Mediconesystems\LivewireDatatables\BooleanColumn;
use Mediconesystems\LivewireDatatables\Column;
use Mediconesystems\LivewireDatatables\DatetimeColumn;
use Mediconesystems\LivewireDatatables\NumberColumn;

class IocUserTable extends IocTable
{
    public $defaultFilters = [
        'active' => '1',
    ];

    public function builder()
    {
        $q = IocUser::query()
            ->leftJoin('ioc_bonjours', function ($leftJoin) {
                $leftJoin
                    ->on('ioc_users.id', '=', 'ioc_bonjours.user')
                    ->where('ioc_bonjours.created_at','=', DB::raw("(SELECT max(created_at) FROM ioc_bonjours WHERE user = ioc_users.id)"));
            })
            ->leftJoin('catalog_agents', function ($leftJoin) {
                $leftJoin
                    ->on('ioc_users.agent', '=', 'catalog_agents.code');
            });

        foreach ($this->customerFeaturesMap() as $customerFeatureToShow) {

            $q->leftJoin("catalog_customer_has_feature as f{$customerFeatureToShow['code']}", function ($join) use ($customerFeatureToShow) {
                $join->on('ioc_users.id', '=', "f{$customerFeatureToShow['code']}.customer_id")
                    ->where("f{$customerFeatureToShow['code']}.feature_id", '=', DB::raw(" (SELECT id from catalog_customer_features WHERE code = '{$customerFeatureToShow['code']}')"));
            });

        }

        return $q;

    }

    public function customerFeaturesMap(): array
    {
        return [];
    }

    public function columns()
    {

        $columns = [

            Column::checkbox(),

            BooleanColumn::name('active')
                ->label('Attivo')
                ->togglable()
                ->booleanFilterable(),

//            Column::name('id')
//                ->label('ID')
//                ->hide(),
//                ->filterable()
//                ->defaultSort('asc'),

            Column::name('code')
                ->label('ID Gestionale')
                ->filterable()
                ->editable()
                ->searchable(),

            Column::name('agent')
                ->label('Codice Agente')
                ->filterable()
                ->hide(),

            Column::name('catalog_agents.name')
                ->label('Agente')
                ->filterable(),

            Column::name('devices.name')
                ->label('Dispositivi')
                ->filterName('devices')
                ->hide()
                ->filterable(),

            Column::callback(['type'], function ($type) {
                return $type == 'D' ? 'Destinazione' : 'Cliente';
            })
                ->label('Tipo')
                ->filterable(['C' => 'Cliente', 'D' => 'Destinazione'])
                ->filterName('type')
                ->hide(),


            Column::name('name')
                ->label('Nome')
                ->filterable()
                ->editable()
                ->searchable(),

            Column::name('ioc_users.username')
                ->label('Username')
                ->filterName('username')
                ->filterable()
                ->editable()
                ->searchable(),

            NumberColumn::name('orders.username:count')
                ->label('Ordini inviati')
                ->filterable()
                ->filterName('orders'),

            Column::name('vat_code')
                ->label('Partita IVA')
                ->filterable()
                ->searchable(),

            Column::name('fiscal_code')
                ->label('Codice Fiscale')
                ->filterable()
                ->searchable()
                ->hide(),

            Column::name('province')
                ->label('Provincia')
                ->filterable()
                ->hide(),

            Column::name('city')
                ->label('Città')
                ->filterable(),

            Column::name('address')
                ->label('Indirizzo')
                ->filterable()
                ->hide(),

            Column::name('cap')
                ->label('Cap')
                ->filterable()
                ->hide(),

            Column::name('email')
                ->label('Email')
                ->filterable()
                ->editable()
                ->searchable()
                ->hide(),

            Column::name('created_at')
                ->label('Creato il')
                ->defaultSort('desc'),
//                ->filterable()
//                ->editable()
//                ->searchable(),

            DatetimeColumn::name('ioc_bonjours.created_at')
                ->label('Ultimo accesso')
                ->format(null, true)
                ->sortable(),

            Column::callback(['id', 'name'], function ($id, $name) {
                return view('datatables::table-actions', [
                    'id' => $id,
                    'name' => $name,
                    'permission' => 'delete-app-users'
                ]);
            })
                ->label('Azioni')
                ->unsortable()
                ->excludeFromExport()
            ,
        ];

        foreach ($this->customerFeaturesMap() as $customerFeatureToShow) {

            $featureColumn = Column::name("f{$customerFeatureToShow['code']}.value")
                ->label($customerFeatureToShow['columnName'])
                ->filterable();

            $indexToInsertAfter = array_search($customerFeatureToShow['insertAfter'], array_column($columns, 'label'));
            if ($indexToInsertAfter !== false) {
                array_splice($columns, $indexToInsertAfter + 1, 0, [$featureColumn]);
            }
        }

        return $columns;


    }

    public function devices() {
        return $this->belongsToMany(Device::class, 'ioc_users_devices', 'user_id','device_id');
    }

}
