<?php

namespace FourWords\AppManager\Ioc\Livewire\Table;

use FourWords\AppManager\Ioc\Models\TaskLog;
use Illuminate\Support\Carbon;
use Illuminate\Support\Str;
use Mediconesystems\LivewireDatatables\BooleanColumn;
use Mediconesystems\LivewireDatatables\Column;
use Mediconesystems\LivewireDatatables\DatetimeColumn;

class TaskLogTable extends IocTable
{

    public $showModal = false;

    public function builder()
    {
        return TaskLog::query()->leftJoin('ioc_tasks', 'ioc_tasks.id', 'ioc_task_logs.task_id');
    }

    public function columns()
    {
        return [

            Column::checkbox(),

            Column::name('id')
                ->label('ID')
                ->hide(),

            Column::name('ioc_tasks.name')
                ->label('Nome')
//                ->filterable()
//                ->editable()
                ->searchable(),

            Column::name('ioc_tasks.target_class')
                ->label('Classe o comando')
//                ->filterable()
//                ->editable()
                ->searchable(),

            DatetimeColumn::name('start')
                ->label('Inizio')
                ->sortable()
                ->format(null, true)
                ->defaultSort('desc'),

            DatetimeColumn::name('end')
                ->sortable()
                ->format(null, true)
                ->label('Fine'),

            DatetimeColumn::callback(['start','end'], function ($start, $end) {
                if (!$start || !$end) {
                    return;
                }
                return Carbon::createFromTimestamp($start)->diffForHumans(Carbon::createFromTimestamp($end), true);
            })
                ->label('Durata'),

            Column::name('error')
            ->label('Errore')   ,

            BooleanColumn::name('isOK')
                ->label('Esito'),

            Column::callback(['ioc_tasks.name','ioc_tasks.target_class','start'], function ($name, $class, $start) {

                $className = last(explode('\\',$class));

                return view('ioc::task.task-actions', [
                    'name' => $name,
                    'taskLogFile' => storage_path('logs/ioc/tasks/'.Str::kebab($className).'-'.$start.'.log'),
                ]);

            })->label('Azioni')
                ->unsortable()
                ->excludeFromExport(),

        ];

    }

}
