<?php

namespace FourWords\AppManager\Ioc\Livewire\Table;

use FourWords\AppManager\Ioc\Helpers\TaskManager;
use FourWords\AppManager\Ioc\Models\Task;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;
use Mediconesystems\LivewireDatatables\BooleanColumn;
use Mediconesystems\LivewireDatatables\Column;
use Mediconesystems\LivewireDatatables\DatetimeColumn;

class TaskTable extends IocTable
{

    public $showModal = false;

    public $jobParams = [];

    public function initialisePerPage()
    {
        $this->getSessionStoredPerPage();

        if (!$this->perPage) {
            $this->perPage = 50;
        }
    }

    public function builder()
    {
        $this->dimOnLoading = false;

        return Task::query()
            ->leftJoin('ioc_task_logs', function ($leftJoin) {
                $leftJoin
                    ->on('ioc_tasks.id', '=', 'ioc_task_logs.task_id')
                    ->where('ioc_task_logs.start','=', DB::raw("(SELECT max(start) FROM ioc_task_logs WHERE task_id = ioc_tasks.id)"));
            });
    }

    public function runTask($id) {
        try {
            TaskManager::runTaskId($id, $this->jobParams);
            $this->emit('refreshLivewireDatatable');
            $this->jobParams = [];
        } catch (\Exception $ex) {
            $this->errorToast('Errore esecuzione Task');
        }
    }

    public function columns()
    {
        return [

            Column::checkbox(),

            Column::name('id')
                ->label('ID')
                ->hide()
//                ->filterable()
                ->defaultSort('asc'),

            Column::name('name')
                ->label('Nome')
//                ->filterable()
//                ->editable()
                ->searchable(),

            Column::name('target_class')
                ->label('Classe o comando')
//                ->filterable()
//                ->editable()
                ->searchable(),

            DatetimeColumn::name('ioc_task_logs.start')
                ->label('Ultimo avvio')
                ->format(null, true),

            Column::callback(['ioc_task_logs.isOK'], function ($isOK) {

                if ($isOK === "" || $isOK === "1") {
                    return '<svg class="h-5 w-5 stroke-current text-green-600 mx-auto" fill="none" viewBox="0 0 24 24" stroke="currentColor">
    <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M9 12l2 2 4-4m6 2a9 9 0 11-18 0 9 9 0 0118 0z"></path>
</svg>';
                } else {
                    return '<svg class="h-5 w-5 stroke-current text-red-300 mx-auto" fill="none" viewBox="0 0 24 24" stroke="currentColor">
    <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M10 14l2-2m0 0l2-2m-2 2l-2-2m2 2l2 2m7-2a9 9 0 11-18 0 9 9 0 0118 0z"></path>
</svg>';
                }
            })->label('Ultimo esito ')
            ->exportCallback(function ($value) {
                return str_contains($value, 'text-green-600') ? "1" : "0";
            }),

            Column::name('frequency')
                ->label('Frequenza'),
//                ->searchable(),
//                ->editable(),
//                ->filterable(),

            Column::name('parameters')
                ->label('Parametri (facoltativo)'),
//                ->searchable()
//                ->editable(),
//                ->filterable(),

            Column::name('ioc_task_logs.error')
                ->label('Errore'),

            Column::callback(['running'], function ($running) {
                return $running ? '<i class="fa-solid text-theme-1 text-lg fa-circle-notch fa-spin"></i>' : '';
            })->label('Stato')->unsortable(),

            BooleanColumn::name('active')
                ->label('Attivo')
                ->togglable()
                ->booleanFilterable(),

            Column::callback(['id', 'name'], function ($id, $name) {

                $iocJob = null;
                $task = Task::find($id);
                if ($task && class_exists(trim($task->target_class))) {
                    $iocJob = app()->make($task->target_class, [$task]);
                }

                    return view('ioc::task.task-actions', [
                        'id' => $id,
                        'name' => $name,
                        'running' => $task->running,
                        'job' => $iocJob,
                        'taskLogFile' => storage_path('logs/ioc/tasks/'.Str::kebab($iocJob ? $iocJob->getClassName() : "class-not-found").'-x.log'),
                        'permission' => 'delete-tasks'
                    ]);


            })->label('Azioni')
                ->unsortable()
                ->excludeFromExport(),

        ];

    }

}
