<?php

namespace FourWords\AppManager\Ioc\Livewire\Task;

use FourWords\AppManager\Ioc\Livewire\IocComponent;
use FourWords\AppManager\Ioc\Models\Task;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Arr;

class TaskCreateEditComponent extends IocComponent
{

    public
        $name,
        $target_class,
        $frequency,
        $parameters,
        $active;

    public $selected_class;

    public $modelId;

    public $showModal = false;

    public $title = "Crea nuovo Task";

    protected $listeners = [
        'onCreateClick',
        'onEditClick',
        'onSave',
        'onCancel'
    ];

    public $frequencies = Task::frequencies;

    public $fields = [
        'id',
        'name',
        'target_class',
        'frequency',
        'parameters',
        'active'
    ];

    protected $rules = [
        'name' => 'required',
        'target_class' => 'required',
        'frequency' => 'required'
    ];

    /**
     * Reseting all inputted fields
     * @return void
     */
    public function resetFields(){

        $this->resetErrorBag();

        foreach ($this->fields as $field) {

            if ($field == 'id') {
                $this->modelId = null;
            } else if ($field == 'active') {
                $this->{$field} = false;
            } else {
                $this->{$field} = '';
            }
        }
    }

    public function render()
    {
        return view('ioc::livewire.task.create-edit');
    }

    public function onCreateClick() {
        if(!$this->checkPermission('create-tasks')) {
            return;
        }

        $this->resetFields();
        $this->title = "Crea Task";
        $this->showModal = true;
    }

    public function setSelectedClass() {

        if($this->selected_class == null) {
            return;
        }

        if (array_search($this->selected_class, director()->jobs) === false) {
            $this->errorToast('Classe non trovata. dump-autoload?');
        }

        $this->target_class = $this->selected_class;
        $this->selected_class = null;
    }

    public function onEditClick($taskId) {
        if(!$this->checkPermission('edit-tasks')) {
            return;
        }

        $this->resetFields();
        $this->title = "Modifica Task";

        try {
            $task = Task::findOrFail($taskId);
            if(!$task) {
                $this->errorToast('Task non trovato');
                return;
            }

            foreach ($this->fields as $field) {

                if ($field == 'id') {
                    $this->modelId = $task->id;
                } else {
                    $this->{$field} = $task->{$field};
                }
            }

            $this->showModal = true;

        } catch (\Exception $ex) {
            $this->errorToast('Errore modifica Task');
        }


    }

    public function onSave() {
        if ($this->modelId) {
            $this->update();
        } else {
            $this->store();
        }
    }

    public function store()
    {
        $this->beforeValidate();
        $this->validate();
        try {
            Task::create([
                'name' => $this->name,
                'target_class' => $this->target_class,
                'frequency' => $this->frequency,
                'parameters' => $this->frequencies[$this->frequency] && !$this->frequencies[$this->frequency]['withParams'] ? '' : $this->parameters,
                'active' => $this->active
            ]);
            $this->emit('refreshLivewireDatatable');
            $this->showModal = false;
            $this->successToast('Task creato con successo');
            $this->resetFields();
        } catch (\Exception $ex) {
            $this->errorToast('Errore nella creazione Task');
        }
    }

    public function beforeValidate() {
        if ($this->frequencies[$this->frequency]['withParams']) {
            $this->rules['parameters'] = 'required';
        } else {
            unset($this->rules['parameters']);
        }
    }

    public function update()
    {
        $this->beforeValidate();
        $this->validate();
        try {

            $task = Task::findOrFail($this->modelId);

            foreach ($this->fields as $field) {

                if ($field == 'id') {
                    $task->id = $this->modelId;
                    continue;
                }

                if ($field == 'parameters' && $this->frequencies[$this->frequency] && !$this->frequencies[$this->frequency]['withParams']) {
                    $this->parameters = '';
                    $task->parameters = '';
                    continue;
                }

                $task->{$field} = $this->{$field};
            }

            $task->save();
            $this->emit('refreshLivewireDatatable');
            $this->showModal = false;
            $this->successToast('Task aggiornato con successo');
            $this->resetFields();
        } catch (\Exception $ex) {
            $this->errorToast('Errore nella modifica Task');
        }
    }

//    /**
//     * show existing task data in edit task form
//     * @param mixed $id
//     * @return void
//     */
//    public function editTask($id){
//        try {
//            $task = Task::findOrFail($id);
//            if( !$task) {
//                $this->errorToast('Task non trovato');
//            } else {
//                $this->name = $task->name;
//                $this->target_class = $task->target_class;
//                $this->taskId = $task->id;
//            }
//        } catch (\Exception $ex) {
//            $this->errorToast('Errore modifica Task');
//        }
//
//    }

//    /**
//     * update the task data
//     * @return void
//     */
//    public function updateTask()
//    {
//        $this->validate();
//        try {
//            Task::whereId($this->taskId)->update([
//                'name' => $this->name,
//                'target_class' => $this->target_class
//            ]);
//            $this->successToast('Task aggiornato con successo');
//            $this->resetFields();
//        } catch (\Exception $ex) {
//            $this->errorToast('Errore aggiornamento Task');
//        }
//    }

//    /**
//     * Cancel Add/Edit form and redirect to task listing page
//     * @return void
//     */
//    public function cancelTask()
//    {
//        $this->resetFields();
//    }

}
