<?php

namespace FourWords\AppManager\Ioc\Livewire\User;

use FourWords\AppManager\Ioc\Livewire\IocComponent;
use FourWords\AppManager\Ioc\Models\IocUser;

class UserCreateEditComponent extends IocComponent
{
    protected $modelName = "Utente App";
    protected $model = IocUser::class;

    public
        $code,
        $type,
        $name,
        $agent,
        $username,
        $password,
        $email,
        $vat_code,
        $fiscal_code,
        $confirmPassword,
        $active;

    public $modelId;

    public $showModal = false;

    public $title = "Crea nuovo";

    protected $listeners = [
        'onCreateClick',
        'onEditClick',
        'onSave',
        'onCancel'
    ];

    public $fields = [
        'id',
        'code',
        'type',
        'agent',
        'name',
        'username',
        'password',
        'email',
        'vat_code',
        'fiscal_code',
        'confirmPassword',
        'active'
    ];

    protected $rules = [
        'type' => 'required',
        'name' => 'required',
        'username' => 'required',
        'vat_code' => 'required_without:fiscal_code',
        'fiscal_code' => 'required_without:vat_code',
    ];

    public function mount() {
        $this->title = "Crea nuovo ".$this->modelName;
    }

    /**
     * Reseting all inputted fields
     * @return void
     */
    public function resetFields(){

        $this->resetErrorBag();

        foreach ($this->fields as $field) {

            if ($field == 'id') {
                $this->modelId = null;
            } else if ($field == 'active') {
                $this->{$field} = false;
            } else {
                $this->{$field} = '';
            }
        }

//        $this->emit('refreshUserDevices', $this->modelId);
    }

    public function render()
    {
        return view('ioc::livewire.user.create-edit');
    }

    public function onCreateClick() {
        if(!$this->checkPermission('create-app-users')) {
            return null;
        }

        $this->title = "Crea ".$this->modelName;
        $this->showModal = true;
    }

    public function onEditClick($modelId) {
        if(!$this->checkPermission('edit-app-users')) {
            return null;
        }

        $this->resetFields();
        $this->title = "Modifica ".$this->modelName;

        try {
            $model = $this->model::findOrFail($modelId);
            if(!$model) {
                $this->errorToast($this->modelName.' non trovato');
                return;
            }

            foreach ($this->fields as $field) {

                if ($field == 'password' || $field == 'confirmPassword') {
                    continue;
                }

                if ($field == 'id') {
                    $this->modelId = $model->id;
                } else {
                    $this->{$field} = $model->{$field};
                }
            }

            $this->showModal = true;

        } catch (\Exception $ex) {
            $this->errorToast('Errore modifica '.$this->modelName);
        }


    }

    public function onSave() {
        if ($this->modelId) {
            $this->update();
        } else {
            $this->store();
        }
    }

    public function updated($name, $value)
    {
        if ($name == 'showModal' && !$value) {
            $this->resetFields();
        }
    }

    public function store()
    {
        $this->beforeValidate();
        $this->validate();
        try {
            $this->model::create([
                'code' => $this->code,
                'type' => $this->type,
                'agent' => $this->agent,
                'name' => $this->name,
                'email' => $this->email,
                'username' => $this->username,
                'password' => $this->password,
                'vat_code' => $this->vat_code,
                'fiscal_code' => $this->fiscal_code,
                'active' => $this->active
            ]);
            $this->emit('refreshLivewireDatatable');
            $this->showModal = false;
            $this->successToast($this->modelName.' creato con successo');
            $this->resetFields();
        } catch (\Exception $ex) {
            $this->errorToast('Errore nella creazione '.$this->modelName);
        }
    }

    public function beforeValidate() {

        if (!$this->modelId) {
            $this->rules['password'] = 'required';
        }

        if (!$this->modelId || $this->password != null) {
            $this->rules['confirmPassword'] = 'required|same:password';
        }
    }

    public function update()
    {
        $this->beforeValidate();
        $this->validate();
        try {

            $model = $this->model::findOrFail($this->modelId);

            foreach ($this->fields as $field) {

                if ($field == 'confirmPassword') {
                    continue;
                }

                if ($field == 'id') {
                    $model->id = $this->modelId;
                    continue;
                }

                if ($field == 'password' && empty($this->password)) {
                    continue;
                }

                $model->{$field} = $this->{$field};
            }

            $model->save();
            $this->emit('refreshLivewireDatatable');
            $this->showModal = false;
            $this->successToast($this->modelName.' aggiornato con successo');
            $this->resetFields();
        } catch (\Exception $ex) {
            $this->errorToast('Errore nella modifica '.$this->modelName);
        }
    }

}
