<?php

namespace FourWords\AppManager\Ioc\Middleware;

use Carbon\Carbon;
use Closure;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Laravel\Sanctum\Events\TokenAuthenticated;
use Laravel\Sanctum\HasApiTokens;
use Laravel\Sanctum\Sanctum;

class HmacOrBearerMiddleware
{

    /**
     * The authentication factory implementation.
     *
     * @var \Illuminate\Contracts\Auth\Factory
     */
    protected $auth;

    /**
     * The number of minutes tokens should be allowed to remain valid.
     *
     * @var int
     */
    protected $expiration;

    /**
     * The provider name.
     *
     * @var string
     */
    protected $provider;

    public function unauthenticated() {
        return response()->json(['data' => ['now' => Carbon::now()->timestamp], 'status' => 'NO_AUTH','message' => 'Token non valido o scaduto']);
    }

    public static function isValidHmac($hmac, $username, $file) {
        $today = gmdate('Ymd');
        $SERVER_KEY = "fuckrussia";
        $serverHmac = base64_encode(hash_hmac('sha1', "$username-$today-$file", $SERVER_KEY, true));
        return hash_equals($serverHmac,$hmac);
    }

    public function getUser($username) {
        $userModel = app()->make(config('auth.providers.ioc_users.model'));
        return $userModel->where('username', $username)->first();
    }

    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure(\Illuminate\Http\Request): (\Illuminate\Http\Response|\Illuminate\Http\RedirectResponse)  $next
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\RedirectResponse|\Illuminate\Http\Response
     */
    public function handle(Request $request, Closure $next)
    {

        if (request()->bearerToken()) {
            $iocApiMiddleware = app()->make(IocApiMiddleware::class);
            $user = $iocApiMiddleware->getUserFromToken(request()->bearerToken());

        } else {

            if ( !request()->has(['user', 'file','token'])) {
                abort(400);
            }

            $username = request('user');
            $file = request('file');
            $token = request('token');

            if (!$token || !$this::isValidHmac($token, $username, $file)) {
                return $this->unauthenticated();
            }

            $user = $this->getUser($username);

        }

        if (!$user) {
            return $this->unauthenticated();
        }

        Auth::login($user);
        return $next($request);

    }

}