<?php

namespace FourWords\AppManager\Ioc\Models;

use FourWords\AppManager\Ioc\Geo\GeoUserData;
use FourWords\AppManager\Ioc\Geo\GeoUserFailed;
use FourWords\AppManager\Ioc\MailMessage\ResetPassword\ResetPasswordItalian;
use FourWords\AppManager\Orders\Model\Order;
use Illuminate\Contracts\Container\BindingResolutionException;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Facades\Hash;
use Laravel\Sanctum\HasApiTokens;

class IocUser extends Authenticatable
{
    use Notifiable, HasApiTokens;


    protected $guarded = [];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password', 'created_at','updated_at','deleted_at', 'active'
    ];

    /**
     * @return BelongsToMany
     * @throws BindingResolutionException
     */
    public function devices() {
        return $this->belongsToMany(Device::class, 'ioc_users_devices', 'user_id','device_id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function bonjours()
    {
        return $this->hasMany(BonjourHistory::class, 'user');
    }

    public function orders()
    {
        return $this->hasMany(Order::class, 'username', 'username');
    }

    public function geo()
    {
        return $this->hasOne(GeoUserData::class, 'code', 'code');
    }

    public function geoFailed()
    {
        return $this->hasOne(GeoUserFailed::class, 'code', 'code');
    }

    public function hasGeoFailed($hash) {
        return $this->geoFailed()
            ->where('hash', $hash)
            ->exists();
    }

    public function setPasswordAttribute($value)
    {
        $this->attributes['password'] = Hash::make($value);
    }

    public function setPasswordMD5Attribute($value)
    {
        $this->attributes['password'] = $value;
    }

    public function sendPasswordResetNotification($token)
    {
        $this->notify(new ResetPasswordItalian($token));
    }

//    protected static function newFactory()
//    {
//        return new IocUserFactory();
//    }
}
