<?php

namespace FourWords\AppManager\Ioc\Models;

use Illuminate\Contracts\Container\BindingResolutionException;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;

class Permission extends Model {

    protected $table = 'ioc_permissions';

    protected $fillable = ['slug','name', 'description'];

    /**
     * Get the route key for the model.
     *
     * @return string
     */
    public function getRouteKeyName() {
        return 'slug';
    }

    /**
     * Get the value of the model's route key.
     *
     * @return mixed
     */
    public function getRouteKey() {
        return $this->slug;
    }

    /**
     * @return BelongsToMany
     * @throws BindingResolutionException
     */
    public function roles() {
        return $this->belongsToMany(Role::class, 'ioc_roles_permissions')->with('ioc_permissions');
    }

    /**
     * @return BelongsToMany
     * @throws BindingResolutionException
     */
    public function users() {
        return $this->belongsToMany(app()->make('User'), 'ioc_users_permissions');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     * @throws BindingResolutionException
     */
    public function groups() {
        return $this->belongsTo(PermissionGroup::class, 'group_id', 'id');
    }

}
