<?php

namespace FourWords\AppManager\Ioc\Models;

use Illuminate\Contracts\Container\BindingResolutionException;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;

class Role extends Model
{
    protected $table = 'ioc_roles';

    const SUPERADMIN = 'superadmin';
    const SUPERADMIN_ID = 1;


    protected $fillable = ['slug','name', 'description'];

    /**
     * Get the route key for the model.
     *
     * @return string
     */
    public function getRouteKeyName()
    {
        return 'slug';
    }

    /**
     * Get the value of the model's route key.
     *
     * @return mixed
     */
    public function getRouteKey()
    {
        return $this->slug;
    }

    /**
     * @return BelongsToMany
     * @throws BindingResolutionException
     */
    public function permissions()
    {
        return $this->belongsToMany(Permission::class, 'ioc_roles_permissions');
    }

//    /**
//     * @return HasMany
//     * @throws BindingResolutionException
//     */
//    public function children()
//    {
//        return $this->hasMany(app()->make('Role'), 'parent_id', 'id');
//    }
//
//    /**
//     * @return HasOne
//     * @throws BindingResolutionException
//     */
//    public function parent()
//    {
//        return $this->hasOne(app()->make('Role'), 'id', 'parent_id');
//    }

    /**
     * @return HasMany
     * @throws BindingResolutionException
     */
    public function users()
    {
        return $this->hasMany(app()->make('User'));
    }


//    /**
//     * Metodo che restituisce tutti i ruoli figli
//     * @return array
//     */
//    public function allChildren()
//    {
//
//        $list = [];
//
//        $list[$this->id] = $this;
//
//        $children = $this->children;
//
//        foreach ($children as $child) {
//
//            $list[$child->id] = $child;
//
//            foreach ($child->allChildren() as $subChild) {
//                $list[$subChild->id] = $subChild;
//            }
//        }
//
//        return $list;
//    }
//
//
//    /**
//     * Metodo che restituisce tutti i ruoli genitori
//     * @return array
//     */
//    public function allParents()
//    {
//        $list = [];
//        array_push($list, $this);//$list[$this->id] = $this;
//
//        $parent = $this->parent;
//        while ($parent != NULL) {
//            array_push($list, $parent);
//            $parent = $parent->parent;
//        }
//        return $list;
//    }
//
//    public function parents()
//    {
//        $list = [];
//        //array_push($list, $this->id);
//        $parent = $this->parent;
//        while ($parent != NULL) {
//            array_push($list, $parent->id);
//            $parent = $parent->parent;
//        }
//        return $list;
//    }

    public function hasP(Permission $permission)
    {
        return $this->permissions->pluck('id')->contains($permission->id);
//        if ($this->parent)
//            return $this->parent->hasP($permission);
//        else
//            return false;
    }

    /**
     * @return \Illuminate\Database\Eloquent\Collection
     * @throws BindingResolutionException
     */
    public function allPermissions()
    {
        if ($this->parent_id == NULL)
            return $this->permissions()->get();
        else
            return $this->permissions()->get()->merge($this->parent->allPermissions());
    }

    public function getTrimmedDescriptionAttribute()
    {
        if (strlen($this->description) > 100)
            return substr($this->description, 0, 100) . '...';
        else
            return $this->description;
    }


//    /**
//     * metodo che ritorna tutti i ruoli presenti
//     * ecludendo quelli configurati in config human_roles_exclude (tipo superadmin)
//     */
//    public function scopeHumanRoles(Builder $query)
//    {
//        // recupero i ruoli da escludere
//        $excludeRoles = config('config.human_roles_exclude');
//
//        $query->whereNotIn('slug', $excludeRoles);
//    }

}
