<?php

namespace FourWords\AppManager\Ioc\Models;

use Illuminate\Contracts\Container\BindingResolutionException;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasOne;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Cache;

class SidebarItem extends Model
{
    protected static function boot()
    {
        parent::boot();

        static::creating(function (SidebarItem $model) {
            if ($model->position == NULL)
                $model->position = SidebarItem::where('parent_id', '=', $model->parent_id)->max('position') + 1;
        });
        static::deleting(function (SidebarItem $model) {
            SidebarItem::where([
                    ['parent_id', '=', $model->parent_id],
                    ['position', '>', $model->position]]
            )->decrement('position');
        });
    }

    public static function clearCache() {
        Cache::forget('sidebar_elements');
    }

    protected $table = 'ioc_sidebar_item';
    public $timestamps = false;

    public $guarded = [];

    /**
     * Metodo che restituisce le categorie figlie
     * @return HasMany
     * @throws BindingResolutionException
     */
    public function children()
    {
        return $this->hasMany(SidebarItem::class, 'parent_id', 'id');
    }

    /**
     * Metodo che restituisce la categoria genitore
     * @return HasOne
     * @throws BindingResolutionException
     */
    public function parent()
    {
        return $this->hasOne(SidebarItem::class, 'id', 'parent_id');
    }

    public function routeBase()
    {
        if ($this->route == NULL)
            return '';
        $index = strpos($this->route, '.');

        if (!$index) {
            return $this->route;
        }

        return substr($this->route, 0, $index) . '\.';
    }

    public function allRouteBase()
    {
        if ($this->children->isEmpty()) {
            if ($this->route != NULL)
                return $this->routeBase();
            else
                return '';
        }
        if ($this->routeBase() != '')
            $string = "{$this->routeBase()}|";
        else
            $string = '';
        foreach ($this->children as $child)
            $string .= "{$child->allRouteBase()}|";
        return substr($string, 0, -1);
    }

    public function allSubRoutes() {

        $subRoutes = [];

        if ($this->children->isEmpty()) {
                return $subRoutes;
        }
        foreach ($this->children as $child)
            $subRoutes[] = $child->route;
        return $subRoutes;
    }

    public function getLevelAttribute()
    {
        return $this->level();
    }

    public function level()
    {
        if ($this->parent_id == NULL)
            return 0;
        else
            return 1 + ($this->parent)->level();
    }

    public function getCanAttribute()
    {
        if ($this->children->isEmpty()) {
            return !is_null($this->permission) && Auth::user()->can($this->permission);
        }

        foreach ($this->children ?? [] as $child) {
            if ($child->can)
                return true;
        }

        return false;
    }

    public function allPermissions()
    {
        if ($this->children->isEmpty())
            if ($this->permission != NULL)
                return [$this->permission];
            else
                return [];

        if ($this->permission != NULL)
            $permissions = [$this->permission];
        else
            $permissions = [];
        foreach ($this->children as $child)
            $permissions = array_merge($permissions, $child->allPermissions());
        return $permissions;
    }

    public function getTranslationPath()
    {
        if (strpos($this->name, '::') === false) {
            $path = 'Ioc::menu.' . $this->name;
        } else $path = $this->name;

        return $path;
    }
}
