<?php

namespace FourWords\AppManager\Ioc\Models;

use Carbon\CarbonInterval;
use Cron\CronExpression;
use Illuminate\Console\Scheduling\CallbackEvent;
use Illuminate\Console\Scheduling\Schedule;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Log;

class Task extends Model
{

    protected $table = 'ioc_tasks';

    protected $guarded = [];

    public static $jobs = [];

    const frequencies = [
        "cron" => ["frequency" => "cron()", "withParams" => true, "hint" => "* * * * *", "detail" => "Ogni minuto"],
        "everyMinute" => ["frequency" => "everyMinute()", "withParams" => false, "detail" => "Ogni minuto"],
        "everyTwoMinutes" => ["frequency" => "everyTwoMinutes()", "withParams" => false, "detail" => "Ogni due minuti"],
        "everyThreeMinutes" => ["frequency" => "everyThreeMinutes()", "withParams" => false, "detail" => "Ogni tre minuti"],
        "everyFourMinutes" => ["frequency" => "everyFourMinutes()", "withParams" => false, "detail" => "Ogni quattro minuti"],
        "everyFiveMinutes" => ["frequency" => "everyFiveMinutes()", "withParams" => false, "detail" => "Ogni cinque minuti"],
        "everyTenMinutes" => ["frequency" => "everyTenMinutes()", "withParams" => false, "detail" => "Ogni dieci minuti"],
        "everyFifteenMinutes" => ["frequency" => "everyFifteenMinutes()", "withParams" => false, "detail" => "Ogni quindici minuti"],
        "everyThirtyMinutes" => ["frequency" => "everyThirtyMinutes()", "withParams" => false, "detail" => "Ogni trenta minuti"],
        "hourly" => ["frequency" => "hourly()", "withParams" => false, "detail" => "Ogni ora"],
        "hourlyAt" => ["frequency" => "hourlyAt(*)", "withParams" => true, "hint" => "25", "detail" => "Ogni ora a 25 minuti"],
        "everyTwoHours" => ["frequency" => "everyTwoHours()", "withParams" => false, "detail" => "Ogni due ore"],
        "everyThreeHours" => ["frequency" => "everyThreeHours()", "withParams" => false, "detail" => "Ogni tre ore"],
        "everyFourHours" => ["frequency" => "everyFourHours()", "withParams" => false, "detail" => "Ogni quattro ore"],
        "everySixHours" => ["frequency" => "everySixHours()", "withParams" => false, "detail" => "Ogni sei ore"],
        "daily" => ["frequency" => "daily()", "withParams" => false, "detail" => "Ogni giorno a mezzanotte"],
        "dailyAt" => ["frequency" => "dailyAt(**:**)", "withParams" => true, "hint" => "14:30", "detail" => "Ogni giorno alle 14:30"],
        "twiceDaily" => ["frequency" => "twiceDaily()", "withParams" => false, "hint" => "12, 18, 30", "detail" => "Ogni giorno alle 12:30 e alle 18:30"],
        "weekly" => ["frequency" => "weekly()", "withParams" => false, "detail" => "Ogni settimana"],
        "weeklyOn" => ["frequency" => "weeklyOn(*, **:**)", "withParams" => false, "hint" => "1, 8:00", "detail" => "Ogni lunedì alle 08:00"],
        "monthly" => ["frequency" => "monthly()", "withParams" => false, "detail" => "Ogni mese"],
        "monthlyOn" => ["frequency" => "monthlyOn(*, **:**)", "withParams" => false, "hint" => "4, 15:00", "detail" => "Ogni 4 del mese alle 15:00"],
        "twiceMonthly" => ["frequency" => "twiceMonthly(*, *, **:**)", "withParams" => false, "hint" => "2, 4, 15:00", "detail" => "Ogni 2 e 4 del mese alle 15:00"],
        "lastDayOfMonth" => ["frequency" => "lastDayOfMonth(**:**)", "withParams" => false, "hint" => "12:15", "detail" => "L'ultimo giorno del mese alle 12:15"],
        "quarterly" => ["frequency" => "quarterly()", "withParams" => false],
        "yearly" => ["frequency" => "yearly()", "withParams" => false],
        "yearlyOn" => ["frequency" => "yearlyOn(*, *, **:**)", "withParams" => true],
        "weekdays" => ["frequency" => "weekdays()", "withParams" => false],
        "weekends" => ["frequency" => "weekends()", "withParams" => false],
        "sundays" => ["frequency" => "sundays()", "withParams" => false],
        "mondays" => ["frequency" => "mondays()", "withParams" => false],
        "tuesdays" => ["frequency" => "tuesdays()", "withParams" => false],
        "wednesdays" => ["frequency" => "wednesdays()", "withParams" => false],
        "thursdays" => ["frequency" => "thursdays()", "withParams" => false],
        "fridays" => ["frequency" => "fridays()", "withParams" => false],
        "saturdays" => ["frequency" => "saturdays()", "withParams" => false],
        "days" => ["frequency" => "days(array|mixed)", "withParams" => true],
        "between" => ["frequency" => "between(startTime, endTime)", "withParams" => true],
        "unlessBetween" => ["frequency" => "unlessBetween(startTime, endTime)", "withParams" => true],
    ];

    public function getRuntimeAttribute()
    {
        return CarbonInterval::seconds($this->runtime)->cascade()->forHumans();
    }

    public function logs()
    {
        return $this->hasMany(TaskLog::class);
    }

    public function getLastStartAttribute()
    {
        return $this->logs()->max('start');
    }

    public function getLastEndAttribute()
    {
        $last_log = $this->logs()->where('start', '=', $this->lastStart)->first();
        return isset($last_log) ? $last_log->end : null;
    }

    public function getNextStartAttribute()
    {

        // instanzio il Kernel della Console per recuperare i task schedulati
        /** @var Schedule $schedule */
//        new Kernel(app(), new Dispatcher());
        $schedule = app()->make(Schedule::class);
        $events = $schedule->events();
        /** @var CallbackEvent $event */
        try {
            foreach ($events as $event) {
                if ($event->description == $this->target_class || str_contains($event->command ?? '', $this->target_class)) {
                    CronExpression::isValidExpression($event->getExpression());
                    $c = CronExpression::factory($event->getExpression());
//                    $c = new CronExpression($event->getExpression());
                    return $c->getNextRunDate()->format('Y-m-d H:i:s');
                }

            }

        } catch (\Exception $e) {
            Log::error($e->getMessage());
        }
    }
}
