<?php

namespace FourWords\AppManager\Ioc\Models;

use FourWords\AppManager\Ioc\MailMessage\ResetPassword\ResetPasswordItalian;
use Illuminate\Contracts\Container\BindingResolutionException;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Facades\Hash;

class User extends Authenticatable
{
    use Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'name', 'email', 'password', 'active'
    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password', 'remember_token',
    ];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

//    /**
//     * The attributes that appends to returned entities.
//     *
//     * @var array
//     */
//    protected $appends = ['photo'];

//    public function getPhotoUrlAttribute()
//    {
//        if ($this->foto !== null) {
//            return url('media/user/' . $this->id . '/' . $this->foto);
//        } else {
//            return url('media-example/no-image.png');
//        }
//    }

    /**
     * @return BelongsTo
     * @throws BindingResolutionException|\Illuminate\Contracts\Container\BindingResolutionException
     */
    public function role()
    {
        return $this->belongsTo(Role::class);
    }

    /**
     * @return BelongsToMany
     * @throws BindingResolutionException
     */
    public function permissions()
    {
        return $this->belongsToMany(Permission::class, 'ioc_users_permissions');
    }

    /**
     * Metodo che verifica se l'utente ha il permesso (o è superadmin)
     * @param Permission $permission
     * @return bool
     * @throws BindingResolutionException
     */
    public function hasPermissionTo(Permission $permission)
    {
        if ($this->isSuperAdmin())
            return true;

        return $this->role->hasP($permission) || $this->permissions()->pluck('ioc_permissions.id')->contains($permission->id);
    }

    public function isSuperAdmin()
    {
        return $this->role  != null && $this->role->name == 'Superadmin';
    }

    public function setPasswordAttribute($value)
    {
        $this->attributes['password'] = Hash::make($value);
    }

    public function getUserPermissions() {
        $userPermissions = RolePermission::where("role_id", "=", $this->role_id)->where("active", "=", true)->get()->pluck('permission_id')->toArray();

        foreach(UserPermission::where('user_id', '=', $this->id)->where('action', '=', true)->get()->pluck('permission_id')->toArray() as $addPermission) {
            $userPermissions[] = $addPermission;
        }

        foreach(UserPermission::where('user_id', '=', $this->id)->where('action', '=', false)->get()->pluck('permission_id')->toArray() as $removePermission) {
            if (($key = array_search($removePermission, $userPermissions)) !== false) {
                unset($userPermissions[$key]);
            }
        }

        $tmp = $userPermissions;
        $userPermissions = [];
        foreach ($tmp as $permission_id) {
            $userPermissions[] = $permission_id;
        }

        return $userPermissions;
    }

    public function sendPasswordResetNotification($token)
    {
        $resetPasswordItalian = new ResetPasswordItalian($token);
        $resetPasswordItalian->route = 'admin.password.reset';
        $this->notify($resetPasswordItalian);
    }

//    protected static function newFactory()
//    {
//        return new UserFactory();
//    }

//    public function getPermissionsArrayAttribute()
//    {
//        if(!Cache::has('user_permissions_'. $this->id)){
//            $permissions_array = [];
//            $freshPermissions = $this->fresh('ioc_permissions')->permissions;
//            foreach ($freshPermissions as $p) {
//                $permissions_array[$p->id] = true;
//            }
//            Cache::add('user_permissions_'. $this->id, $permissions_array);
//        }else{
//            $permissions_array = Cache::get('user_permissions_' . $this->id);
//        }
//        return $permissions_array;
//    }

//    public function reloadPermissionArray()
//    {
//        Cache::forget('user_permissions_'. $this->id);
//        $this->getPermissionsArrayAttribute();
//    }

//    public function newsNotification()
//    {
//        return $this->morphMany(NewsNotificationRecipient::class, 'newsNotificationRecipient');
//    }

//    public function scopeHumanUsers(Builder $query)
//    {
//        // recupero i ruoli visibili
//        $humanRolesId = app()->make('Role')->humanRoles()->get();
//
//        // recupero gli utenti che hanno uno dei ruoli visibili
//        $query->whereIn('users.role_id', $humanRolesId->pluck('id')->toArray());
//    }

}
