<?php

namespace FourWords\AppManager\Ioc\Providers;

use FourWords\AppManager\Ioc\Models\Permission;
use FourWords\AppManager\Ioc\Models\User;
use Illuminate\Foundation\Support\Providers\AuthServiceProvider as ServiceProvider;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Gate;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Schema;

class AuthServiceProvider extends ServiceProvider
{

    /**
     * Register any authentication / authorization services.
     *
     * @return void
     */
    public function boot()
    {
        Config::set('auth.providers.users.model', User::class);
        $this->registerPolicies();

        try {
            if (Schema::hasTable('ioc_permissions')) {
                foreach (Permission::all() as $permission) {
                    Gate::define($permission->slug, function ($user) use ($permission) {
                        if (!($user instanceof User))
                            $user = app()->make('User')->find($user->id);
                        return $user->hasPermissionTo($permission);
                    });
                }
            }
        } catch (\Exception $e) {
            Log::error($e->getMessage());
        }
    }
}
