<?php

namespace FourWords\AppManager\Ioc\Providers;

use FourWords\AppManager\Ioc\Models\SidebarItem;
use FourWords\AppManager\Ioc\View\Composers\FakerComposer;
use FourWords\AppManager\Ioc\View\Composers\LoggedInUserComposer;
use FourWords\AppManager\Ioc\View\FileViewFinder;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\View;

class ViewServiceProvider extends \Illuminate\View\ViewServiceProvider
{

    public function boot()
    {
        View::composer('*', FakerComposer::class);
        View::composer('*', LoggedInUserComposer::class);

        if (!Cache::has('sidebar_elements')) {

            $elements = collect([]);
            try {
                $elements = SidebarItem::with(['children','children.children.children', 'parent'])->orderBy('position')->get();
            } catch(\Exception $ignored) {}

            Cache::forget('sidebar_elements');
//            Cache::forget('sidebar_level0');
            if (!$elements->isEmpty()) Cache::add('sidebar_elements', $elements);
//            Cache::add('sidebar_level0', $level0);
        }

        View::share([
            'side_menu' => Cache::get('sidebar_elements') ?? [],
            'theme' => 'light',
            'layout' => 'side-menu'
        ]);
    }

    /**
     * Register the view finder implementation.
     *
     * @return void
     */
    public function registerViewFinder() {
        $this->app->bind('view.finder', function ($app) {
            return new FileViewFinder($app['files'], $app['config']['view.paths']);
        });
    }
}
