<?php

namespace FourWords\AppManager\Ioc\Ps;

use Exception;
use FourWords\AppManager\Ioc\Ps\Traits\PSConnection;

class GetAppRelease {

    use PSConnection;

    protected $app;
    protected $token;
    protected $psDB;

    /**
     * @param $app
     */
    public function __construct()
    {
        // Solamente per la _COOKIE_KEY_ e la connessione al DB
        $this->psDB = $this->loadPSConnection();
    }


    /**
     * @throws Exception
     */
    public function get($app, $token)
    {

        $this->app = $app;
        $this->token = $token;

        //imposto il nome della sessione
        session_id(md5(_COOKIE_KEY_));

        //comando di avvio della sessione
        session_start();

        //controllo se è presente in sessione un array chiamato icm_token
        if(!isset($_SESSION['icm_devices']))
        {
            $_SESSION['icm_devices'] = array();
        }

        if(!isset($_SESSION['icm_devices'][$this->token]) || !isset($_SESSION['icm_devices'][$this->token]['expiration_date'])) {
            throw new Exception('unauthorized');
        }

        // Controllo se il token è scaduto, in caso lo annullo
        $timestamp = (int)$_SESSION['icm_devices'][$this->token]['expiration_date'];

        if ($timestamp != 'MAX') {
            if ($timestamp < time()) {
                unset($_SESSION['icm_devices'][$this->token]);
                throw new Exception('unauthorized');
            } else {
                // Lo rinnovo di 2h (Dovrei leggere i file di configurazione ICM ma preferisco non includerli per fare solo un rinnovo)
                $_SESSION['icm_devices'][$this->token]['expiration_date'] = mktime(date("H") + 2, date("i") + 0, date("s") + 0);
            }
        }

        //recupero le informazioni
        $info = $this->psDB->selectOne("
				SELECT DISTINCT iai.id_app_info,iah.news,iah.version,iah.size,iah.releasedate,iai.description
				FROM icm_app_infos as iai LEFT JOIN icm_app_history as iah
				ON iai.id_app_info = iah.id_app_info
				WHERE iai.id_app_info = $this->app AND iah.current = 1 AND iah.disabled = 0 AND iah.status = 'Completo'
				");

        //se non ho trovato nulla
        if(!$info) throw new Exception('unavailable');

        //recupero le icone
        $icons = $this->psDB->select("
				SELECT name,filename
				FROM icm_app_infos iais left join icm_app_images as iai on iai.id_app_info = iais.id_app_info
				WHERE (name = 'normal' OR name = 'retina') AND iais.id_app_info = $this->app AND filename != '' 
				AND filename != 'null'
				");

        //recupero la storia dei rilasci
        $history = $this->psDB->select("
				SELECT news,version,releasedate
				FROM icm_app_history
				WHERE current = 0 AND id_app_info = $this->app AND disabled = 0
					AND releasedate < (
						SELECT releasedate
						FROM icm_app_history
						WHERE current = 1 AND id_app_info = $this->app
					)
				ORDER BY releasedate DESC
				");

        $return = array();
        foreach($icons as $icon) {
            $return[($icon->name == 'normal') ? "icon" : "icon2x"] = absuri(config('ioc.ps.dir')).'/modules/icm/app/img/'.$icon->filename;
        }
        $return["news"] = $info->news;
        $return["url"] = absuri(config('ioc.ps.dir')).'/modules/icm/app/store.php';
        $return["version"] = $info->version;
        $return["releasedate"] = $info->releasedate;
        $return["size"] = $info->size.' MB';
        $return["description"] = str_replace("<p>", "\r\n\r\n", rtrim($info->description,"</p>"));
        $return["history"] = $history;

        return $return;

    }

}