<?php

namespace FourWords\AppManager\Ioc\Ps;

use Exception;
use FourWords\AppManager\Ioc\Events\GetDatabaseEvent;
use FourWords\AppManager\Ioc\Ps\Traits\PSConnection;

class GetResource {

    use PSConnection;

    protected $type;
    protected $file_name;
    protected $token;
    protected $userCode;
    protected $psDB;

    /**
     * @throws Exception
     */
    public function __construct()
    {
        // Solamente per la _COOKIE_KEY_ e la connessione al DB
        $this->psDB = $this->loadPSConnection();
    }

    /**
     * @throws Exception
     */
    public function get($file_name, $type, $token, $userCode = ""): string
    {

        $this->type = $type;
        $this->file_name = $file_name;
        $this->userCode = $userCode;
        $this->token = $token;

        //imposto il nome della sessione
        session_id(md5(_COOKIE_KEY_));

        //comando di avvio della sessione
        session_start();

        if(!isset($_SESSION['icm_devices']))
        {
            $_SESSION['icm_devices'] = array();
        }

        $today = gmdate('Ymd');
        $SERVER_KEY = "fuckrussia";

        try
        {

            $HMAC = base64_encode(hash_hmac('sha1', "$this->userCode-$today-$file_name", $SERVER_KEY, true));
            $isAuthorized = false;

            if (($this->type == 'img' || $this->type == 'vid' || $this->type == 'doc') && hash_equals($HMAC,$this->token)) {
                $isAuthorized = true;
            } else if(isset($_SESSION['icm_devices'][$this->token]) && isset($_SESSION['icm_devices'][$this->token]['expiration_date'])) {

                // Prendo lo userCode
                $this->userCode = $_SESSION['icm_devices'][$this->token]['userCode'];

                // Controllo se il token è scaduto, in caso lo annullo
                $timestamp = (int)$_SESSION['icm_devices'][$this->token]['expiration_date'];

                $isAuthorized = true;
                if ($timestamp != 'MAX') {
                    if ($timestamp < time()) {
                        unset($_SESSION['icm_devices'][$this->token]);
                        $isAuthorized = false;
                    } else {
                        // Lo rinnovo di 2h (Dovrei leggere i file di configurazione ICM ma preferisco non includerli per fare solo un rinnovo)
                        $_SESSION['icm_devices'][$this->token]['expiration_date'] = mktime(date("H") + 2, date("i") + 0, date("s") + 0);
                    }
                }
            }

            if (!$isAuthorized) { throw new Exception('unauthorized'); }

            //se il file è di tipo contratto rimuovo dal suo nome il timestamp
            if($this->type == 'cont') {
                list($customer_code,$timestamp,$ext) = explode('.',$file_name);
                $file_name = $customer_code.'.'.$ext;
            }

            //se il file è di tipo categoria rimuovo dal suo nome il timestamp
            if($this->type == 'cat') {
                $tmp_list = explode('.', $file_name);
                unset($tmp_list[0]);
                $file_name = implode('.', $tmp_list);
            }

            //se il file è di tipo .db, quindi il dispositivo sta per ricevere il nuovo database sqlite,
            //salvo nel database icm_device la nuova versione del dispositivo corrente
            if($this->type == 'db')
            {

                try {
                    $dbPath = pushEvent(new GetDatabaseEvent(['type' => $this->type, 'filename' => $this->file_name, 'userCode' => $this->userCode]));
                } catch (\Exception $e) {
                    throw new Exception('file_not_found');
                }

                //recupero il codice dispositivo
                $device_oudid = $_SESSION['icm_devices'][$this->token]['code'];

                //recupero il timestamp del db che si sta per inviare
                //smonto la stringa rispetto il carattere '.'
                $list = explode('.', $file_name);

                //salvo il nuovo lastdb_timestamp del dispositivo (solo se inizia con icmdb)
                if (strpos($list[0], 'icmdb') === 0) {

                    $timestamp = $list[1];

                    $timestamp = is_null($timestamp) ? time() : date("Y-m-d H:i:s",$timestamp-date("Z"));

                    $this->psDB->unprepared("
                        UPDATE icm_devices
                        SET lastdb_timestamp = '".$timestamp."'
                        WHERE code = '$device_oudid'
                        ");
                }

                return $dbPath;

            } else {
                $file_path = self::getFilePath($this->type,$file_name,$this->userCode);
            }

            //se è un documento o un video devo cercare il nome del file senza estensione
            if($this->type == 'doc' || $this->type == 'vid')
            {
                //smonto la stringa rispetto il carattere '.'
                $list = explode('.', $file_name);

                //salvo il nome del file senza estensione
                unset($list[sizeof($list)-1]);
                $filename_no_ext = implode('.',$list);

                //copio il file mettendo l'estensione
                $file_name = $filename_no_ext;
            }

            session_commit();


            return $file_path.$file_name;

        }
        catch (Exception $e)
        {
            session_commit();

            throw $e;
        }

    }

    /**
     * @throws Exception
     */
    public static function getFilePath($type, $file, $userCode): string
    {

        //smonto la stringa rispetto il carattere '.'
        $list = explode('.', $file);

        //salvo il nome del file senza estensione
        unset($list[sizeof($list)-1]);
        $filename = implode('.',$list);

        //recupero tutti i percorsi delle cartelle dove sono presenti i possibili
        //file da trasferire
        switch ($type)
        {
            case 'img': $paths = array('/var/www/html/ioc/storage/catalog/images/',config('ioc.ps.icmImgCategoriesDir'), config('ioc.ps.icmImgGalleriesDir'), config('ioc.ps.icmImgProductsDir'), config('ioc.ps.icmImgCombinationsDir'));break;
            case 'doc':  case 'vid': $paths = array(config('ioc.ps.downloadDir'));break;
            default: $paths = null;
        }


        //se il path è ancora nullo significa che il type passato dall'app non è stato
        //riconosciuto
        if(!$paths)
            throw new Exception('type_problem');

        //se è un documento o un video devo cercare il nome del file senza estensione
        if($type == 'doc' || $type == 'vid')
            $file = $filename;

        foreach($paths as $path)
        {
            if(is_readable($path.$file))
            {
                return $path;
            }
        }

        //se non ho trovato il file lancio un eccezione
        throw new Exception('file_not_found');

    }

}