<?php

namespace FourWords\AppManager\Ioc\Ps;

use FourWords\AppManager\Ioc\Contracts\UserImporter;
use FourWords\AppManager\Ioc\Jobs\IocJob;
use FourWords\AppManager\Ioc\Ps\Traits\PSConnection;

class ImportPsUsers extends IocJob implements UserImporter
{

    use PSConnection;

    protected $psDB;
    protected  $idLang;

    public function handleJob()
    {
        parent::handleJob();

        $this->import();

    }

    function import()
    {
        /**
         * Entry point base per far partire l'importazione dei clienti da chiamata REST
         */

        try {

            $this->psDB = $this->loadPSConnection();

            // Creo il profilo 'Cliente' qualora non vi fosse e prendo l'id
            $res = (array) $this->psDB->table('ps_profile_lang')->select('*')->where('name', 'Cliente')->first();

            $id_profile_cliente = (int) $res['id_profile'];

            if (empty($res)) {
                $id_profile_cliente = $this->psDB->table('ps_profile')->max('id_profile') + 1;
                $this->psDB->insert('INSERT INTO ps_profile (id_profile) VALUES (?)', [$id_profile_cliente]);
                $this->psDB->insert('INSERT INTO ps_profile_lang (id_lang,id_profile,name) VALUES (?,?,?)', [1, $id_profile_cliente, 'Cliente']);

            }

            // Se la lunghezza di 'code' in icm_employees è < 20 la porto a 20
            $codeColumnLength = $this->psDB->select("
                                                    SELECT CHARACTER_MAXIMUM_LENGTH as code_length
                                                    FROM information_schema.columns
                                                    WHERE table_schema = ?
                                                    AND table_name = ?
                                                    AND column_name = ?
                                                ", [_DB_NAME_, 'icm_employees', 'code'])[0]->code_length;
            if (!empty($codeColumnLength) && ((int)$codeColumnLength) < 20) {
                $this->psDB->statement('ALTER TABLE icm_employees MODIFY COLUMN code varchar(20) NOT NULL AFTER id_employee');
            }

            $users = $this->getUsers();

            $this->idLang = $this->psDB->table(_DB_PREFIX_.'lang')->select('id_lang')->where('iso_code','it')->first()->id_lang;

            foreach ($users as $user) {

                // Importo tutti gli utenti in prestashop con active = false
                $this->saveOrUpdateEmployee($user['firstname'],$user['username'],$user['code'],$user['passwd'],$id_profile_cliente, 0);
//                $this->saveOrUpdateEmployee($user['descrizione'],$user['login'],$user['codice_cliente'],$user['PASSWORDENC'],$id_profile_cliente, $user['flag_sospensione'] !== 'S' ? "1" : "0", $logger);
            }

            $this->logger->info('Importazione conclusa');

        } catch (\Exception $e) {

            echo $e;
            $this->logger->error($e);

        }

    }

    function getUsers() {
        //TODO: Prendere dati da tabelle standard Galileo
       dd('TODO');
    }

    function saveOrUpdateEmployee($firstname, $username, $code, $passwd, $id_profile, $active) {

        /** Username */

        // Rimuovo spazi in eccesso, caratteri speciali e sostituisco spazi con trattini per creare la mail
        $username = preg_replace('/\s+/', ' ', $username);
        $username = str_replace(' ', '-', $username);
        $username = preg_replace('/[^A-Za-z0-9\-]/', '', $username);

        $email = strtolower($username) . config('ioc.ps.client.mailSuffix', '');

        /** Firstname */

        // Rimuovo alcuni caratteri speciali dal firstname
        $firstname = preg_replace('/[^A-Za-z&\'\s.-]/', ' ', $firstname);
        // Sostituisco spazi multipli con uno spazio solo
        $firstname = preg_replace('/\s+/', ' ', $firstname);

        /** Lastname */
        $lastname = '-';

        $psUser = $this->psDB->table(_DB_PREFIX_.'employee')
            ->select(['id_employee', 'firstname', 'lastname', 'passwd'])
            ->where('email', $email)->first();

        //se il dipendente esiste creo l'oggetto del dipendente altrimenti ne creo uno vuoto
        if($psUser) {

//        if ($employee->email != $email) {
//            $this->logger->writeLog(_ICM_DATE_LOG_FILE_ . '_user.txt', $employee->email." modificato in ".$email);
//            $this->logger->writeLog(_ICM_DATE_LOG_FILE_ . '_user.txt','');

            $changable = ['firstname','lastname', 'passwd'];
            $userChanged = false;

            foreach ($changable as $field) {
                if ($psUser->{$field} != ${$field}) {
                    $this->logger->info("Aggiornato campo $field per utente $username: da ".$psUser->{$field}." a ".${$field});
                    $userChanged = true;
                }
            }

            if ($userChanged) {
                $psUser->firstname = $firstname;
                $psUser->lastname = $lastname;
                $psUser->passwd = $passwd;
//                $employee->active = $active;
                try {
                    $this->psDB->table(_DB_PREFIX_.'employee')->where('id_employee',$psUser->id_employee)->update(['firstname' => $firstname, 'lastname' => $lastname, 'passwd' => $passwd]);
                } catch (\Exception $exception) {
                    throw new \Exception($exception->getMessage()."\n".json_encode($psUser),$exception->getCode());
                }

            }

        }
        else {
            $psUser = (object)[];
//        $passwd = $this->randomSixCharactersPassword();
            $psUser->passwd = $passwd;
            $psUser->firstname = $firstname;
            $psUser->lastname = $lastname;

            $psUser->id_lang = $this->idLang;
            //se l'utente recuperato non è un agente (tipo AG) gli assegno il profilo pubblico (quindi visualizza solo il db generale)
            $psUser->id_profile = $id_profile;
            $psUser->active = $active;

            $psUser->email = $email;

            $newUserId = null;

            try {
                $newUserId = $this->psDB->table(_DB_PREFIX_.'employee')->insertGetId([
                    'firstname' => $psUser->firstname,
                    'lastname' => $psUser->lastname,
                    'passwd' => $psUser->passwd,
                    'id_lang' => $this->idLang,
                    'id_profile' => $psUser->id_profile,
                    'active' => $psUser->active,
                    'email' => $psUser->email,
                    'last_passwd_change' => date('Y-m-d'),
                    'last_connection_date' => date('Y-m-d')
                    ]);
            } catch (\Exception $exception) {
                throw new \Exception($exception->getMessage()."\n".json_encode($psUser),$exception->getCode());
            }

        }

        if ($newUserId) {
            $this->logger->notice("Nuovo utente: $username | Password: $passwd");
            $this->psDB->table('icm_employees')->insert(['id_employee' => $newUserId, 'id_parent' => 0, 'code' => $code, 'status' => 'ENABLED']);
        } else {
            $this->psDB->table('icm_employees')->where('id_employee', $psUser->id_employee)->update(['id_parent' => 0, 'code' => $code]);
        }

        return $psUser->id;

    }

    public function getModel()
    {
        //
    }

    public function saveOrUpdateUser(&$userData)
    {
        //
    }

    public function recordToUserMap()
    {
        //
    }

    public function prepareUserData(&$userData)
    {
        //
    }
}