<?php

namespace FourWords\AppManager\Ioc\Ps;

use DateTimeZone;
use Exception;
use FourWords\AppManager\Ioc\Events\BonjourFailedEvent;
use FourWords\AppManager\Ioc\Events\PSBonjourPayload;
use FourWords\AppManager\Ioc\Ps\Traits\PSConnection;
use Illuminate\Support\Str;
use Monolog\Formatter\LineFormatter;
use Monolog\Handler\RotatingFileHandler;
use Monolog\Logger;

class PSBonjour
{

    use PSConnection;

    protected $logger;

    protected $now;
    protected $response;
    protected $isDemo = false;

    public function isTokenExpired($token)
    {
        //recupero il timestamp
        $timestamp = (int)$_SESSION['icm_devices'][$token]['expiration_date'];

        //se è settato a MAX è valido per tutta la durata della sessione server
        if($timestamp == 'MAX')
            return false;

        //se il token è ancora valido aggiorno la sua validità
        if($timestamp > time()) {

            //rinnovo la validità
            $_SESSION['icm_devices'][$token]['expiration_date'] = $this->getExpirationDate(_ICM_TOKEN_NORMAL_EXPIRE_TIME_);

            return false;
        }

        //è scaduto quindi lo rimuovo dalla sessione
        unset($_SESSION['icm_devices'][$token]);

        return true;
    }

    public function getExpirationDate($type)
    {
        //se è settato a MAX è valido per tutta la durata della sessione server
        if($type == 'MAX')
            return $type;

        //recupero quante ore minuti e secondi avrà il token di validità
        list($ora,$min,$sec) = explode(",",$type);

        //ritorno l'ora corrente più le ore minuti e secondi di validità per formare
        //il timestamp di fine validità del token
        return mktime(date("H")+$ora,date("i")+$min,date("s")+$sec);
    }

    public function __construct()
    {

        $this->now = gmmktime();

        $this->response =  ['data' => [
            'now' => $this->now
        ]];

        // Log
        $this->logger = new Logger('bonjour');
        $this->logger->setTimezone(new DateTimeZone('Europe/Rome'));

        if (!file_exists(config('ioc.log.dir'))) {
            mkdir(config('ioc.log.dir'),config('ioc.log.dirPermissions'), true);
        }

        // debug.log
        $LdebugHandler = new RotatingFileHandler(config('ioc.log.dir').'bonjour/debug.log',14,Logger::DEBUG, true, config('ioc.log.filePermissions'));
        $debugDateFormat = "Y-m-d H:i:s";
        $debugOutput = "[%datetime%] %channel%.%level_name% : %message% \n";
        $debugFormatter = new LineFormatter($debugOutput, $debugDateFormat,true);
        $LdebugHandler->setFormatter($debugFormatter);
        $this->logger->pushHandler($LdebugHandler);
    }

    /**
     * @throws \Exception
     */
    //TODO:>> Tradurre i messaggi di errore?
    public function login () {

        try {


            $psDB = $this->loadPSConnection();

            $loggedUser = array();

            //imposto il nome della sessione
            session_id(md5(_COOKIE_KEY_));

            //comando di avvio della sessione
            session_start();

            //controllo se è presente in sessione un array chiamato icm_token
            if (!isset($_SESSION['icm_devices']))
                $_SESSION['icm_devices'] = array();

            //recupero le variabili dal request html
            $token = request()->get('token');
            $code = request()->get('oudid');
            $user = request()->get('user');
            $password = request()->get('pwd');
            $system_name = request()->get('system_name');
            $system_version = request()->get('system_version');
            $model = request()->get('model');
            $app_version = request()->get('app_version');
            $lastdb_version = request()->get('lastdb_version', '');
            $subUser = request()->get('subUser');
            $id_app_info = request()->get('app','');

            if (strtolower($user) == "clappconnect" || strtolower($user) == "clitest" ) {
                $this->isDemo = true;
            }

            if (!Str::contains($user, '@')) {
                $user = $user . config('ioc.ps.client.mailSuffix','');
            }


            //se il modulo è disabilitato impedisco l'accesso
//            if (!_ICM_ACTIVE_ || _ICM_MAINTENANCE_) {
//                $this->response['status'] = 'ICM_DISABLED';
//                $this->response['message'] = 'Servizio in manutenzione';
//                return pushEvent(new BonjourFailedEvent($this->response));
//            }

            // controllo la licenza
//            if(($license_devices = checkLicense()) === false)
//            {
//                $this->logger->warn('Licenza d\'uso scaduta');
//                $this->response['status'] = 'LICENSE_EXPIRED';
//                $this->response['message'] = 'Licenza d\'uso scaduta';
//                return pushEvent(new BonjourFailedEvent($this->response));
//            }

            //se è stato scelto di avviare la sincronizzazione da bonjour
            //controllo se è possibile sincronizzare i db, nel caso li creo
//            if (_ICM_BONJOUR_SYNC_DB_CHECK_) {
//                syncDbCheck();
//            }

            $device = $psDB->table('icm_devices')->select('*')->where('code', $code)->first();

            //controllo se il device è disabilitato
            if ($device && $device->status == 'DISABLED') {
                //tolgo il token dalla sessione
                unset($_SESSION['icm_devices'][$token]);
                $this->logger->warn('L\'utente ' . $user . ' ha il device disabilitato');
                $this->response['status'] = 'DEV_NO_AUTH';
                $this->response['message'] = 'Dispositivo non abilitato';
                return pushEvent(new BonjourFailedEvent($this->response));
            }

            //se lo stato del device è deleted invio il comando di cancellazione dati sul dispositivo
            if ($device && $device->status == 'DELETED') {
                //tolgo il token dalla sessione
                unset($_SESSION['icm_devices'][$token]);
                $this->response['status'] = 'WIPE';
                $this->response['message'] = 'Dispositivo non abilitato';
                return pushEvent(new BonjourFailedEvent($this->response));
            }

            //boolean che mi ricorda se mi sono autenticato con il token
            $authWithToken = false;

            //verifico se mi è stato passato il token
            if ($token) {
                //verifico se il token del device è presente in sessione ed è ancora valido,
                //se lo è autentico l'utente
                if (isset($_SESSION['icm_devices'][$token]) && isset($_SESSION['icm_devices'][$token]['expiration_date']) && !$this->isTokenExpired($token)) {

                    //recupero tutti i dati del devide e dell'utente presenti in sessione
                    $user = $_SESSION['icm_devices'][$token]['user'];
                    $code = $_SESSION['icm_devices'][$token]['code'];
                    $subUser = $_SESSION['icm_devices'][$token]['subUser'];
                    $userCode = $_SESSION['icm_devices'][$token]['userCode'];
                    $userRole = $_SESSION['icm_devices'][$token]['userRole'];
                    $subUsersList = $_SESSION['icm_devices'][$token]['subUsersList'];
                    $actions = $_SESSION['icm_devices'][$token]['actions'];
                    $authWithToken = true;
                } else {
                    //token presente ma è scaduto, lo cancello
                    if (isset($_SESSION['icm_devices'][$token]))
                        unset($_SESSION['icm_devices'][$token]);

                    $this->response['status'] = 'TOKEN_EXPIRED';
                    $this->response['message'] = 'Token scaduto o non valido';
                    return pushEvent(new BonjourFailedEvent($this->response));
                }
            }

            $query = $psDB->table(_DB_PREFIX_.'employee')->join('icm_employees','icm_employees.id_employee','=',_DB_PREFIX_.'employee.id_employee')
                ->select([
                    _DB_PREFIX_.'employee.id_employee',
                    _DB_PREFIX_.'employee.id_profile',
                    _DB_PREFIX_.'employee.lastname',
                    _DB_PREFIX_.'employee.firstname',
                    _DB_PREFIX_.'employee.email',
                    _DB_PREFIX_.'employee.active',
                    'icm_employees.code',
                    'icm_employees.status',
                ])
                ->where('email', $user);

            if (!config('ioc.ps.loginViaB2B')) {
                $query->where('passwd',$password);
            }

            $psUser = $query->first();

            //verifico se l'utente passato è presente
            if (!$psUser) {
                $this->response['status'] = 'USER_NO_AUTH';
                $this->response['message'] = 'Username e/o password errati';
                return pushEvent(new BonjourFailedEvent($this->response));
            }

            $psUser->username = (string)preg_replace('/@(.*)/', '', $psUser->email);

            if ($device) {
                $this->response['data']['clean'] = (bool)$device->clean_data;
            }


            if (!$this->isDemo && (!$psUser->active || $psUser->status !== 'ENABLED') ) {
                $this->response['status'] = 'DISABLED_USER';
                $this->response['message'] = 'Utente non abilitato';
                return pushEvent(new BonjourFailedEvent($this->response));
            }

            $id_employee = $psUser->id_employee;
            $userCode = $psUser->code;
            $id_profile = $psUser->id_profile;

            //inizializzo a null la variabile che mi servirà per creare un oggetto Icm_device
            $device_obj = null;

            //se non c'è il device nel db con l'oudid passato significa che sto ricevendo una
            //richiesta di attivazione del servizio per questo dispositivo. Inserisco i dati
            //necessari dell'utente e del device nel db
            if (!$device) {

                $device = (object) [
                    'code' => $code,
                    'employee' => $id_employee,
                    'system_name' => $system_name,
                    'system_version' => $system_version,
                    'model' => $model,
                    'app_version' => $app_version,
                    'lastdb_version' => $lastdb_version,
                    'id_app_info' => $id_app_info
                ];

                //controllo che le variabili richieste siano state settate
                if(is_null($device->code) || is_null($device->model) || is_null($device->system_name)
                    || is_null($device->system_version) || is_null($device->employee))
                    throw new Exception('Icm_device: non posso salvare il device nel db, mancano dei valori che non possono essere nulli');

                if(empty($device->id_app_info))
                    $device->id_app_info = 1;

                //controllo il timestamp
                if(is_null($device->lastdb_timestamp))
                    $device->lastdb_timestamp = null;

                $pushDeviceToken = !is_null($device->push_device_token) ? "'{$device->push_device_token}'" : NULL;
                $pusherChannels = !is_null($device->pusher_channels) ? "'".$device->pusher_channels."'" : NULL;
                    $res = $psDB->insert('INSERT INTO icm_devices (code,system_name,system_version,model,app_version,lastdb_timestamp,status,id_app_info,
				                         push_device_token,push_status,pusher_channels) VALUES (?,?,?,?,?,?,?,?,?,?,?)', [$device->code, $device->system_name, $device->system_version, $device->model, $device->app_version, $device->lastdb_timestamp,
                        'ENABLED', $device->id_app_info, $pushDeviceToken, $device->push_status, $pusherChannels]);

                if($res)
                {
                    $d = $psDB->table('icm_devices')->select('id_device')->where('code', $device->code)->first();


                    $device->id_device = $d->id_device;
                    $device->device_employee_status = 'WAIT';

                    $psDB->insert('INSERT INTO icm_devices_employees (id_device,id_employee,status) VALUES (?,?,?)', [$d->id_device, $device->employee, 'WAIT']);
                }

            }


            //controllo se il dipendente ha già usato il dispositivo, nel caso sia il primo accesso
            //aggiungo la riga nel db e setto lo stato a WAIT

            $id_device = $psDB->table('icm_devices')
                ->select('id_device')
                ->where('code',$code)
			    ->first();

            $deviceEmployee = $psDB->table('icm_devices_employees as ide')
                ->select('*')
                ->where('id_device', $id_device ? $id_device->id_device : 0)
                ->where('id_employee',$id_employee)
                ->first();

            if(!$deviceEmployee)
            {

                $psDB->insert('INSERT INTO icm_devices_employees (id_device,id_employee,status) VALUES (?,?,?)', [$device->id_device, $id_employee, 'WAIT']);
                $deviceEmployee = (object) ['id_device' => $device->id_device, 'id_employee' => $id_employee, 'status' => 'WAIT', 'last_access' => date('Y-m-d H:i:s')];
                $firstAccess = true;

            }
            $firstAccess = false;


//            $firstAccess = \Icm_device_employee::checkIfIsFirstUse($code,$id_employee);

            //se la variabile è ancora null, dunque non ho ancora creato l'oggetto essendoci già
            //nel db il device con oudid passato, creo l'oggetto
//            if (is_null($device_obj))
//                $device_obj = new Icm_device_custom($code, $id_employee);

            //se è la prima volta che ha effettuato il login
//            if (_ICM_MANAGE_AUTHORIZATIONS_ && $firstAccess) {
//                //scrivo nel log che un utente ha effettuato l'accesso con un dispositivo
//                //ed è in attesa di esser abilitato all'utilizzo
//                $this->logger->warn("L'utente con codice ". $userCode . " - " . $employeeInfo['firstname'] . " " . $employeeInfo['lastname'] . " sta eseguendo il primo accesso ed è in attesa del permesso per l'utilizzo del dispositivo $device_obj->system_name");
//            }

            //se lo stato della coppia utente/dispositivo è disabilitata mando json con status disabled_device_user
            if ($deviceEmployee->status === 'DISABLED') {
                $this->response['status'] = 'USER_DEV_NO_AUTH';
                $this->response['message'] = 'Utente non abilitato all\'uso del dispositivo';
                return pushEvent(new BonjourFailedEvent($this->response));
            }

            if (!$this->isDemo && intval(config('ioc.maxDevicesForUser')) > 0) {
                $r = $psDB->select($psDB->raw("SELECT COUNT(*) as n_dispositivi FROM icm_devices id JOIN icm_devices_employees ide ON id.id_device = ide.id_device AND ide.id_employee = '" . $id_employee . "' WHERE ide.status = 'OK'"));

                $nDispositivi = count($r) > 0 ? (int)$r[0]->n_dispositivi : 0;
                if ($deviceEmployee->status == "WAIT" && $nDispositivi >= config('ioc.maxDevicesForUser')) {

                    $psDB->table('icm_devices_employees')->where('id_device',$deviceEmployee->id_device)->where('id_employee',$id_employee)->update(['status' => 'WAIT']);
                    $deviceEmployee->status = 'WAIT';
                    $this->logger->warn("L'utente con codice " . $userCode . " - " . $psUser->firstname . " " . $psUser->lastname . " è in attesa del permesso per l'utilizzo del dispositivo $system_name");
                } else if ($deviceEmployee->status !== 'OK') {
                    $psDB->table('icm_devices')->where('id_device',$deviceEmployee->id_device)->update(['status' => 'ENABLED']);
                    $psDB->table('icm_devices_employees')->where('id_device',$deviceEmployee->id_device)->where('id_employee',$id_employee)->update(['status' => 'OK']);
                    $device->status = 'ENABLED';
                    $deviceEmployee->status = 'OK';
                }
            } else if ($deviceEmployee->status !== 'OK') {
                $psDB->table('icm_devices')->where('id_device',$deviceEmployee->id_device)->update(['status' => 'ENABLED']);
                $psDB->table('icm_devices_employees')->where('id_device',$deviceEmployee->id_device)->where('id_employee',$id_employee)->update(['status' => 'OK']);
                $device->status = 'ENABLED';
                $deviceEmployee->status = 'OK';
            }

            //se lo stato della coppia utente/dispositivo è in WAIT mando json con status wait a meno che non sia in whitelist
            if ($deviceEmployee->status === 'WAIT') {

                // if((!_ICM_MANAGE_AUTHORIZATIONS_ && ($license_devices - \Icm_device::getNumberOfLicensedDevices()) > 0) || $device_obj->isInWhitelist() || \Icm_employee::isInWhiteList($id_employee)) {
                if ($this->isDemo) {
                    $psDB->table('icm_devices')->where('id_device',$deviceEmployee->id_device)->update(['status' => 'ENABLED']);
                    $psDB->table('icm_devices_employees')->where('id_device',$deviceEmployee->id_device)->where('id_employee',$id_employee)->update(['status' => 'OK']);
                    $device->status = 'ENABLED';
                    $deviceEmployee->status = 'OK';
                } else {


                    //se ci sono ancora licenze disponibili e il dispositivo è in whitelist o l'utente è in whitelist
//                    if ((($license_devices - \Icm_device::getNumberOfLicensedDevices()) > 0) && ($device_obj->isInWhitelist() || \Icm_employee::isInWhiteList($id_employee))) {
//                        //abilito il dispositivo e anche la coppia dipendente/dispositivo
//                        $device_obj->setDeviceStatusEnabled();
//                        $device_obj->setDeviceEmployeeStatusOk();
//                        $device_obj->update();
//                    } else {
                        $this->response['status'] = 'WAIT_AUTH';
                        $this->response['message'] = 'In attesa di abilitazione da parte dell\'amministratore';
                        return pushEvent(new BonjourFailedEvent($this->response));
//                    }
                }
            }

            //se lo stato della coppia utente/dispositivo è abilitato lo imposto a OK
            if($deviceEmployee->status === 'ENABLED')
            {
                $psDB->table('icm_devices_employees')->where('id_device',$deviceEmployee->id_device)->where('id_employee',$id_employee)->update(['status' => 'OK']);
                $deviceEmployee->status = 'OK';
            }

            //se ok verifico password e se valida invio json con status ok, data, key e salvo il token in sessione.
            if ($deviceEmployee->status === 'OK') {
                //verifico user e password dell'utente se non mi sono autenticato con il token
                if (!$authWithToken) {

                    //se il modulo che recupera i dati dall'AS400 per effettuare gli ordine è attivo, effettuo il Bonjour
                    //sul B2B, altrimenti effettuo l'autenticazione in PS
                    if(config('ioc.ps.loginViaB2B')) {
                        try {

                            //TODO:!!!!
                            //TODO:!!!!
                            $this->logger->warn('TODO!!! Un utente ha cercato di effettuare il login tramite b2b ma non è ancora configurato');
                            $this->response['status'] = 'NO_AUTH_B2B';
                            $this->response['message'] = 'Todo';
                            return pushEvent(new BonjourFailedEvent($this->response));

//                            parse_str($_SERVER['QUERY_STRING'], $queryString);
//                            $queryString['user'] = strtoupper($queryString['user']);
//                            $queryString = http_build_query($queryString);
//
//                            //effettuo il login sul B2B
//                            $response = $this->bonjourB2b($queryString); //TODO:!!!!
//
//                            if($response['status'] != 'OK') {
//                                throw new \Exception();
//                            } else {
//
//                                //recupero quei dati che voglio mantenere nella successiva risposta
//                                //ricevuti dal B2B
//                                $token = $response['token'];
//                                $subUser = isset($response['data']['subUser']) ? $response['data']['subUser'] : "";
//
//                                //se non è presente un codice agente nel B2B recupero il codice presente in PS
//                                $userCode = ($response['data']['userCode'] != '') ? $response['data']['userCode'] : $psDB->table('icm_employees')->select('code')->where('id_employee', $id_employee)->first()->code;
//
//                                $userRole = $response['data']['userRole'];
//                                $subUsersList = isset($response['data']['subUsersList']) ? $response['data']['subUsersList'] : array();
//                                $actions = $response['data']['actions'];
//                            }

                        }
                        catch(\Exception $e) {
                            $this->response['status'] = 'NO_AUTH_B2B';
                            $this->response['message'] = $response['data']['error'] ?? "";
                            return pushEvent(new BonjourFailedEvent($this->response));
                        }
                    }
                    else {

                        $auth = $psDB->table(_DB_PREFIX_.'employee')->join('icm_employees','icm_employees.id_employee','=',_DB_PREFIX_.'employee.id_employee')
                            ->select([
                                _DB_PREFIX_.'employee.id_employee',
                                _DB_PREFIX_.'employee.id_profile',
                                _DB_PREFIX_.'employee.lastname',
                                _DB_PREFIX_.'employee.firstname',
                                _DB_PREFIX_.'employee.email',
                                _DB_PREFIX_.'employee.active',
                                'icm_employees.code',
                                'icm_employees.status',
                            ])
                            ->where('email', $user)
                            ->where('passwd',$password)
                            ->where('active', 1)->first();

                        if ($auth) {

                            $dateString = date('Y-m-d H:i:s');
                            $data_ora = explode(' ', $dateString);
                            $data = explode('-', $data_ora[0]);
                            $ora = explode(':', $data_ora[1]);
                            $ts = gmmktime((int)$ora[0], (int)$ora[1], (int)$ora[2], (int)$data[1], (int)$data[2], (int)$data[0]);

                            $token = md5($ts . $code);
                        }

                    }

                    // se è l'utente utilizzato per l'apple business manager aggiungo la stringa ABM al codice
//                    if (strtolower($user) == strtolower(_ICM_USERNAME_OF_DEMO_USER_)) $userCode = \Icm_employee::getAbmCodeByUserCode($userCode);

                    $_SESSION['icm_devices'][$token]['code'] = $code;
                    $_SESSION['icm_devices'][$token]['expiration_date'] = mktime(date("H")+2,date("i")+0,date("s")+0);
                    $_SESSION['icm_devices'][$token]['user'] = $user;
                    $_SESSION['icm_devices'][$token]['subUser'] = $subUser ?? '';
                    $_SESSION['icm_devices'][$token]['userCode'] = $userCode ?? '';
                    $_SESSION['icm_devices'][$token]['userRole'] = $userRole ?? '';
                    $_SESSION['icm_devices'][$token]['subUsersList'] = $subUsersList ?? '';
                    $_SESSION['icm_devices'][$token]['actions'] = $actions ?? '';
                    $_SESSION['icm_devices'][$token]['device'] = !is_null($id_app_info) ? $id_app_info : '1';    //TODO: da rimuovere, l'app deve sempre inviare l'id

                    //scrivo nel log che ha effettuato l'accesso l'$user con il dispositivo $code
                    $this->logger->info("L'utente con codice ". $userCode . " - " . $psUser->firstname . " " . $psUser->lastname . " ha effettuato l'accesso con il dispositivo " . $system_name . " e versione ".request('app_version', ''));
                } else {

                    //scrivo nel log che l'utente ha interrato il servizio
                    $this->logger->info("L'utente con codice ". $userCode . " - " . $psUser->firstname . " " . $psUser->lastname . " ha interrogato i servizi con il dispositivo " . $system_name . " e versione ".request('app_version', ''));
                }

                //aggiorno la data dell'ultimo accesso del dipendente con il dispositivo
                $psDB->table('icm_devices_employees')->where('id_device',$deviceEmployee->id_device)->where('id_employee',$id_employee)->update(['last_access' => date('Y-m-d H:i:s')]);

                //aggiorno le infomazioni del dispositivo salvandole nel db
                if (!empty($app_version))
                    $device->app_version = $app_version;
                if (!empty($system_name))
                    $device->system_name = $system_name;
                if (!empty($system_version))
                    $device->system_version = $system_version;
                if (!empty($model))
                    $device->model = $model;
                if (!empty($id_app_info))
                    $device->id_app_info = $id_app_info;

                //se il timestamp ricevuto dall'applicazione è vuoto setto a 0 il lastdb_timestamp nel db mysql
                //altrimenti setto il timestamp ricevuto
                if (empty($lastdb_version))
                    $device->lastdb_timestamp = null;
                else $device->lastdb_timestamp = date("Y-m-d H:i:s",(empty($lastdb_version ?? '') ? time() : $lastdb_version) - date("Z"));

                //aggiorno il device
                $psDB->table('icm_devices')->where('id_device',$deviceEmployee->id_device)->update([
                    'code' => $device->code,
                    'system_name' => $device->system_name,
                    'system_version' => $device->system_version,
                    'model' => $device->model,
                    'app_version' => $device->app_version,
                    'lastdb_timestamp' => $device->lastdb_timestamp,
                    'status' => $device->status,
                    'id_app_info' => $device->id_app_info
                   ]);
                
                
                //                $psDB->table('icm_devices_employees')->where('id_device',$deviceEmployee->id_device)->where('id_employee',$id_employee)->update(['status' => 'OK']);

//                // Aggiungo il codice cliente fatturazione
//                $de = new DataExtractor();
//                $res = $de->executeS("SELECT CSPECA as codice_cliente_fatturazione FROM TRD80SMI.wCGANA00F WHERE CONTCA = '".$userCode."'")[0];
//
//                $codiceClienteFatturazione = $res->codice_cliente_fatturazione == null ? "" : $res->codice_cliente_fatturazione;
//                $loggedUser['clientCode'] = $codiceClienteFatturazione;
//                $loggedUser['clientType'] = $codiceClienteFatturazione != "" ? 'D' : 'C';


                //aggiorno in sessione l'id profilo (che potrebbe variare)
                $_SESSION['icm_devices'][$token]['id_profile'] = $id_profile;

                //recupero i permessi utente definiti in PS e li accodo a quelli presenti nell'array actions
                $actions = array();
//                $actions = array_merge($actions, \Icm_actions::getActionsOfEmployee($id_employee));

                //se l'utente non ha il profilo di agente inserisco un parametro il action che indica all'app di mostrare
                //solo la parte catalogo
//        if ((_ICM_DATAEXTRACTOR_ACTIVE_) && $id_profile != _IDE_EMPLOYEE_DEFAULT_ID_PROFILE_) {
//            $actions[] = _IDE_ACTIONS_SHOW_ONLY_CATALOG_;
//        }

                //salvo in sessione i permessi utente
                $_SESSION['icm_devices'][$token]['actions'] = $actions;

                // se la password è scaduta
//                if (!\Icm_employee::isPasswordValid($id_employee)) {
//
//                    //invio lo stato specifico all'app
//                    $this->response['status'] = "PWD_EXPIRED";
//                    $this->response['message'] = "Password scaduta";
//                    $this->response['token'] = $token;
//                    return pushEvent(new BonjourFailedEvent($this->response));
//                } else {

                    //recupero tutti i dati da inviare all'app
                    $loggedUser['userCode'] = $userCode;
                    $loggedUser['firstname'] = $psUser->firstname;
                    $loggedUser['lastname'] = $psUser->lastname;
                    $loggedUser['username'] = $psUser->username;
                    $loggedUser['email'] = $psUser->email;
                    $appVersion = $psDB->table('icm_app_history')->select('version')->where('current','1')->where('id_app_info', $id_app_info)->where('disabled','0')->first();
                    $loggedUser['appVersion'] = $appVersion ? $appVersion->version : '-';
//                }

                try {
                    $res = $psDB->insert('INSERT INTO icm_stats_bonjour (id_employee, id_device, date_access) VALUES (?,?,?)', [$id_employee, $device->id_device, date('Y-m-d H:i:s')]);
                } catch (Exception $ignored) {}

                //log in datadog
//                \IcmStatsd::getInstance()->load('icm.bonjour', 1, 1.0, [
//                    'status' => $response['status'],
//                    'user' => $_SESSION['icm_devices'][$token]['user'],
//                    'user_code' => $response['data']['userCode'],
//                    'system_name' => $system_name,
//                    'system_version' => $system_version,
//                    'model' => $model,
//                    'app_version' => $app_version,
//                    'lastdb_version' => $lastdb_version,
//                    'subUser' => $subUser,
//                    'id_app_info' => $id_app_info,
//                    'shop_name' => 'ioc-no-shop-name',
//                ]);


                $payload = ['data' => $loggedUser, 'token' => $token, 'status' => 'OK'];
                $payload = pushEvent(new PSBonjourPayload($payload));

//                $loggedUser = pushEvent(new BonjourEvent($loggedUser));

                /** Scrivo finalmente il json di risposta */
//                $this->response['token'] = $token;
//                $this->response['status'] = "OK";
//                $this->response['data'] = array_merge($this->response['data'],$loggedUser);

                return $payload;

            }
        } catch (\Exception $e) { // | \Throwable php > 7.0
            if (isset($this->logger)) {
                $this->logger->error('========================');
                $this->logger->error('Bonjour Fallito, $_REQUEST:');
                $this->logger->error(json_encode($_REQUEST));
                $this->logger->error('### Exception:');
                $this->logger->error($e->getMessage());
                $this->logger->error('========================');
            }
            throw($e);
        } finally {
            session_commit();
        }

        return pushEvent(new BonjourFailedEvent($this->response));
    }

}
