<?php

/**
 * Copyright (c) 2020/2
 * Created by Federico Papetti
 */


namespace FourWords\AppManager\Ioc\Ps\Traits;


use Exception;
use FourWords\AppManager\Director\DatabaseManager;
use FourWords\AppManager\Director\FWUtils;

/**
 * Trait PSConnection
 * Utilizzando questo trait si ha accesso ai metodi per il caricamento dei file di Prestashop, dei moduli icm e la connessione al db ps (Illuminate\Database\Connection)
 */

trait PSConnection
{
    /**
     * Includo Prestashop per poter utilizzare le sue classi
     */
    public function loadPrestashop() {
        $oldErrorReporting = error_reporting();
        error_reporting(E_ERROR);
        require_once(config('ioc.ps.dir') . 'config/config.inc.php');
        error_reporting($oldErrorReporting);
    }

    /**
     * Includo solamente la configurazione di Prestashop
     */
    public function loadPrestashopSettings() {

        if (!file_exists(FWUtils::addTrailingSlashes(config('ioc.ps.dir')) . 'config/settings.inc.php')) {
            throw new \Exception("Impossibile raggiungere Prestashop: require settings.inc.php");
        }
        require_once(config('ioc.ps.dir') . 'config/settings.inc.php');
    }

    /**
     * Includo il modulo icmordersfromas per poter utilizzare le sue classi
     * @throws Exception
     */
    public function loadICMOrdersFromASModule() {
        $this->loadPrestashop();

        if (!\ModuleCore::isInstalled('icmordersfromas')) {
            throw new Exception("Modulo icmordersfromas non installato in prestashop");
        }

        \ModuleCore::getInstanceByName('icmordersfromas');

    }

    /**
     * Aggiungo la connessione al database di PS in capsule
     * @throws Exception
     */
    public function loadPSConnection() {

        $this->loadPrestashopSettings();

        try {
            DatabaseManager::getConnection('ps');
        } catch (Exception $e) {

            DatabaseManager::addConnection([
                'driver' => 'mysql',
                'host' => _DB_SERVER_,
                'port' => '3306',
                'database' => _DB_NAME_,
                'username' => _DB_USER_,
                'password' => _DB_PASSWD_,
                'unix_socket' => '',
                'charset' => 'utf8mb4',
                'collation' => 'utf8mb4_unicode_ci',
                'prefix' => '',
                'prefix_indexes' => true,
                'strict' => true
            ],'ps');

        }

        return DatabaseManager::getConnection('ps');

    }

}
