<?php

namespace FourWords\AppManager\Ioc\Resource;

use Exception;
use FourWords\AppManager\Director\FWUtils;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;

class ResourceRepository {

    protected $typesAndPaths;

    public function __construct($typesAndPaths = []) {
        $this->typesAndPaths = $typesAndPaths;
    }

    public function addType(string $type, array $paths) {

        if (Arr::has($this->typesAndPaths, $type)) {
            $this->typesAndPaths[$type] = array_merge($this->typesAndPaths[$type], $paths);
        } else {
            $this->typesAndPaths[$type] = $paths;
        }
    }

    public function addPath(string $path, array $types) {
        foreach ($types as $type) {
            $this->addType($type, [$path]);
        }
    }

    public function downloadFile($name, $type, $params = null) {
        $path = $this->getFile($name, $type, $params);
        if(is_null($path)) {
            abort(404);
        }

        return response()->download($path);
    }

    protected function searchFileInPaths($filename, $paths) {
        foreach ($paths as $path) {
            if (File::exists($path)) {
                $filePath = FWUtils::addTrailingSlashes($path).$filename;
                if (File::exists($filePath)) return $filePath;
            }
        }
        return null;
    }

    public function getFile($name, $type, $params = null) {

        try {

            if (empty($this->typesAndPaths)) throw new Exception(get_class() . " :: La lista di path in cui cercare le risorse è vuota.");

            $type = Str::startsWith($type, '.') ? Str::substr($type, 1) : $type;

            $pathsToSearch = [];

            if (array_key_exists($type, $this->typesAndPaths)) {
                $paths = $this->typesAndPaths[$type];
                foreach ($paths as $path) {
                    $pathsToSearch[] = Str::startsWith($path, "/") ? $path : base_path($path);
                }
            }

            if (array_key_exists('*', $this->typesAndPaths)) {
                $paths = $this->typesAndPaths['*'];
                foreach ($paths as $path) {
                    $pathsToSearch[] = Str::startsWith($path, "/") ? $path : base_path($path);
                }
            }

            if ($params) {
                foreach ($params as $param => $value) {
                    foreach ($pathsToSearch as &$path) {
                        $path = Str::replaceFirst($param, $value, $path);
                    }
                }
            }

        } catch (Exception $e) {
            Log::error($e);
            return null;
        }

        if (empty($pathsToSearch)) return null;

        if ($type == 'db') {

            // Divido il path in path + codice cliente e path + warehouse
            $oldPaths = $pathsToSearch;
            $pathsToSearch = [];

            $pathsNumbers = count($oldPaths);

            for ($i = 0; $i < $pathsNumbers; $i++) {
                $pathsToSearch[] = $oldPaths[$i] . '/' . auth()->user()->code;
                $pathsToSearch[] = $oldPaths[$i] . '/warehouse';
            }
        } else if ($type == 'doc' || $type == 'vid') {

            //TODO: Truffe di prestashop? Tenere se installato ps.
            // Rimuovo l'estensione dal nome del file
            $name = pathinfo($name)['filename'];

        //TODO: Hotfix per vecchia versione iOS, rimuovere
        } else if ($type == 'img' && str_contains($name,'@2x') && !env('SARENI_POINTING_TO_PRESTASHOP')) {
            $name = str_replace('@2x', '', $name);
        }

        return $this->searchFileInPaths($name,array_unique($pathsToSearch));
    }

}