<?php

namespace FourWords\AppManager\Ioc\Traits;

use Exception;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\File;

trait OverridesViews
{

    /**
     * Sostituisco view in views-nomepackage alle view del rispettivo package
     *
     * es. $viewsDir-appstore/nomeview.blade.php sostituirà appstore::nomeview
     * es. $viewsDir-orders/test/nomeview.blade.php sostituirà orders::test.nomeview
     * @param $viewsDir
     * @return void
     */
    public function overrideCustomViews($viewsDir)
    {
        $providerName = basename(str_replace('\\', '/', trim(get_class($this), '\\')));

        $customViews = Cache::remember($providerName.'_custom_views', app()->environment() == 'production' ? 86400 : 60, function () use ($viewsDir) {

            $cv = [];

            foreach (director()->modules as $packageName) {

                $customViewDir = $viewsDir . '-' . $packageName;
                if (File::isDirectory($customViewDir)) {
                    $cv[$packageName] = $customViewDir;
                }
            }

            return $cv;
        });

        foreach ($customViews as $packageName => $dir) {
            $this->loadViewsFrom($dir, $packageName);
        }

    }

}