<?php

namespace Mediconesystems\LivewireDatatables;

use Illuminate\Support\Carbon;

class DatetimeColumn extends Column
{
    public $type = 'datetime';
    public $callback;

    public function __construct()
    {
        $this->format();
    }

    public function format($format = null, $showDiff = false)
    {
        $this->callback = function ($value) use ($format, $showDiff) {

            $ts = !is_numeric($value) ? Carbon::parse($value) : Carbon::createFromTimestamp($value);
            $dateTime = $value ? $ts->format($format ?? config('livewire-datatables.default_datetime_format')) : null;

            if ($dateTime && $showDiff) {
                $dateTime = $dateTime . ' ('.$ts->diffForHumans(Carbon::now(), true).' fa)';
            }
            return $dateTime;
        };

        return $this;
    }

    public function humanDiff($absolute = true)
    {
        $this->callback = function ($value) use ($absolute) {
            $ts = Carbon::createFromTimestamp($value);
            return $value ? $ts->diffForHumans(Carbon::now(), $absolute) . ($absolute ? ' fa' : '') . " alle ".$ts->format('H:i') : null;
        };

        return $this;
    }

}
