<?php

namespace FourWords\AppManager\Orders\Livewire\Table;

use FourWords\AppManager\Ioc\Livewire\Table\IocTable;
use FourWords\AppManager\Orders\Model\Order;
use Mediconesystems\LivewireDatatables\Column;
use Mediconesystems\LivewireDatatables\DatetimeColumn;
use Mediconesystems\LivewireDatatables\NumberColumn;

class OrdersTable extends IocTable
{

    public function builder()
    {
        return app()->make(Order::class)->query()->leftJoin('ioc_users', 'ioc_users.username', 'orders.username');
    }

    public function columns()
    {
        return [

            DatetimeColumn::name('created_at')
                ->label('Data primo invio')
                ->sortable()
                ->format(null, true),

            Column::name('ioc_users.name')
                ->label('Name')
                ->filterable()
                ->searchable(),

            Column::name('username')
                ->label('Username')
                ->filterable()
                ->searchable(),

            Column::name('code')
                ->label('Codice gestionale')
                ->filterable()
                ->searchable(),

            Column::name('agent')
                ->label('Agente')
                ->filterable(),

//            DatetimeColumn::name('creation_date')
//                ->label('Data creazione (App)')
//                ->sortable()
//                ->format(null, true),

            DatetimeColumn::name('response_date')
                ->label('Data responso')
                ->hide()
                ->format(null, true),

            Column::name('attempts')
                ->hide()
                ->label('Tentativi'),

            Column::name('response_error')
                ->label('Errore'),

            Column::name('quantity_total')
                ->sortable()
                ->label('Quantità'),

            Column::name('colli')
                ->sortable()
                ->label('Colli'),

            NumberColumn::callback(['total'], function ($total) {
                return currency($total)."€";
            })
                ->label('Totale')
                ->filterable(),

            Column::name('reference')
                ->label('Riferimento')
                ->filterable()
                ->hide()
                ->searchable(),

            Column::callback(['status'], function ($status) {

                if ($status === Order::$OK) {
                    return '<svg class="h-5 w-5 stroke-current text-green-600 mx-auto" fill="none" viewBox="0 0 24 24" stroke="currentColor">
    <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M9 12l2 2 4-4m6 2a9 9 0 11-18 0 9 9 0 0118 0z"></path>
</svg>';
                } else if ($status === Order::$ERROR) {
                    return '<svg class="h-5 w-5 stroke-current text-red-300 mx-auto" fill="none" viewBox="0 0 24 24" stroke="currentColor">
    <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M10 14l2-2m0 0l2-2m-2 2l-2-2m2 2l2 2m7-2a9 9 0 11-18 0 9 9 0 0118 0z"></path>
</svg>';
                } else {
                    return '<div class="text-center"><i class="fa-light fa-clock fa-lg"></i></div>';
                }
            })->label('Stato')
                ->exportCallback(function ($value) {
                    return $value;
                })
                ->filterable(['WAIT', 'OK', 'ERROR']),

            Column::callback('id', function ($id) {
                return view('orders::icon-view', ['id' => $id]);
            })->label('Azioni')
                ->excludeFromExport()

        ];
    }
}
