<?php

namespace FourWords\AppManager\Orders\Providers;

use FourWords\AppManager\Orders\B2B;
use FourWords\AppManager\Orders\Interfaces\OrderRowsRepositoryInterface;
use FourWords\AppManager\Orders\Interfaces\OrderServiceInterface;
use FourWords\AppManager\Orders\Interfaces\OrderRepositoryInterface;
use FourWords\AppManager\Orders\Livewire\OrderRowsView;
use FourWords\AppManager\Orders\Livewire\Table\OrderRowsTable;
use FourWords\AppManager\Orders\Livewire\Table\OrdersTable;
use FourWords\AppManager\Orders\Model\OrderCounter;
use FourWords\AppManager\Orders\Model\Order;
use FourWords\AppManager\Orders\Repository\OrderRepository;
use FourWords\AppManager\Orders\Repository\OrderRowsRepository;
use FourWords\AppManager\Orders\Services\B2BOrderService;
use Illuminate\Support\ServiceProvider;
use Livewire\Livewire;

class OrdersServiceProvider extends ServiceProvider
{

    public function register() {
        $this->app->bind(Order::class, Order::class);
        $this->app->bind(OrderCounter::class, OrderCounter::class);
        $this->app->bind(OrderRepositoryInterface::class, OrderRepository::class);
        $this->app->bind(OrderRowsRepositoryInterface::class, OrderRowsRepository::class);
        $this->app->bind(OrderServiceInterface::class, B2BOrderService::class);
        $this->app->bind(B2B::class, B2B::class);
    }

    public function boot()
    {
        config(require __DIR__ . '/../../config/orders.php');
        $this->loadRoutesFrom(__DIR__ . '/../../routes/routes.php');
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'orders');

        $this->registerLivewireComponents();
    }

    protected function registerLivewireComponents()
    {
        Livewire::component('orders-table', OrdersTable::class);
        Livewire::component('order-rows-table', OrderRowsTable::class);
        Livewire::component('orders-rows-view', OrderRowsView::class);
    }

}