<?php

namespace FourWords\AppManager\Orders\Repository;

use Carbon\Carbon;
use FourWords\AppManager\Orders\Interfaces\OrderRepositoryInterface;
use FourWords\AppManager\Orders\Model\Order;
use Illuminate\Support\Collection;

class OrderRepository implements OrderRepositoryInterface
{
    public function all(): Collection
    {
        return app()->make(Order::class)::all();
    }

    public function create(array $attributes): ?Order
    {
        return app()->make(Order::class)::create(array_merge($attributes, ['created_at' => Carbon::now()->toDateTimeString()]));
    }

    public function insert(array $attributes): bool
    {
        return app()->make(Order::class)::insert(array_merge($attributes, ['created_at' => Carbon::now()->toDateTimeString()]));
    }

    public function delete($creationDate, $username): bool
    {
        $order = $this->find($creationDate, $username);
        return $order->delete();
    }

    public function find($creationDate, $username): ?Order
    {
        return app()->make(Order::class)::where('creation_date', $creationDate)->where('username',$username)->first();
    }

    public function update($creationDate, string $username, array $attributes): bool
    {
        $order = $this->find($creationDate, $username);
        return $order->update($attributes);
    }
}
