<?php

namespace FourWords\AppManager\Orders\Repository;

use FourWords\AppManager\Orders\Interfaces\OrderRowsRepositoryInterface;
use FourWords\AppManager\Orders\Model\Order;
use FourWords\AppManager\Orders\Model\OrderRow;
use Illuminate\Support\Collection;

class OrderRowsRepository implements OrderRowsRepositoryInterface
{

    public function all(): Collection
    {
        return app()->make(Order::class)::all();
    }

    public function allOf($orderId): Collection
    {
        $order = app()->make(Order::class)->find($orderId);
        return $order->rows ?? collect();
    }

    public function insertRows($orderId, array $attributes): bool
    {
        /** @var Order $order */
        $order = app()->make(Order::class)->find($orderId);
        return $order && $order->rows()->insert($attributes);
    }

    public function insertRowsFromApp($orderId, array $attributes): bool
    {
        /** @var Order $order */
        $order = app()->make(Order::class)->find($orderId);
        if (!$order) return false;

        foreach ($attributes as $orderRow) {
            $orderRow = OrderRow::fromApp($orderRow, $order->id, $order->created_at);
            $orderRow->save();
        }

        return true;
    }

    public function deleteRows($orderId): bool
    {
        /** @var Order $order */
        $order = app()->make(Order::class)->find($orderId);
        return $order && $order->rows()->delete();
    }
}
