<?php

namespace FourWords\AppManager\Orders\Services;

use FourWords\AppManager\Ioc\Bonjour\B2BBonjour;
use FourWords\AppManager\Orders\Model\AppModels\OrderResponse;
use Illuminate\Support\Facades\Http;

class B2BOrderService extends AbstractOrderService
{
    protected $token;
    protected $b2bBonjour;

    public function __construct(B2BBonjour $b2BBonjour)
    {
        parent::__construct();
        $this->b2bBonjour = $b2BBonjour;
    }

    /**
     * @throws \Exception
     */
    public function storeRemotely(object $order): ?OrderResponse
    {

        $response = $this->b2bBonjour->authenticateRemotely();

        if ($response['status'] != 'OK') {
            throw new \Exception($response['data']['error'] ?? '');
        }

        $this->token = $response['token'];

        $fields = [
            "action" => request('action') ?? '2',
            "ordine" => json_encode($order),
            "token" => $response['token']
        ];

        $loadOrderResponse = Http::withOptions([
            'verify' => false,
            'timeout' => 300
        ])->post(config('orders.b2b.url').'loadOrder'.'?'. http_build_query($fields) );

        $result = json_decode(utf8_encode($loadOrderResponse->body()), true);

        if ($loadOrderResponse->status() != 200) {
            throw new \Exception("Errore invio ordine B2B, HTTP Status: ".$loadOrderResponse->status());
        }

        if ($result['status'] != 'OK') {
            throw new \Exception("Errore invio ordine B2B, Body: ".$loadOrderResponse->body());
        }

        return new OrderResponse($result['status'], 'Ordine inviato correttamente', $order->cliente, $result['data']['numero']);
    }
}