<?php

namespace FourWords\AppManager\Orders\Services;

use FourWords\AppManager\DataExtractor\DataExtractor;
use FourWords\AppManager\Orders\Model\AppModels\OrderResponse;
use FourWords\AppManager\Orders\Model\OrderCounter;
use Illuminate\Support\Facades\DB;

class JGalileoOrderService extends AbstractOrderService
{

    protected $orderNumberReference;
    protected $currentDate;
    const CUSTOMER_CODE = 'cliente';
    const CUSTOMER_VAT = 'partitaIva';
    const CUSTOMER_CF = 'codiceFiscale';
    const PROSPECT_CODE = 'prospect_code';
    const NEWCUSTOMER = 'nuovoCliente';
    const NEWCUSTOMER_YES = 'S';
    const NEWCUSTOMER_NO = 'N';

    const ORDER_DATA = 'data_add';
    const ORDER_NUMBER = 'numeroOrdine';
    const ORDER_COMMENT = 'commento';
    const ORDER_COMMENT_ROW_NUMBER = 'commento_num_riga';
    const ORDER_COMMENT_TEXT = 'testo_commento';
    const ORDER_NORECALC = 'noRecalc';
    const ORDER_AGENT = 'agente';
    const ORDER_DOCUMENTTYPE = 'tipoDocumento';
    const ORDER_RECEIVER = 'destinatario';
    const ORDER_DIFFERENTDESTINATIONCODE = 'codiceDiversaDestinazione';
    const ORDER_PAYMENTTYPE = 'tipoPagamento';
    const ORDER_PAYMENTDISCOUNT = 'scontoPagamento';
    const ORDER_DELIVERYDATE = 'dataConsegna';
    const ORDER_CREATIONDATE = 'dataCreazioneOrdine';
    const ORDER_DB_CREATIONDATE = 'dataCreazioneDbOrdine';
    const ORDER_USERNAME = 'username';
    const ORDER_NOPARK = 'noPark';

    const ORDER_NOPARK_S = 'S';
    const ORDER_NOPARK_N = 'N';

    const ROW_NUMBER = 'numeroRiga';
    const ROW_PRODUCTCODE = 'codiceProd';
    const ROW_QUANTITY = 'qta';
    const ROW_PACKAGES = 'colli';
    const ROW_PRICE_L = 'prezzoLordo';
    const ROW_PRICE_N = 'prezzoNetto';
    const ROW_MOVEMENTTYPE = 'tipoMovimento';
    const ROW_DISCOUNT1 = 'sconto1';
    const ROW_DISCOUNT2 = 'sconto2';
    const ROW_DISCOUNT3 = 'sconto3';
    const ROW_DISCOUNT4 = 'sconto4';
    const ROW_DISCOUNT5 = 'sconto5';
    const ROW_DISCOUNT6 = 'sconto6';
    const ROW_DISCOUNT7 = 'sconto7';
    const ROW_DISCOUNT8 = 'sconto8';

    const ROW_MOVEMENTTYPE_SELL = 'VE';
    const ROW_MOVEMENTTYPE_FREE = 'OM';

    protected static $orderHead = [
        self::ORDER_DATA => ['name' => 'DT01OO', 'size' => 8],
        self::ORDER_DOCUMENTTYPE => ['name' => 'TDOCOO', 'size' => 1],
        self::ORDER_NUMBER => ['name' => 'NROROO', 'size' => 7],
        self::ROW_NUMBER => ['name' => 'NRRGOO', 'size' => 3],
        self::CUSTOMER_CODE => ['name' => 'CDCFOO', 'size' => 10],
        self::ORDER_PAYMENTTYPE => ['name' => 'CDPAOO', 'size' => 3],
        self::ORDER_PAYMENTDISCOUNT => ['name' => 'SCPAOO', 'size' => 7],
        self::ORDER_AGENT => ['name' => 'CDAGOO', 'size' => 3],
        self::ORDER_DELIVERYDATE => ['name' => 'DTCNOO', 'size' => 8],
        self::PROSPECT_CODE => ['name' => 'NWCLOO', 'size' => 7],
    ];

    protected static $orderRow = [
        self::ORDER_DATA => ['name' => 'DT01OO', 'size' => 8],
        self::ORDER_DOCUMENTTYPE => ['name' => 'TDOCOO', 'size' => 1],
        self::ORDER_NUMBER => ['name' => 'NROROO', 'size' => 7],
        self::ROW_NUMBER => ['name' => 'NRRGOO', 'size' => 3],
        self::ROW_PRODUCTCODE => ['name' => 'CDAROO', 'size' => 18],
        self::ROW_MOVEMENTTYPE => ['name' => 'TIMOOO', 'size' => 2],
        self::ROW_PRICE_L => ['name' => 'PRZUOO', 'size' => 26],
        self::ROW_PRICE_N => ['name' => 'PZNEOO', 'size' => 26],
        self::ROW_DISCOUNT1 => ['name' => 'SCN1OO', 'size' => 7],
        self::ROW_DISCOUNT2 => ['name' => 'SCN2OO', 'size' => 7],
        self::ROW_DISCOUNT3 => ['name' => 'SCN3OO', 'size' => 7],
        self::ROW_DISCOUNT4 => ['name' => 'SCN4OO', 'size' => 7],
        self::ROW_DISCOUNT8 => ['name' => 'AUPROO', 'size' => 26],
        self::ROW_PACKAGES => ['name' => 'NCOLOO', 'size' => 5],
        self::ROW_QUANTITY => ['name' => 'QTOROO', 'size' => 17],
    ];

    protected static $orderComment = [
        self::ORDER_NUMBER => 'NROROC',
        self::ROW_NUMBER => 'NRRGOC',
        self::ORDER_COMMENT_ROW_NUMBER => 'RGCOOC',
        self::ORDER_COMMENT_TEXT => 'TCOMOC',
    ];

    protected $orderCounterModel;
    protected $dataExtractor;

    public function __construct(OrderCounter $orderCounterModel, DataExtractor $dataExtractor)
    {
        parent::__construct();
        $this->orderCounterModel = $orderCounterModel;
        $this->dataExtractor = $dataExtractor;
    }

    /**
     * @throws \Exception
     */
    public function storeRemotely(object $order): ?OrderResponse
    {

        $this->orderNumberReference = $this->getNewOrderNumber();
        $this->currentDate = date('Ymd');

        $this->addOrderHeader();
        $this->addOrderRows();
        $this->addOrderComments();

        return new OrderResponse("OK", 'Ordine inviato correttamente', $this->order->cliente, $this->orderNumberReference);

    }

    protected function getNewOrderNumber() {

        DB::beginTransaction();
        $this->orderCounterModel::first()->increment('number');
        $counterNumber = $this->orderCounterModel::first()->number;
        DB::commit();

        return (string) $counterNumber;
    }

    protected function sanitizeForAS($string, $size = null, $isNumeric = false) {

        //se la stringa è nulla assegno striga vuota
        if(is_null($string))
            $string = '';

        if ($isNumeric && !is_numeric($string))
            $string = '0';

        //decodifico la stringa
        $string = urldecode($string);

        //ripristino la & 
        $string = str_replace(array('%26'), array('&'), $string);

        //urldecode e codifico il utf8
        $string = utf8_encode($string);

        //converto le lettere accentate in lettera apice
        $string = $this->replaceAccent($string);

        //se è stata impostata una dimensione massima e la stringa è più lunga
        if(!is_null($size) && strlen($string) > $size) {

            //riduco la stringa alla dimensione massima disponibile per il campo
            $string = substr($string, 0, $size);
        }

        return $string;
    }

    protected function replaceAccent($string, $replacement = "'")
    {
        $a = array('À', 'Á', 'Â', 'Ã', 'Ä', 'Å', 'Æ', 'Ç', 'È', 'É', 'Ê', 'Ë', 'Ì', 'Í', 'Î', 'Ï', 'Ð', 'Ñ', 'Ò', 'Ó', 'Ô', 'Õ', 'Ö', 'Ø', 'Ù', 'Ú', 'Û', 'Ü', 'Ý', 'ß', 'à', 'á', 'â', 'ã', 'ä', 'å', 'æ', 'ç', 'è', 'é', 'ê', 'ë', 'ì', 'í', 'î', 'ï', 'ñ', 'ò', 'ó', 'ô', 'õ', 'ö', 'ø', 'ù', 'ú', 'û', 'ü', 'ý', 'ÿ', 'Ā', 'ā', 'Ă', 'ă', 'Ą', 'ą', 'Ć', 'ć', 'Ĉ', 'ĉ', 'Ċ', 'ċ', 'Č', 'č', 'Ď', 'ď', 'Đ', 'đ', 'Ē', 'ē', 'Ĕ', 'ĕ', 'Ė', 'ė', 'Ę', 'ę', 'Ě', 'ě', 'Ĝ', 'ĝ', 'Ğ', 'ğ', 'Ġ', 'ġ', 'Ģ', 'ģ', 'Ĥ', 'ĥ', 'Ħ', 'ħ', 'Ĩ', 'ĩ', 'Ī', 'ī', 'Ĭ', 'ĭ', 'Į', 'į', 'İ', 'ı', 'Ĳ', 'ĳ', 'Ĵ', 'ĵ', 'Ķ', 'ķ', 'Ĺ', 'ĺ', 'Ļ', 'ļ', 'Ľ', 'ľ', 'Ŀ', 'ŀ', 'Ł', 'ł', 'Ń', 'ń', 'Ņ', 'ņ', 'Ň', 'ň', 'ŉ', 'Ō', 'ō', 'Ŏ', 'ŏ', 'Ő', 'ő', 'Œ', 'œ', 'Ŕ', 'ŕ', 'Ŗ', 'ŗ', 'Ř', 'ř', 'Ś', 'ś', 'Ŝ', 'ŝ', 'Ş', 'ş', 'Š', 'š', 'Ţ', 'ţ', 'Ť', 'ť', 'Ŧ', 'ŧ', 'Ũ', 'ũ', 'Ū', 'ū', 'Ŭ', 'ŭ', 'Ů', 'ů', 'Ű', 'ű', 'Ų', 'ų', 'Ŵ', 'ŵ', 'Ŷ', 'ŷ', 'Ÿ', 'Ź', 'ź', 'Ż', 'ż', 'Ž', 'ž', 'ſ', 'ƒ', 'Ơ', 'ơ', 'Ư', 'ư', 'Ǎ', 'ǎ', 'Ǐ', 'ǐ', 'Ǒ', 'ǒ', 'Ǔ', 'ǔ', 'Ǖ', 'ǖ', 'Ǘ', 'ǘ', 'Ǚ', 'ǚ', 'Ǜ', 'ǜ', 'Ǻ', 'ǻ', 'Ǽ', 'ǽ', 'Ǿ', 'ǿ');
        $b = array("A'", "A'", "A", "A", "A", "A", "AE", "C", "E'", "E'", "E", "E", "I'", "I'", "I", "I", "D", "N", "O'", "O'", "O", "O", "O", "O", "U'", "U'", "U", "U", "Y", "ß", "a'", "a'", "a", "a", "a", "a", "ae", "c", "e'", "e'", "e", "e", "i'", "i'", "i", "i", "n", "o'", "o'", "o", "o", "o", "o", "u'", "u'", "u", "u", "y", "y", "A", "a", "A", "a", "A", "a", "c", "c", "C", "c", "C", "c", "C", "c", "D", "d", "D", "d", "E", "e", "E", "e", "E", "e", "E", "e", "E", "e", "G", "g", "G", "g", "G", "g", "G", "g", "H", "h", "H", "n", "I", "i", "I", "i", "I", "i", "I", "i", "I", "i", "IJ", "ij", "J", "j", "K", "k", "L", "l", "L", "l", "L", "l", "L", "l", "l", "l", "N", "n", "N", "n", "N", "n", "n", "O", "o", "O", "o", "O", "o", "CE", "ce", "R", "i", "R", "r", "R", "r", "S", "s", "S", "s", "S", "s", "S", "s", "T", "t", "T", "t", "T", "t", "U", "u", "U", "u", "U", "u", "U", "u", "U", "u", "U", "u", "W", "w", "Y", "y", "Y", "Z", "z", "Z", "z", "Z", "z", "i", "f", "O", "o", "U", "u", "A", "a", "I", "i", "O", "o", "U", "u", "U", "u", "U", "u", "U", "ǚ", "U", "u", "A", "a", "AE", "ae", "O", "o");
        return str_replace($a, $b, $string);
    }

    protected function doubleQuote($string) {
        return str_replace(array("'"), array("''"), $string);
    }

    /**
     * @return void
     * @throws \Exception
     */
    protected function addOrderHeader() {

        $order = (array) $this->order;

        //preparo la riga di testata da inserire
        $head = array(
            self::$orderHead[self::ORDER_NUMBER]['name'] => $this->orderNumberReference,
            self::$orderHead[self::ORDER_DATA]['name'] => $this->currentDate,
            self::$orderHead[self::ROW_NUMBER]['name'] => 0,
            self::$orderHead[self::CUSTOMER_CODE]['name'] => ($order[self::NEWCUSTOMER] == self::NEWCUSTOMER_NO) ? $this->sanitizeForAS($order[self::CUSTOMER_CODE]) : 0,
            self::$orderHead[self::ORDER_DOCUMENTTYPE]['name'] => $this->sanitizeForAS($order[self::ORDER_DOCUMENTTYPE], self::$orderHead[self::ORDER_DOCUMENTTYPE]['size']),
            self::$orderHead[self::ORDER_PAYMENTTYPE]['name'] => $this->sanitizeForAS($order[self::ORDER_PAYMENTTYPE], self::$orderHead[self::ORDER_PAYMENTTYPE]['size']),
            self::$orderHead[self::ORDER_PAYMENTDISCOUNT]['name'] => $this->sanitizeForAS($order[self::ORDER_PAYMENTDISCOUNT], self::$orderHead[self::ORDER_PAYMENTDISCOUNT]['size'], true),
            self::$orderHead[self::ORDER_AGENT]['name'] => $this->sanitizeForAS($order[self::ORDER_AGENT], self::$orderHead[self::ORDER_AGENT]['size']),
            self::$orderHead[self::PROSPECT_CODE]['name'] => ($order[self::NEWCUSTOMER] == self::NEWCUSTOMER_YES) ? $this->sanitizeForAS($order[self::PROSPECT_CODE]) : 0,
            self::$orderHead[self::ORDER_DELIVERYDATE]['name'] => $this->sanitizeForAS($order[self::ORDER_DELIVERYDATE], self::$orderHead[self::ORDER_DELIVERYDATE]['size'], true),
        );

        $query = "INSERT INTO ".config('orders.as.library').config('orders.as.headTable'). "(".$this->getColumns($head).") VALUES (".$this->getValues($head).")";

        $res = $this->dataExtractor->execute($query);
        if ($res == 0) throw new \Exception('Testata non inserita: '.json_encode($head, JSON_PRETTY_PRINT));
    }

    /**
     * @throws \Exception
     */
    protected function addOrderRows() {


        foreach ($this->order->rows as $row) {

            $row = (array) $row;

            try {

                $rowValue = [];

                //preparo la riga
                $rowValue = [
                    self::$orderRow[self::ORDER_DATA]['name'] => $this->currentDate,
                    self::$orderRow[self::ORDER_DOCUMENTTYPE]['name'] => $this->sanitizeForAS($this->order->tipoDocumento, self::$orderRow[self::ORDER_DOCUMENTTYPE]['size']),
                    self::$orderRow[self::ORDER_NUMBER]['name'] => $this->orderNumberReference,
                    self::$orderRow[self::ROW_NUMBER]['name'] => $this->sanitizeForAS($row[self::ROW_NUMBER], self::$orderRow[self::ROW_NUMBER]['size'], true),
                    self::$orderRow[self::ROW_PRODUCTCODE]['name'] => $this->sanitizeForAS($row[self::ROW_PRODUCTCODE], self::$orderRow[self::ROW_PRODUCTCODE]['size']),

                    self::$orderRow[self::ROW_MOVEMENTTYPE]['name'] => $this->sanitizeForAS($row[self::ROW_MOVEMENTTYPE], self::$orderRow[self::ROW_MOVEMENTTYPE]['size']),
                    self::$orderRow[self::ROW_PRICE_L]['name'] => $this->sanitizeForAS($row[self::ROW_PRICE_L], self::$orderRow[self::ROW_PRICE_L]['size']),
                    self::$orderRow[self::ROW_PRICE_N]['name'] => $this->sanitizeForAS($row[self::ROW_PRICE_N], self::$orderRow[self::ROW_PRICE_N]['size']),
                    self::$orderRow[self::ROW_DISCOUNT1]['name'] => $this->sanitizeForAS($row[self::ROW_DISCOUNT1], self::$orderRow[self::ROW_DISCOUNT1]['size'], true),
                    self::$orderRow[self::ROW_DISCOUNT2]['name'] => $this->sanitizeForAS($row[self::ROW_DISCOUNT2], self::$orderRow[self::ROW_DISCOUNT2]['size'], true),
                    self::$orderRow[self::ROW_DISCOUNT3]['name'] => $this->sanitizeForAS($row[self::ROW_DISCOUNT3], self::$orderRow[self::ROW_DISCOUNT3]['size'], true),
                    self::$orderRow[self::ROW_DISCOUNT4]['name'] => $this->sanitizeForAS($row[self::ROW_DISCOUNT4], self::$orderRow[self::ROW_DISCOUNT4]['size'], true),
                    self::$orderRow[self::ROW_DISCOUNT8]['name'] => $this->sanitizeForAS($row[self::ROW_DISCOUNT8], self::$orderRow[self::ROW_DISCOUNT8]['size'], true),
                    self::$orderRow[self::ROW_QUANTITY]['name'] => $this->sanitizeForAS($row[self::ROW_QUANTITY], self::$orderRow[self::ROW_QUANTITY]['size']),
                    self::$orderRow[self::ROW_PACKAGES]['name'] => $this->sanitizeForAS($row[self::ROW_PACKAGES], self::$orderRow[self::ROW_PACKAGES]['size']),
                ];

                $query = "INSERT INTO " . config('orders.as.library').config('orders.as.rowsTable') . "(".$this->getColumns($rowValue).") VALUES (".$this->getValues($rowValue).")";

                $res = $this->dataExtractor->execute($query);
                if ($res == 0) throw new \Exception('Riga non inserita: ' . json_encode($row, JSON_PRETTY_PRINT));

            } catch (\Exception | \Throwable $e) {

                try {
                    $this->deleteHead();
                    $this->deleteRows();
                } catch (\Exception $e2) {
                    throw new \Exception("Errore invio ordine + cancellazione dati: $e\n\n$e2\n\n".json_encode($rowValue, JSON_PRETTY_PRINT));
                }

                throw $e;

            }

        }


    }

    protected function getColumns($map) {
        return implode(', ', array_keys($map));
    }

    protected function getValues($map) {
        $values = '';
        foreach ($map as $value)
            $values .= "'".$this->doubleQuote($value)."', ";

        return rtrim($values, ', ');
    }

    protected function addOrderComments() {

        $comment = $this->sanitizeForAS(self::$orderHead[self::ORDER_COMMENT]);

        //Se la stringa è vuota ritorno
        if(empty($comment)) return;

        //Divido il commento in più righe, massimo 40 caratteri
        $comments = str_split($comment, 40);

        $rows = array();

        //per ogni riga di commento
        foreach ($comments as $n => $commentText) {

            //preparo la riga
            $rows[] = array(
                self::$orderComment[self::ORDER_NUMBER] => $this->orderNumberReference,
                self::$orderComment[self::ROW_NUMBER] => 0,
                self::$orderComment[self::ORDER_COMMENT_ROW_NUMBER] => $n + 1,
                self::$orderComment[self::ORDER_COMMENT_TEXT] => $commentText,
            );
        }

        $query = "INSERT INTO " . config('orders.as.library').config('orders.as.commentTable') . "(".$this->getColumns($rows).") VALUES (".$this->getValues($rows).")";

        //inserisco le righe del commento dell'ordine
        try {

            $res = $this->dataExtractor->execute($query);
            if ($res < count($rows)) throw new \Exception('Commenti non inseriti: ' . json_encode($rows, JSON_PRETTY_PRINT));

        } catch (\Exception | \Throwable $e) {

            try {
                $this->deleteHead();
                $this->deleteRows();
            } catch (\Exception $e2) {
                throw new \Exception("Errore invio ordine + cancellazione dati: $e\n\n$e2\n\n".json_encode($rows, JSON_PRETTY_PRINT));
            }

            throw $e;

        }
    }

    protected function deleteHead() {
        $this->dataExtractor->executeS("DELETE FROM ".config('orders.as.library').config('orders.as.headTable').
            " WHERE ".self::$orderHead[self::ORDER_NUMBER]['name']." = ".$this->orderNumberReference);
    }

    protected function deleteRows() {
        $this->dataExtractor->executeS("DELETE FROM ".config('orders.as.library').config('orders.as.rowsTable').
            " WHERE ".self::$orderRow[self::ORDER_NUMBER]['name']." = ".$this->orderNumberReference);
    }

    protected function deleteComments() {
        $this->dataExtractor->executeS("DELETE FROM ".config('orders.as.library').config('orders.as.commentTable').
            " WHERE ".self::$orderComment[self::ORDER_NUMBER]['name']." = ".$this->orderNumberReference);
    }

}