<?php

namespace FourWords\AppManager\PushNotifications\Controllers;

use Carbon\Carbon;
use FourWords\AppManager\Director\FWUtils;
use FourWords\AppManager\Ioc\Controllers\IocController;
use FourWords\AppManager\PushNotifications\Model\PushNotification;
use FourWords\AppManager\PushNotifications\Model\PushNotificationSentUser;

class PushNotificationController extends IocController
{
    public function index() {
        $this->checkPermission('view-push-notifications');
        return view('push::index');
    }

    public function create() {
        $this->checkPermission('create-push-notifications');
        return view('push::create');
    }

    public function edit($id) {
        $this->checkPermission('edit-push-notifications');

        $notification = PushNotification::find($id);
        if(!$notification) {
            abort(404);
        }

        return view('push::edit', ['notification' => $notification]);
    }

    public function show($id) {
        $this->checkPermission('view-push-notifications');

        $notification = PushNotification::find($id);
        if(!$notification) {
            abort(404);
        }

        return view('push::show', ['pushNotification' => $notification]);
    }

    public function groups() {
        $this->checkPermission('view-push-notifications-groups');
        return view('push::groups.index');
    }

//    public function segments() {
//        $this->checkPermission('view-push-notifications-segments');
//        return view('push::segments.index');
//    }

    //API
    public function getUserNotifications() {
        $user = auth()->user();
        if (!$user) return response()->json([],400);

        $pushNotificationSentToUser = PushNotificationSentUser::where('user', '=', $user->id)->with('notification')->get();

        $pushNotificationsResult = [];
        foreach ($pushNotificationSentToUser as $obj) {

            $pushNotificationData = [];

            $pushNotificationData['id'] = $obj['notification']['id'];
            $pushNotificationData['link'] = $obj['notification']['url'];
            $pushNotificationData['product'] = $obj['notification']['product'];
            $pushNotificationData['title'] = $obj['notification']['title'];
            $pushNotificationData['message'] = $obj['notification']['message'];
            $pushNotificationData['read'] = $obj['read'];
            $pushNotificationData['opened'] = $obj['opened'];
            $pushNotificationData['sent_at'] = (new Carbon($obj['created_at']))->getTimestamp();
            if (!empty($obj['notification']['image'])) {
                $pushNotificationData['image_url'] = PushNotification::imageAsUrl($obj['notification']['image']);
            }

            $pushNotificationsResult[] = FWUtils::removeNullsAndEmptyString($pushNotificationData);
        }

        return response()->json(['notifications' => $pushNotificationsResult]);
    }
}
