<?php

namespace FourWords\AppManager\PushNotifications\Database;

use FourWords\AppManager\PushNotifications\Model\PushNotificationSegment;
use FourWords\AppManager\PushNotifications\Model\Segments\PushNotificationSubsegment;
use FourWords\AppManager\PushNotifications\Model\Segments\PushNotificationSubsegmentFilter;
use FourWords\AppManager\PushNotifications\Model\Segments\PushNotificationSubsegmentOperator;
use Illuminate\Database\Seeder;

class PushSegmentsSeeder extends Seeder
{
    /**
     * Seed the application's database.
     *
     * @return void
     */
    public function run()
    {
        $this->createSubsegmentsFilters();
        $this->createSubsegmentsOperators();

        $subsegmentOrders = PushNotificationSubsegment::create([
            'name' => 'Ordini',
            'query' => 'SELECT u.id
                FROM ioc_users u
                LEFT JOIN orders o ON u.username = o.username
                WHERE u.active = true AND {$1}
                GROUP BY u.id, u.username
                HAVING {$2}',
        ]);

        $subsegmentOrders->filters()->attach(PushNotificationSubsegmentFilter::find(1), [
            'column' => 'COUNT(o.username)',
            'placeholder' => '2',
        ]);
        $subsegmentOrders->filters()->attach(PushNotificationSubsegmentFilter::find(3), [
            'column' => 'o.created_at',
            'placeholder' => '1',
        ]);

        $subsegmentCatalog = PushNotificationSubsegment::create([
            'name' => 'Classifica utenti',
            'query' => 'SELECT u.id
                FROM ioc_users u
                {$-2}
                WHERE u.active = true AND 
                {$-1}',
            'list' => 'SELECT DISTINCT value AS value, value AS description, feature_id FROM catalog_customer_has_feature',
            'catalog_customer_has_feature' => 'SELECT id AS value, name AS description FROM catalog_customer_features'
        ]);

        $subsegmentCatalog->filters()->attach(PushNotificationSubsegmentFilter::find(4), [
            'filter_id' => 5,
            'column' => 'cchf.value',
            'placeholder' => -1,
        ]);

        //Utenti con ordini
        $segmentUsersWithOrders = PushNotificationSegment::create([
            'name' => 'Utenti con ordini',
        ]);
        $segmentUsersWithOrders->subsegment()->attach($subsegmentOrders, [
            'operator_id' => PushNotificationSubsegmentOperator::where('filter_id', '=', 1)->where('operator', '=', '>')->first()->id,
            'value' => 0,
            'position' => 0
        ]);
        $segmentUsersWithOrders->subsegment()->attach($subsegmentOrders, [
            'operator_id' => PushNotificationSubsegmentOperator::where('filter_id', '=', 3)->where('operator', '=', '>')->first()->id,
            'value' => NULL,
            'position' => 0
        ]);

        //Utenti senza ordini
        $segmentUsersWithoutOrders = PushNotificationSegment::create([
            'name' => 'Utenti senza ordini',
        ]);
        $segmentUsersWithoutOrders->subsegment()->attach($subsegmentOrders, [
            'operator_id' => PushNotificationSubsegmentOperator::where('filter_id', '=', 1)->where('operator', '=', '=')->first()->id,
            'value' => 0,
            'position' => 0
        ]);
        $segmentUsersWithoutOrders->subsegment()->attach($subsegmentOrders, [
            'operator_id' => PushNotificationSubsegmentOperator::where('filter_id', '=', 3)->where('operator', '=', '>')->first()->id,
            'value' => NULL,
            'position' => 0
        ]);
    }

    private function createSubsegmentsFilters() {
        $data = [
            ['name' => 'number'],
            ['name' => 'boolean'],
            ['name' => 'date'],
            ['name' => 'list'],
            ['name' => 'catalog_customer_has_feature'],
        ];

        foreach ($data as $item) {
            PushNotificationSubsegmentFilter::create([
                'name' => $item['name'],
            ]);
        }
    }

    private function createSubsegmentsOperators() {
        $data = [
            ['filter_id' => 1, 'operator' => '>', 'operator_name' => 'maggiore di'],
            ['filter_id' => 1, 'operator' => '>=', 'operator_name' => 'maggiore o uguale di'],
            ['filter_id' => 1, 'operator' => '<', 'operator_name' => 'minore di'],
            ['filter_id' => 1, 'operator' => '<=', 'operator_name' => 'minore o uguale di'],
            ['filter_id' => 1, 'operator' => '=', 'operator_name' => 'uguale a'],
            ['filter_id' => 2, 'operator' => '=', 'operator_name' => 'è'],
            ['filter_id' => 3, 'operator' => '>', 'operator_name' => 'dopo il'],
            ['filter_id' => 3, 'operator' => '>=', 'operator_name' => 'dal'],
            ['filter_id' => 3, 'operator' => '<', 'operator_name' => 'prima del'],
            ['filter_id' => 3, 'operator' => '<=', 'operator_name' => 'prima e compreso il'],
            ['filter_id' => 3, 'operator' => '=', 'operator_name' => 'il'],
            ['filter_id' => 1, 'operator' => '<>', 'operator_name' => 'diverso da'],
            ['filter_id' => 3, 'operator' => '<>', 'operator_name' => 'tutti tranne il'],
            ['filter_id' => 4, 'operator' => '=', 'operator_name' => 'è'],
            ['filter_id' => 4, 'operator' => '<>', 'operator_name' => 'non è'],
            ['filter_id' => 2, 'operator' => '<>', 'operator_name' => 'non è'],
            ['filter_id' => 5, 'operator' => '=', 'operator_name' => 'è'],
        ];

        foreach ($data as $item) {
            PushNotificationSubsegmentOperator::create([
                'filter_id' => $item['filter_id'],
                'operator' => $item['operator'],
                'operator_name' => $item['operator_name'],
            ]);
        }
    }
}
