<?php

namespace FourWords\AppManager\PushNotifications\Jobs;

use Carbon\Carbon;
use FourWords\AppManager\Appstore\Models\AppstoreType;
use FourWords\AppManager\Ioc\Jobs\IocJob;
use FourWords\AppManager\Ioc\Models\Device;
use FourWords\AppManager\Ioc\Models\IocUser;
use FourWords\AppManager\PushNotifications\Model\PushNotification;
use FourWords\AppManager\PushNotifications\Model\PushNotificationSent;
use FourWords\AppManager\PushNotifications\Model\PushNotificationSentUser;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Support\Facades\DB;

class SendPushNotifications extends IocJob
{

    public $queue = 'notifications';
    protected $googleSent = 0;
    protected $googleFailed = 0;

    protected $appleSent = 0;
    protected $appleFailed = 0;

    private $userIds, $devices;

    public function handle()
    {
        // Imposto questo flag a false prima dell'handle della superclasse per evitare di riempire log se non ci sono invii da effettuare
        $this->shouldLog = false;
        parent::handle();
    }

    public function getDevices($pushNotification) {

        /** @var Collection $userIds */
        $userIds = collect();

        $devicesWithPushActivated = Device::query()
            ->where('push_device_token', '!=', '')
            ->where('push_status', '>=','2');

        /** @var Collection $groups */
        $groups = $pushNotification->groups;
        /** @var Collection $segments */
        $segments = $pushNotification->segments;

        if ($groups->isEmpty() && $segments->isEmpty()) {
            $userIds = IocUser::where('active', '=', true)->pluck('id');
            $this->logger->debug('Invio a tutti: '. count($userIds->first()).' utenti');
        } else {

            if ($groups->count() > 0) {
                $this->logger->debug('Gruppi:');
                foreach ($groups as $group) {
                    // Prendo tutti gli user id
                    $users = $group->users()->pluck('user_id')->all();
                    $userIds = $userIds->merge($users);
                    $this->logger->debug($group->name.': '.count($users).' utenti');
                }
            }

            if ($segments->count() > 0) {
                $this->logger->debug('Segmenti:');
                foreach ($segments as $segment) {
                    //Nell'array inserisco l'id del segmento come chiave
                    $subsegmentsQuery[$segment->id] = DB::table('push_notifications_segment_has_subsegments AS ss')
                        ->select('subs.query', 'ss_has_f.placeholder', 'ss_has_f.filter_id', 'ss_has_f.column', 'so.operator', 'ss.value', 'ss.position')
                        ->join('push_notifications_subsegments AS subs', 'ss.subsegment_id', '=', 'subs.id')
                        ->join('push_notifications_subsegments_operators AS so', 'ss.operator_id', '=', 'so.id')
                        ->join('push_notifications_subsegments_filters AS ssf', 'so.filter_id', '=', 'ssf.id')
                        ->join('push_notifications_subsegments_has_filters AS ss_has_f', function ($join) {
                            $join->on('ss.subsegment_id', '=', 'ss_has_f.subsegment_id')
                                ->on('ssf.id', '=', 'ss_has_f.filter_id');
                        })
                        ->where('ss.segment_id', $segment->id)
                        ->get()->toArray();
                }

                $usersFromSegment = [];
                foreach ($segments as $segment) {
                    //prendo il primo risultato della colonna "query" dei sottosegmenti
                    //(tutti i risultati di un sottosegmento hanno la colonna "query" uguale)
                    $query = $subsegmentsQuery[$segment->id][0]->query;
                    $position = $subsegmentsQuery[$segment->id][0]->position;
                    //in base ai risultati della query iniziale per estrare le query dei sottosegmenti
                    //ottiene il numero di filtri per sottosegmento.
                    //I valori ottenuti saranno (Colonna, operatore, valore e posizione di placeholder)
                    $firstUsersFromSegment = true;
                    //SOLO PER I CATALOG USERS
                    $feature_id = [];
                    foreach ($subsegmentsQuery[$segment->id] as $filter) {
                        if($filter->position != $position) {
                            if($query != null) {
                                $query = str_replace('OR {$-1}', '', $query);
                                $query = str_replace('AND {$-1}', '', $query);
                                $query = str_replace('{$-2}', '', $query);
                                $query = preg_replace('/,\{\$-3\.\w\}/', '', $query);

                                $results = DB::select($query);
                                $userIdFromSubsegmentsQuery = array_map(function($result) {
                                    return $result->id;
                                }, $results);

                                if($firstUsersFromSegment) {
                                    $usersFromSegment = $userIdFromSubsegmentsQuery;
                                    $firstUsersFromSegment = false;
                                } else {
                                    $usersFromSegment = array_intersect($userIdFromSubsegmentsQuery, $usersFromSegment);
                                }
                            }

                            $query = $filter->query;
                            $position = $filter->position;
                        }
                        //Assemblaggio della clausola:
                        if($filter->value == null) {
                            //se il campo di valore di un filtro è stato lasciato vuoto allora nel placeholder della query
                            //si metterà 1=1 per non variare il risultato della query
                            $placeholderValue = '1=1';
                        } else {
                            $placeholderValue = $filter->column.' '.$filter->operator.' '.$filter->value;
                        }

                        //Una volta che la clausula è stata assemblata andrà a sositutire il placeholder della query del sottosegmento
                        //ES:
                        //Clausola: orders.date > 2022-02-12
                        //Query: SELECT * FROM orders WHERE {$1} ---> SELECT * FROM orders WHERE orders.date > 2022-02-12
                        if($filter->placeholder != -1) {
                            $query = str_replace('{$'.$filter->placeholder.'}', $placeholderValue, $query);
                        } else {
                            if($placeholderValue != '1=1') {
                                //Catalog USERS
                                if($filter->filter_id == 5) {
                                    $checkFeatureId = DB::table('catalog_customer_has_feature')->where('value', '=', str_replace("'", "", $filter->value))->first()->feature_id;
                                    if(!in_array($checkFeatureId, $feature_id)) {
                                        $feature_id[] = $checkFeatureId;
                                        $query = str_replace('{$-2}', 'JOIN catalog_customer_has_feature cchf'.$checkFeatureId.' ON cchf'.$checkFeatureId.'.customer_id = u.id {$-2}', $query);
                                        $query = str_replace('{$-1}', ' (cchf'.$checkFeatureId.'.feature_id = '.$checkFeatureId.' AND cchf'.$checkFeatureId.'.value IN ({$-3.'.$checkFeatureId.'})) AND {$-1}', $query);
                                    }

                                    $query = str_replace('{$-3.'.$checkFeatureId.'}', $filter->value . ',{$-3.'.$checkFeatureId.'}', $query);
                                } else {
                                    $query = str_replace('{$' . $filter->placeholder . '}', $placeholderValue . ' OR {$-1}', $query);
                                }
                            }
                        }
                    }

                    //Una volta ottenuto tutte le query dei sottosegmenti le eseguo e salvo gli user ID in un array
                    //da utilizzare nella stima degli utenti / dispositivi
                    $query = str_replace('OR {$-1}', '', $query);
                    $query = str_replace('AND {$-1}', '', $query);
                    $query = str_replace('{$-2}', '', $query);
                    $query = preg_replace('/,\{\$-3\.\w\}/', '', $query);

                    $results = DB::select($query);
                    $userIdFromSubsegmentsQuery = array_map(function($result) {
                        return $result->id;
                    }, $results);

                    //dopo essermi salvato il risultato delle query in user ID di ogni segmento selezionato per la notifica unisco i risultati
                    //per ricavare la stima degli utenti / dispositivi
                    if($firstUsersFromSegment) {
                        $usersFromSegment = $userIdFromSubsegmentsQuery;
                    } else {
                        $usersFromSegment = array_intersect($userIdFromSubsegmentsQuery, $usersFromSegment);
                    }

                    $userIds = $userIds->merge($usersFromSegment);
                    $this->logger->debug($segment->name.': '.count($usersFromSegment).' utenti');
                }
            }
        }

        $this->userIds = $userIds->unique()->toArray();

        $this->logger->debug('Utenti univoci: '. count($this->userIds));

        $os = [];
        if($pushNotification->ios_send) {
            $os[] = '1';
        }
        if($pushNotification->android_send) {
            $os[] = '2';
        }

        //Prendo tutti i devices da questi user id
        $this->devices = $devicesWithPushActivated->whereIn('app_id', $os)->
        whereHas('users', function ($query) use ($userIds) {
            $query->whereIn('user_id', $this->userIds);
        })->where('active', '=', true)->get()->unique('id');



        $this->logger->debug('Device univoci con push token e notifiche attivate: '. count($this->devices));

        return $this->devices;
    }

    public function handleJob()
    {
        $pushNotificationsToSend = PushNotification::getPendingNotifications();

        if (count($pushNotificationsToSend) > 0) {
            $this->logger->info('Inizio Task: ' . $this->task->name);
        }

        // Invio tutte le notifiche push che devono essere inviate
        foreach ($pushNotificationsToSend as $pushNotification) {

            // Segno i token a cui ho mandato una notifica per evitare invii multipli, doppio controllo per sicurezza
            $pushTokens = [];
            if (config('push-notifications.dryRun')) {
                $this->logger->warning('== !!DRY RUN!! ==: ');
            }
            $this->logger->info('Notifica push: '.$pushNotification->name);
            $this->logger->info('Invio Apple: '.($pushNotification->ios_send ? 'Si' : 'NO').' - '.'Invio Google: '.($pushNotification->android_send ? 'Si' : 'NO'));
            $this->logger->info('Data invio schedulata: '.$pushNotification->send_at);
            $this->logger->debug('Titolo: '.$pushNotification->title);
            $this->logger->debug('Messaggio: '.$pushNotification->message);

            $devices = $this->getDevices($pushNotification);

            // Lo imposto come "in invio"
            $pushNotification->sent = -1;
            $pushNotification->sent_at = Carbon::now();
            $pushNotification->save();

            $this->createPushNotificationSent($pushNotification);

            if (count($devices) > 0)
                $this->shouldLog = true;

            // Loop dei dispositivi collegati alla notifica (o dispositivi totali)
            foreach ($devices as $device) {
                /** @var Device $device */

                $hiddenToken = substr($device->push_device_token, 0, 6).'.....'.substr($device->push_device_token,-6);

                // Se il device è iOS
                if ($device->app_id == AppstoreType::$IOS && $pushNotification->ios_send) {

                    $hadErrors = false;
                    try {
                        $payload = $device->sendApple($pushNotification);
                        if (config('push-notifications.dryRun')) {
                            $this->logger->info('Invio fittizio a: ' . $device->name . ' - token: ' . $hiddenToken."\n".json_encode($payload, JSON_PRETTY_PRINT));
                        } else {
                            $this->logger->info('Invio apple riuscito: ' . $device->name . ' - token: ' . $hiddenToken);
                        }

                        $this->appleSent += 1;
                        $this->storeUserSent($device, $pushNotification->id);

                        $this->updatePushNotificationSentCounter($pushNotification->id);
                    } catch (\Exception $e) {
                        $hadErrors = true;
                        $this->logger->error('INVIO APPLE FALLITO: '.$device->name.' - token: '.$hiddenToken);
                        $this->logger->error($e->getMessage());
                        $this->appleFailed += 1;
                    } finally {
                        if ($hadErrors && config('push-notifications.storeIfFailed')) {
                            $this->storeUserSent($device, $pushNotification->id);
                        }
                    }

                // Se il device è Android
                } else if ($device->app_id == AppstoreType::$ANDROID && $pushNotification->android_send) {

                    $hadErrors = false;
                    try {
                        $payload = $device->sendGoogle($pushNotification);
                        if (config('push-notifications.dryRun')) {
                            $this->logger->info('Invio fittizio a: ' . $device->name . ' - token: ' . $hiddenToken."\n".json_encode($payload, JSON_PRETTY_PRINT));
                        } else {
                            $this->logger->info('Invio google riuscito: ' . $device->name . ' - token: ' . $hiddenToken);
                        }

                        $this->googleSent += 1;
                        $this->storeUserSent($device, $pushNotification->id);

                        $this->updatePushNotificationSentCounter($pushNotification->id);
                    } catch (\Exception $e) {
                        $hadErrors = true;
                        $this->logger->error('INVIO GOOGLE FALLITO: '.$device->name.' - token: '.$hiddenToken);
                        $this->logger->error($e->getMessage());
                        $this->googleFailed += 1;
                    } finally {
                        if ($hadErrors && config('push-notifications.storeIfFailed')) {
                            $this->storeUserSent($device, $pushNotification->id);
                        }
                    }

                } else {
                    $this->logger->error("Il device non è ne Andriod ne iOS");
                }

            }

            //tabella push_notification_sent
            $pushNotification->sent = 1;
            $pushNotification->save();

        }

        if (count($pushNotificationsToSend) > 0) {
            $this->logger->debug('Totale invii: '. ($this->appleSent+$this->googleSent).'/'.count($devices));
            if ($this->appleFailed > 0 || $this->googleFailed > 0) {
                $this->logger->error('INVII FALLITI: ' . ($this->appleFailed + $this->googleFailed).'/'.count($devices));
                $this->logger->error('FALLITI GOOGLE: ' . $this->googleFailed);
                $this->logger->error('FALLITI APPLE: ' . $this->appleFailed);
            }
        }

    }

    private function storeUserSent($device, $pushNotificationId) {
        $deviceUsersIds = $device->users->pluck('id');
        foreach($deviceUsersIds as $userId) {
            try {
                if (!PushNotificationSentUser::where('push_notification_id',$pushNotificationId)->where('user', $userId)->exists()) {
                    PushNotificationSentUser::create([
                        'push_notification_id' => $pushNotificationId,
                        'user' => $userId,
                    ]);
                }
            } catch(\Exception $e) {}
        }
    }

    private function createPushNotificationSent($pushNotification) {
        $pushNotificationSent = new PushNotificationSent();
        $pushNotificationSent->push_notification_id = $pushNotification->id;

        $groups_names = null;
        foreach ($pushNotification->groups as $group) {
            $groups_names[] = $group->name;
        }
        if($groups_names != null) {
            $pushNotificationSent->groups_names =  json_encode($groups_names);
        }

        $segments_names = null;
        foreach ($pushNotification->segments as $segment) {
            $segments_names[] = $segment->name;
        }
        if($segments_names != null) {
            $pushNotificationSent->segments_names = json_encode($segments_names);
        }

        $pushNotificationSent->estimated_users = count($this->userIds);
        $pushNotificationSent->estimated_devices = count($this->devices);

        $pushNotificationSent->sent_at = Carbon::now();
        $pushNotificationSent->save();
    }

    private function updatePushNotificationSentCounter($pushNotificationId) {
        try {
            $sent = PushNotificationSent::where('push_notification_id', '=', $pushNotificationId)->first();
            $sent->sent_to += 1;
            $sent->save();
        } catch(\Exception $e) {
            $this->logger->error('ERRORE NEL SALVATAGGIO DELL\'UTENTE NELLA TABELLA PUSH_NOTIFICATION_SENT_USERS');
        }
    }

//    static public function sendAppleBackgroundNotification($push_device_token, $notification_title, $notification_body, $data = [])
//    {
//
//        $key_file = config('push-notifications.apple.key_file');
//        $keyid = config('push-notifications.apple.keyid');
//        $teamid = config('push-notifications.apple.teamid');
//        $bundleid = config('push-notifications.apple.bundleid');
//        $url = config('push-notifications.apple.url');
//
//        $keyfile_path = storage_path("private/$key_file");
//        $key = openssl_pkey_get_private('file://' . $keyfile_path);
//
//        $header = ['alg' => 'ES256', 'kid' => $keyid];
//        $claims = ['iss' => $teamid, 'iat' => time()];
//
//        $header_encoded = self::base64($header);
//        $claims_encoded = self::base64($claims);
//
//        $signature = '';
//        openssl_sign($header_encoded . '.' . $claims_encoded, $signature, $key, 'sha256');
//        $jwt = $header_encoded . '.' . $claims_encoded . '.' . base64_encode($signature);
//
//        $headers = [
//            'apns-topic' => "{$bundleid}",
//            'apns-expiration' => 0,
//            'apns-priority' => 5,
//            'apns-push-type' => 'background'
//        ];
//
//        $completeUrl = "$url/3/device/$push_device_token";
//        $post = [
//            'aps' => [
//                'content-available' => 1
//            ],
//            'title' => $notification_title,
//            'body' => $notification_body,
//            'news' => $data['news_id']
//        ];
//
//        if(isset($data['image_url'])) {
//            $post['image_url'] = $data['image_url'];
//        }
//
//        $options = [
//            RequestOptions::VERSION => 2.0
//        ];
//
//        $response = Http::withToken($jwt)->withHeaders($headers)->withOptions($options)->post($completeUrl, $post);
//
//        if ($response->status() != 200) {
//            throw new Exception("Errore invio notifica push");
//        }
//
//        return $response->body();
//    }

}