<?php

namespace FourWords\AppManager\PushNotifications\Livewire;

use FourWords\AppManager\Ioc\Livewire\IocComponent;
use FourWords\AppManager\Ioc\Models\Device;
use FourWords\AppManager\Ioc\Models\IocUser;
use FourWords\AppManager\PushNotifications\Model\PushNotificationGroup;
use FourWords\AppManager\PushNotifications\Model\PushNotificationSegment;
use FourWords\AppManager\PushNotifications\Model\PushNotificationSent;
use FourWords\AppManager\PushNotifications\Model\Segments\PushNotificationSubsegment;
use FourWords\AppManager\PushNotifications\Model\Segments\PushNotificationSubsegmentFilter;
use FourWords\AppManager\PushNotifications\Model\Segments\PushNotificationSubsegmentOperator;
use Illuminate\Support\Facades\DB;
use Livewire\WithFileUploads;

class AudiencePushNotificationSection extends IocComponent {
    use WithFileUploads;

    public $notification, $pushNotificationSent, $readonly;
    public $showModal, $modalTitle;

    public
        $audience = 'push-notification-audience-all-users',

        $initAudienceSectionValue,
        $audienceSectionsValue,
        $audienceSectionValueSelected,

        $initSegmentValue,
        $segmentValue,
        $segmentValueSelected,

        $groupsSaved = [],
        $segmentsSaved = [],

        $editSegmentSelected,

        $segmentName,
        $filterOperator = [],
        $filterValue = [],
        $selects = [],
        $select_parent_position = [];

    public bool
        $androidSend = true,
        $iosSend = true;

    public int
        $dataPickersCount = 0,
        $estimateUsersNotificationsSent = 0,
        $estimateDevicesNotificationsSent = 0,
        $estimateDevicesNotificationsActivatedSent = 0,

        $datePickerId = 0;

    public $fields = [
        'segmentName'
    ];

    public $subsegments;
    public array
        $subsegmentsAddedList;


    protected $rules = [
        'audienceSectionValueSelected' => 'required',
    ];

    protected $listeners = ['setFilterDataPickerValue', 'saveSegment', 'removeSubsegment', 'addSelectCatalogCustomerFeatures', 'removeSelectCatalogCustomerFeatures'];

    public function mount() {
        $this->pushNotificationSent = PushNotificationSent::where('push_notification_id', '=', $this->notification->id)->first();
        $this->readonly = $this->pushNotificationSent != null;

        $this->getSpecificUserGroups();
        $this->getUsersSegment();

        $this->subsegments = PushNotificationSubsegment::all();

        if($this->notification) {
            $this->setAudience($this->notification->id);
            $this->setAudienceSegment($this->notification->id);
        }
    }

    public function render() {
        return view('push::livewire.audience-section');
    }

    public function updated($name, $value) {
        if($name == 'audience') {
            if($value == 'push-notification-audience-all-users') {
                $this->emitUp('setAudience', []);
            } else {
                $this->emitUp('setAudience', $this->groupsSaved);
                $this->emitUp('setAudienceSegment', $this->segmentsSaved);
            }
            $this->setEstimateNotificationsSent();
        }

        if($name == 'androidSend' || $name == 'iosSend') {
            $this->setEstimateNotificationsSent();
            $this->emit('setAudienceOS', $this->androidSend, $this->iosSend);
        }

        if ($name == 'showModal' && !$value) {
            $this->emit('resetDataPickerValue');
        }
    }

    public function chooseGroup() {
        if($this->audienceSectionsValue) {
            $this->showModal = true;
            $this->modalTitle = 'Aggiungi Gruppo';
            $this->audienceSectionValueSelected = array_keys($this->audienceSectionsValue)[0];
        }
    }

    public function getSpecificUserGroups() {
        $this->audienceSectionsValue = PushNotificationGroup::all()->pluck('name', 'id')->toArray();
        $this->initAudienceSectionValue = $this->audienceSectionsValue;
    }

    public function getUsersSegment() {
        $this->segmentValue = PushNotificationSegment::all()->pluck('name', 'id')->toArray();
        $this->initSegmentValue = $this->segmentValue;
    }

    public function setAudienceOSToSend() {
        if(!$this->notification) {
            return;
        }

        $this->androidSend = $this->notification->android_send;
        $this->iosSend = $this->notification->ios_send;
        $this->emit('setAudienceOS', $this->androidSend, $this->iosSend);
    }

    public function onSaveAudience() {
        if($this->audienceSectionsValue) {
            $this->groupsSaved[$this->audienceSectionValueSelected] = $this->audienceSectionsValue[$this->audienceSectionValueSelected];
            $this->audienceSectionsValue = array_diff($this->audienceSectionsValue, $this->groupsSaved);
            $this->showModal = false;

            $this->setEstimateNotificationsSent();

            $this->emitUp('setAudience', $this->groupsSaved);
        }
    }

    public function removeGroup($group) {
        unset($this->groupsSaved[$group]);
        $this->audienceSectionsValue[$group] = $this->initAudienceSectionValue[$group];

        $this->setEstimateNotificationsSent();
        $this->emitUp('setAudience', $this->groupsSaved);
    }

    public function setAudience($id) {
        $this->groupsSaved = DB::table('push_notifications_has_groups')
            ->join('push_notifications_groups', 'push_notifications_has_groups.group_id', '=', 'push_notifications_groups.id')
            ->where('push_notifications_has_groups.notification_id', $id)
            ->pluck('push_notifications_groups.name','push_notifications_groups.id')
            ->all();

        $this->audienceSectionsValue = array_diff($this->audienceSectionsValue, $this->groupsSaved);

        if($this->groupsSaved) {
            $this->audience = 'push-notification-audience-specific-users';
        }
    }

    public function setAudienceSegment($id) {
        $this->segmentsSaved = DB::table('push_notifications_has_segments')
            ->join('push_notifications_segments', 'push_notifications_has_segments.segment_id', '=', 'push_notifications_segments.id')
            ->where('push_notifications_has_segments.notification_id', $id)
            ->pluck('push_notifications_segments.name','push_notifications_segments.id')
            ->all();

        $this->segmentValue = array_diff($this->segmentValue, $this->segmentsSaved);

        if($this->segmentsSaved) {
            $this->audience = 'push-notification-audience-specific-users';
        }
    }

    public function sendAudienceData() {
        $this->emitUp('setAudience', $this->groupsSaved);
        $this->emitUp('setAudienceSegment', $this->segmentsSaved);

        $this->setAudienceOSToSend();
        $this->setEstimateNotificationsSent();
    }

    public function chooseSegment() {
        $this->subsegmentsAddedList = [];
        $this->showModal = true;
        $this->modalTitle = 'Aggiungi Segmento';

        if($this->segmentValue) {
            $this->segmentValueSelected = array_keys($this->segmentValue)[0];
        }
    }

    public function onSaveSegment() {
        if($this->segmentValue) {
            $this->segmentsSaved[$this->segmentValueSelected] = $this->segmentValue[$this->segmentValueSelected];
            $this->segmentValue = array_diff($this->segmentValue, $this->segmentsSaved);

            $this->setEstimateNotificationsSent();
            $this->showModal = false;

            $this->emitUp('setAudienceSegment', $this->segmentsSaved);
        }
    }

    public function removeSegment($segment) {
        unset($this->segmentsSaved[$segment]);
        $this->segmentValue[$segment] = $this->initSegmentValue[$segment];
        $this->sendAudienceData();
    }

    private function setEstimateNotificationsSent() {
        $os = [];
        if($this->iosSend) {
            $os[] = '1';
        }
        if($this->androidSend) {
            $os[] = '2';
        }

        if($this->audience != 'push-notification-audience-all-users') {
            $usersFromGroup = PushNotificationGroup::whereIn('id', array_keys($this->groupsSaved))->get()->pluck('users')->flatten()->unique('id');

            $usersFromSegment = $this->getEstimateUsersFromSegmentsQuery();
            $usersId = array_unique(array_merge($usersFromSegment, $usersFromGroup->pluck('id')->toArray()));
        } else {
            $usersId = IocUser::where('active', '=', 1)->get()->pluck('id')->toArray();
        }

        //tira fuori solo i dispositivi collegati, non gli stessi dispositivi con più users
        $devices = Device::whereIn('app_id', $os)
            ->whereHas('users', function ($query) use ($usersId) {
                $query->whereIn('user_id', $usersId);
            })->where('active', '=', true)->get()->unique('id');

        $this->estimateDevicesNotificationsSent = $devices->count();//DB::table('ioc_users_devices')->where('status', '=', 'OK')->get()->count();
        //tira fuori solo gli utenti con almeno un dispositivo collegato
        $this->estimateUsersNotificationsSent = count($usersId);
        $this->estimateDevicesNotificationsActivatedSent = $devices
                                                            ->where('push_device_token', '!=', '')
                                                            ->where('push_status', '>=', 2)->count();
    }

    public function selectSegmentValue($segmentId) {
        $this->segmentValueSelected = $segmentId;
        $this->onSaveSegment();
    }

    public function onCreateNewSegment() {
        $this->modalTitle = 'Crea Segmento';
        $this->resetFields();
    }

    public function addSubsegment($subsegment) {
        $this->subsegmentsAddedList[] = $subsegment;

        $filterValue = [];
        $filterOperator = [];

        //per ogni sottosegmento estraggo il numero di filtri [del tipo di filtro (filter ID)]
        //ES: filter_id = 1 avrà i seguenti operatori impostati dal DB (>, >=, < ecc ecc)
        foreach (PushNotificationSubsegment::find($subsegment['id'])->filters as $i => $filter) {

            //imposto i valori vuoti (o il primo risultato se il filtro è lista) e il primo operatore impostato dal DB
            if($filter->id == 4/*tipo di filtro per le liste*/) {
                $filterValue[] = array_key_first($this->getSubsegmentList($subsegment['list']));
                $filterOperator[] = $filter->operator->first()->id;
            } else if($filter->id == 5 ) {
                foreach($this->getSubsegmentList($subsegment['catalog_customer_has_feature']) as $groupListDescription) {
                    $filterValue[] = null;//DB::table('catalog_customer_has_feature')->where('feature_id', '=', $groupListValue)->first()->value;
                    $filterOperator[] = $filter->operator->first()->id;
                }
            } else {
                $filterValue[] = null;
                $filterOperator[] = $filter->operator->first()->id;
            }

            if($filter->id == 3 /*tipo di filtro per le date*/) {
                //creo il date picker JS
                $this->emit('setDatePickerSubsegment', $this->dataPickersCount, (count($this->subsegmentsAddedList)-1), $i);
                $this->dataPickersCount++;
            }
        }

        //Ogni sottosegmento avrà il la sua posizione nell'array e a sua volta avra la posizione dei suoi filtri
        //esempio $filterValue[0][1] indica il secondo filtro del primo sottosegmento inserito per la creazione del segmento
        $this->filterValue[] = $filterValue;
        $this->filterOperator[] = $filterOperator;
    }

    public function getSubsegmentFilters($subsegment) {
        return PushNotificationSubsegment::find($subsegment)->filters;
    }

    public function getSubsegmentList($subsegmentQuery) {
        $results = DB::select($subsegmentQuery);

        $keys = array_map(function($result) {
            return $result->value;
        }, $results);
        $values = array_map(function($result) {
            return $result->description;
        }, $results);

        return array_combine($keys, $values);
    }

    public function getSubsegmentList_CatalogCustomerHasFeature($subsegmentQuery, $feature_id) {
        $results = collect(DB::select($subsegmentQuery))->where('feature_id', '=', $feature_id)->values()->toArray();

        $keys = array_map(function($result) {
            return $result->value;
        }, $results);
        $values = array_map(function($result) {
            return $result->description;
        }, $results);

        return array_combine($keys, $values);
    }

    public function saveSegment($segmentId = null) {
        if(!$this->subsegmentsAddedList) {
            return;
        }
        $this->validate(['segmentName' => 'required']);

        try {
            //Se segment è null crea un nuovo segmento, altrimenti o modifica
            if($segmentId == null) {
                $segment = new PushNotificationSegment();
            }

            DB::transaction(function() use (&$segment) {
                $segment->name = $this->segmentName;
                $segment->save();

                foreach ($this->subsegmentsAddedList as $i => $subsegment) {
                    $isValueSet = false;
                    foreach ($this->filterValue[$i] as $j => $value) {

                        $valueWithoutQuotes = trim($value, '"');
                        if (!is_numeric($valueWithoutQuotes) && $valueWithoutQuotes != null) {
                            if (!(substr($value, 0, 1) === "'" && substr($value, -1) === "'")) {
                                $value = '\''.$value.'\'';
                            }
                        }

                        if($this->select_parent_position[$i][$j] != null && $this->filterOperator[$i][$j] == null) {
                            continue;
                        }

                        DB::table('push_notifications_segment_has_subsegments')->insert([
                            'segment_id' => $segment->id,
                            'subsegment_id' => $subsegment['id'],
                            'operator_id' => $this->filterOperator[$i][$j],
                            'value' => $value,
                            'position' => $i,
                            'select_parent_position' => $this->select_parent_position[$i][$j]
                        ]);

                        if($value != null && !$isValueSet) {
                            $isValueSet = true;
                        }
                    }

                    if(!$isValueSet) {
                        throw new \Exception('Almeno un valore del filtro deve essere impostato');
                    }
                }
            });
        } catch (\Exception $e) {
            $this->errorToast($e->getMessage());
            return;
        }

        $this->segmentValue[$segment->id] = $segment->name;
        $this->initSegmentValue[$segment->id] = $segment->name;
        $this->selectSegmentValue($segment->id);
        $this->resetFields();
        $this->successToast('Segmento creato con successo');
    }

    public function removeSubsegment($subsegmentAddedPosition, $datePickerCount) {
        unset($this->subsegmentsAddedList[$subsegmentAddedPosition]);
        unset($this->filterValue[$subsegmentAddedPosition]);
        unset($this->filterOperator[$subsegmentAddedPosition]);
        unset($this->selects[$subsegmentAddedPosition]);
        unset($this->select_parent_position[$subsegmentAddedPosition]);

        $this->subsegmentsAddedList = array_values($this->subsegmentsAddedList);
        $this->filterValue = array_values($this->filterValue);
        $this->filterOperator = array_values($this->filterOperator);

        $this->dataPickersCount = $datePickerCount;
    }

    public function resetFields() {
        $this->subsegmentsAddedList = [];
        $this->filterValue = [];
        $this->filterOperator = [];
        $this->dataPickersCount = 0;
        $this->selects = [];
        $this->select_parent_position = [];

        $this->resetErrorBag();

        foreach ($this->fields as $field) {

            if ($field == 'id') {
                //$this->modelId = null;
            } else if ($field == 'active') {
                $this->{$field} = false;
            } else {
                $this->{$field} = '';
            }
        }
        $this->emit('resetDataPickerValue');
    }

    public function getEstimateUsersFromSegmentsQuery() {
        //prendo i segmenti inseriti - l'id dell'segmento sono le dhiavi cell'array $segmentsSaved
        $segments = PushNotificationSegment::whereIn('id', array_keys($this->segmentsSaved))->get();
        $subsegmentsQuery = [];
        //Eseguo una query per estrarre la query da eseguire per prendere gli ID degli utenti e tutte le colonne,operatori,
        //valori,posizioni del placeholder che andranno a rimpiazzare tutti i placeholder della query impostata ai sottosegmenti (ES: {$1})
        //e la converto in un array.
        foreach ($segments as $segment) {
            //Nell'array inserisco l'id del segmento come chiave
            $subsegmentsQuery[$segment->id] = DB::table('push_notifications_segment_has_subsegments AS ss')
                ->select('subs.query', 'ss_has_f.placeholder', 'ss_has_f.filter_id', 'ss_has_f.column', 'so.operator', 'ss.value', 'ss.position')
                ->join('push_notifications_subsegments AS subs', 'ss.subsegment_id', '=', 'subs.id')
                ->join('push_notifications_subsegments_operators AS so', 'ss.operator_id', '=', 'so.id')
                ->join('push_notifications_subsegments_filters AS ssf', 'so.filter_id', '=', 'ssf.id')
                ->join('push_notifications_subsegments_has_filters AS ss_has_f', function ($join) {
                    $join->on('ss.subsegment_id', '=', 'ss_has_f.subsegment_id')
                        ->on('ssf.id', '=', 'ss_has_f.filter_id');
                })
                ->where('ss.segment_id', $segment->id)
                ->get()->toArray();
        }

        $usersFromSegment = [];
        foreach ($segments as $segment) {
            //prendo il primo risultato della colonna "query" dei sottosegmenti
            //(tutti i risultati di un sottosegmento hanno la colonna "query" uguale)
            $query = $subsegmentsQuery[$segment->id][0]->query;
            $position = $subsegmentsQuery[$segment->id][0]->position;
            //in base ai risultati della query iniziale per estrare le query dei sottosegmenti
            //ottiene il numero di filtri per sottosegmento.
            //I valori ottenuti saranno (Colonna, operatore, valore e posizione di placeholder)
            $firstUsersFromSegment = true;
            //SOLO PER I CATALOG USERS
            $feature_id = [];
            foreach ($subsegmentsQuery[$segment->id] as $filter) {
                if($filter->position != $position) {
                    if($query != null) {
                        $query = str_replace('OR {$-1}', '', $query);
                        $query = str_replace('AND {$-1}', '', $query);
                        $query = str_replace('{$-2}', '', $query);
                        $query = preg_replace('/,\{\$-3\.\w\}/', '', $query);

                        $results = DB::select($query);
                        $userIdFromSubsegmentsQuery = array_map(function($result) {
                            return $result->id;
                        }, $results);

                        if($firstUsersFromSegment) {
                            $usersFromSegment = $userIdFromSubsegmentsQuery;
                            $firstUsersFromSegment = false;
                        } else {
                            $usersFromSegment = array_intersect($userIdFromSubsegmentsQuery, $usersFromSegment);
                        }
                    }

                    $query = $filter->query;
                    $position = $filter->position;
                }
                //Assemblaggio della clausola:
                if($filter->value == null) {
                    //se il campo di valore di un filtro è stato lasciato vuoto allora nel placeholder della query
                    //si metterà 1=1 per non variare il risultato della query
                    $placeholderValue = '1=1';
                } else {
                    $placeholderValue = $filter->column.' '.$filter->operator.' '.$filter->value;
                }

                //Una volta che la clausula è stata assemblata andrà a sositutire il placeholder della query del sottosegmento
                //ES:
                //Clausola: orders.date > 2022-02-12
                //Query: SELECT * FROM orders WHERE {$1} ---> SELECT * FROM orders WHERE orders.date > 2022-02-12
                if($filter->placeholder != -1) {
                    $query = str_replace('{$'.$filter->placeholder.'}', $placeholderValue, $query);
                } else {
                    if($placeholderValue != '1=1') {
                        //Catalog USERS
                        if($filter->filter_id == 5) {
                            $checkFeatureId = DB::table('catalog_customer_has_feature')->where('value', '=', str_replace("'", "", $filter->value))->first()->feature_id;
                            if(!in_array($checkFeatureId, $feature_id)) {
                                $feature_id[] = $checkFeatureId;
                                $query = str_replace('{$-2}', 'JOIN catalog_customer_has_feature cchf'.$checkFeatureId.' ON cchf'.$checkFeatureId.'.customer_id = u.id {$-2}', $query);
                                $query = str_replace('{$-1}', ' (cchf'.$checkFeatureId.'.feature_id = '.$checkFeatureId.' AND cchf'.$checkFeatureId.'.value IN ({$-3.'.$checkFeatureId.'})) AND {$-1}', $query);
                            }

                            $query = str_replace('{$-3.'.$checkFeatureId.'}', $filter->value . ',{$-3.'.$checkFeatureId.'}', $query);
                        } else {
                            $query = str_replace('{$' . $filter->placeholder . '}', $placeholderValue . ' OR {$-1}', $query);
                        }
                    }
                }
            }

            //Una volta ottenuto tutte le query dei sottosegmenti le eseguo e salvo gli user ID in un array
            //da utilizzare nella stima degli utenti / dispositivi
            $query = str_replace('OR {$-1}', '', $query);
            $query = str_replace('AND {$-1}', '', $query);
            $query = str_replace('{$-2}', '', $query);
            $query = preg_replace('/,\{\$-3\.\w\}/', '', $query);

            $results = DB::select($query);
            $userIdFromSubsegmentsQuery = array_map(function($result) {
                return $result->id;
            }, $results);

            //dopo essermi salvato il risultato delle query in user ID di ogni segmento selezionato per la notifica unisco i risultati
            //per ricavare la stima degli utenti / dispositivi
            if($firstUsersFromSegment) {
                $usersFromSegment = $userIdFromSubsegmentsQuery;
            } else {
                $usersFromSegment = array_intersect($userIdFromSubsegmentsQuery, $usersFromSegment);
            }
        }

        return $usersFromSegment;
    }

    public function setFilterDataPickerValue($values, $update) {
        //emit in cui riceverà i valori inseriti da possibili datpicker creati
        //l'array values contiene la data come valore e subsegmentsCount, subsegmentsFilters come posizione del sottosegmento, filtro
        foreach ($values as $value) {
            if($value['date'] == null) {
                $this->filterValue[$value['subsegmentsCount']][$value['filtersCount']] = null;
            } else {
                $this->filterValue[$value['subsegmentsCount']][$value['filtersCount']] = \Carbon\Carbon::createFromFormat('d M, Y', $value['date'])->format('Y-m-d');
            }
        }

        if(!$update) {
            $this->saveSegment();
        } else {
            $this->updateSegment($this->editSegmentSelected->id);
        }
    }

    public function onEditSegmentClick($segmentId) {
        $this->showModal = true;
        $this->modalTitle = 'Modifica Segmento';
        $this->resetFields();

        $this->editSegmentSelected = PushNotificationSegment::find($segmentId);
        $this->segmentName = $this->editSegmentSelected->name;

        $subsegments = DB::table('push_notifications_segment_has_subsegments')
                     ->where('segment_id', '=', $this->editSegmentSelected->id)
                     ->get();

        foreach ($subsegments->groupBy('position') as $subsegment) {
            $this->subsegmentsAddedList[] = PushNotificationSubsegment::find($subsegment->first()->subsegment_id);
        }

        foreach ($subsegments as $subsegment) {
            $valueWithoutQuotes = str_replace("'", "", $subsegment->value); // Rimuovi gli apici singoli dalla stringa

            $this->filterOperator[$subsegment->position][] = $subsegment->operator_id;
            $this->filterValue[$subsegment->position][] = $valueWithoutQuotes;

            if($subsegment->select_parent_position !== null) {
                $this->selects[$subsegment->position][$subsegment->select_parent_position][] = count($this->filterValue[$subsegment->position]) - 1;
                $this->select_parent_position[$subsegment->position][count($this->filterValue[$subsegment->position]) - 1] = $subsegment->select_parent_position;
            }

            if(PushNotificationSubsegmentOperator::find($subsegment->operator_id)->filter_id == 3) {
                $date = null;
                if($valueWithoutQuotes != null) {
                    $date = \Carbon\Carbon::createFromFormat('Y-m-d', $valueWithoutQuotes)->format('j M, Y');
                }

                $this->emit('setDatePickerSubsegment', $this->dataPickersCount, $subsegment->position,
                    count($this->filterValue[$subsegment->position]) - 1, $date);
                $this->dataPickersCount++;
            }
        }
    }

    public function updateSegment($segmentId) {
        if(!$this->subsegmentsAddedList) {
            return;
        }
        $this->validate(['segmentName' => 'required']);

        try {
            $segment = PushNotificationSegment::find($segmentId);

            DB::transaction(function() use (&$segment) {
                $segment->name = $this->segmentName;
                $segment->save();

                DB::table('push_notifications_segment_has_subsegments')
                    ->where('segment_id', $this->editSegmentSelected->id)
                    ->delete();

                foreach ($this->subsegmentsAddedList as $i => $subsegment) {
                    $isValueSet = false;
                    foreach ($this->filterValue[$i] as $j => $value) {
                        $valueWithoutQuotes = trim($value, '"');
                        if (!is_numeric($valueWithoutQuotes) && $valueWithoutQuotes != null) {
                            if (!(substr($value, 0, 1) === "'" && substr($value, -1) === "'")) {
                                $value = '\''.$value.'\'';
                            }
                        }

                        DB::table('push_notifications_segment_has_subsegments')->insert([
                            'segment_id' => $segment->id,
                            'subsegment_id' => $subsegment['id'],
                            'operator_id' => $this->filterOperator[$i][$j],
                            'value' => $value,
                            'position' => $i,
                            'select_parent_position' => $this->select_parent_position[$i][$j]
                        ]);

                        if($value != null && !$isValueSet) {
                            $isValueSet = true;
                        }
                    }

                    if(!$isValueSet) {
                        throw new \Exception('Almeno un valore del filtro deve essere impostato');
                    }
                }
            });
        } catch (\Exception $e) {
            $this->errorToast($e->getMessage());
            return;
        }

        $this->segmentValue[$segment->id] = $segment->name;
        $this->initSegmentValue[$segment->id] = $segment->name;
        $this->selectSegmentValue($segment->id);
        $this->resetFields();
    }

    public function deleteSegment() {
        try {
            $segmentIdToDelete = $this->editSegmentSelected->id;
            $this->editSegmentSelected->id = null;

            DB::table('push_notifications_segment_has_subsegments')
                ->where('segment_id', $segmentIdToDelete)
                ->delete();

            DB::table('push_notifications_segments')
                ->where('id', $segmentIdToDelete)
                ->delete();

            unset($this->segmentsSaved[$segmentIdToDelete]);
        } catch (\Exception $e) {
            $this->successToast($e->getMessage());
            return;
        }

        $this->emit('closeDropdowns');
        $this->resetFields();

        $this->sendAudienceData();
        $this->getUsersSegment();
        $this->showModal = false;

        $this->successToast('Segmento eliminato con successo');
        $this->setEstimateNotificationsSent();
    }

    public function closeModal() {
        $this->showModal = false;
    }

    public function addSelectCatalogCustomerFeatures($subsegmentPosition, $filterPosition) {
        $newFilterPosition = count($this->filterValue[$subsegmentPosition]);

        $this->selects[$subsegmentPosition][$filterPosition][] = $newFilterPosition;
        $this->select_parent_position[$subsegmentPosition][$newFilterPosition] = $filterPosition;

        $this->filterValue[$subsegmentPosition][$newFilterPosition] = null;
        $this->filterOperator[$subsegmentPosition][$newFilterPosition] = $this->filterOperator[$subsegmentPosition][0];
    }

    public function removeSelectCatalogCustomerFeatures($subsegmentPosition, $selectParentPosition, $filterPosition) {
        unset($this->selects[$subsegmentPosition][$selectParentPosition][count($this->selects[$subsegmentPosition][$selectParentPosition]) - 1]);
        unset($this->select_parent_position[$subsegmentPosition][$filterPosition]);
        unset($this->filterValue[$subsegmentPosition][$filterPosition]);
        unset($this->filterOperator[$subsegmentPosition][$filterPosition]);

        $this->filterValue = array_values($this->filterValue);
        $this->filterOperator = array_values($this->filterOperator);
    }
}
