<?php

namespace FourWords\AppManager\PushNotifications\Livewire;

use Carbon\Carbon;
use FourWords\AppManager\Ioc\Livewire\IocComponent;
use FourWords\AppManager\PushNotifications\Model\PushNotificationSent;
use Livewire\WithFileUploads;

class DeliveryPushNotificationSection extends IocComponent {
    public $notification, $pushNotificationSent, $readonly;

    protected $listeners = ['setDateScheduled'];

    public
        $radioDateSchedule = 'push-notification-send-immediately',
        $dateScheduled,

        $hours,
        $minutes,
        $hourSelected,
        $minuteSelected;


    public function mount() {
        if($this->notification) {
            $this->pushNotificationSent = PushNotificationSent::where('push_notification_id', '=', $this->notification->id)->first();
            $this->readonly = $this->pushNotificationSent != null;

            $this->initDateSchedule();
        } else {
            $this->initDefaultDateSchedule();
        }
    }

    public function render() {
        return view('push::livewire.delivery-section');
    }

    private function initDefaultDateSchedule() {
        $time = Carbon::now();
        $this->dateScheduled = $time->format('d M, Y');

        $this->hours = ['00','01','02','03','04','05','06','07','08','09','10','11','12','13','14','15','16','17','18','19','20','21','22','23'];
        $this->minutes = ['00', '15', '30', '45'];

        $temp = $time->format('i') / 15;
        if($temp >= 3) {
            $this->hourSelected = $time->addHours()->format('H');
            $this->minuteSelected = $this->minutes[0];
        } else {
            $this->hourSelected = $time->format('H');
            $this->minuteSelected = $this->minutes[$temp+1];
        }
    }

    public function initDateSchedule() {
        $this->radioDateSchedule = 'push-notification-send-specific-time';
        $this->hours = ['00','01','02','03','04','05','06','07','08','09','10','11','12','13','14','15','16','17','18','19','20','21','22','23'];
        $this->minutes = ['00', '15', '30', '45'];

        $time = strtotime($this->notification->send_at);
        $this->dateScheduled = date('d M, Y' , $time);
        $this->hourSelected = date('H' , $time);

        $temp = date('i' , $time) / 15;
        $this->minuteSelected = $this->minutes[$temp];
    }
    public function setDateScheduled($date, $toSend) {

        $sendNow = false;

        if($this->radioDateSchedule == 'push-notification-send-specific-time') {
            $date = \Carbon\Carbon::createFromFormat('d M, Y', $date);
            $this->dateScheduled = $date->format('Y-m-d');

            if(Carbon::createFromFormat('Y-m-d H:i', $this->dateScheduled.' '.$this->hourSelected.':'.$this->minuteSelected)->setTimezone('Europe/Rome') < Carbon::now() && $toSend) {
                $this->errorToast('Inserisci una data programmata futura');
                return;
            }
        } else {
            $date = Carbon::now();
            $this->dateScheduled = $date->format('Y-m-d');
            $this->hourSelected = $date->format('H');
            $this->minuteSelected = $date->format('i');
            $sendNow = true;
        }


        $this->emitUp('setDelivery', ['date' => $this->dateScheduled, 'hour' => $this->hourSelected, 'minute' => $this->minuteSelected], $toSend, $sendNow);
    }
}
