<?php

namespace FourWords\AppManager\PushNotifications\Livewire;

use Carbon\Carbon;
use FourWords\AppManager\Ioc\Helpers\TaskManager;
use FourWords\AppManager\Ioc\Livewire\IocComponent;
use FourWords\AppManager\PushNotifications\Jobs\SendPushNotifications;
use FourWords\AppManager\PushNotifications\Model\PushNotificationSent;
use Illuminate\Support\Facades\Storage;
use Livewire\WithFileUploads;

class EditPushNotificationPage extends IocComponent {
    use WithFileUploads;

    public $notification, $showMode;
    public $title;

    public
        $audience = [],
        $audienceSegment = [],
        $message = [],
        $delivery = [],
        $sendNow = false;

    public bool
        $androidSend,
        $iosSend;


    protected $listeners = ['setAudience', 'setAudienceSegment', 'setAudienceOS', 'setMessage', 'setDelivery'];

    protected $rules = [
        'title' => 'required|min:4',
    ];

    public function mount() {
        $this->initNotification();
        $this->showMode = PushNotificationSent::where('push_notification_id', '=', $this->notification->id)->first() != null;
    }

    public function render() {
        return view('push::livewire.create-page');
    }

    private function initNotification() {
        $this->title = $this->notification->name;
    }
    
    public function edit($toSend)
    {
        $this->validate();
        if(!$this->androidSend && !$this->iosSend) {
            $this->errorToast('Seleziona almeno un sistema operativo a cui inviare la notifica');
            return null;
        }

        try {
            $this->editNotificationDB($toSend);

            if ($this->sendNow) {
                TaskManager::runJob(SendPushNotifications::class);
            }

        } catch (\Exception $e) {
            $this->errorToast('Errore nella modifica: '. $e->getMessage());
            return null;
        }

        return $this->sendNow ? redirect(route('push.show', ['id' => $this->notification->id])) : redirect(route('push.index'));
    }
    
    private function editNotificationDB($toSend) {
        $this->notification->name = $this->title;

        $this->notification->title = $this->message['title'];
        //$this->notification->subtitle = $this->message['title'];
        $this->notification->message = $this->message['description'];

        if($this->message['image'] != null) {
            $this->notification->image = $this->uploadImage($this->notification);
        }

        $this->notification->url = $this->message['url'];
        $this->notification->product = $this->message['productCode'];

        $this->notification->send_at = $this->getDateScheduled();

        $this->notification->android_send = $this->androidSend;
        $this->notification->ios_send = $this->iosSend;

        $this->notification->draft = !$toSend;
        $this->notification->save();

        $this->notification->groups()->sync(array_keys($this->audience));
        $this->notification->segments()->sync(array_keys($this->audienceSegment));
    }

    private function uploadImage($table) {
        $oldFilePath = 'public/notificationImages/'.$this->message['image'];
        $newFileName = 'notification-image'.$table->id.'.'.explode('.',$this->message['image'])[1];
        $newFilePath = 'public/notificationImages/'.$newFileName;

        Storage::delete('public/notificationImages/'.$newFileName);
        Storage::move($oldFilePath, $newFilePath);
        return $newFileName;
    }

    public function setAudience($value) {
        $this->audience = $value;
    }

    public function setAudienceSegment($value) {
        $this->audienceSegment = $value;
    }

    public function setAudienceOS($android, $ios) {
        $this->androidSend = $android ?? true;
        $this->iosSend = $ios ?? true;
    }

    public function setMessage($value) {
        $this->message = $value;
    }

    public function setDelivery($value, $toSend, $sendNow) {
        $this->delivery = $value;
        $this->sendNow = $sendNow;
        $this->edit($toSend);
    }

    private function getDateScheduled() {
        return Carbon::createFromFormat('Y-m-d H:i', $this->delivery['date'].' '.$this->delivery['hour'].':'.$this->delivery['minute'])->setTimezone('Europe/Rome');
    }

}
