<?php

namespace FourWords\AppManager\PushNotifications\Livewire;

use Carbon\Carbon;
use FourWords\AppManager\Appstore\Models\AppstoreApp;
use FourWords\AppManager\Catalog\Model\Product;
use FourWords\AppManager\Ioc\Livewire\IocComponent;
use FourWords\AppManager\Ioc\Models\IocUser;
use FourWords\AppManager\PushNotifications\Model\PushNotification;
use FourWords\AppManager\PushNotifications\Model\PushNotificationGroup;
use FourWords\AppManager\PushNotifications\Model\PushNotificationSent;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Livewire\WithFileUploads;

class MessagePushNotificationSection extends IocComponent {
    use WithFileUploads;

    public $notification, $pushNotificationSent;

    public
        $messageTitle,
        //$messageSubtitle,
        $messageDescription,
        $appLogo,
        $showMessageImage = null,
        $messageImage,
        $messageImageTemp,
        $messageUrl,
        $messageProductCode,
        $messageProductName;

    public
        $products,
        $isEmpty,
        $search,
        $showUrl = false;


    protected $rules = [
        'messageDescription' => 'required|min:4',
    ];

    public function mount() {
        $this->pushNotificationSent = PushNotificationSent::where('push_notification_id', '=', $this->notification->id)->first();
        $this->readonly = $this->pushNotificationSent != null;

        $this->initMessage();
    }

    public function render() {
        if (!is_null($this->search)) {
            $this->products = Product::where('name', 'LIKE', '%' . $this->search . '%')
                ->orWhere('code', 'LIKE', '%' . $this->search . '%')
                ->take(8)
                ->get();
        } else {
            $this->products = [];
        }
        $this->isEmpty = __('Prodotto non trovato.');

        return view('push::livewire.message-section');
    }

    public function updated($name) {
        if($name == 'messageImage') {
            if($this->checkImage()) {
                $this->uploadImage();
                //$this->messageImageTemp = $this->messageImage->getFileName();
            } else {
                $this->messageImage = null;
            }
        }

        $this->sendMessageData();
    }

    private function initMessage() {
        if($this->notification) {
            $this->messageTitle = $this->notification->title;
            $this->messageSubtitle = $this->notification->subtitle;
            $this->messageDescription = $this->notification->message;
            $this->showMessageImage = $this->notification->image;

            $this->messageUrl = $this->notification->url;
            if($this->messageUrl !== null) {
                $this->showUrl = true;
            } else {
                $this->messageProductCode = $this->notification->product;
                if($this->messageProductCode !== null) {
                    $this->messageProductName = Product::where('code', '=', $this->messageProductCode)->first()->name;
                }
            }
        } else {
            $this->messageTitle = 'Nuovo messaggio!';
            $this->messageDescription = 'Inserisci il tuo nuovo messaggio';
        }

        $this->appLogo = AppstoreApp::first()->icon;
    }

    private function uploadImage() {
        if($this->messageImage != null) {
            if($this->messageImageTemp) {
                Storage::delete('public/notificationImages/'.$this->messageImageTemp);
            }

            $this->messageImageTemp = $this->messageImage->getFileName();
            $this->messageImage->storeAs('notificationImages', $this->messageImageTemp, 'public');
        }
    }

    private function checkImage() {
        try {
            $this->validate(['messageImage' => 'max:30720']); //Controllo logo
            if($this->messageImage != null) {
                if(!in_array($this->messageImage->extension(), config('appstore.admin.imageFileExtensions'))) {
                    $this->errorToast('Il logo deve essere un\'immagine');
                    return false;
                }
            }
        } catch (\Exception $e) {
            $this->errorToast('Il logo deve pesare meno di 30MB');
            return false;
        }
        return true;
    }

    public function sendMessageData() {
        if($this->showUrl) {
            $this->messageProductCode = null;
        } else {
            $this->messageUrl = null;
        }

        $messageData = ['title' => $this->messageTitle, 'description' => $this->messageDescription, 'image' => $this->messageImageTemp, 'url' => $this->messageUrl, 'productCode' => $this->messageProductCode];
        $this->emitUp('setMessage', $messageData);
    }

    public static function search($searchKey)
    {
        return self::where('name', 'LIKE', '%' . $searchKey . '%')
            ->orWhere('code', 'LIKE', '%' . $searchKey . '%');
    }

    public function setMessageProductCode($productName, $productCode) {
        $this->messageProductName = $productName;
        $this->messageProductCode = $productCode;
        $this->search = null;
        $this->sendMessageData();
    }
}
