<?php

namespace FourWords\AppManager\PushNotifications\Livewire;

use FourWords\AppManager\Ioc\Livewire\Table\IocTable;
use FourWords\AppManager\PushNotifications\Model\PushNotification;
use FourWords\AppManager\PushNotifications\Model\PushNotificationGroup;
use Mediconesystems\LivewireDatatables\Column;

class PushTable extends IocTable {

    protected $listeners = [
        'onEditClick',
        'onShowClick',
    ];

    public function initialisePerPage()
    {
        $this->getSessionStoredPerPage();

        if (!$this->perPage) {
            $this->perPage = 50;
        }
    }

    public function builder()
    {
        return PushNotification::query();
    }

    public function columns()
    {
        return [
            Column::name('id')
                ->label('ID')
                ->hide()
//                ->filterable()
                ->defaultSort('desc'),

            Column::name('name')
                ->label('Nome')
//                ->filterable()
//                ->editable()
                ->searchable(),


            Column::callback(['draft'], function ($draft) {

                if (!$draft) {
                    return '<svg class="h-5 w-5 stroke-current text-green-600 mx-auto" fill="none" viewBox="0 0 24 24" stroke="currentColor">
    <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M9 12l2 2 4-4m6 2a9 9 0 11-18 0 9 9 0 0118 0z"></path>
</svg>';
                } else {
                    return '<svg class="h-5 w-5 stroke-current text-red-300 mx-auto" fill="none" viewBox="0 0 24 24" stroke="currentColor">
    <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M10 14l2-2m0 0l2-2m-2 2l-2-2m2 2l2 2m7-2a9 9 0 11-18 0 9 9 0 0118 0z"></path>
</svg>';
                }
            })->label('Da inviare')->exportCallback(function ($value) {
                return strval($value);
            }),

            Column::callback(['sent'], function ($sent) {
                if ($sent == 1) {
                    return '<svg class="h-5 w-5 stroke-current text-green-600 mx-auto" fill="none" viewBox="0 0 24 24" stroke="currentColor">
    <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M9 12l2 2 4-4m6 2a9 9 0 11-18 0 9 9 0 0118 0z"></path>
</svg>';
                } else if ($sent == -1) {
                    return '<div class="text-center"><i class="fa-light fa-clock fa-lg"></i></div>';

                } else {
                    return '<svg class="h-5 w-5 stroke-current text-red-300 mx-auto" fill="none" viewBox="0 0 24 24" stroke="currentColor">
    <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M10 14l2-2m0 0l2-2m-2 2l-2-2m2 2l2 2m7-2a9 9 0 11-18 0 9 9 0 0118 0z"></path>
</svg>';
                }
            })->label('Inviato')
                ->exportCallback(function ($value) {
                    return $value;
                }),

            Column::name('groups.name')
                ->label('Gruppi')
                ->filterable(),

            Column::name('segments.name')
                ->label('Segmenti')
                ->filterable(),

            Column::name('send_at')
                ->label('Invia il')
                ->searchable(),

            Column::name('sent_at')
                ->label('Inviato il')
                ->searchable(),

            Column::callback(['android_send', 'ios_send'], function ($android_send, $ios_send) {
                return $android_send ? ($ios_send ? 'Android & iOS' : 'Andorid') : ($ios_send ? 'iOS' : '');
            })->unsortable()
                ->label('OS'),

            Column::callback(['id', 'name', 'sent'], function ($id, $name, $sent) {
                return view('datatables::table-actions', [
                    'id' => $id,
                    'name' => $name,
                    'permission' => 'edit-push-notifications',
                    'action' => $sent == 0 ? 'edit' : 'show'
                ]);
            })->unsortable()
                ->label('Azioni')
            ->excludeFromExport(),
        ];
    }

    public function onEditClick($id) {
        return redirect(route('push.edit', ['id' => $id]));
    }

    public function onShowClick($id) {
        return redirect(route('push.show', ['id' => $id]));
    }

    public function groups() {
        return $this->belongsToMany(PushNotificationGroup::class, 'push_notifications_has_groups', 'notification_id','group_id');
    }
}
