<?php

namespace FourWords\AppManager\PushNotifications\Livewire\Segments;

use FourWords\AppManager\Ioc\Livewire\IocComponent;
use FourWords\AppManager\Ioc\Models\IocUser;
use FourWords\AppManager\Ioc\Models\Role;
use FourWords\AppManager\Ioc\Models\User;
use FourWords\AppManager\PushNotifications\Model\PushNotificationGroup;
use FourWords\AppManager\PushNotifications\Model\PushNotificationSegment;
use Illuminate\Support\Facades\DB;

class PushSegmentCreateEditComponent extends IocComponent
{
    protected $modelName = "segmento utenti";
    protected $model = PushNotificationSegment::class;

    public $modelId;

    public $showModal = false;

    public $title = "Crea nuovo";

    public string
        $name,
        $query;


    protected $listeners = [
        'onCreateClick',
        'onEditClick',
        'onSave',
        'onCancel'
    ];

    public $fields = [
        'id',
        'name',
        'query'
    ];

    protected $rules = [
        'name' => 'required|min:2',
        'query' => 'required',
    ];

    public function mount() {
        $this->title = "Crea nuovo ".$this->modelName;
    }

    /**
     * Reseting all inputted fields
     * @return void
     */
    public function resetFields(){

        $this->resetErrorBag();

        foreach ($this->fields as $field) {

            if ($field == 'id') {
                $this->modelId = null;
            } else if ($field == 'active') {
                $this->{$field} = false;
            } else {
                $this->{$field} = '';
            }
        }
    }

    public function render()
    {
        return view('push::livewire.segments.create-edit');
    }

    public function onCreateClick() {
        if(!$this->checkPermission('create-push-notifications')) {
            return;
        }

        $this->resetFields();
        $this->title = "Crea ".$this->modelName;

        $this->showModal = true;
    }

    public function onEditClick($modelId) {
        if(!$this->checkPermission('edit-push-notifications')) {
            return;
        }

        $this->resetFields();
        $this->title = "Modifica ".$this->modelName;

        try {
            $model = $this->model::findOrFail($modelId);
            if(!$model) {
                $this->errorToast($this->modelName.' non trovato');
                return;
            }

            foreach ($this->fields as $field) {
                if ($field == 'id') {
                    $this->modelId = $model->id;
                } else {
                    $this->{$field} = $model->{$field};
                }
            }

            $this->showModal = true;

        } catch (\Exception $ex) {
            $this->errorToast('Errore modifica '.$this->modelName);
        }
    }

    public function onSave() {
        if ($this->modelId) {
            if($this->checkPermission('edit-push-notifications')) {
                $this->update();
            }
        } else {
            if($this->checkPermission('create-push-notifications')) {
                $this->store();
            }
        }
    }

    public function store()
    {
        $this->validate();
        
        try {
            $push = new $this->model();
            $push->name = $this->name;
            $push->query = $this->query;
            $push->save();

            $this->showModal = false;
            $this->emit('refreshLivewireDatatable');
            $this->successToast($this->modelName.' creato con successo');
            $this->resetFields();
        } catch (\Exception $ex) {
            $this->errorToast($ex->getMessage());
            $this->errorToast('Errore nella creazione '.$this->modelName);
        }
    }

    public function update()
    {
        $this->validate();
        try {

            $model = $this->model::findOrFail($this->modelId);

            foreach ($this->fields as $field) {

                if ($field == 'id') {
                    $model->id = $this->modelId;
                    continue;
                }

                $model->{$field} = $this->{$field};
            }

            $model->save();

            $this->showModal = false;
            $this->emit('refreshLivewireDatatable');
            $this->successToast($this->modelName.' aggiornato con successo');
            $this->resetFields();
        } catch (\Exception $ex) {
            $this->errorToast('Errore nella modifica '.$this->modelName);
        }
    }
}
