<?php

namespace FourWords\AppManager\PushNotifications\Model;

use Exception;
use GuzzleHttp\RequestOptions;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Storage;


class PushNotification extends Model
{
    protected $table = "push_notifications";
//    use HasFactory;

    var $timestamps = true;

    var $guarded = [];

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function customData()
    {
        return $this->hasMany(PushNotificationCustomData::class, 'notification_id');
    }

    public function sentData()
    {
        return $this->hasMany(PushNotificationSentUser::class, 'push_notification_id', 'id');
    }

    /**
     * @return BelongsToMany
     */
    public function groups()
    {
        return $this->belongsToMany(PushNotificationGroup::class, 'push_notifications_has_groups', 'notification_id', 'group_id');
    }

    /**
     * @return BelongsToMany
     */
    public function segments()
    {
        return $this->belongsToMany(PushNotificationSegment::class, 'push_notifications_has_segments', 'notification_id', 'segment_id');
    }

    public static function getPendingNotifications()
    {
        return static::where('draft', false)
            ->where('sent', '=', '0')
            ->where('send_at', '<=', now())
            ->get();
    }

    public function sendTest() {
        $deviceToken = '01d8bd3c7b727e1077317032e4cda0812f99fa8b8a54200a22c9994170e6215c';
        $bundleId = 'it.sanmarcoinformatica.ioc.demo';
        $keyPath = storage_path('AuthKey_6YF86ZW98M.p8'); // Path to your APNs .p8 key file
        $keyId = 'N4USFAKS2J'; // Your Key ID
        $teamId = '6ZT6VP526E'; // Your Team ID
        $production = false; // Set to true for production

        // Prepare the notification payload
        $payload = [
            'aps' => [
                'alert' => 'Hello World',
                'sound' => 'ping.aiff',
            ],
            'id' => 123, // Your custom payload data
        ];

        $headers = [
            'Authorization: Bearer ' . $this->generateJwtToken($keyPath, $keyId, $teamId),
            'apns-topic: ' . $bundleId,
            'Content-Type' => 'application/json',
            'apns-priority' => '10'
        ];

        $options = [
            RequestOptions::VERSION => 2.0
        ];

        // Create the HTTP client
        $client = Http::withHeaders($headers)->withOptions($options);

        // Send the notification
        $response = $client->post('https://api.' . ($production ? 'push' : 'sandbox.push') . '.apple.com/3/device/' . $deviceToken, [
            'json' => $payload,
        ]);

        // Check the response for any errors
        if ($response->successful()) {
            $result = $response->json();
            // Handle the result
            dd($result);
        } else {
            // Handle the error
            dd('Notification sending failed', $response->status(), $response->body());
        }
    }

    private function generateJwtToken($keyPath, $keyId, $teamId)
    {
        // Load your private key
        $privateKey = file_get_contents($keyPath);

        // Create the JWT payload
        $header = json_encode(['alg' => 'ES256', 'kid' => $keyId]);
        $payload = json_encode(['iss' => $teamId, 'iat' => time()]);

        // Encode the payload
        $base64Header = base64_encode($header);
        $base64Payload = base64_encode($payload);

        // Sign the payload
        openssl_sign($base64Header . '.' . $base64Payload, $signature, $privateKey, 'sha256');

        // Base64 encode the signature
        $base64Signature = base64_encode($signature);

        // Create the JWT token
        return $base64Header . '.' . $base64Payload . '.' . $base64Signature;
    }

    public static function imageAsUrl($image) {
        return env('APP_URL').Storage::url('notificationImages/'.$image);
    }

    /**
     * @return array[]
     *@throws Exception
     */
    public function sendApple($devicePushToken) {

        $key_file = config('push-notifications.apple.key_file');
        $keyid = config('push-notifications.apple.keyid');
        $teamid = config('push-notifications.apple.teamid');
        $bundleid = config('push-notifications.apple.bundleid');
        $url = config('push-notifications.apple.url');

        $keyfile_path = storage_path("$key_file");

        $headers = [
            'Content-Type' => 'application/json',
            'apns-topic' => "{$bundleid}",
            'apns-priority' => '10'
        ];

        $completeUrl = "$url/3/device/$devicePushToken";

        $key = 'link';
        $value = $this->url;
        if($this->url === null) {
            $key = 'product';
            $value = $this->product;
        }

        $payload = [
            'aps' => [
                'alert' => [
                    'title' => $this->title,
                    'subtitle' => $this->subtitle,
                    'body' => $this->message
                ],
                'sound' => 'default'
            ],
            $key => $value
        ];

        if(!empty($this->image)) {
            $payload['aps']['mutable-content'] = 1;
            $payload['image_url'] = self::imageAsUrl($this->image);
        }

        $options = [
            RequestOptions::VERSION => 2.0
        ];

        if (config('push-notifications.dryRun')) {
            return $payload;
        }

        $response = Http::withToken($this->apns_jwt_token($teamid, $keyid, $keyfile_path))->withHeaders($headers)->withOptions($options)->post($completeUrl, $payload);

        if ($response->status() != 200) {
            throw new Exception("Errore invio notifica push Apple: ".($response->body() ?? ''));
        }

        return $payload;
    }

    /**
     * @return array
     *@throws Exception
     */
    public function sendGoogle($devicePushToken) {

        $server_key = config('push-notifications.google.server_key');
        $url = config('push-notifications.google.url');
        $click_action = config('push-notifications.google.click_action');

        $key = 'link';
        $value = $this->url;
        if($this->url === null) {
            $key = 'product';
            $value = $this->product;
        }

        $payload = [
            'to' => $devicePushToken,
            'notification' => [
                'title' => $this->title,
                'body' => $this->message,
                'sound' => 'default',
            ],
            'data' => [
                $key => $value
            ]
        ];


        foreach ($this->customData as $data) {
            $payload['data'][$data->key] = $data->value;
        }

        if (!empty($click_action))
            $payload['notification']['click_action'] = $click_action;

        if(!empty($this->image)) {
            $payload['notification']['image'] = self::imageAsUrl($this->image);
        }

        if (config('push-notifications.dryRun')) {
            return $payload;
        }

        $response = Http::withToken($server_key,'key=')->post($url, $payload);

        if ($response->status() != 200) {
            throw new Exception("Errore invio notifica push Google: ".($response->body() ?? ''));
        }

        return $payload;
    }

    static protected function base64($data)
    {
        return rtrim(strtr(base64_encode(json_encode($data)), '+/', '-_'), '=');
    }

    static protected function base64url_encode($binary_data) { return strtr(rtrim(base64_encode($binary_data), '='), '+/', '-_'); }

    function apns_jwt_token($team_id, $key_id, $private_key_pem_str)
    {
        if (! function_exists('openssl_get_md_methods') || ! in_array('sha256', openssl_get_md_methods())) throw new Exception('Requires openssl with sha256 support');

        $private_key = openssl_pkey_get_private('file://' . $private_key_pem_str);
        if (! $private_key) throw new Exception('Cannot decode private key');

        $msg = self::base64url_encode(json_encode([ 'alg' => 'ES256', 'kid' => $key_id ])) . '.' . self::base64url_encode(json_encode([ 'iss' => $team_id, 'iat' => time() ]));
        openssl_sign($msg, $der, $private_key, 'sha256');

        // DER unpacking from https://github.com/firebase/php-jwt
        $components = [];
        $pos = 0;
        $size = strlen($der);
        while ($pos < $size) {
            $constructed = (ord($der[$pos]) >> 5) & 0x01;
            $type = ord($der[$pos++]) & 0x1f;
            $len = ord($der[$pos++]);
            if ($len & 0x80) {
                $n = $len & 0x1f;
                $len = 0;
                while ($n-- && $pos < $size) $len = ($len << 8) | ord($der[$pos++]);
            }

            if ($type == 0x03) {
                $pos++;
                $components[] = substr($der, $pos, $len - 1);
                $pos += $len - 1;
            } else if (! $constructed) {
                $components[] = substr($der, $pos, $len);
                $pos += $len;
            }
        }
        foreach ($components as &$c) $c = str_pad(ltrim($c, "\x00"), 32, "\x00", STR_PAD_LEFT);

        return $msg . '.' . self::base64url_encode(implode('', $components));
    }
}
