<?php
/**
 * Copyright (c) 2020/2
 * Created by Federico Papetti
 */

namespace FourWords\AppManager\PushNotifications\Providers;

use FourWords\AppManager\Ioc\Events\DidCopyFrontendFiles;
use FourWords\AppManager\PushNotifications\Livewire\AudiencePushNotificationSection;
use FourWords\AppManager\PushNotifications\Livewire\CreatePushNotificationPage;
use FourWords\AppManager\PushNotifications\Livewire\DeliveryPushNotificationSection;
use FourWords\AppManager\PushNotifications\Livewire\EditPushNotificationPage;
use FourWords\AppManager\PushNotifications\Livewire\Groups\AddGroupUsersTable;
use FourWords\AppManager\PushNotifications\Livewire\Groups\PushCreateEditComponent;
use FourWords\AppManager\PushNotifications\Livewire\Segments\PushSegmentCreateEditComponent;
use FourWords\AppManager\PushNotifications\Livewire\Groups\PushGroupUsersTable;
use FourWords\AppManager\PushNotifications\Livewire\MessagePushNotificationSection;
use FourWords\AppManager\PushNotifications\Livewire\PushTable;
use FourWords\AppManager\PushNotifications\Livewire\Segments\PushSegmentUsersTable;
use FourWords\AppManager\PushNotifications\Livewire\ShowPushNotificationPage;
use Illuminate\Support\Facades\Event;
use Illuminate\Support\ServiceProvider;
use Illuminate\Support\Facades\File;
use Livewire\Livewire;

class PushNotificationServiceProvider extends ServiceProvider
{

    public function register() {
        $this->bindClasses('Model');
    }

    public function bindClasses($type) {
        $filesDirectory = __DIR__.'/../'.$type;

        $filesToInclude = File::files($filesDirectory);

        foreach ($filesToInclude as $classFile) {
            $className = pathinfo($classFile, PATHINFO_FILENAME);
            $modelClass = "FourWords\\AppManager\\Notifications\\$type\\{$className}";

            // Binding
            $this->app->bind($modelClass, $modelClass);
        }
    }

    public function boot() {

        // Includo le configurazioni
        config(require __DIR__.'/../../config/push-notifications.php');

        //Migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');
        //Views
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'push');
        //Routes
        $this->loadRoutesFrom(__DIR__ . '/../../routes/routes.php');

        //Livewire
        $this->registerLivewireComponents();

        Event::listen(DidCopyFrontendFiles::class, function (DidCopyFrontendFiles $event) {
            $this->updateFrontend();
        });

    }

    private function registerLivewireComponents() {
        Livewire::component('push-table', PushTable::class);
        Livewire::component('create-push-notification-page', CreatePushNotificationPage::class);
        Livewire::component('edit-push-notification-page', EditPushNotificationPage::class);
        Livewire::component('show-push-notification-page', ShowPushNotificationPage::class);

        Livewire::component('audience-section', AudiencePushNotificationSection::class);
        Livewire::component('message-section', MessagePushNotificationSection::class);
        Livewire::component('delivery-section', DeliveryPushNotificationSection::class);

        Livewire::component('push-group-users-table', PushGroupUsersTable::class);
        Livewire::component('push-group-users-create', PushCreateEditComponent::class);
        Livewire::component('push-group-users-add-table', AddGroupUsersTable::class);

        Livewire::component('push-segment-users-table', PushSegmentUsersTable::class);
        Livewire::component('push-segment-users-create', PushSegmentCreateEditComponent::class);
    }

    public function updateFrontend()
    {

        /*
        |--------------------------------------------------------------------------
        | File statici
        |--------------------------------------------------------------------------
        | Il contenuto della cartella 'push-notifications/resources/static' verrà copiato in public/static
        */

        $sourceFolder = __DIR__ . '/../../resources/static';
        $destinationFolder = public_path('static');
        $iterator = new \DirectoryIterator($sourceFolder);

        foreach ($iterator as $item) {
            if ($item->isFile()) {
                $sourceFilePath = $item->getPathname();
                $destinationFilePath = $destinationFolder . DIRECTORY_SEPARATOR . $item->getFilename();
                File::copy($sourceFilePath, $destinationFilePath);
            } elseif ($item->isDir() && !$item->isDot()) {
                $sourceSubFolder = $item->getPathname();
                $destinationSubFolder = $destinationFolder . DIRECTORY_SEPARATOR . $item->getFilename();
                File::copyDirectory($sourceSubFolder, $destinationSubFolder);
            }
        }

    }


}
