<?php

namespace FourWords\AppManager\SqliteGen\Archive;

use FourWords\AppManager\Director\FWUtils;

abstract class Archiver implements ArchiverInterface
{
    protected $quality = null;
    protected $filesToCompress;
    protected $archivePathWithoutExt = null;

    protected $uncompressedSize = 0;
    protected $compressedSize = 0;
    protected $compressionTime = 0;
    protected $compressionPercentage = 0.0;

    abstract protected function compress();

    /**
     * @param $quality
     */
    public function __construct($quality = null)
    {
        $this->quality = $quality;
    }

    public function setQuality($quality)
    {
        if ($quality == null) return;

        if (is_integer($quality) || ctype_digit($quality)) {
            $this->quality = "w".$quality;
        } else {
            $this->quality = $quality;
        }

    }

    public function getQuality()
    {
        return $this->quality;
    }

    /**
     * @throws ArchiveException
     */
    public function addFile($file, $nameInArchive)
    {
        if (!file_exists($file)) {
            throw new ArchiveException('Il file '.$file.' non è stato trovato');
        }

        if ($size = filesize($file)) {
            $this->uncompressedSize += $size;
        } else {
            throw new ArchiveException('Impossibile recuperare dimensioni file '.$file);
        }

        $this->filesToCompress[] = ['file' => $file, 'nameInArchive' => $nameInArchive];
    }

    /**
     * @throws ArchiveException
     */
    public function archive($pathWithoutExt)
    {
        $this->setArchivePath($pathWithoutExt);

        $startTime = time();
        $compressed = $this->compress();
        $endTime = time();

        if (!$compressed) {
            throw new ArchiveException('Impossibile creare l\'archivio: '.$this->getArchivePath());
        }

        $this->setCompressionTime($endTime - $startTime);
        $this->setCompressedSize();
        $this->setCompressionPercentage();


        return true;
    }

    protected function setArchivePath($path)
    {
        $this->archivePathWithoutExt = $path;
    }

    public function getArchivePath()
    {
        return $this->archivePathWithoutExt.$this->getExtension();
    }

    protected function setCompressionTime($seconds)
    {
        $this->compressionTime = $seconds;
    }

    protected function setCompressionPercentage()
    {
        $this->compressionPercentage = round(100-(($this->getCompressedSize() * 100) / $this->getUncompressedSize()),1);
    }

    protected function setCompressedSize()
    {
        if (file_exists($this->getArchivePath()) && ($size = filesize($this->getArchivePath()))) {
            $this->compressedSize = $size;
        }
    }

    public function getUncompressedSize($readable = false)
    {
        if ($readable) {
            return FWUtils::humanFilesize($this->uncompressedSize);
        } else {
            return $this->uncompressedSize;
        }
    }

    public function getCompressedSize($readable = false)
    {
        if ($readable) {
            return FWUtils::humanFilesize($this->compressedSize);
        } else {
            return $this->compressedSize;
        }
    }

    public function getCompressionTime($readable = false)
    {
        if ($readable) {
            return FWUtils::humanSeconds($this->compressionTime);
        } else {
            return $this->compressionTime;
        }
    }

    public function getCompressionPercentage($readable = false)
    {
        if ($readable) {
            return $this->compressionPercentage."%";
        } else {
            return $this->compressionPercentage;
        }
    }

}