<?php

namespace FourWords\AppManager\SqliteGen\Commands;


use Exception;
use FourWords\AppManager\Ioc\Handlers\LaravelConsoleHandler;
use FourWords\AppManager\SqliteGen\SqliteGenerator;
use Illuminate\Console\Command;
use Illuminate\Filesystem\Filesystem;
use Monolog\Formatter\LineFormatter;
use Monolog\Logger;

class GenerateDatabasesCommand extends Command
{

    /**
     * The console command name.
     *
     * @var string
     */
    protected $name = 'database:generate';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Genero i database cliente';

    protected $signature = 'database:generate 
        {--db= : Nome del database} 
        {--skip= : Database da skippare} 
        {--owners= : Gli owners separati da virgola} 
        {--forced : Se forzare la generazione} 
        {--stdout : Se mostrare log in stdout}';

    /**
     * Execute the console command.
     *
     * @param Filesystem $filesystem
     *
     * @return void
     */
    public function handle(Filesystem $filesystem)
    {

        $this->info('Lancio la generazione database');

        /** @var SqliteGenerator $sqg */
        $sqg = app('sqlitegen');

        if ($this->option('stdout')) {
            $dateFormat = "Y-m-d H:i:s";
            $output = "[%datetime%] [%extra.memory_usage%] %channel%.%level_name% %extra.function%: %message% \n";
            $formatter = new LineFormatter($output, $dateFormat,true);
            $handler = new LaravelConsoleHandler($this, Logger::DEBUG, true);
            $handler->setFormatter($formatter);
            $sqg->pushHandler($handler);
        }

        try {
            app('sqlitegen')->generate($this->option('db'))->skipping($this->option('skip'))->forced($this->option('forced'))->forOwners($this->option('owners') ? explode(',', $this->option('owners')) : null);
        } catch (Exception $e) {
            $this->error('Errore nella generazione database: '.$e->getMessage());
        }

        $this->info('Conclusa generazione database');


    }

}
