<?php
/**
 * Copyright (c) 2020/3
 * Created by Federico Papetti
 */

namespace FourWords\AppManager\SqliteGen\Controllers;

use Detection\MobileDetect;
use Exception;
use FourWords\AppManager\SqliteGen\SQLiteDB;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Str;

class DatabaseController extends Controller
{
    protected $db;
    protected $skip;
    protected $owners;
    protected $forced;

    public function __construct(Request $request)
    {
        $this->db = $request->get('db');
        $this->skip = $request->get('skip');
        $this->owners = $request->get('owners');
        $this->forced = (bool) $request->get('forced');
    }

    public function download()
    {
        $this->logRequest();

        try {

            // Prendo il database
            /** @var SQLiteDB $database */
            $database = app()->make('sqlitegen')->getDatabase($this->db ?? 'Specific');

            // Se ha degli owner
            if ($database->hasOwners ?? true) {

                // Imposto l'owner specifico
                $database->setOwner(Auth::user()['code'] ?? Auth::user()['username']);

            }

            $fileToDownload = null;

            // Se voglio scaricare un file specifico
            if ($specificFile = request()->get('file')) {
                foreach ($database->getVersions() as $version) {
                    if (Str::contains($version, $specificFile)) {
                        $fileToDownload = $version;
                        break;
                    }
                }

            } else if (!$fileToDownload = $database->getLatestVersionedCompressedDb()) {
                // Se non esiste il file da scaricare
                throw new Exception("Il database non è ancora stato generato o non esiste il suo zip nel percorso di esportazione.");
            }

        } catch (Exception $e) {

            $this->logException($e);
            abort(500);
        }

        return response()
            ->download($fileToDownload,
                $specificFile ? $specificFile : $database->getLatestVersionedCompressedDb(true, true),
                ['Content-type' => 'application/zip']);
    }

    /**
     * Gestisce la richiesta di generazione del database
     *
     * @param Request $request
     * @return JsonResponse
     * @throws Exception
     */
    public function generate(Request $request)
    {
        $this->logRequest();

        try {
            app('sqlitegen')->generate($this->db)->skipping($this->skip)->forced($this->forced)->forOwners($this->owners);
        } catch (Exception $e) {
            $this->logException($e);
            throw new Exception($e->getMessage(), preg_replace("/[^0-9]/", "", $e->getCode()));
        }

        return response()->json(["status" => "OK","message" => "Generazione completata"],200);
    }

    /**
     * Logga la richiesta passata al controller
     */
    protected function logRequest() {

        $fullUrl = '['.$_SERVER['REQUEST_METHOD'].'] ' . ((isset($_SERVER['HTTPS']) && $_SERVER['HTTPS']=='on') ? 'https' : 'http')  . '://' .  $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];

        app('sqlitegen')->logger->notice("=========================");
        app('sqlitegen')->logger->notice("**** NUOVA RICHIESTA ****");
        app('sqlitegen')->logger->notice((new MobileDetect())->getUserAgent());
        app('sqlitegen')->logger->notice($fullUrl);
        app('sqlitegen')->logger->debug('Dump $_REQUEST: ' . json_encode($_REQUEST));
    }

    /**
     * Logga l'eccezione lanciata
     */
    protected function logException($exception) {

        app('sqlitegen')->logger->error("-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-");
        app('sqlitegen')->logger->error($exception->getMessage());
        app('sqlitegen')->logger->error("-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-");

//        if (!$exception instanceof Exception) {
//            $stackTrace = [];
//            foreach ($exception->getTrace() as &$trace) {
//                $arr['file'] = ($trace['file'] ?? '') . ' -- line ' . ($trace['line'] ?? '');
//                $arr['call'] = ($trace['class'] ?? '') . ($trace['type'] ?? '') . ($trace['function'] ?? '');
//                $arr['args'] = ($trace['args'] ?? '');
//                $stackTrace[] = $arr;
//            }
//
//            app('sqlitegen')->logger->error('StackTrace: ' . json_encode(array_splice($stackTrace, 0, 10), JSON_UNESCAPED_SLASHES | JSON_PRETTY_PRINT));
//            app('sqlitegen')->logger->error("-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-");
//        }
    }

}
