<?php

/**
 * Copyright (c) 2022/9
 * Created by Enricomaria Marchiori
 */

namespace FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities;

use Illuminate\Database\Schema\Blueprint;
use FourWords\AppManager\SqliteGen\Model;

class Images extends Model
{
    public $sqliteTable = 'images';
    protected $primaryKey = 'id';
    public $incrementing = true;
    public $timestamps = false;

    // Velocizzo gli insert
    protected $useMutators = false;
    protected $logQueries = false;

    // colonna in mysql => colonna in sqlite
    protected $map = [
        'gallery' => 'collection',
        'catalog_products.id' => 'product',
        'position' => 'rank',
        'IFNULL(description, filename)' => 'desc_lang_1',
        'imported_filename' => 'img_big_lang_1'
    ];

    protected function autoImportConnection()
    {
        return 'mysql';
    }

    protected function autoImportUp(Blueprint $table)
    {
        $table->id();
        $table->integer('product');
        $table->integer('collection')->default(1);
        $table->integer('rank')->default(1);

        for ($i = 1; $i <= 10; $i++) {
            $table->string("desc_lang_$i")->nullable();
            $table->string("img_small_lang_$i")->nullable();
            $table->integer("img_small_size_lang_$i")->nullable();
            $table->integer("img_small_width_lang_$i")->nullable();
            $table->integer("img_small_height_lang_$i")->nullable();
            $table->string("img_big_lang_$i")->nullable();
            $table->integer("img_big_size_lang_$i")->nullable();
            $table->integer("img_big_width_lang_$i")->nullable();
            $table->integer("img_big_height_lang_$i")->nullable();
        }

        $table->integer('_disabled')->default(0);
        $table->integer('_modified')->nullable();
    }

    protected function autoImportQuery()
    {
        $selectColumns = '';
        foreach ($this->map as $key => $value) {
            $selectColumns .= "{$key} AS {$value},\n";
        }
        $selectColumns = rtrim($selectColumns, ",\n");

        return "SELECT 
                $selectColumns
                FROM catalog_products_images JOIN catalog_products ON catalog_products.code = catalog_products_images.product_code";
    }

}