<?php

/**
 * Copyright (c) 2021/6
 * Created by Federico Papetti
 */

namespace FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities;

use FourWords\AppManager\Director\FWUtils;
use FourWords\AppManager\SqliteGen\Model;
use Illuminate\Database\Schema\Blueprint;

class PriceList extends Model
{
    public $sqliteTable = 'price_list';
    protected $primaryKey = ['code','product','currency','extension','feature','brand','start_date'];
    public $incrementing = false;
    public $timestamps = false;

    // Velocizzo gli insert
    protected $useMutators = false;
    protected $logQueries = false;

    protected function onInsertRecord(&$record)
    {
        $record['start_date'] = FWUtils::asDateToTimestamp($record['start_date']);
        $record['end_date'] = FWUtils::asDateToTimestamp($record['end_date']);
    }

    protected function autoImportUp(Blueprint $table)
    {
        $table->string('code');
        $table->string('product')->nullable();
        $table->string('currency')->nullable();
        $table->string('extension')->nullable();
        $table->string('feature')->nullable();
        $table->string('brand')->nullable();
        $table->double('start_date')->default(0);
        $table->double('end_date')->default(0);
        $table->double('gross_price')->default(0);
        $table->double('discount_1')->default(0);
        $table->double('discount_2')->default(0);
        $table->double('discount_3')->default(0);
        $table->double('discount_4')->default(0);
        $table->double('discount_5')->default(0);
        $table->double('discount_6')->default(0);
        $table->double('discount_7')->default(0);
        $table->double('discount_8')->default(0);
        $table->string('discount_type_1')->nullable();
        $table->string('discount_type_2')->nullable();
        $table->string('discount_type_3')->nullable();
        $table->string('discount_type_4')->nullable();
        $table->string('discount_type_5')->nullable();
        $table->string('discount_type_6')->nullable();
        $table->string('discount_type_7')->nullable();
        $table->string('discount_type_8')->nullable();
        $table->double('target_qta_1')->default(0);
        $table->double('target_qta_2')->default(0);
        $table->double('target_qta_3')->default(0);
        $table->double('target_qta_4')->default(0);
        $table->double('target_qta_5')->default(0);
        $table->double('target_qta_6')->default(0);
        $table->double('target_discount_1')->default(0);
        $table->double('target_discount_2')->default(0);
        $table->double('target_discount_3')->default(0);
        $table->double('target_discount_4')->default(0);
        $table->double('target_discount_5')->default(0);
        $table->double('target_discount_6')->default(0);
        $table->string('targets_discount_type')->nullable();
    }

    protected function autoImportQuery()
    {
        return "
SELECT 
    CLISML as code,
    CDARML as product,
    CDVAML as currency,
    CTGMML as extension,
    CLASML as feature,
    CDMAML as brand,
    DTUAML as start_date,
    DTLVML as end_date,
    PRZLML as gross_price,
    -1*AS01ML as discount_1,
    -1*AS02ML as discount_2,
    -1*AS03ML as discount_3,
    -1*AS04ML as discount_4,
    -1*AS05ML as discount_5,
    SCCAML as discount_6,
    '' as discount_7,
    AUPRML as discount_8,
    CASE WHEN FAUSML = '' THEN '%' ELSE FAUSML END AS discount_type_1,
    CASE WHEN FAUSML = '' THEN '%' ELSE FAUSML END AS discount_type_2,
    CASE WHEN FAUSML = '' THEN '%' ELSE FAUSML END AS discount_type_3,
    CASE WHEN FAUSML = '' THEN '%' ELSE FAUSML END AS discount_type_4,
    CASE WHEN FAUSML = '' THEN '%' ELSE FAUSML END AS discount_type_5,
    FSCCML as discount_type_6,
    '' as discount_type_7,
    FAUPML as discount_type_8,
    TAO1ML as target_qta_1,
    TAO2ML as target_qta_2,
    TAO3ML as target_qta_3,
    TAO4ML as target_qta_4,
    TAO5ML as target_qta_5,
    TAO6ML as target_qta_6,
    SCT1ML as target_discount_1,
    SCT2ML as target_discount_2,
    SCT3ML as target_discount_3,
    SCT4ML as target_discount_4,
    SCT5ML as target_discount_5,
    SCT6ML as target_discount_6,
    CASE WHEN FTAOML = '' THEN '%' ELSE FTAOML END AS targets_discount_type
FROM ".config('ioc.as.library')."MGLIS01F 
WHERE DTUAML <= ".date('Ymd')." AND (DTLVML = 0 OR DTLVML >= ".date('Ymd').") AND CLISML <> ''";
    }

}
