<?php

/**
 * Copyright (c) 2020/2
 * Created by Federico Papetti
 */

namespace FourWords\AppManager\SqliteGen\Databases\IOC\Generic;

use FourWords\AppManager\Catalog\Model\Category;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\AttributesProducts;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\AttributesValues;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\Categories;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\CategoriesParents;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\CategoriesProducts;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\Features;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\FeaturesCollections;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\FeaturesProducts;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\Images;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\ImagesCollections;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\Languages;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\Parameters;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\Payments;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\PriceList;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\Products;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\ProductsBarcodes;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\ProductsHaveResources;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\ProductsRelated;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\Resources;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\ResourcesCollections;
use FourWords\AppManager\SqliteGen\Databases\IOC\Generic\Entities\ShippingModes;
use FourWords\AppManager\SqliteGen\IocSqliteDB;

class Generic extends IocSqliteDB {

    public $order = 1;
    public $exportUncompressedDb = false;
    public $exportCompressedDb = false;
    public $timestampVersioning = false;
    public $keepOnlyInTemp = true;

    public $hasOwners = false;

    public function populate() {

        // Tabelle di base
        Languages::populate([['iso' => 'it', 'name' => 'Italiano (Italian)', 'col_suffix' => 'lang_1','main' => true]]);
        Parameters::populate();

        /** Dati recuperati dal Mysql, precedentemente estratti dall'ERP con classi Importer */
        Products::autoImport();
        ProductsBarcodes::autoImport();
        AttributesValues::autoImport();
        AttributesProducts::autoImport();
        ProductsRelated::autoImport();

        // Feature
        FeaturesCollections::populate([['name_lang_1' => 'Caratteristiche']]);
        Features::autoImport();
        FeaturesProducts::autoImport();

        // Immagini
        ImagesCollections::autoImport();
        Images::autoImport();

        // Risorse aziendali
        ResourcesCollections::autoImport();
        Resources::autoImport();
        ProductsHaveResources::autoImport();

        // Categorie gerarchiche
        $this->generateCategoriesAndTrees();

        /** Dati recuperati direttamente da ERP */
        Payments::autoImport();
        ShippingModes::autoImport();
        PriceList::autoImport();

    }

    public function getNameOfExportedDatabase(): string
    {
        return $this->getName() . '.sqlite';
    }

    public function getNameOfExportedDatabaseWithoutExtension(): string
    {
        return $this->getName();
    }

    protected function generateCategoriesAndTrees($excludeEmptyCategories = true)
    {
        $catalogCategories = Category::with('subcategories')->get();
        $categoriesData = [];
        $categoriesParentsData = [];

        foreach ($catalogCategories as $category) {

            $productsTotalNumber = 0;
            Category::getAllSubcategoryProductsCount($category, $productsTotalNumber);

            // Escludo la categoria se non ho un prodotto collegato a essa o alle sue sotto categorie
            if ($excludeEmptyCategories && $productsTotalNumber == 0) continue;

            $subCategoriesList = [];
            Category::getAllSubcategoryIds($category, $subCategoriesList);

            $categoriesData[] = [
                'id' => $category->id,
                'code' => $category->code,
                'subcategories_list' => implode(",",$subCategoriesList),       // Id separati da virgola di tutte le sottocategorie in modo ricorsivo fino all'ultima
                'subcategories_total' => $category->subcategories->count(),             // Conta delle categorie direttamente sotto la categoria attuale
                'name_lang_1' => $category->name,
                'products_total' => $productsTotalNumber
            ];

            $categoriesParentsData[] = [
                'category' => $category->id,
                'parent' => $category->parent_id ?? 0
            ];

        }

        Categories::populate($categoriesData);
        CategoriesParents::populate($categoriesParentsData);
        CategoriesProducts::autoImport();
    }

}
