<?php

/**
 * Copyright (c) 2021/6
 * Created by Federico Papetti
 */

namespace FourWords\AppManager\SqliteGen\Databases\IOC\Specific\Entities;

use Illuminate\Database\Schema\Blueprint;
use FourWords\AppManager\SqliteGen\Model;

class CustomerInfo extends Model
{
    public $sqliteTable = 'customer_info';
    protected $primaryKey = 'id';
    public $incrementing = false;
    public $timestamps = false;

    protected $throwIfEmptyResult = true;

    protected function autoImportUp(Blueprint $table)
    {
        $table->unsignedBigInteger('id');
        $table->string('type')->nullable();
        $table->string('code')->nullable();
        $table->string('company')->nullable();
        $table->string('address')->nullable();
        $table->string('city')->nullable();
        $table->string('state')->nullable();
        $table->string('cap')->nullable();
        $table->string('vat')->nullable();
        $table->string('cf')->nullable();
        $table->string('agent')->nullable();
        $table->string('subagent')->nullable();
        $table->string('template')->nullable();
        $table->string('pricelist')->nullable();
        $table->string('pricelist_str')->nullable();
        $table->double('currency')->default(0);
        $table->string('currency_desc')->nullable();
        $table->string('feature_1')->nullable();
        $table->string('feature_2')->nullable();
        $table->string('feature_3')->nullable();
        $table->string('feature_4')->nullable();
        $table->string('feature_5')->nullable();
        $table->string('phone')->nullable();
        $table->string('email')->nullable();
        $table->double('discount_1')->default(0);
        $table->double('discount_2')->default(0);
        $table->string('bill_to')->nullable();
        $table->string('payment_code')->nullable();
        $table->string('iso_code')->nullable();
        $table->string('lang_code')->nullable();
        $table->string('iban')->nullable();
        $table->string('closure_day')->nullable();
        $table->integer('payment_status')->default(0);
        $table->string('shipping_mode')->nullable();
        $table->integer('transport_mode')->default(0);
        $table->string('packaging_mode')->nullable();
        $table->integer('to_expire')->default(0);
        $table->integer('expired')->default(0);
        $table->string('person')->nullable();
        $table->string('subagent2')->nullable();
        $table->string('subagent3')->nullable();
        $table->string('hq')->nullable();
        $table->string('zone')->nullable();
        $table->string('delivery_mode')->nullable();
        $table->integer('credit')->default(0);
        $table->string('cod_discount_1')->nullable();
        $table->string('cod_discount_2')->nullable();
        $table->string('fax')->nullable();
        $table->string('cell')->nullable();
        $table->string('suspended')->nullable();
        $table->string('country')->nullable();
        $table->string('last_visit')->nullable();
        $table->string('last_invoice')->nullable();
        $table->double('latitude')->default(0);
        $table->double('longitude')->default(0);
        $table->double('current_ordered')->default(0);
        $table->double('previous_ordered')->default(0);
        $table->double('previous_year_ordered')->default(0);

    }

    protected function autoImportQuery()
    {
        return "SELECT DISTINCT 
					A2.CLFOCP AS type,
					A1.CONTCA AS code,
					A2.DSCOCP AS company,
					A1.INDICA AS address,
					A1.LOCACA AS city,
					A1.PROVCA AS state,
					A1.CAPOCA AS cap,
					CONCAT(A1.CISOCA, A1.PIVACA) AS vat,
					A1.CDFICA AS cf,
					A1.CDAGCA AS agent,
					A1.CDSACA AS subagent,
					'' AS template,
					A1.CDLICA AS pricelist,
					E.CDL2AD AS pricelist_str,
					A1.CDVACA AS currency,
					SUBSTRING(D.XDATTB, 48, 50) AS currency_desc,
					A1.CLA1CA AS feature_1,
					A1.CLA2CA AS feature_2,
					A1.CLA3CA AS feature_3,
					A1.CLA4CA AS feature_4,
					A1.CLA5CA AS feature_5,
					A1.NTELCA AS phone,
					A1.INELCA AS email,
					A1.SCC1CA AS discount_1,
					A1.SCC2CA AS discount_2,
					E.CDGCAD AS bill_to,
					IFNULL(IFNULL(NULLIF(A1.CDPGCA, ''), A3.CDPGCA), '') AS payment_code,
					A1.CISOCA AS iso_code,
					A1.CDLNCA AS lang_code,
					A1.IBANCA AS iban,
					E.GCHIAD AS closure_day,
					0 AS payment_status,
					A1.MSPECA AS shipping_mode,
					A1.MCONCA AS transport_mode,
					A1.MIMBCA AS packaging_mode,
					0 AS to_expire,
					0 AS expired,
					A1.PEFSCA AS person,
					'' AS subagent2,
					'' AS subagent3,
					'' AS hq,
					'' AS zone,
					TMECAD AS delivery_mode,
					0 AS credit,
					'' AS cod_discount_1,
					'' AS cod_discount_2,
                    A1.NFAXCA AS fax,
                    A1.SWEBCA AS cell,
                    GESTCP AS suspended,
                    '' AS country,
                    '' AS last_visit,
                    '' AS last_invoice,
                    '' AS latitude,
                    '' AS longitude,
                    '' AS current_ordered,
                    '' AS previous_ordered,
                    '' AS previous_year_ordered
					FROM 
					". config('ioc.as.library') ."CGANA00F A1 
                    JOIN ". config('ioc.as.library') ."CGPCO00F A2 ON A1.CONTCA = A2.CONTCP 
					LEFT JOIN ". config('ioc.as.library') ."CGANA02F B ON A1.CONTCA = B.CONTAC 
					LEFT JOIN ". config('ioc.as.library') ."SMTAB00F C ON C.XCODTB = A1.NAZICA AND C.XTIPTB = '01NZ' 
					LEFT JOIN ". config('ioc.as.library') ."SMTAB00F D ON D.XCODTB = A1.CDVACA AND D.XTIPTB = '01VA' 
					LEFT JOIN ". config('ioc.as.library') ."CGANA03F E ON A1.CONTCA = E.CONTAD 
					LEFT JOIN ". config('ioc.as.library') ."CGANA00F A3 ON E.CDGCAD = A3.CONTCA  
					WHERE 
					CLFOCP IN ('C', 'D')
					AND A1.CONTCA = '".$this->owner."' OR E.CDGCAD = '".$this->owner."'";
                    //  AND GESTCP = '' AND IFNULL(BLORAC, '') = ''
    }

}
