<?php

/**
 * Copyright (c) 2023/3
 * Created by Marco Santambrogio
 */

namespace FourWords\AppManager\SqliteGen\Databases\IOC\Specific\Entities;

use FourWords\AppManager\Director\FWUtils;
use FourWords\AppManager\SqliteGen\Model;
use Illuminate\Database\Schema\Blueprint;

class Stock extends Model
{
    public $sqliteTable = 'stock2';
    protected $primaryKey = 'id';
    public $incrementing = true;
    public $timestamps = false;

    // Velocizzo gli insert
    protected $useMutators = false;
    protected $logQueries = false;

    protected function onInsertRecord(&$record)
    {
        $record['last_purchase1'] = FWUtils::asDateToTimestamp($record['last_purchase1']);
        $record['last_purchase2'] = FWUtils::asDateToTimestamp($record['last_purchase2']);
        $record['last_purchase3'] = FWUtils::asDateToTimestamp($record['last_purchase3']);
    }

    protected function autoImportUp(Blueprint $table)
    {
        $table->unsignedBigInteger('id', true);
        $table->string('document_type');
        $table->string('movement_type');
        $table->string('customer');
        $table->string('product');
        $table->double('gross_price')->default(0);
        $table->double('reduced_price')->default(0);
        $table->double('stock_discount1')->default(0);
        $table->double('stock_discount2')->default(0);
        $table->double('stock_discount3')->default(0);
        $table->double('stock_discount4')->default(0);
        $table->string('last_purchase1')->default(0);
        $table->string('last_purchase2')->default(0);
        $table->string('last_purchase3')->default(0);
        $table->double('history_qta_sell')->default(0);
        $table->double('history_qta_free')->default(0);
        $table->double('qta_sell')->default(0);
        $table->double('qta_shipped')->default(0);
        $table->double('cartons_sell')->default(0);
        $table->double('qta_free')->default(0);
//        $table->string('destination')->nullable();
//        $table->string('last_purchase_price1')->nullable();
//        $table->string('last_purchase_price2')->nullable();
//        $table->string('last_purchase_price3')->nullable();
//        $table->string('last_purchase_qta1')->nullable();
//        $table->string('last_purchase_qta2')->nullable();
//        $table->string('last_purchase_qta3')->nullable();
//        $table->string('order_num1')->nullable();
//        $table->string('order_num2')->nullable();
        $table->string('doc_num')->nullable();

        $table->index(['product', 'customer']);
    }

    protected function autoImportQuery()
    {
        return "
SELECT 
    TES.TDOCOO AS document_type,
    RIG.TIMOOO AS movement_type,
    case when CSPEOO <> ' ' then CSPEOO else CDCFOO end as customer,
    CDAROO AS product,
    PRZUOO AS gross_price,
    PZNEOO AS reduced_price,
    SCN1OO AS stock_discount1,
    SCN2OO AS stock_discount2,
    SCN3OO AS stock_discount3,
    SCN4OO AS stock_discount4,
    DTOROO AS last_purchase1,
    '' AS last_purchase2,
    '' AS last_purchase3,
    '' AS history_qta_sell,
    '' AS qta_free,
    QTOROO AS qta_sell,
    QTGSOO AS qta_shipped,
    TES.NROROO AS doc_num
FROM ".config('ioc.as.library')."OCMOV00F TES
JOIN ".config('ioc.as.library')."OCMOV01F RIG ON RIG.CDDTOO = TES.CDDTOO AND TES.NROROO = RIG.NROROO AND TES.TDOCOO = RIG.TDOCOO
WHERE TES.CDDTOO  = '01' AND (CDCFOO = '".$this->owner."' OR CSPEOO = '".$this->owner."') AND CDAROO IS NOT NULL AND CDAROO != '' AND DTOROO >= ".date('Ymd',strtotime('-365 days'));
    }

}
