<?php

/**
 * Copyright (c) 2020/2
 * Created by Federico Papetti
 */


namespace FourWords\AppManager\SqliteGen;

use FourWords\AppManager\Director\FWUtils;
use FourWords\AppManager\Ioc\Ps\Traits\PSConnection;
use Illuminate\Database\Capsule\Manager;
use FourWords\AppManager\SqliteGen\Exceptions\HandledException;
use ZipArchive;

/**
 * Class IcmSQLiteDB
 * @package SqliteGen
 *
 * Utilizzando questa classe il db verrà esportato come nel modulo icm, nella cartella icm/sqlite del prestashop specificato in icm.ps.icmSqliteDir, con relativo pathing e emptyDB
 */
class IcmSQLiteDB extends SQLiteDB
{

    use PSConnection;

    public $shouldExportEmptyDB = true;

    protected $didExportEmptyDB = false;
    protected $exportedICMDatabaseFullPath;

    public function getNameOfExportedDatabase() {
        return 'icmdb.db';
    }

    public function exportDir() {
        return config('icm.ps.icmSqliteDir');
    }

    public function checkICMFolder() {

        // Se la cartella sqlite in modules/icm non esiste vuol dire che il modulo non è installato
        if (!file_exists($this->exportDir())) {
            throw new HandledException('La cartella di esportazione "'.$this->exportDir().'" non esiste, il modulo ICM non è installato?');
        }
    }

    public function getExportedDatabaseFullPath() {

        $exportDir = FWUtils::addTrailingSlashes($this->exportDir());

        /**  Se è impostato un owner il path per esportarlo è modules/icm/sqlite/$owner/icmdb.db */
        if ($this->owner) {
            $this->exportedICMDatabaseFullPath = $exportDir . $this->owner . '/' . $this->getNameOfExportedDatabase();
        } else {
            /** Altrimento è modules/icm/sqlite/icmdb.db */
            $this->exportedICMDatabaseFullPath = $exportDir . $this->getNameOfExportedDatabase();
        }

        return $this->exportedICMDatabaseFullPath;

    }

    public function export() {

        $this->checkICMFolder();

        // Se sto forzando il rilascio, mi assicuro di aver valorizzato il path di esportazione icm
//        if (!$this->exportedICMDatabaseFullPath) {
        $this->getExportedDatabaseFullPath();
//        }

        $exportDir = FWUtils::addTrailingSlashes($this->exportDir());

        $timestamp = time();

        $zip = new ZipArchive;

        $prefixWithoutExt = substr($this->getNameOfExportedDatabase(),0,strpos($this->getNameOfExportedDatabase(),'.'));

        /**  Se è impostato un owner esporto il db in modules/icm/sqlite/$owner/icmdb.db con relativo zip */
        if ($this->owner) {
            if (!file_exists($exportDir . $this->owner)) {
                mkdir($exportDir . $this->owner, config('sqlitegen.databases.dirPermissions'));
            }
            if ($this->exportUncompressedDb) {
                copy($this->getDatabaseFullPath(), $this->exportedICMDatabaseFullPath);
                chmod($this->exportedICMDatabaseFullPath, config('sqlitegen.databases.filePermissions'));
                $this->sqliteGenerator->logger->debug('Esportato in ' . $this->exportedICMDatabaseFullPath);
            }

            // Zippo il file con il timestamp
            if ($zip->open($exportDir . $this->owner . '/' . $prefixWithoutExt. '.'.$timestamp.'.zip', ZipArchive::CREATE) === TRUE)
            {
                $zip->addFile($this->getDatabaseFullPath(),$this->getNameOfExportedDatabase());
                $zip->close();
                $this->sqliteGenerator->logger->debug('Zippato come '.$this->getNameOfExportedDatabase());

                chmod($exportDir . $this->owner . '/' . $prefixWithoutExt. '.'.$timestamp.'.zip', config('sqlitegen.databases.filePermissions'));

            } else {
                throw new HandledException('Impossibile creare l\'archivio del database icmdb');
            }

            /** Nell'icm l'emptyDB è il database per agente senza dati */
            /** Copio il db agente nella cartella emptyDB, lo svuoto e creo il relativo zip */
            if ($this->shouldExportEmptyDB && !$this->didExportEmptyDB) {

                $this->sqliteGenerator->logger->debug('Esporto l\'emptyDB');

                $this->didExportEmptyDB = true;

                if (!file_exists($exportDir . 'emptyDB/'))
                    mkdir($exportDir . 'emptyDB/', config('sqlitegen.databases.dirPermissions'));

                copy($this->getDatabaseFullPath(), $exportDir . 'emptyDB/'.$this->getNameOfExportedDatabase());
                chmod($exportDir . 'emptyDB/'.$this->getNameOfExportedDatabase(), config('sqlitegen.databases.filePermissions'));
                $this->sqliteGenerator->logger->debug('Esportato in '.$exportDir . 'emptyDB/'.$this->getNameOfExportedDatabase());

                $capsule = new Manager();
                $capsule->addConnection(['driver' => 'sqlite',
                    'database' => $exportDir . 'emptyDB/'.$this->getNameOfExportedDatabase()
                ], 'emptyDB');

                $tables = $capsule->getConnection('emptyDB')->select('SELECT name FROM sqlite_master WHERE type=\'table\'');
                foreach ($tables as $table) {
                    $capsule->getConnection('emptyDB')->select('DELETE FROM '.$table->name); // Il truncate non funziona
                }

                // Zippo il file con il timestamp
                if ($zip->open($exportDir . 'emptyDB/'.$this->getNameOfExportedDatabase().'.zip', file_exists($exportDir . 'emptyDB/'.$this->getNameOfExportedDatabase().'.zip') ? ZipArchive::OVERWRITE : ZipArchive::CREATE) === TRUE) {
                    $zip->addFile($exportDir . 'emptyDB/'.$this->getNameOfExportedDatabase(),$this->getNameOfExportedDatabase());
                    $zip->close();

                    chmod($exportDir . 'emptyDB/'.$this->getNameOfExportedDatabase().'.zip', config('sqlitegen.databases.filePermissions'));

                } else {
                    throw new HandledException('Impossibile creare l\'archivio del database icmdb');
                }

            }

        } else {
            /** Altrimento lo esporto in modules/icm/sqlite/icmdb.db con relativo zip */

            if ($this->exportUncompressedDb) {
                copy($this->getDatabaseFullPath(), $this->exportedICMDatabaseFullPath);
                chmod($this->exportedICMDatabaseFullPath, config('sqlitegen.databases.filePermissions'));
                $this->sqliteGenerator->logger->debug('Esportato in ' . $this->exportedICMDatabaseFullPath);
            }

            // Zippo il file con il timestamp
            if ($zip->open($exportDir . $prefixWithoutExt. '.'.$timestamp.'.zip', ZipArchive::CREATE) === TRUE)
            {
                $zip->addFile($this->getDatabaseFullPath(),$this->getNameOfExportedDatabase());
                $zip->close();
                $this->sqliteGenerator->logger->debug('Zippato come '.$this->getNameOfExportedDatabase());

                chmod($exportDir . $prefixWithoutExt . '.'.$timestamp.'.zip', config('sqlitegen.databases.filePermissions'));


            } else {
                throw new HandledException('Impossibile creare l\'archivio del database icmdb');
            }

        }

        // Rimuovo le vecchie versioni
        $this->removeOlderVersions();

    }
}
