<?php

/**
 * Copyright (c) 2020/2
 * Created by Federico Papetti
 */


namespace FourWords\AppManager\SqliteGen;

use FourWords\AppManager\SqliteGen\Parameters\ParametersInterface;

/**
 * Class IcmClientiSQLiteDB
 * @package SqliteGen
 *
 * Utilizzando questa classe il db verrà esportato compresso e versionato, con il nome icmdb-clienti
 */
class IocSqliteDB extends SQLiteDB
{

    public $exportUncompressedDb = false;
    public $exportCompressedDb = true;
    public $timestampVersioning = true;
    /** @var ParametersInterface  */
    protected $parametersRepository = null;

    /**
     * @return ParametersInterface
     */
    public function parameters(): ParametersInterface
    {
        if ($this->parametersRepository && !empty($this->parametersRepository->all())) { return $this->parametersRepository; }
        $this->parametersRepository = app()->make(ParametersInterface::class,[$this]);
        return $this->parametersRepository;
    }

    public function getNameOfExportedDatabase() {
        return 'icmdb-clienti.db';
    }

    public function getNameOfExportedDatabaseWithoutExtension() {
        return 'icmdb-clienti';
    }

    public function exportDir() {
        return config('sqlitegen.databases.exportDir');
    }

    /**
     * @param $categoriesNames array
     * @param false $removeSubcategories
     */
    public function removeCategories(array $categoriesNames, bool $removeSubcategories = false, bool $removeProducts = false) {

        $deleted = 0;

        $categories = $this->getConnection()->table('categories')->select('id', 'code', 'subcategories_list')->whereIn('code', $categoriesNames)->get('id', 'code', 'subcategories_list');

        if ($removeProducts) {
            $productsIds = $this->getConnection()->table('products')->join('categories_products','id','=','product')->whereIn('category',$categories->pluck('id'))->select('id')->get('id')->pluck('id');
            $deleted += $this->removeProducts($productsIds->toArray());
        }

        $deleted = $this->getConnection()->table('categories')->whereIn('code',$categoriesNames)->delete();
        $this->getConnection()->table('categories_products')->whereIn('category',$categories->pluck('id'))->delete();
        $this->getConnection()->table('categories_parents')->whereIn('category',$categories->pluck('id'))->delete();

        if ($removeSubcategories) {
            $subCategoriesIds = explode(",",join(",",$categories->pluck('subcategories_list')->toArray()));
            $deleted += $this->removeSubCategories($subCategoriesIds);

            if ($removeProducts) {
                $chunk = collect($subCategoriesIds)->chunk(500);
                $productsIds = collect([]);
                foreach ($chunk as $ids) {
                    $productsIds = $productsIds->merge($this->getConnection()->table('products')->join('categories_products', 'id', '=', 'product')->whereIn('category', $ids)->select('id')->get('id')->pluck('id'));
                }
                $this->removeProducts($productsIds->toArray());
            }
        }

        return $deleted;
    }

    public function removeSubCategories(array $subCategoriesIds) {
        $chunk = collect($subCategoriesIds)->chunk(500);
        $deleted = 0;
        foreach ($chunk as $ids) {
            $deleted += $this->getConnection()->table('categories')->whereIn('id',$ids)->delete();
            $this->getConnection()->table('categories_products')->whereIn('category',$ids)->delete();
            $this->getConnection()->table('categories_parents')->whereIn('category',$ids)->delete();
        }


        return $deleted;
    }

    public function removeProducts(array $productsIds) {
        $chunk = collect($productsIds)->chunk(500);
        $deleted = 0;
        foreach ($chunk as $ids) {
            $this->getConnection()->table('products')->whereIn('id',$ids)->delete();
        }

        return $deleted;
    }

}
