<?php
/**
 * Copyright (c) 2020/2
 * Created by Federico Papetti
 */

namespace FourWords\AppManager\SqliteGen\Providers;

use FourWords\AppManager\Ioc\Resource\ResourceRepository;
use FourWords\AppManager\SqliteGen\Archive\ArchiverInterface;
use FourWords\AppManager\SqliteGen\Archive\GZipArchiver;
use FourWords\AppManager\SqliteGen\Commands\GenerateDatabasesCommand;
use FourWords\AppManager\SqliteGen\Parameters\ParametersInterface;
use FourWords\AppManager\SqliteGen\Parameters\ParametersRepository;
use FourWords\AppManager\SqliteGen\SqliteGenerator;
use Illuminate\Support\ServiceProvider;

class SqliteGeneratorServiceProvider extends ServiceProvider
{

    public function register() {

        $this->app->singleton('sqlitegen', function($app) {
            return new SqliteGenerator();
        });

        $this->app->bind(ParametersInterface::class, function ($app, $params) {
            return new ParametersRepository($params[0]);
        });

        $this->app->bind(ArchiverInterface::class,function($app) {
            return new GZipArchiver();
        });

        if ($this->app->runningInConsole()) {
            $this->commands(GenerateDatabasesCommand::class);
        }

    }

    public function boot() {

        // Includo le configurazioni
        config(require __DIR__ . '/../../config/sqlitegen.php');

        // Includo le rotte
        $this->loadRoutesFrom(__DIR__ . '/../../routes/routes.php');

        /** @var ResourceRepository $resourceRepository */
        $resourceRepository = $this->app->make(ResourceRepository::class);
        $resourceRepository->addPath(config('sqlitegen.databases.exportDir'), ['db']);

    }

}
